const { stringify } = require('querystring');
const config = require('config');
const Promise = require('bluebird');
const bytes = require('bytes');
const mimeType = require('mime-types');
const AWS = require('aws-sdk');
const { homedir } = require('os');
const chokidar = require('chokidar');
const {
    join, isNil, compact, keys, reduce
} = require('lodash');
const {
    TYPE_FILE, TYPE_DIRECTORY, TYPE_SYMBOLIC, SQS_DEFAULT_REGION
} = require('../consts');

const RETRIABLE_ERRORS = ['MalformedXML', 'InvalidPart', 'NoSuchUpload', 'MethodNotAllowed'];
const getLogger = require('../logger/logger');

const logger = getLogger();

AWS.config.logger = {
    log: line => {
        logger.debug(line);
    }
};

chokidar.watch(join(homedir(), '.aws', 'credentials'), { usePolling: { interval: 3000 } })
    .on('change', path => {
        try {
            logger.info(`Shared credentials file ${path} changed, clearing cache...`);
            AWS.util.iniLoader.clearCachedFiles();
            logger.info('Shared credentials file cache cleared');
        } catch (err) {
            logger.error('Failed to clear shared credentials file cache', err);
        }
    });

AWS.events
    .on('retry', response => {
        if (RETRIABLE_ERRORS.includes(response.error.code)) {
            logger.warn('Retrying AWS S3 error:', response.error.code);
            response.error.retryable = true;
        }
    })
    .on('build', request => {
        request.httpRequest.headers['User-Agent'] = 'APN/1.0 NetApp/1.0 CloudSync/1.0';
    });

async function listBuckets(s3) {
    const response = await s3.listBuckets().promise();
    return response.Buckets.map(bucket => ({ name: bucket.Name }));
}

async function getBucketLocation(s3, bucket) {
    const response = await s3.getBucketLocation({ Bucket: bucket }).promise();
    return response.LocationConstraint === '' ? SQS_DEFAULT_REGION : response.LocationConstraint;
}

async function getBucketTagging(s3, bucket) {
    try {
        const response = await s3.getBucketTagging({ Bucket: bucket }).promise();
        return response.TagSet;
    } catch (err) {
        if (['NoSuchTagSet'].includes(err.code)) {
            return [];
        }
        throw err;
    }
}

async function headBucket(s3, bucket) {
    return new Promise((resolve, reject) => {
        s3.headBucket({ Bucket: bucket })
            .on('complete', res => {
                if (res.httpResponse.headers['x-amz-bucket-region']) {
                    resolve(res.httpResponse.headers['x-amz-bucket-region']);
                } else {
                    const { statusCode } = res.httpResponse;
                    reject(new Error(`Got unexpected status code ${statusCode} - for headBucket operation on bucket ${bucket}`));
                }
            })
            .send();
    });
}

async function deleteObject(s3, bucket, key) {
    return s3.deleteObject({ Bucket: bucket, Key: key }).promise();
}

function getReadStream(s3, bucket, key, size) {
    const rs = s3.getObject({ Bucket: bucket, Key: key }).createReadStream();

    // patch the stream coming from SDK with object size
    // without the size SDK will be unable to determine stream length
    // and will do multipart using default 5MB part size
    // which means files bigger then 50GB will have more then 10000 parts limit
    rs.length = size;

    return rs;
}

async function getObject(s3, bucket, key) {
    const { Body } = await s3.getObject({ Bucket: bucket, Key: key }).promise();
    return Body;
}

async function getObjectTagging(s3, bucket, key) {
    try {
        const tags = await s3.getObjectTagging({ Bucket: bucket, Key: key }).promise();
        return tags.TagSet;
    } catch (err) {
        // in case of a folder without tags returns NoSuchKey
        // in Intuit we get MethodNotAllowed
        if (['NoSuchKey', 'MethodNotAllowed'].includes(err.code)) {
            return [];
        }
        throw err;
    }
}

async function getObjectTags(s3, bucket, key) {
    const s3Tags = await getObjectTagging(s3, bucket, key);
    return reduce(s3Tags, (result, current) => (Object.assign(result, { [(current.Key).toLowerCase()]: current.Value })), {});
}

async function headObject(s3, bucket, key) {
    try {
        const metadata = await s3.headObject({ Bucket: bucket, Key: key }).promise();
        return metadata.Metadata;
    } catch (err) {
        if (err.code === 'NotFound') { // in case of a folder without tags return NoSuchKey
            return {};
        }
        throw err;
    }
}

// returns array of only files and dirs
async function listObjects(s3, bucket, prefix, onlyDirectories, continuationToken, dirsAndFiles = []) {
    let fixedPrefix = isNil(prefix) || prefix === '' ? undefined : prefix;
    if (fixedPrefix) {
        fixedPrefix = fixedPrefix.endsWith('/') ? fixedPrefix : `${fixedPrefix}/`;
    }
    const objects = await s3.listObjects({
        Bucket: bucket, Prefix: fixedPrefix, Delimiter: '/', Marker: continuationToken
    }).promise();
    let updatedResult = dirsAndFiles;
    if (onlyDirectories) {
        updatedResult = compact(dirsAndFiles.concat(compact(objects.CommonPrefixes.map(commonPrefix => commonPrefix.Prefix.replace(/\/$/, '').replace(fixedPrefix, '')))));
    } else {
        const directories = objects.CommonPrefixes.map(resultPrefix => ({
            key: resultPrefix.Prefix,
            type: TYPE_DIRECTORY,
            size: 0
        }));
        const files = objects.Contents.map(obj => ({
            key: obj.Key, size: obj.Size, type: TYPE_FILE, lastModified: obj.LastModified
        }));
        updatedResult = compact(dirsAndFiles.concat(files.concat(directories)));
    }
    if (objects.IsTruncated) {
        return listObjects(s3, bucket, prefix, onlyDirectories, objects.NextMarker, updatedResult);
    }
    return updatedResult;
}

async function listObjectsV2(s3, bucket, prefix, onlyDirectories, continuationToken, dirsAndFiles = []) {
    let fixedPrefix = isNil(prefix) || prefix === '' ? undefined : prefix;
    if (fixedPrefix) {
        fixedPrefix = fixedPrefix.endsWith('/') ? fixedPrefix : `${fixedPrefix}/`;
    }
    const objects = await s3.listObjectsV2({
        Bucket: bucket, Prefix: fixedPrefix, Delimiter: '/', ContinuationToken: continuationToken
    }).promise();
    let updatedResult = dirsAndFiles;
    if (onlyDirectories) {
        updatedResult = compact(dirsAndFiles.concat(compact(objects.CommonPrefixes.map(commonPrefix => commonPrefix.Prefix.replace(/\/$/, '').replace(fixedPrefix, '')))));
    } else {
        const directories = objects.CommonPrefixes.map(resultPrefix => ({
            key: resultPrefix.Prefix,
            type: TYPE_DIRECTORY,
            size: 0
        }));
        const files = objects.Contents.map(obj => ({
            key: obj.Key, size: obj.Size, type: TYPE_FILE, lastModified: obj.LastModified
        }));
        updatedResult = compact(dirsAndFiles.concat(files.concat(directories)));
    }
    if (objects.IsTruncated) {
        return listObjectsV2(s3, bucket, prefix, onlyDirectories, objects.NextContinuationToken, updatedResult);
    }
    return updatedResult;
}

function upload(s3, bucket, bodyStream, key, options = {}) {
    const { tags, ...rest } = options;
    const contentType = mimeType.lookup(key) || 'application/octet-stream';

    const params = {
        ...{
            Bucket: bucket,
            Key: key,
            Body: bodyStream,
            ContentType: contentType
        },
        ...rest
    };
    return new AWS.S3.ManagedUpload({
        partSize: bytes(config.get('protocols.s3.partSize')),
        queueSize: config.get('protocols.s3.queueSize'),
        params,
        leavePartsOnError: false,
        tags,
        service: s3
    });
}

function createS3Metadata(entry, tags) {
    // metadata values must be strings -
    const metadata = {
        'netapp-cs-uid': `${entry.uid}`,
        'netapp-cs-gid': `${entry.gid}`,
        'netapp-cs-mode': `${entry.mode}`,
        'netapp-cs-mtimesec': `${entry.mtimeSec}`
    };

    if (entry.type === TYPE_SYMBOLIC) {
        metadata['netapp-cs-slink'] = 'True';
    }

    if (tags) {
        tags.forEach(tag => {
            const { Key, Value } = tag;
            metadata[Key] = Value;
        });
    }

    return metadata;
}

function calculateObjectStoragePath(dir, path) {
    return dir ? `${dir}/${path}` : path;
}

function createTagsArray(requestTags, entry) {
    const convertTag = {
        'netapp-cs-uid': 'uid', 'netapp-cs-gid': 'gid', 'netapp-cs-mode': 'mode', 'netapp-cs-mtimesec': 'mtimeSec'
    };

    const propertiesToTags = keys(convertTag).map(property => ({
        Key: property,
        Value: `${entry[convertTag[property]]}`
    }));
    if (entry.type === TYPE_SYMBOLIC) {
        propertiesToTags.push({ Key: 'netapp-cs-slink', Value: 'True' });
    }

    return requestTags ? requestTags.concat(propertiesToTags) : propertiesToTags;
}

function convertTagsArrayToString(tags) {
    const tagsObj = tags.reduce((accumulator, item) => {
        accumulator[item.Key] = item.Value;
        return accumulator;
    }, {});
    return stringify(tagsObj);
}

async function createMultipartUpload(s3, targetBucket, targetKey, extraParams) {
    const params = {
        Bucket: targetBucket,
        Key: targetKey
    };
    Object.assign(params, extraParams);
    const multipartUpload = await s3.createMultipartUpload(params).promise();

    return multipartUpload.UploadId;
}

async function completeMultipartUpload(s3, targetBucket, targetKey, copyPartResults, uploadId) {
    return s3.completeMultipartUpload(
        {
            Bucket: targetBucket,
            Key: targetKey,
            MultipartUpload: {
                Parts: copyPartResults
            },
            UploadId: uploadId
        }
    ).promise();
}

async function copyPart(s3, targetBucket, targetKey, copySource, partNum, uploadId, startBytes, endBytes) {
    return s3.uploadPartCopy({
        Bucket: targetBucket,
        CopySource: copySource,
        Key: targetKey,
        PartNumber: partNum,
        UploadId: uploadId,
        CopySourceRange: `bytes=${startBytes}-${endBytes}`
    }).promise();
}

async function copyRegularObject(s3, destinationBucket, copySource, destinationKey, extraParams) {
    const params = {
        Bucket: destinationBucket,
        CopySource: copySource,
        Key: destinationKey
    };
    Object.assign(params, extraParams);
    return s3.copyObject(params).promise();
}

async function putS3Object(s3, bucket, bodyStream, path) {
    return s3.putObject({ Bucket: bucket, Key: path, Body: bodyStream })
        .on('sign', request => {
            // ONTAP S3 dont know how to handle multipart upload and dont send 100-continue response -
            // we need to override the expect header so the sdk wont get stuck
            // need to change https to http because in case of http there must be body signning and then the sdk cant handle passThrough stream and only readStream
            logger.debug('deleting expect header and changing https to http');
            request.httpRequest.endpoint.protocol = 'http';
            request.httpRequest.endpoint.port = 80;
            request.httpRequest.endpoint.href = `http://${request.httpRequest.endpoint.host}/`;
            delete request.httpRequest.headers.Expect;
            delete request.httpRequest.headers.expect;
        });
}

async function restoreS3Object(s3, bucket, path, storageClassObject) {
    return s3.restoreObject({
        Bucket: bucket,
        Key: path,
        RestoreRequest: {
            Days: storageClassObject.days,
            GlacierJobParameters: {
                Tier: storageClassObject.tier
            }
        }
    }).promise();
}

module.exports = {
    listBuckets,
    getBucketTagging,
    getBucketLocation,
    headBucket,
    listObjects,
    listObjectsV2,
    getObjectTagging,
    createS3Metadata,
    calculateObjectStoragePath,
    headObject,
    getReadStream,
    deleteObject,
    getObject,
    createTagsArray,
    convertTagsArrayToString,
    upload,
    copyPart,
    completeMultipartUpload,
    createMultipartUpload,
    copyRegularObject,
    getObjectTags,
    restoreS3Object,
    putS3Object
};
