const cluster = require('cluster');
const AWS = require('aws-sdk');
const https = require('https');
const proxy = require('proxy-agent');
const {
    compact, mapValues, chunk, isArray, flatten
} = require('lodash');
const ms = require('ms');
const zlib = require('zlib');
const getLogger = require('../logger/logger');
const {
    SQS_DEFAULT_MESSAGE_VISIBILITY_TIMEOUT,
    SQS_DEFAULT_WAIT_TIMEOUT,
    SQS_MAX_NUMBER_OF_MESSAGES,
    SQS_DEFAULT_REGION,
    AWS_SQS_CREDENTIALS
} = require('../consts');

const logger = getLogger();
const messagesLogger = getLogger('messages');
const MAX_ITEMS_PER_BATCH = 10;

const RETRIABLE_ERRORS = ['AccessDenied', 'AWS.SimpleQueueService.NonExistentQueue', 'InvalidClientTokenId'];

AWS.events.on('retry', response => {
    if (response.request.service.config.endpoint === `sqs.${SQS_DEFAULT_REGION}.amazonaws.com` && RETRIABLE_ERRORS.includes(response.error.code)) {
        logger.debug('Retrying AWS SQS error:', response.error.code);
        response.error.retryable = true;
    }
});

logger.info('Proxy configured:', Boolean(process.env.http_proxy));

let sqsClient;

function getSQS() {
    if (!sqsClient) {
        sqsClient = new AWS.SQS({
            region: SQS_DEFAULT_REGION,
            accessKeyId: AWS_SQS_CREDENTIALS.ACCESS_KEY_ID,
            secretAccessKey: AWS_SQS_CREDENTIALS.SECRET_ACCESS_KEY,
            retryDelayOptions: {
                base: 5000
            },
            correctClockSkew: true,
            httpOptions: process.env.http_proxy ? {
                agent: proxy(process.env.http_proxy)
            } : { agent: new https.Agent({ keepAlive: true }) }
        });
    }
    return sqsClient;
}

async function deleteMessage(queueUrl, receiptHandle) {
    const sqs = getSQS();
    try {
        await sqs.deleteMessage({
            QueueUrl: queueUrl,
            ReceiptHandle: receiptHandle
        }).promise();
    } catch (err) {
        if (err.code === 'AWS.SimpleQueueService.NonExistentQueue') {
            logger.warn(`Queue ${queueUrl} does not exist`);
        } else {
            logger.error(`Failed to delete message ${queueUrl}`, err);
        }
    }
}

async function zip(stringToZip) {
    return new Promise((resolve, reject) => {
        zlib.deflate(stringToZip, (err, buffer) => {
            if (err) {
                reject(err);
            } else {
                resolve(buffer.toString('base64'));
            }
        });
    });
}

async function unzip(zippedString) {
    return new Promise((resolve, reject) => {
        const buffer = Buffer.from(zippedString, 'base64');
        zlib.unzip(buffer, (err, bufferUnzipped) => {
            if (err) {
                reject(err);
            } else {
                resolve(bufferUnzipped.toString());
            }
        });
    });
}

async function parseMessages(messages, queueUrl) {
    const parsedMessagesPromises = messages.map(async message => {
        let results;
        try {
            const body = message.MessageAttributes.zipped ? JSON.parse(await unzip(message.Body)) : JSON.parse((message.Body));
            const attributes = Object.assign(mapValues(message.MessageAttributes, value => value.StringValue),
                { approximateReceiveCount: message.Attributes.ApproximateReceiveCount });
            if (isArray(body)) {
                results = body.map(entry => ({
                    receiveTimeStamp: Date.now(),
                    lastVisibilityTimeoutChange: Date.now(),
                    body: entry,
                    receiptHandle: message.ReceiptHandle,
                    attributes
                }));
            } else {
                results = [{
                    receiveTimeStamp: Date.now(),
                    lastVisibilityTimeoutChange: Date.now(),
                    body,
                    receiptHandle: message.ReceiptHandle,
                    attributes
                }];
            }
        } catch (err) {
            logger.warn(`Failed to parse message body with ReceiptHandle: ${message.ReceiptHandle}`, err); // Don't log message body - could contain passwords
            deleteMessage(queueUrl, message.ReceiptHandle);
        }
        return results;
    });
    const parsedMessages = await Promise.all(parsedMessagesPromises);
    return compact(flatten(parsedMessages));
}

async function receiveParsedMessages(queueUrl, maxNumberOfMessages = SQS_MAX_NUMBER_OF_MESSAGES, waitTimeSeconds = SQS_DEFAULT_WAIT_TIMEOUT) {
    const sqs = getSQS();
    try {
        const response = await sqs.receiveMessage({
            QueueUrl: queueUrl,
            VisibilityTimeout: ms(SQS_DEFAULT_MESSAGE_VISIBILITY_TIMEOUT) / 1000,
            WaitTimeSeconds: waitTimeSeconds,
            MessageAttributeNames: ['All'],
            AttributeNames: ['ApproximateReceiveCount'],
            MaxNumberOfMessages: maxNumberOfMessages
        }).promise();
        return await parseMessages(response.Messages || [], queueUrl);
    } catch (err) {
        logger.error(`Failed to receive messages from queue ${queueUrl}`, err);
        throw err;
    }
}

async function receiveMessages(queueUrl) {
    // TODO Avoid zip when number of entries is low + cancel server zip CS-4537
    const messages = await receiveParsedMessages(queueUrl);

    messages.forEach(msg => {
        const { attributes: { approximateReceiveCount } } = msg;
        if (approximateReceiveCount > 1) {
            logger.warn(`Message received ${approximateReceiveCount} times, message`, JSON.stringify(msg));
        }
    });

    if (messages.length > 0 && cluster.isMaster) {
        logger.info('RECEIVE MESSAGES:', JSON.stringify(messages, null, 4));
        messagesLogger.info('RECEIVE MESSAGES:', JSON.stringify(messages, null, 4));
    }

    return messages;
}

async function sendMessage(queueUrl, message, messageAttributes = {}) {
    if (cluster.isMaster && messageAttributes.type !== 'health') {
        messagesLogger.info('SEND MESSAGE:', JSON.stringify({ message, messageAttributes }, null, 4));
        logger.info('SEND MESSAGE:', JSON.stringify({ message, messageAttributes }, null, 4));
    }
    const MessageAttributes = mapValues(messageAttributes, value => ({ StringValue: value, DataType: 'String' }));
    let MessageBody;
    if (queueUrl.includes('STATUSES')) {
        MessageBody = JSON.stringify(message);
        delete MessageAttributes.zipped;
    } else {
        MessageBody = await zip(JSON.stringify(message));
        MessageAttributes.zipped = { StringValue: 'True', DataType: 'String' };
    }

    const sqs = getSQS();
    return sqs.sendMessage({
        QueueUrl: queueUrl,
        MessageBody,
        MessageAttributes
    }).promise();
}

async function changeMessageVisibilityBatch(queueUrl, receiptHandles, visibilityTimeout = ms(SQS_DEFAULT_MESSAGE_VISIBILITY_TIMEOUT) / 1000) {
    const sqs = getSQS();
    const chunks = chunk(receiptHandles, MAX_ITEMS_PER_BATCH);
    const promises = chunks.map(chunky => {
        const Entries = chunky.map((receiptHandle, index) => ({
            Id: index.toString(),
            ReceiptHandle: receiptHandle,
            VisibilityTimeout: visibilityTimeout
        }));

        return sqs.changeMessageVisibilityBatch({
            Entries,
            QueueUrl: queueUrl
        }).promise();
    });
    const results = await Promise.all(promises);
    results.forEach(result => result.Failed.forEach(fail => logger.warn('Failed to change visibility timeout', fail)));
}

module.exports = {
    deleteMessage, receiveMessages, sendMessage, changeMessageVisibilityBatch, receiveParsedMessages
};
