const {
    intersectionWith, pullAllWith, get
} = require('lodash');
const config = require('config');
const {
    ONTAP_S3_PROVIDER, ALIBABA_PROVIDER, IBM_PROVIDER, SGWS_PROVIDER, S3_PROVIDER
} = require('../consts');

async function listS3ObjectsOnce(s3, bucket, prefix, continuationToken, useEtags) {
    const result = await s3.listObjects({
        Bucket: bucket, Prefix: prefix ? `${prefix}/` : undefined, Marker: continuationToken
    }).promise();
    if (continuationToken === undefined && prefix) { // In the first iteration, need to filter the key which equals to the prefix (in V2 we use StartAfter)
        result.Contents = result.Contents.filter(obj => obj.Key !== `${prefix}/`);
    }
    const objects = result.Contents.map(obj => ({
        path: prefix ? obj.Key.slice(prefix.length + 1) : obj.Key, size: obj.Size, mtimeSec: parseInt(obj.LastModified.getTime() / 1000), etag: useEtags ? obj.ETag.slice(1, -1) : undefined
    }));
    return [objects, result.NextMarker];
}

async function listS3ObjectsV2Once(s3, bucket, prefix, continuationToken, useEtags) {
    const result = await s3.listObjectsV2({
        Bucket: bucket, Prefix: prefix ? `${prefix}/` : undefined, ContinuationToken: continuationToken, StartAfter: prefix ? `${prefix}/` : undefined
    }).promise();
    const objects = result.Contents.map(obj => ({
        path: prefix ? obj.Key.slice(prefix.length + 1) : obj.Key, size: obj.Size, mtimeSec: parseInt(obj.LastModified.getTime() / 1000), etag: useEtags ? obj.ETag.slice(1, -1) : undefined
    }));
    return [objects, result.NextContinuationToken];
}

async function listBlobsOnce(containerClient, containerName, prefix, continuationToken/* , useEtags */) {
    const blobs = [];
    const iterator = containerClient.listBlobsFlat({ prefix: prefix ? `${prefix}/` : undefined }).byPage({ continuationToken, maxPageSize: 1000 });
    const response = (await iterator.next()).value;
    for (const blob of response.segment.blobItems) {
        blobs.push({
            path: prefix ? blob.name.slice(prefix.length + 1) : blob.name, size: blob.properties.contentLength, mtimeSec: parseInt(blob.properties.lastModified.getTime() / 1000)// , etag: useEtags ? blob.properties.etag : undefined
        });
    }
    return [blobs, response.continuationToken];
}

async function listGcpObjectsOnce(gcpClient, bucketName, prefix, continuationToken, useEtags) {
    const bucket = gcpClient.bucket(bucketName);
    const response = await bucket.getFiles({
        prefix: prefix ? `${prefix}/` : undefined, autoPaginate: false, pageToken: continuationToken
    });
    const objectsPromises = response[0].map(async object => {
        const metadata = await gcpClient.bucket(bucketName).file(object.name).getMetadata();
        const mtimeSec = new Date(get(metadata[0], 'updated'));
        // check if possible to avoid metadata call
        return {
            path: prefix ? object.name.slice(prefix.length + 1) : object.name, size: parseInt(metadata[0].size), mtimeSec: parseInt(mtimeSec.getTime() / 1000), etag: useEtags ? object.metadata.md5Hash : undefined
        };
    });

    const result = await Promise.all(objectsPromises);
    return [result, response[1] !== null ? response[1].pageToken : undefined];
}

async function listObjectsOnce(provider, protocol, client, bucket, prefix, continuationToken, useEtags) {
    if (continuationToken === null) { return [[], null]; }
    switch (protocol) {
        case 's3':
            switch (provider) {
                case S3_PROVIDER:
                case ALIBABA_PROVIDER:
                    return listS3ObjectsV2Once(client, bucket, prefix, continuationToken, useEtags);
                case IBM_PROVIDER:
                case SGWS_PROVIDER:
                case ONTAP_S3_PROVIDER:
                    return config.get(`protocols.${provider}.list-v2`)
                        ? listS3ObjectsV2Once(client, bucket, prefix, continuationToken, useEtags) : listS3ObjectsOnce(client, bucket, prefix, continuationToken, useEtags);
                default:
                    throw new Error(`Unknown provider ${provider} received ${provider === 's3'}`);
            }
        case 'azure':
            return listBlobsOnce(client, bucket, prefix, continuationToken, useEtags);
        case 'gcp':
            return listGcpObjectsOnce(client, bucket, prefix, continuationToken, useEtags);
        default:
            throw new Error(`Unknown protocol ${protocol} received`);
    }
}

async function isEndpointEmpty(provider, protocol, client, bucket, prefix) {
    const objects = (await listObjectsOnce(provider, protocol, client, bucket, prefix))[0];
    return objects.length === 0;
}

function areCloudObjectsEqual(obj1, obj2) {
    return obj1.etag ? obj1.path === obj2.path && obj1.etag === obj2.etag
        : obj1.path === obj2.path && obj1.size === obj2.size && obj1.mtimeSec < obj2.mtimeSec;
}

function areCloudObjectsEqualExceptMtime(obj1, obj2) {
    return obj1.etag ? obj1.path === obj2.path && obj1.etag === obj2.etag
        : obj1.path === obj2.path && obj1.size === obj2.size;
}

function filterEqualObjects(sourceObjects, targetObjects) {
    const equalObjects = intersectionWith(sourceObjects, targetObjects, areCloudObjectsEqual);
    pullAllWith(sourceObjects, equalObjects, areCloudObjectsEqualExceptMtime);
    pullAllWith(targetObjects, equalObjects, areCloudObjectsEqualExceptMtime);
    return [sourceObjects, targetObjects, equalObjects];
}

module.exports = {
    listObjectsOnce, isEndpointEmpty, filterEqualObjects, areCloudObjectsEqual
};
