const {
    readdir, lstat, unlink, chmod, chown
} = require('fs-extra');
const config = require('config');
const { cpus } = require('os');
const { join } = require('path');
const { compact, get } = require('lodash');
const Promise = require('bluebird');
const axios = require('axios/index');
const throat = require('throat');
const { escape } = require('querystring');
const { fsErrorsDescriptionObject } = require('./fs-error-descriptions');
const getLogger = require('../logger/logger');

const LOCAL_HOST = 'http://127.0.0.1';
const logger = getLogger();
let nfs4Acl; // in cases where we fail to install this module, we want to ignore the error and continue
try {
    nfs4Acl = require('nfs4-acl'); /* eslint-disable-line global-require, import/no-unresolved */
} catch (e) {
    logger.info('Could not require nfs4-acl package');
    nfs4Acl = undefined;
}

const {
    TYPE_FILE, TYPE_DIRECTORY, TYPE_SYMBOLIC, TYPE_OTHER
} = require('../consts');

function gracefulJoin(...paths) {
    return join(...compact(paths));
}

function getType(stats) {
    let type = TYPE_OTHER;

    if (stats.isFile()) {
        type = TYPE_FILE;
    } else if (stats.isDirectory()) {
        type = TYPE_DIRECTORY;
    } else if (stats.isSymbolicLink()) {
        type = TYPE_SYMBOLIC;
    }

    return type;
}

function getNfsAcl(path) {
    const acesToIgnore = config.get('protocols.nfs.acl.aces-prefix-to-ignore');
    return nfs4Acl.get(path, acesToIgnore);
}

function setNfsAcl(path, acl, isDir) {
    nfs4Acl.set(path, acl, isDir);
}

function getRandomPort() {
    const startRange = config.get('workers.acl-server.port-start-range');
    const numberInRange = Math.floor(Math.random() * (cpus().length * config.get('workers.acl-server.processes-cpu-ratio')));
    return startRange + numberInRange;
}

async function getCifsAcl(path) {
    const ACL_SERVER_PORT = getRandomPort();
    try {
        const { data } = await axios.get(`${LOCAL_HOST}:${ACL_SERVER_PORT}/smb/get-acl/${escape(path)}`);
        return data;
    } catch (e) {
        throw get(e, 'response.data.message', e);
    }
}

async function setCifsAcl(path, acl) {
    const ACL_SERVER_PORT = getRandomPort();
    try {
        return await axios.put(`${LOCAL_HOST}:${ACL_SERVER_PORT}/smb/set-acl`, { path, acl });
    } catch (e) {
        throw get(e, 'response.data.message', e);
    }
}

async function chmodAndChown(path, entry) {
    const targetPath = gracefulJoin(path, entry.path);
    await Promise.all([chmod(targetPath, entry.mode), chown(targetPath, entry.uid, entry.gid)]);
}

async function chownIfNeeded(path, uid, gid) {
    if (process.getuid() !== uid || process.getgid() !== gid) {
        try {
            await chown(path, uid, gid);
        } catch (err) {
            if (err.code === 'EINVAL') {
                // squashed NFS mount
                logger.warn('Unable to chown due to EINVAL', { path, uid, gid });
            } else {
                throw err;
            }
        }
    }
}

async function stat(path, entry, copyAcl) {
    const stats = await lstat(join(path, entry));
    const type = getType(stats);

    const statObj = {
        path: entry,
        type,
        size: (type === TYPE_DIRECTORY) ? 0 : stats.size,
        mode: stats.mode,
        uid: stats.uid,
        gid: stats.gid,
        mtimeSec: parseInt(stats.mtimeMs / 1000)
    };
    if (copyAcl === 'nfs') {
        Object.assign(statObj, { nfs4Acl: getNfsAcl(join(path, entry)) });
    }
    if (copyAcl === 'cifs') {
        Object.assign(statObj, { cifsAcl: await getCifsAcl(join(path, entry)) });
    }
    return statObj;
}

async function list(path, copyAcl) {
    const entries = await readdir(path);
    if (entries.length > 500000) {
        logger.error(`Huge directory found (${entries.length} entries):`, path);
    }
    const wrappedEntries = await Promise.all(entries.map(throat(100, async entry => {
        try {
            const value = await stat(path, entry, copyAcl);
            return { status: 'fulfilled', value };
        } catch (err) {
            return { status: 'rejected', reason: err };
        }
    })));
    return wrappedEntries.filter(entry => entry.status === 'fulfilled').map(entry => entry.value);
}

async function removeCoreFiles() {
    logger.info('Looking for core files...');
    try {
        const entries = await readdir('.');
        const coreFiles = entries.filter(entry => /^core\.\d+$/.test(entry));
        logger.info(`${coreFiles.length} core files found`);
        const promises = coreFiles.map(coreFile => unlink(coreFile));
        await Promise.all(promises);
    } catch (err) {
        logger.warn('Failed to remove core files', err);
    }
}

function fixErrorCodeIfNeeded(errorCode) {
    if (typeof (errorCode) === 'string' && errorCode.includes('.')) {
        return errorCode.replace(/\./g, '_');
    }
    return errorCode;
}

function generateErrorStatisticsObject(error) {
    const errorObject = {};
    if (get(error, 'errors[0].reason')) { // then probably it's a GCP related error
        errorObject.code = fixErrorCodeIfNeeded(error.errors[0].reason);
    } else if (get(error, 'details.errorCode')) { // then it's an Azure related error
        errorObject.code = fixErrorCodeIfNeeded(error.details.errorCode);
    } else if (error.code) { // if error code exists, most likely that it's realted to FS or S3.
        if (get(error, 'request.url')) { // if we have request.url with 'windows' inside, then it's azure
            const url = get(error, 'request.url');
            if (url.includes('windows')) {
                errorObject.code = fixErrorCodeIfNeeded(error.code);
            }
        } else if (fsErrorsDescriptionObject[error.code]) { // then probably it's file system
            errorObject.code = error.code;
            errorObject.description = fsErrorsDescriptionObject[error.code];
        } else if (error.extendedRequestId) { // then probably it's s3 error
            errorObject.code = fixErrorCodeIfNeeded(error.code);
            errorObject.description = error.message;
        } else {
            errorObject.code = 'unknownError';
        }
    } else {
        errorObject.code = 'unknownError';
    }
    return errorObject;
}

module.exports = {
    list, gracefulJoin, removeCoreFiles, chmodAndChown, chownIfNeeded, setNfsAcl, setCifsAcl, getCifsAcl, generateErrorStatisticsObject
};
