const {
    hostname, release
} = require('os');
const md5 = require('md5');
const {
    emptyDir, writeFile, readFile, lstat, createReadStream, pathExists
} = require('fs-extra');
const moment = require('moment');
const https = require('https');
const { pipeline } = require('stream');
const execute = require('../executor');
const {
    DATA_BROKER_RUNTIME_ABOUT, DATA_BROKER_RUNTIME_CONFIG, LOGS_DIRECTORY_RELATIVE_PATH, LOCAL_CONFIG_FILE
} = require('../consts');
const createZipDirectory = require('./zip-directory');
const getLogger = require('../logger/logger');

const logger = getLogger();

const X_HEADER_DATA_FILE = 'X-HEADER-DATA';
const PM2_LOGS_FILE = 'pm2logs.json';
const TEMPORARY_DIRECTORY = './tmp/asup';

async function getPm2Logs() {
    logger.debug('Getting pm2 logs');

    const pm2LogsContent = await execute('pm2 prettylist');
    logger.debug(`Writing Pm2 logs content to file: ${PM2_LOGS_FILE}`, pm2LogsContent);

    await writeFile(`${TEMPORARY_DIRECTORY}/${PM2_LOGS_FILE}`, pm2LogsContent);
}

async function buildHeaders(serialNumber) {
    logger.debug('Building ASUP headers');

    const hostOs = await execute('uname -a');

    return {
        'Content-Type': 'application/x-7z-compressed',
        'user-agent': 'filer/Netapp',
        'X-Netapp-asup-model-name': 'CloudSync',
        'X-Netapp-asup-os-version': release(),
        'X-Netapp-asup-product-line': 'NetApp',
        'X-Netapp-asup-host-os': `${hostOs.trim()}`,
        'X-Netapp-asup-system-id': DATA_BROKER_RUNTIME_CONFIG['data-broker-id'],
        'X-NetApp-asup-serial-num': serialNumber,
        'X-Netapp-asup-hostname': hostname(),
        'X-Netapp-asup-confidentiality': 'NetApp Confidential',
        'X-Netapp-asup-subject': `Cloud Sync Notification from ${hostname()} (USER_TRIGGERED) INFO`,
        'X-Netapp-asup-generated-on': moment().utc().format('ddd MMMM D HH:mm:ss ZZ YYYY'),
        'X-Netapp-asup-version': 3,
        'X-Netapp-asup-from': 'test',
        'X-Netapp-asup-sequence': 1,
        'X-Netapp-asup-product-build-no': DATA_BROKER_RUNTIME_ABOUT.version
    };
}

async function send(fileName, requestId, asupUrl, headers) {
    return new Promise((resolve, reject) => {
        const req = https.request({
            hostname: asupUrl,
            path: `/put/AsupPut/${fileName}`,
            method: 'PUT',
            headers

        }, res => {
            logger.info(`ASUP requestId - ${requestId}, status: ${res.statusCode}`);
            logger.debug(`ASUP request headers: ${JSON.stringify(res.headers)}`);

            resolve();
        });

        req.on('error', err => reject(err));

        pipeline(
            createReadStream(`${TEMPORARY_DIRECTORY}/${fileName}`),
            req,
            err => {
                if (err) {
                    reject(err);
                } else {
                    logger.debug('Streaming done');
                }
            }
        );
    });
}

async function handleAsupRequest(requestId, serialNumber, dataBrokerId, asupUrl) {
    logger.debug('Handling ASUP request: ', {
        requestId, serialNumber, dataBrokerId, asupUrl
    });

    await emptyDir(TEMPORARY_DIRECTORY);

    await getPm2Logs();

    const headers = await buildHeaders(serialNumber);

    await writeFile(`${TEMPORARY_DIRECTORY}/${X_HEADER_DATA_FILE}`, Object.entries(headers).map(entry => entry.join(': ')).join('\n'));

    const isLocalConfigFileExist = await pathExists(LOCAL_CONFIG_FILE);

    const name = `${DATA_BROKER_RUNTIME_CONFIG['data-broker-id']}-${moment().format('DD-MMMM-YYYY_hh-mm-ss')}`;

    const content = (isLocalConfigFileExist) ? [`${TEMPORARY_DIRECTORY}/${X_HEADER_DATA_FILE}`, `${TEMPORARY_DIRECTORY}/${PM2_LOGS_FILE}`, `${LOGS_DIRECTORY_RELATIVE_PATH}`, `${LOCAL_CONFIG_FILE}`] : [`${TEMPORARY_DIRECTORY}/${X_HEADER_DATA_FILE}`, `${TEMPORARY_DIRECTORY}/${PM2_LOGS_FILE}`, `${LOGS_DIRECTORY_RELATIVE_PATH}`];

    await createZipDirectory(TEMPORARY_DIRECTORY, name, content);

    const fileName = `${name}.7z`;
    const stat = await lstat(`${TEMPORARY_DIRECTORY}/${fileName}`);
    const data = await readFile(`${TEMPORARY_DIRECTORY}/${fileName}`);

    try {
        logger.info('Executing ASUP request: ', {
            fileName, requestId, serialNumber, dataBrokerId
        });

        await send(fileName, requestId, asupUrl, {
            ...headers,
            ...{
                'Content-Length': stat.size,
                'X-Netapp-asup-payload-checksum': md5(data)
            }
        });

        logger.debug('Done executing ASUP request');
    } catch (err) {
        logger.error('Failed to send ASUP request. ', err);
    }
}

module.exports = handleAsupRequest;
