const HttpsProxyAgent = require('https-proxy-agent');
const download = require('download');
const config = require('config');
const { readFile } = require('fs-extra');
const { chmod } = require('fs').promises;
const ms = require('ms');
const { getLocalVaultHealthStatus } = require('../vault/vault-operations');
const sleep = require('../../utils/utils');
const getLogger = require('../logger/logger');
const execute = require('../executor');

const logger = getLogger();

async function updateNodeVersion() {
    logger.info('Installed node version', process.versions.node);
    const aboutFile = await readFile(config.get('about-file'));
    const about = JSON.parse(aboutFile);
    logger.info('update node version - ABOUT file is', about);

    if (about.nodeVersion && about.nodeVersion !== process.versions.node) {
        logger.info('Installing node version', about.nodeVersion);
        try {
            await execute(`N_PREFIX=// n ${about.nodeVersion}`);
            logger.info('Removing all versions except the current version...');
            await execute('N_PREFIX=// n prune');
        } catch (err) {
            logger.error('Failed to update node', err);
        }
    }
}

async function updateVaultVersion(options) {
    // get current vault version
    const { version: currentVaultVersion } = await getLocalVaultHealthStatus();
    logger.info('Installed vault version', currentVaultVersion);
    const aboutFile = await readFile(config.get('about-file'));
    const about = JSON.parse(aboutFile);
    logger.info('update vault version - ABOUT file is', about);
    if (about.vaultVersion && about.vaultVersion !== currentVaultVersion) {
        logger.info('Installing vault version', about.vaultVersion);
        logger.info('>>> Stopping vault process <<<');
        await execute('pm2 stop vault');
        await sleep(ms('3s'));
        logger.info('>>> Downloading new version <<<');
        await download(`https://repo.cloudsync.netapp.com/vault_${about.vaultVersion}_linux_amd64.zip`, '/usr/bin', options);
        await chmod('/usr/bin/vault', '775');
        logger.info('>>> Restarting vault process <<<');
        await execute('pm2 restart vault');
        await sleep(ms('3s')); // give some time for vault to reboot
    }
}

async function performUpdate(bundleVersion, bundleUrl) {
    const options = process.env.http_proxy ? { extract: true, agent: { https: new HttpsProxyAgent(process.env.http_proxy) } } : { extract: true };
    try {
        logger.info('Downloading and extracting...', { bundleVersion, bundleUrl });
        await download(bundleUrl, '/opt/netapp/databroker', options);
        await sleep(ms('3s')); // give time for logger to restart
    } catch (err) {
        logger.error(`Failed to download and extract ${bundleUrl}`, err);
        throw err;
    }

    try {
        await updateNodeVersion();
    } catch (err) {
        logger.error('Failed to update node', err);
    }

    try {
        await updateVaultVersion(options);
    } catch (err) {
        logger.info('Failed to update vault version', err);
    }

    logger.info('Installing production modules...');
    await execute('npm i --production');

    try {
        logger.info('Install latest PM2 version...');
        await execute('npm i pm2 -g');
    } catch (err) {
        logger.error('Failed to install latest PM2 version', err);
    }

    logger.info('Restarting...');
    await execute('pm2 restart data-broker');
}

module.exports = performUpdate;
