const LRU = require('lru-cache');
const ms = require('ms');
const { readFileSync, pathExistsSync } = require('fs-extra');
const { get, isString } = require('lodash');
const {
    getSecret, createSecret, exportVaultSecretData
} = require('./vault-operations');
const getLogger = require('../logger/logger');
const {
    ALIBABA_PROVIDER, IBM_PROVIDER, SGWS_PROVIDER, ONTAP_S3_PROVIDER
} = require('../consts');

const options = { max: 100, maxAge: ms('10m') };
const cache = new LRU(options);

const logger = getLogger();

const META_DATA_FILE = './metadata.json';

async function writeMetadata(path, data) {
    logger.debug('Writing metadata for path: ', path);

    cache.set(path, data);
    await createSecret(path, data);
}

async function readMetadata(path) {
    logger.debug('Reading metadata for path: ', path);

    const data = cache.get(path);

    if (data) {
        return data;
    }

    logger.debug(`Metadata was not found in cache, try to get the data for path: ${path} from Vault`);
    let metadata = await getSecret(path);

    if (metadata) {
        cache.set(path, metadata);
    } else {
        logger.debug(`Metadata was not found in Vault, try to get the data for path: ${path} from file (old relationship)`);

        /* This part of code will be used to forward new relationships credentials from metadata.json to vault
           The migration will be used as lazy migration, first we will try to find the relevant path in the cache/vault.
           In case of miss we will try to find the file and read the credentials from it.
           Either the cache & vault will be update with the new data */

        if (pathExistsSync(META_DATA_FILE)) {
            logger.debug('metadata.json file exist, trying to extract metadata)');

            try {
                const [relationshipId, endPoint, provider] = path.split('.');
                metadata = get(JSON.parse(readFileSync(META_DATA_FILE)), `${relationshipId}.${endPoint}.${provider}`);

                if (isString(metadata)) {
                    await writeMetadata(path, { storageAccountConnectionString: metadata });
                } else {
                    await writeMetadata(path, metadata);
                }
            } catch (err) {
                logger.error('Could not parse metadata.json', err);
            }
        } else {
            logger.info('metadata.json file does not exist');
        }
    }

    return metadata;
}

async function registerMetadata(source, target, sourcePassword, targetPassword, relationshipId) {
    logger.debug(`Registering metadata for relationshipId: ${relationshipId}: `, { source, target });

    if (source.azure) {
        const { storageAccountName, storageAccountConnectionString } = source.azure;

        if (storageAccountName && storageAccountConnectionString) {
            await writeMetadata(`${relationshipId}.src.azure`, { storageAccountConnectionString });
        }
    }

    if (target.azure) {
        const { storageAccountName, storageAccountConnectionString } = target.azure;

        if (storageAccountName && storageAccountConnectionString) {
            await writeMetadata(`${relationshipId}.trg.azure`, { storageAccountConnectionString });
        }
    }

    if (source.cifs) {
        const {
            username, workgroup
        } = source.cifs;

        if (username && sourcePassword) {
            await writeMetadata(`${relationshipId}.src.cifs`, { username, password: sourcePassword, workgroup });
        }
    }

    if (target.cifs) {
        const {
            username, workgroup
        } = target.cifs;

        if (username && targetPassword) {
            await writeMetadata(`${relationshipId}.trg.cifs`, { username, password: targetPassword, workgroup });
        }
    }

    if (source.s3) {
        const {
            accessKey, secretKey, provider
        } = source.s3;

        if ([ALIBABA_PROVIDER, SGWS_PROVIDER, IBM_PROVIDER, ONTAP_S3_PROVIDER].includes(provider)) {
            if (accessKey && secretKey) {
                await writeMetadata(`${relationshipId}.src.${provider}`, { accessKey, secretKey });
            }
        }
    }

    if (target.s3) {
        const {
            accessKey, secretKey, provider
        } = target.s3;

        if ([ALIBABA_PROVIDER, SGWS_PROVIDER, IBM_PROVIDER, ONTAP_S3_PROVIDER].includes(provider)) {
            if (accessKey && secretKey) {
                await writeMetadata(`${relationshipId}.trg.${provider}`, { accessKey, secretKey });
            }
        }
    }
}

async function exportMetadata() {
    logger.debug('Exporting metadata');

    return exportVaultSecretData();
}

async function importMetadata(metadata) {
    logger.debug('Importing metadata');

    const cacheDump = cache.dump();
    const updatedMetadata = [...new Set([...cacheDump, ...metadata])];

    await Promise.all(updatedMetadata.map(secret => createSecret(secret.k, secret.v)));
}

module.exports = {
    getMetadata: readMetadata,
    registerMetadata,
    exportMetadata,
    importMetadata
};
