const Joi = require('@hapi/joi');
const bucketSchema = require('../schemas/bucket-schema');
const containerSchema = require('../schemas/container-schema');
const metadataSchema = require('../schemas/metadata-schema');
const kmsKeySchema = require('../schemas/kms-key-schema');
const {
    TYPE_SYMBOLIC, TYPE_FILE, TYPE_DIRECTORY, TYPE_OTHER, STATUS_FAILED, STATUS_SUCCEEDED, ACTION_DELETE_SOURCE, ACTION_DELETE_TARGET, ACTION_COPY,
    ACTION_GRACE, ACTION_IGNORE, SCAN_STEP, TRANSFER_STEP, ACTION_DELETE_FAILED, LIST_STEP,
    ACTION_COPY_FAILED, ACTION_ABORT, ALIBABA_PROVIDER, ONTAP_S3_PROVIDER, S3_PROVIDER, SGWS_PROVIDER, IBM_PROVIDER
} = require('../../lib/consts');
const sgwsBucketSchema = require('../schemas/sgws-bucket-schema');

const masterListS3BucketsResponse = Joi.object().keys({
    provider: Joi.string().valid(ONTAP_S3_PROVIDER, ALIBABA_PROVIDER, S3_PROVIDER, SGWS_PROVIDER, IBM_PROVIDER).required(),
    buckets: Joi.when('provider',
        { is: [SGWS_PROVIDER, IBM_PROVIDER, ONTAP_S3_PROVIDER], then: Joi.array().items(sgwsBucketSchema), otherwise: Joi.array().items(bucketSchema) }),
    status: Joi.string().valid(STATUS_SUCCEEDED),
    host: Joi.when('provider',
        { is: [SGWS_PROVIDER, IBM_PROVIDER, ONTAP_S3_PROVIDER], then: Joi.string().required() }),
    port: Joi.when('provider',
        { is: [SGWS_PROVIDER, IBM_PROVIDER], then: Joi.string().required() })
});

const masterListGcpBucketFoldersResponse = Joi.object().keys({
    bucket: bucketSchema,
    prefix: Joi.string().allow(''),
    folders: Joi.array().items(Joi.string()).required(),
    status: Joi.string().valid(STATUS_SUCCEEDED)
});

const masterListGcpBucketsResponse = Joi.object().keys({
    buckets: Joi.array().items(bucketSchema),
    status: Joi.string().valid(STATUS_SUCCEEDED)
});

const masterListS3BucketFoldersResponse = Joi.object().keys({
    bucket: Joi.when('provider',
        { is: [SGWS_PROVIDER, IBM_PROVIDER, ONTAP_S3_PROVIDER], then: sgwsBucketSchema, otherwise: bucketSchema }),
    prefix: Joi.string().allow(''),
    folders: Joi.array().items(Joi.string()).required(),
    host: Joi.when('provider',
        { is: [SGWS_PROVIDER, IBM_PROVIDER, ONTAP_S3_PROVIDER], then: Joi.string().required() }),
    port: Joi.when('provider',
        { is: [SGWS_PROVIDER, IBM_PROVIDER], then: Joi.string().required() }),
    status: Joi.string().valid(STATUS_SUCCEEDED)
});
const masterListAzureBlobStorageContainersResponse = Joi.object().keys({
    containers: Joi.array().items(containerSchema),
    status: Joi.string().valid(STATUS_SUCCEEDED)
});

const masterListKmsKeysResponse = Joi.object().keys({
    bucket: Joi.string().required(),
    keys: Joi.array().items(kmsKeySchema),
    status: Joi.string().valid(STATUS_SUCCEEDED)
});

const masterListExportsResponse = Joi.object().keys({
    exports: Joi.array().items(Joi.string()),
    host: Joi.string().required(),
    status: Joi.string().valid(STATUS_SUCCEEDED)
});

const masterListExportFoldersResponse = Joi.object().keys({
    export: Joi.string().required(),
    host: Joi.string().required(),
    path: Joi.string().allow(''),
    folders: Joi.array().items(Joi.string()).required(),
    status: Joi.string().valid(STATUS_SUCCEEDED)
});

const masterListAzureBlobStorageContainerFoldersResponse = Joi.object().keys({
    container: Joi.string().required(),
    prefix: Joi.string(),
    folders: Joi.array().items(Joi.string()).required(),
    status: Joi.string().valid(STATUS_SUCCEEDED)
});

const masterSharesResponse = Joi.object().keys({
    shares: Joi.array().items(Joi.string()),
    host: Joi.string().required(),
    status: Joi.string().valid(STATUS_SUCCEEDED)
});

const masterShareFoldersResponse = Joi.object().keys({
    share: Joi.string().required(),
    host: Joi.string().required(),
    path: Joi.string(),
    folders: Joi.array().items(Joi.string()).required(),
    status: Joi.string().valid(STATUS_SUCCEEDED)
});

const metaDataAWS = Joi.object().keys({
    subnetId: Joi.string().required(),
    subnetCidrBlock: Joi.string().ip({ cidr: 'required' }).required(),
    vpcId: Joi.string().required(),
    vpcCidrBlock: Joi.string().required(),
    instanceId: Joi.string().required(),
    region: Joi.string().required(),
    availabilityZone: Joi.string().required(),
    awsAccountId: Joi.string().regex(/^[0-9]+$/, 'numbers').length(12).required()
});

const metaDataGCP = Joi.object().keys({
    instanceId: Joi.string().required(),
    zone: Joi.string().required(),
    subnetCidrBlock: Joi.string().required(),
    serviceAccount: Joi.string().required(),
    network: Joi.string().required()
});

const metaDataAzure = Joi.object().keys({
    instanceId: Joi.string().required(),
    region: Joi.string().required(),
    resourceGroupName: Joi.string().required(),
    subnetCidrBlock: Joi.string().ip({ cidr: 'required' })
});

const placementMessageOnPrem = Joi.object().keys({
    version: Joi.string().required(),
    hostname: Joi.string().required(),
    privateIp: Joi.string().ip().required(),
    publicIp: Joi.string().allow(''),
    os: Joi.string().required(),
    platform: Joi.string().required(),
    release: Joi.string().required(),
    osTotalMem: Joi.number().positive().required(),
    node: Joi.string().required(),
    cpus: Joi.number().positive().required(),
    processMaxMem: Joi.number().positive().required(),
    awsAccountId: Joi.string().regex(/^[0-9]+$/, 'numbers').length(12),
    gcpProjectId: Joi.string(),
    proxy: Joi.string(),
    certificate: Joi.string()
});

const placementMessageAWS = metaDataAWS.concat(placementMessageOnPrem);
const placementMessageGCP = metaDataGCP.concat(placementMessageOnPrem);
const placementMessageAzure = metaDataAzure.concat(placementMessageOnPrem);

const healthMessage = Joi.object().keys({
    transferrers: Joi.number().min(0).required(),
    scanners: Joi.number().min(0).required(),
    memory: Joi.object().keys({
        rss: Joi.number().positive().required(),
        heapTotal: Joi.number().positive().required(),
        heapUsed: Joi.number().positive().required(),
        external: Joi.number().required()
    }),
    cpuUsage: Joi.object().keys({
        user: Joi.number().positive().required(),
        system: Joi.number().positive().required()
    }),
    transferRate: Joi.number()
});

const statisticsScehma = Joi.object().keys({
    bytes: Joi.number().min(0).required(),
    items: Joi.number().min(0).required()
});

const countersSchema = Joi.object().keys({
    [TYPE_FILE]: statisticsScehma,
    [TYPE_DIRECTORY]: statisticsScehma,
    [TYPE_SYMBOLIC]: statisticsScehma,
    [TYPE_OTHER]: statisticsScehma
}).or(TYPE_FILE, TYPE_DIRECTORY, TYPE_SYMBOLIC, TYPE_OTHER);

const scanAndTransferResponse = Joi.object().keys({
    relationshipId: Joi.string().required(),
    [SCAN_STEP]: Joi.object().keys({
        [STATUS_FAILED]: countersSchema,
        [ACTION_DELETE_SOURCE]: countersSchema,
        [ACTION_DELETE_TARGET]: countersSchema,
        [ACTION_COPY]: countersSchema,
        [ACTION_GRACE]: countersSchema,
        [ACTION_IGNORE]: countersSchema,
        [ACTION_ABORT]: countersSchema
    }).or(ACTION_DELETE_SOURCE, ACTION_DELETE_TARGET, ACTION_COPY, ACTION_GRACE, ACTION_IGNORE, STATUS_FAILED, ACTION_ABORT),
    [TRANSFER_STEP]: Joi.object().keys({
        [ACTION_DELETE_SOURCE]: countersSchema,
        [ACTION_DELETE_TARGET]: countersSchema,
        [ACTION_COPY]: countersSchema,
        [ACTION_DELETE_FAILED]: countersSchema,
        [ACTION_COPY_FAILED]: countersSchema,
        [ACTION_ABORT]: countersSchema,
        relationshipErrors: Joi.object()
    }).or(ACTION_DELETE_SOURCE, ACTION_DELETE_TARGET, ACTION_COPY, ACTION_ABORT, ACTION_DELETE_FAILED, ACTION_COPY_FAILED, 'relationshipErrors'),
    [LIST_STEP]: Joi.object().keys({
        [LIST_STEP]: countersSchema,
        [STATUS_FAILED]: countersSchema,
        relationshipErrors: Joi.object()
    })
}).or(SCAN_STEP, TRANSFER_STEP, LIST_STEP);

const masterMetadataResponseSchema = Joi.object().keys({
    targetDataBrokerId: Joi.string().required(),
    metadata: metadataSchema,
    status: Joi.string().valid(STATUS_SUCCEEDED)
});

const sqsMasterOutgoingMessagesSchemes = {
    's3-buckets': masterListS3BucketsResponse,
    's3-bucket-folders': masterListS3BucketFoldersResponse,
    'gcp-buckets': masterListGcpBucketsResponse,
    'gcp-bucket-folders': masterListGcpBucketFoldersResponse,
    exports: masterListExportsResponse,
    'export-folders': masterListExportFoldersResponse,
    shares: masterSharesResponse,
    'share-folders': masterShareFoldersResponse,
    'azure-blob-storage-containers': masterListAzureBlobStorageContainersResponse,
    'azure-blob-storage-container-folders': masterListAzureBlobStorageContainerFoldersResponse,
    'kms-keys': masterListKmsKeysResponse,
    'placement-aws': placementMessageAWS,
    'placement-gcp': placementMessageGCP,
    'placement-azure': placementMessageAzure,
    'placement-on-prem': placementMessageOnPrem,
    health: healthMessage,
    statistics: scanAndTransferResponse,
    metadata: masterMetadataResponseSchema
};

module.exports = sqsMasterOutgoingMessagesSchemes;
