const { cloneDeep } = require('lodash');
const { copyStatisticsSchema } = require('../../../src/messages/schemas/statuses-outgoing-messages');
const {
    STATUS_SUCCEEDED, STATUS_IGNORED, TYPE_FILE, ACTION_COPY, SCAN_STEP, TRANSFER_STEP, LIST_STEP, ACTION_LIST, ACTION_DELETE_SOURCE,
    TYPE_DIRECTORY, TYPE_SYMBOLIC, TYPE_OTHER, ACTION_IGNORE, ACTION_DELETE_TARGET, ACTION_GRACE, STATUS_FAILED
} = require('../../../src/lib/consts');

describe('sqs-statuses-outgoing-messages validations - copyStatisticsSchema ', () => {
    const validStatsMessage = {
        step: SCAN_STEP,
        relationshipId: '1',
        status: STATUS_SUCCEEDED,
        bytes: 52645,
        objectType: TYPE_FILE,
        targetRequiredAction: ACTION_COPY,
        path: 'test12',
        attributes: { type: 'copy' }
    };

    const validIgnoredStatsMessage = {
        step: SCAN_STEP,
        relationshipId: '1',
        status: STATUS_IGNORED,
        bytes: 52645,
        objectType: TYPE_OTHER,
        targetRequiredAction: ACTION_IGNORE,
        path: 'test12',
        reason: { err: 'special file' },
        attributes: { type: 'copy' }
    };

    test('sqs-statuses-outgoing-messages valid COPY stats message', () => {
        expect(copyStatisticsSchema.validate(validStatsMessage).error).toBeUndefined();
    });

    test('sqs-statuses-outgoing-messages not valid COPY stats message- no valid step', () => {
        const invalidStatsMessage = cloneDeep(validStatsMessage);
        invalidStatsMessage.step = 'Type';
        expect(copyStatisticsSchema.validate(invalidStatsMessage).error.details[0])
            .toEqual(expect.objectContaining({ message: `"step" must be one of [${SCAN_STEP}, ${TRANSFER_STEP}, ${LIST_STEP}]` }));
    });

    test('sqs-statuses-outgoing-messages not valid COPY stats message- no relationshipId ', () => {
        const invalidStatsMessage = cloneDeep(validStatsMessage);
        delete invalidStatsMessage.relationshipId;
        expect(copyStatisticsSchema.validate(invalidStatsMessage).error.details[0])
            .toEqual(expect.objectContaining({ message: '"relationshipId" is required' }));
    });

    test('sqs-statuses-outgoing-messages not valid COPY stats message- no bytes ', () => {
        const invalidStatsMessage = cloneDeep(validStatsMessage);
        delete invalidStatsMessage.bytes;
        expect(copyStatisticsSchema.validate(invalidStatsMessage).error.details[0])
            .toEqual(expect.objectContaining({ message: '"bytes" is required' }));
    });

    test('sqs-statuses-outgoing-messages not valid COPY stats message- invalid objectType ', () => {
        const invalidStatsMessage = cloneDeep(validStatsMessage);
        invalidStatsMessage.objectType = 'ABC';
        expect(copyStatisticsSchema.validate(invalidStatsMessage).error.details[0])
            .toEqual(expect.objectContaining({ message: `"objectType" must be one of [${TYPE_FILE}, ${TYPE_DIRECTORY}, ${TYPE_SYMBOLIC}]` }));
    });

    test('sqs-statuses-outgoing-messages not valid COPY stats message- invalid targetRequiredAction ', () => {
        const invalidStatsMessage = cloneDeep(validStatsMessage);
        invalidStatsMessage.targetRequiredAction = 'ABC';
        expect(copyStatisticsSchema.validate(invalidStatsMessage).error.details[0])
            .toEqual(expect.objectContaining({ message: `"targetRequiredAction" must be one of [${ACTION_COPY}, ${ACTION_DELETE_SOURCE}, ${ACTION_DELETE_TARGET}, ${ACTION_GRACE}, ${STATUS_FAILED}, ${ACTION_LIST}]` }));
    });

    test('sqs-statuses-outgoing-messages valid IGNORE stats message', () => {
        expect(copyStatisticsSchema.validate(validIgnoredStatsMessage).error).toBeUndefined();
    });

    test('sqs-statuses-outgoing-messages not valid IGNORE stats message - no valid step', () => {
        const invalidIgnoreStatsMessage = cloneDeep(validIgnoredStatsMessage);
        invalidIgnoreStatsMessage.step = TRANSFER_STEP;
        expect(copyStatisticsSchema.validate(invalidIgnoreStatsMessage).error.details[0])
            .toEqual(expect.objectContaining({ message: `"step" must be [${SCAN_STEP}]` }));
    });

    test('sqs-statuses-outgoing-messages not valid IGNORE stats message- invalid targetRequiredAction ', () => {
        const invalidIgnoreStatsMessage = cloneDeep(validIgnoredStatsMessage);
        invalidIgnoreStatsMessage.targetRequiredAction = 'ABC';
        expect(copyStatisticsSchema.validate(invalidIgnoreStatsMessage).error.details[0].message)
            .toEqual(`"targetRequiredAction" must be one of [${ACTION_COPY}, ${ACTION_DELETE_SOURCE}, ${ACTION_DELETE_TARGET}, ${ACTION_GRACE}, ${STATUS_FAILED}, ${ACTION_LIST}, ${ACTION_IGNORE}]`);
    });
});
