const AWS = require('aws-sdk-mock');
const { cloneDeep } = require('lodash');
const {
    compareFiles, compareDirs, listObjectsOperation
} = require('../../src/workers/scanner');
const {
    TYPE_FILE, TYPE_OTHER, TYPE_DIRECTORY, TYPE_SYMBOLIC
} = require('../../src/lib/consts');

const date = new Date('2018-01-30T13:09:52.000Z');
const mtimeSec = parseInt(date.getTime() / 1000);

describe('compareFiles', () => {
    const file = {
        path: 'test1',
        size: 123,
        type: TYPE_FILE,
        mode: 33206,
        uid: 1,
        gid: 0,
        mtimeSec: 1507638051
    };

    test('compareFiles - tags are the same', () => {
        expect(compareFiles([file], [file])).toEqual({
            TASK_TYPE_COPY: [],
            TASK_TYPE_DELETE_SOURCE: [{
                gid: 0,
                mode: 33206,
                mtimeSec: 1507638051,
                path: 'test1',
                size: 123,
                type: 'FILE',
                uid: 1
            }],
            TASK_TYPE_DELETE_TARGET: [],
            TASK_TYPE_IGNORE: []
        });
    });

    test('compareFiles - source has different tag value', () => {
        const source = cloneDeep(file);
        source.uid = 0;
        expect(compareFiles([source], [file])).toEqual({
            TASK_TYPE_COPY: [source], TASK_TYPE_DELETE_SOURCE: [], TASK_TYPE_DELETE_TARGET: [], TASK_TYPE_IGNORE: []
        });
    });

    test('compareFiles - target has different tag value', () => {
        const target = cloneDeep(file);
        target.uid = 0;
        expect(compareFiles([file], [target])).toEqual({
            TASK_TYPE_COPY: [file], TASK_TYPE_DELETE_SOURCE: [], TASK_TYPE_DELETE_TARGET: [], TASK_TYPE_IGNORE: []
        });
    });

    test('compareFiles - target has different time value', () => {
        const target = cloneDeep(file);
        target.mtimeSec = 1507638042;
        expect(compareFiles([file], [target])).toEqual({
            TASK_TYPE_COPY: [file], TASK_TYPE_DELETE_SOURCE: [], TASK_TYPE_DELETE_TARGET: [], TASK_TYPE_IGNORE: []
        });
    });

    test('compareFiles - source has different tags values', () => {
        const source = cloneDeep(file);
        source.uid = 0;
        source.gid = 1;
        expect(compareFiles([source], [file])).toEqual({
            TASK_TYPE_COPY: [source], TASK_TYPE_DELETE_SOURCE: [], TASK_TYPE_DELETE_TARGET: [], TASK_TYPE_IGNORE: []
        });
    });

    test('compareFiles - target has different tag values', () => {
        const target = cloneDeep(file);
        target.uid = 0;
        target.gid = 1;
        expect(compareFiles([file], [target])).toEqual({
            TASK_TYPE_COPY: [file], TASK_TYPE_DELETE_SOURCE: [], TASK_TYPE_DELETE_TARGET: [], TASK_TYPE_IGNORE: []
        });
    });

    test('compareFiles - source has ignored file tag value', () => {
        const source = cloneDeep(file);
        source.type = TYPE_OTHER;
        source.path = 'ignored';
        expect(compareFiles([source], [])).toEqual({
            TASK_TYPE_COPY: [], TASK_TYPE_DELETE_SOURCE: [], TASK_TYPE_DELETE_TARGET: [], TASK_TYPE_IGNORE: [source]
        });
    });

    test('compareFiles - source and target has ignored files with same path', () => {
        const source = cloneDeep(file);
        const target = cloneDeep(file);
        source.type = TYPE_OTHER;
        target.type = TYPE_OTHER;
        source.path = 'ignored';
        target.path = 'ignored';
        source.size = 1;
        expect(compareFiles([source], [target])).toEqual({
            TASK_TYPE_COPY: [], TASK_TYPE_DELETE_SOURCE: [], TASK_TYPE_DELETE_TARGET: [], TASK_TYPE_IGNORE: [source, target]
        });
    });

    test('compareFiles - target has ignored files that source doesn\'t have', () => {
        const target = cloneDeep(file);
        target.type = TYPE_OTHER;
        target.path = 'ignored';
        expect(compareFiles([], [target])).toEqual({
            TASK_TYPE_COPY: [], TASK_TYPE_DELETE_SOURCE: [], TASK_TYPE_DELETE_TARGET: [], TASK_TYPE_IGNORE: [target]
        });
    });

    test('compareFiles - source is a special file and target is a regular file - target file should be deleted', () => {
        const source = cloneDeep(file);
        const target = cloneDeep(file);
        source.type = TYPE_OTHER;
        source.path = 'ignored';
        expect(compareFiles([source], [target])).toEqual({
            TASK_TYPE_COPY: [], TASK_TYPE_DELETE_SOURCE: [], TASK_TYPE_DELETE_TARGET: [target], TASK_TYPE_IGNORE: [source]
        });
    });

    test('compareFiles - target is a special file but source is normal - should try to copy', () => {
        const source = cloneDeep(file);
        const target = cloneDeep(file);
        target.type = TYPE_OTHER;
        target.path = 'ignored';
        expect(compareFiles([source], [target])).toEqual({
            TASK_TYPE_COPY: [source], TASK_TYPE_DELETE_SOURCE: [], TASK_TYPE_DELETE_TARGET: [], TASK_TYPE_IGNORE: [target]
        });
    });

    test('compareFiles - symbolic file & tags are the same', () => {
        const symbolic = cloneDeep(file);
        symbolic.type = TYPE_SYMBOLIC;
        expect(compareFiles([symbolic], [symbolic])).toEqual({
            TASK_TYPE_COPY: [],
            TASK_TYPE_DELETE_SOURCE: [{
                gid: 0,
                mode: 33206,
                mtimeSec: 1507638051,
                path: 'test1',
                size: 123,
                type: 'SYMBOLIC',
                uid: 1
            }],
            TASK_TYPE_DELETE_TARGET: [],
            TASK_TYPE_IGNORE: []
        });
    });

    test('compareFiles - symbolic file only in source', () => {
        const symbolic = cloneDeep(file);
        symbolic.type = TYPE_SYMBOLIC;
        expect(compareFiles([symbolic], [])).toEqual({
            TASK_TYPE_COPY: [symbolic], TASK_TYPE_DELETE_SOURCE: [], TASK_TYPE_DELETE_TARGET: [], TASK_TYPE_IGNORE: []
        });
    });

    test('compareFiles - symbolic file only in target', () => {
        const symbolic = cloneDeep(file);
        symbolic.type = TYPE_SYMBOLIC;
        expect(compareFiles([], [symbolic])).toEqual({
            TASK_TYPE_COPY: [], TASK_TYPE_DELETE_SOURCE: [], TASK_TYPE_DELETE_TARGET: [symbolic], TASK_TYPE_IGNORE: []
        });
    });

    test('compareFiles - source is regular file & target symbolic', () => {
        const symbolic = cloneDeep(file);
        symbolic.type = TYPE_SYMBOLIC;
        expect(compareFiles([file], [symbolic])).toEqual({
            TASK_TYPE_COPY: [file], TASK_TYPE_DELETE_SOURCE: [], TASK_TYPE_DELETE_TARGET: [], TASK_TYPE_IGNORE: []
        });
    });

    test('compareFiles - source is symbolic file & target regular', () => {
        const symbolic = cloneDeep(file);
        symbolic.type = TYPE_SYMBOLIC;
        expect(compareFiles([symbolic], [file])).toEqual({
            TASK_TYPE_COPY: [symbolic], TASK_TYPE_DELETE_SOURCE: [], TASK_TYPE_DELETE_TARGET: [], TASK_TYPE_IGNORE: []
        });
    });
});
describe('compareDirs', () => {
    const dir = {
        path: 'dir1',
        size: 0,
        type: TYPE_DIRECTORY,
        mode: 16877,
        uid: 1,
        gid: 0,
        mtimeSec: 1507638051
    };

    test('compareDirs - only source', () => {
        expect(compareDirs([dir], [])).toEqual({
            TASK_TYPE_EQUAL: [], TASK_TYPE_SOURCE_ONLY: [dir], TASK_TYPE_TARGET_ONLY: [], TASK_TYPE_IGNORE: [], TASK_TYPE_EQUAL_AND_COPY: []
        });
    });
    test('compareDirs - only target', () => {
        expect(compareDirs([], [dir])).toEqual({
            TASK_TYPE_EQUAL: [], TASK_TYPE_SOURCE_ONLY: [], TASK_TYPE_TARGET_ONLY: [dir], TASK_TYPE_IGNORE: [], TASK_TYPE_EQUAL_AND_COPY: []
        });
    });
    test('compareDirs- tags are the same', () => {
        expect(compareDirs([dir], [dir])).toEqual({
            TASK_TYPE_EQUAL: [dir], TASK_TYPE_SOURCE_ONLY: [], TASK_TYPE_TARGET_ONLY: [], TASK_TYPE_IGNORE: [], TASK_TYPE_EQUAL_AND_COPY: []
        });
    });
    test('compareDirs - source and target have different path', () => {
        const target = cloneDeep(dir);
        target.path = 'differentDir';
        expect(compareDirs([dir], [target])).toEqual({
            TASK_TYPE_EQUAL: [], TASK_TYPE_SOURCE_ONLY: [dir], TASK_TYPE_TARGET_ONLY: [target], TASK_TYPE_IGNORE: [], TASK_TYPE_EQUAL_AND_COPY: []
        });
    });
    test('compareDirs - source has .snapshot', () => {
        const src = cloneDeep(dir);
        src.path = '.snapshot';
        expect(compareDirs([src], [])).toEqual({
            TASK_TYPE_EQUAL: [], TASK_TYPE_SOURCE_ONLY: [], TASK_TYPE_TARGET_ONLY: [], TASK_TYPE_IGNORE: [src], TASK_TYPE_EQUAL_AND_COPY: []
        });
    });
    test('compareDirs - target has .snapshot', () => {
        const target = cloneDeep(dir);
        target.path = '.snapshot';
        expect(compareDirs([], [target])).toEqual({
            TASK_TYPE_EQUAL: [], TASK_TYPE_SOURCE_ONLY: [], TASK_TYPE_TARGET_ONLY: [], TASK_TYPE_IGNORE: [target], TASK_TYPE_EQUAL_AND_COPY: []
        });
    });
    test('compareDirs - source & target has .snapshot', () => {
        const src = cloneDeep(dir);
        src.path = '.snapshot';
        const target = cloneDeep(dir);
        target.path = '.snapshot';
        expect(compareDirs([dir, src], [target])).toEqual({
            TASK_TYPE_EQUAL: [], TASK_TYPE_SOURCE_ONLY: [dir], TASK_TYPE_TARGET_ONLY: [], TASK_TYPE_IGNORE: [src, target], TASK_TYPE_EQUAL_AND_COPY: []
        });
    });
    test('compareDirs - source and target have different mode', () => {
        const target = cloneDeep(dir);
        target.mode = 19000;
        expect(compareDirs([dir], [target])).toEqual({
            TASK_TYPE_EQUAL: [dir], TASK_TYPE_SOURCE_ONLY: [], TASK_TYPE_TARGET_ONLY: [], TASK_TYPE_IGNORE: [], TASK_TYPE_EQUAL_AND_COPY: [dir]
        });
    });
    test('compareDirs - mtimeSec is ignored', () => {
        const target = cloneDeep(dir);
        target.mtimeSec = Date.now();
        expect(compareDirs([dir], [target])).toEqual({
            TASK_TYPE_EQUAL: [dir], TASK_TYPE_SOURCE_ONLY: [], TASK_TYPE_TARGET_ONLY: [], TASK_TYPE_IGNORE: [], TASK_TYPE_EQUAL_AND_COPY: []
        });
    });
});
describe('listObjectsOperation - s3', () => {
    function s3FileObject(path, size, isSymbolic = false, uid = 0, gid = 0) {
        return {
            path,
            size,
            type: (isSymbolic) ? TYPE_SYMBOLIC : TYPE_FILE,
            mode: 33206,
            mtimeSec,
            uid,
            gid
        };
    }
    function s3DirectoryObject(path, uid = 0, gid = 0) {
        return {
            path,
            size: 0,
            type: TYPE_DIRECTORY,
            mode: 16822,
            mtimeSec,
            uid,
            gid
        };
    }

    function sgwsFileObject(path, size, isSymbolic = false, uid = 0, gid = 0) {
        return {
            path,
            size,
            type: (isSymbolic) ? TYPE_SYMBOLIC : TYPE_FILE,
            mode: 33206,
            mtimeSec,
            uid,
            gid
        };
    }
    function sgwsDirectoryObject(path, uid = 0, gid = 0) {
        return {
            path,
            size: 0,
            type: TYPE_DIRECTORY,
            mode: 16822,
            mtimeSec,
            uid,
            gid
        };
    }

    const listObjectsV2Function = (params, callback) => {
        let result;
        if (params.ContinuationToken) {
            result = {
                IsTruncated: false,
                Contents: [{
                    Key: 'temp/lastObject.png',
                    LastModified: date,
                    ETag: '71e749519a5cdf0a437aa540d98d079a',
                    Size: 815104,
                    StorageClass: 'STANDARD'
                },
                {
                    Key: 'temp/',
                    LastModified: date,
                    ETag: '71e749519a5cdf0a437aa540d98d079a',
                    Size: 0,
                    StorageClass: 'STANDARD'
                }],
                Name: 'test',
                Prefix: 'temp/',
                Delimiter: '/',
                MaxKeys: 1000,
                CommonPrefixes: [{ Prefix: 'temp/dir1/dir2' }],
                KeyCount: 3
            };
        } else {
            result = {
                IsTruncated: true,
                Contents: [{
                    Key: 'temp/testdoc.doc',
                    LastModified: date,
                    ETag: '71e749519a5cdf0a437aa540d98d079a',
                    Size: 815104,
                    StorageClass: 'STANDARD'
                },
                {
                    Key: 'temp/testdoc.html',
                    LastModified: date,
                    ETag: 'cab26c0e9c32031ee4d679b83eea13a8',
                    Size: 75121,
                    StorageClass: 'STANDARD'
                }],
                Name: 'test',
                Prefix: 'temp/',
                Delimiter: '/',
                MaxKeys: 1000,
                CommonPrefixes: [{ Prefix: 'temp/dir1/' }],
                KeyCount: 3,
                NextContinuationToken: 'next'
            };
        }
        callback(null, result);
    };
    const getObjectTaggingFunc = (params, callback) => {
        const files = ['temp/testdoc.html', 'temp/lastObject.png'];
        let result;
        if (files.includes(params.Key)) {
            result = { TagSet: [{ Key: 'netapp-cs-mode', Value: '33206' }, { Key: 'netapp-cs-uid', Value: '0' }, { Key: 'netapp-cs-gid', Value: '0' }, { Key: 'netapp-cs-mtimeSec', Value: mtimeSec.toString() }] };
        } else if (params.Key === 'temp/testdoc.doc') {
            result = { TagSet: [{ Key: 'netapp-cs-mode', Value: '33206' }, { Key: 'netapp-cs-uid', Value: '0' }, { Key: 'netapp-cs-gid', Value: '0' }, { Key: 'netapp-cs-slink', Value: 'True' }, { Key: 'netapp-cs-mtimeSec', Value: mtimeSec.toString() }] };
        } else {
            result = { TagSet: [{ Key: 'netapp-cs-mode', Value: '16822' }, { Key: 'netapp-cs-uid', Value: '0' }, { Key: 'netapp-cs-gid', Value: '0' }, { Key: 'netapp-cs-mtimeSec', Value: mtimeSec.toString() }] };
        }
        callback(null, result);
    };
    const listObjectsFunction = (params, callback) => {
        let result;
        if (params.Marker) {
            result = {
                IsTruncated: false,
                Contents: [{
                    Key: 'temp/lastObject.png',
                    LastModified: date,
                    ETag: '71e749519a5cdf0a437aa540d98d079a',
                    Size: 815104,
                    StorageClass: 'STANDARD'
                },
                {
                    Key: 'temp/',
                    LastModified: date,
                    ETag: '71e749519a5cdf0a437aa540d98d079a',
                    Size: 0,
                    StorageClass: 'STANDARD'
                }],
                Name: 'test',
                Prefix: 'temp/',
                Delimiter: '/',
                MaxKeys: 1000,
                CommonPrefixes: [{ Prefix: 'temp/dir1/dir2' }],
                KeyCount: 3
            };
        } else {
            result = {
                IsTruncated: true,
                Contents: [{
                    Key: 'temp/testdoc.doc',
                    LastModified: date,
                    ETag: '71e749519a5cdf0a437aa540d98d079a',
                    Size: 815104,
                    StorageClass: 'STANDARD'
                },
                {
                    Key: 'temp/testdoc.html',
                    LastModified: date,
                    ETag: 'cab26c0e9c32031ee4d679b83eea13a8',
                    Size: 75121,
                    StorageClass: 'STANDARD'
                }],
                Name: 'test',
                Prefix: 'temp/',
                Delimiter: '/',
                MaxKeys: 1000,
                CommonPrefixes: [{ Prefix: 'temp/dir1/' }],
                KeyCount: 3,
                NextMarker: 'next'
            };
        }
        callback(null, result);
    };
    const getMetadata = (params, callback) => {
        const files = ['temp/testdoc.html', 'temp/lastObject.png'];
        let result;
        if (files.includes(params.Key)) {
            result = {
                AcceptRanges: 'bytes',
                LastModified: date,
                ContentLength: 19223,
                ETag: '"e938eed8ca783dc4bfab0946cad368a5"',
                ContentType: 'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
                Metadata:
                    {
                        'netapp-cs-gid': '0',
                        'netapp-cs-mode': '33206',
                        'netapp-cs-mtimesec': mtimeSec.toString(),
                        'netapp-cs-uid': '0'
                    }
            };
        } else if (params.Key === 'temp/testdoc.doc') {
            result = {
                AcceptRanges: 'bytes',
                LastModified: date,
                ContentLength: 19223,
                ETag: '"e938eed8ca783dc4bfab0946cad368a5"',
                ContentType: 'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
                Metadata:
                    {
                        'netapp-cs-gid': '0',
                        'netapp-cs-mode': '33206',
                        'netapp-cs-mtimesec': mtimeSec.toString(),
                        'netapp-cs-uid': '0',
                        'netapp-cs-slink': 'True'
                    }
            };
        } else {
            result = {
                AcceptRanges: 'bytes',
                LastModified: date,
                ContentLength: 19223,
                ETag: '"e938eed8ca783dc4bfab0946cad368a5"',
                ContentType: 'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
                Metadata:
                    {
                        'netapp-cs-gid': '0',
                        'netapp-cs-mode': '16822',
                        'netapp-cs-mtimesec': mtimeSec.toString(),
                        'netapp-cs-uid': '0'
                    }
            };
        }
        callback(null, result);
    };

    afterEach(() => {
        AWS.restore('S3', 'listObjectsV2');
        AWS.restore('S3', 'listObjects');
    });

    test('listS3Objects - no valid bucket', async () => {
        AWS.mock('S3', 'listObjectsV2', (params, callback) => callback({
            message: 'The specified bucket is not valid.',
            code: 'InvalidBucketName'
        }, {}));

        await expect(listObjectsOperation({ bucket: 'test', region: 'us-east-1', provider: 's3' })).rejects.toEqual({
            message: 'The specified bucket is not valid.',
            code: 'InvalidBucketName'
        });
    });

    test('listS3Objects - Access Denied', async () => {
        AWS.mock('S3', 'listObjectsV2', (params, callback) => callback({ code: 'AccessDenied', message: 'Access Denied' }, {}));
        await expect(listObjectsOperation({ bucket: 'test', region: 'us-east-1', provider: 's3' })).rejects.toEqual({ code: 'AccessDenied', message: 'Access Denied' });
    });

    test('listObjects- s3 with recursive calls', async () => {
        AWS.mock('S3', 'listObjectsV2', listObjectsV2Function);
        AWS.mock('S3', 'getObjectTagging', getObjectTaggingFunc);
        const listObjectResult = await listObjectsOperation({ bucket: 'test', region: 'us-east-1', provider: 's3' });
        expect(listObjectResult).toEqual(expect.arrayContaining([
            s3FileObject('testdoc.doc', 815104, true), s3FileObject('testdoc.html', 75121), s3FileObject('lastObject.png', 815104),
            s3DirectoryObject('dir1'), s3DirectoryObject('dir2')]));
    });

    test('listObjects- alibaba with recursive calls', async () => {
        AWS.mock('S3', 'listObjectsV2', listObjectsV2Function);
        AWS.mock('S3', 'headObject', getMetadata);
        const listObjectResult = await listObjectsOperation({
            bucket: 'test', region: 'us-east-1', provider: 'alibaba', accessKey: 'a', secretKey: 'b'
        });
        expect(listObjectResult).toEqual(expect.arrayContaining([
            s3FileObject('testdoc.doc', 815104, true), s3FileObject('testdoc.html', 75121), s3FileObject('lastObject.png', 815104),
            s3DirectoryObject('dir1'), s3DirectoryObject('dir2')]));
    });

    test('listObjects - sgws- with recursive calls', async () => {
        AWS.mock('S3', 'listObjects', listObjectsFunction);
        AWS.mock('S3', 'headObject', getMetadata);
        const listObjectResult = await listObjectsOperation({
            bucket: 'test', host: '1.1.1.1', provider: 'sgws', port: 111, accessKey: 'a', secretKey: 'a', path: 'test'
        });

        expect(listObjectResult).toEqual(expect.arrayContaining([
            sgwsFileObject('testdoc.doc', 815104, true),
            sgwsFileObject('testdoc.html', 75121),
            sgwsFileObject('lastObject.png', 815104),
            sgwsDirectoryObject('dir1'), sgwsDirectoryObject('dir2')]));
    });

    test('listObjects - ibm- with recursive calls', async () => {
        AWS.mock('S3', 'listObjects', listObjectsFunction);
        AWS.mock('S3', 'headObject', getMetadata);
        const listObjectResult = await listObjectsOperation({
            bucket: 'test', host: '1.1.1.1', provider: 'ibm', port: 111, accessKey: 'a', secretKey: 'a', path: 'test'
        });

        expect(listObjectResult).toEqual(expect.arrayContaining([
            sgwsFileObject('testdoc.doc', 815104, true),
            sgwsFileObject('testdoc.html', 75121),
            sgwsFileObject('lastObject.png', 815104),
            sgwsDirectoryObject('dir1'), sgwsDirectoryObject('dir2')]));
    });
});
