const fs = require('fs');
const {flatten, first, sortBy, groupBy} = require('lodash');
const Path = require('path');
const Promise = require('bluebird');
const config = require('config');
const {IPA_REGISTERED_STATUS} = require('../src/utils/consts');
const getLogger = require('../src/utils/logger');
const logger = getLogger();


function hasAutomaticExtension(email) {
    return ['netapp.com', 'unitek.com', 'fastlaneus.com', 'ihcnetappu.com', 'trainocate.in', 'netapp-test.com'].some(domain => (email).includes(domain));
}

async function updateIndexesForCollection(collection) {
    try {
        const indexes = await collection.indexes();
        await Promise.all(indexes.filter(index => index.key.user).map(async index => {
            try {
                const keys = index.key;
                logger.info('drop index with name', index.name);
                await collection.dropIndex(index.name);
                delete keys.user;
                keys.account = 1;
                logger.info('Creating index with keys', keys);
                await collection.createIndex(keys);
            } catch (err) {
                logger.error('Failed handling index', index, err);
            }
        }));
    } catch (err) {
        logger.error('ERROR updating indexes for collection', err);
        throw err;
    }
}

async function updateCollectionWithAccount(collection, usersIds, account) {
    try {
        logger.info('Updating collection', collection.namespace);
        await collection.updateMany({user: {$in: usersIds}}, {$set: {account}, $unset: {user: ''}});
        logger.info('done updating collection', collection.namespace);
    } catch (e) {
        logger.error('ERROR', usersIds, account, e);
    }
}

function findUsersData(users) {
    if (users.length > 1) {
        const licenses = flatten(users.map(user => user.byol.licenses));
        const trialExpiry = first(sortBy(users, 'trialExpiry')).trialExpiry;
        //assumption all users in the same account have the same subscription
        const subscribedUser = users.find(user => user.subscription.status === 'subscribe-success');
        const userWithNssCredentials = users.find(user => user.nssCredentials);
        const registeredUser = users.find(user => user.support.status === IPA_REGISTERED_STATUS);
        const automaticExtension = users.some(user => hasAutomaticExtension(user.email));
        return {
            byol: {licenses},
            automaticExtension,
            trialExpiry,
            subscription: subscribedUser ? subscribedUser.subscription : first(users).subscription,
            nssCredentials: userWithNssCredentials ? userWithNssCredentials.nssCredentials : undefined,
            support: registeredUser ? registeredUser.support : first(users).support
        };

    } else {
        const [user] = users;
        // eslint-disable-next-line no-unused-vars
        const {_id, email, name, company, isOccm, isQstack, portalId, updatedAt, ...rest} = user;
        const automaticExtension = hasAutomaticExtension(email);
        return {...rest, automaticExtension};
    }
}


/**
 * Make any changes you need to make to the database here
 * and return promise
 * 'this' is the mongoose object
 */
async function up() {

    try {
        const {db} = this.connection;
        const usersDb = db.collection('users');
        const accountsDb = db.collection('accounts');
        const relationshipsDB = db.collection('relationships');
        const costsDb = db.collection('costs');
        const databrokersDb = db.collection('databrokers');
        const relationshipStatsDb = db.collection('relationshipstats');
        const timelineV2Db = db.collection('timelinev2');
        const groupsDb = db.collection('groups');
        const collectionsToUpdate = [relationshipsDB, databrokersDb, relationshipStatsDb, timelineV2Db, costsDb, groupsDb];

        //change indexes
        await Promise.all(collectionsToUpdate.map(collection => updateIndexesForCollection(collection)));
        //update no exceptions
        logger.info('read file');
        const tenancyData = JSON.parse(fs.readFileSync(Path.resolve(__dirname, config.get('tenancy-user-to-accounts')), 'utf8'));
        const tenancyDataByAccount = groupBy(tenancyData.users, 'account_public_id');
        logger.info('tenancyDataByAccount', tenancyDataByAccount);

        Object.keys(tenancyDataByAccount).map(accountId => {
            tenancyDataByAccount[accountId] = tenancyDataByAccount[accountId].reduce((accumulator, account) => {
                const {account_public_id, account_name, auth0_id} = account;
                if (!accumulator.accountId) {
                    accumulator.accountId = account_public_id;
                    accumulator.name = account_name;
                }
                accumulator.users.push(auth0_id);
                return accumulator;
            }, {users: []});
        });

        logger.info('*Accounts to update', Object.values(tenancyDataByAccount), Object.values(tenancyDataByAccount).length);
        await Promise.map(Object.values(tenancyDataByAccount), async externalAccount => {
            const {users: auth0Ids, accountId: tenancyAccountId, name} = externalAccount;
            const users = await usersDb.find({portalId: {$in: auth0Ids}, tenancyAccountId: {$exists: false}}).toArray();
            const userIds = users.map(user => user._id);
            logger.info('updating external users', userIds, tenancyAccountId);
            if (userIds.length) { //create data for user
                const accountData = findUsersData(users);
                const {value: AccountFromDB} = await accountsDb.findOneAndUpdate({tenancyAccountId}, {
                    $set: {
                        ...accountData, tenancyAccountId, name
                    }
                }, {upsert: true, returnOriginal: false});
                const {_id: account} = AccountFromDB;
                //update collections
                await Promise.all(collectionsToUpdate.map(collection => updateCollectionWithAccount(collection, userIds, account)));
                logger.log('after updating all collections userIds', userIds, 'tenancyAccountId', tenancyAccountId);
                await usersDb.updateMany({_id: {$in: userIds}}, {$set: {tenancyAccountId}}, {strict: false});
            }
        }, {concurrency:50});
    } catch (err) {
        logger.error('Error', err);
        throw err;
    }
}

/**
 * Make any changes that UNDO the up function side effects here (if possible)
 * and return promise
 * 'this' is the mongoose object
 */
async function down() {
    return this.model('User').countDocuments();
}

module.exports = {up, down};

