const ms = require('ms');
const server = require('./app').default;
const DataBroker = require('./models/data-broker');
const Relationship = require('./models/relationship');
const {Account} = require('./models/account');
const {SAAS_REGISTRATION_STATUS_UNSUBSCRIBE_SUCCESS, SAAS_REGISTRATION_STATUS_SUBSCRIBE_SUCCESS} = require('./utils/consts');
const getLogger = require('./utils/logger');
const logger = getLogger();
// preload all AWS services, since mock-aws doesn't know to work with lazy loaded service
for (const service in global.AWS) {
    const instance = global.AWS[service];
    delete global.AWS[service];
    global.AWS[service] = instance;
}

// put mock AWS SDK instead of real one
global.AWS = require('mock-aws');

// load mocks
const {sendSaaSSubscriptionMessage, sendHealthMessage, sendPlacementMessage} = require('./simulator/sqs-mock');
require('./simulator/ipa-mock');
require('./simulator/marketplace-metering-mock');
require('./simulator/metadata-mock');
require('./simulator/s3-mock');
require('./simulator/sns-mock');
require('./simulator/iam-mock');
require('./simulator/ecs-mock');

function startDataBroker(dataBroker) {
    const {name} = dataBroker;
    if (!['Waiting', 'azureFailed'].includes(name)) {
        sendPlacementMessage(dataBroker);
    }
}

// start all data brokers
setImmediate(async() => {
    const dataBrokers = await DataBroker.find().exec();
    dataBrokers.forEach(dataBroker => startDataBroker(dataBroker));
});

setInterval(async() => {
    const dataBrokers = await DataBroker.find({statusesQueueUrl: {$exists: true}}).exec();
    const promises = dataBrokers.map(async dataBroker => {
        const {name: dataBrokerName} = dataBroker;
        if (dataBroker.hostname) {
            if (['FailedDataBrokerAfterRelationship', 'asterixFailedDataBrokerAfterRelationship'].includes(dataBrokerName)) {
                const relationship = await Relationship.findOne({dataBroker}).exec();
                // do not send heart beat to this data broker once a relationship exists
                if (relationship) {
                    return;
                }
            }
            if (['FAILED', 'Unknown'].includes(dataBrokerName)) {
                return;
            }
            sendHealthMessage(dataBroker);
        }
    });
    await Promise.all(promises);

}, ms('10s'));

setInterval(async() => {
    // send data broker heart beat
    const dataBrokers = await DataBroker.find({statusesQueueUrl: {$exists: true}}).exec();
    dataBrokers.forEach(dataBroker => {
        if (!dataBroker.hostname && dataBroker.name !== 'azureProgress') {
            startDataBroker(dataBroker);
        }
    });
}, ms('5s'));

setInterval(async() => {
    // send data broker heart beat
    const dataBroker = await DataBroker.findOne({name: 'azureProgress', statusesQueueUrl: {$exists: true}}).exec();
    if (dataBroker) {
        startDataBroker(dataBroker);
    }
}, ms('30s'));

// Unsubscribe specific account after 1 minute
setInterval(async() => {
    const tenancyAccountId = 'account-h2VqfaT6';
    const account = await Account.findOne({
        tenancyAccountId,
        'subscription.status': SAAS_REGISTRATION_STATUS_SUBSCRIBE_SUCCESS
    }).exec();

    if (account) {
        logger.info(`Going to unsubscribe AccountId: ${tenancyAccountId}`);
        sendSaaSSubscriptionMessage(account.subscription.customerIdentifier, SAAS_REGISTRATION_STATUS_UNSUBSCRIBE_SUCCESS);
    }

}, 60000);

module.exports = server;
