const https = require('https');
const ms = require('ms');
const config = require('config');
const format = require('string-template');
const path = require('path');
const fsPromises = require('fs').promises;
const { SERVICE_REGION } = require('../../utils/consts');
const getLogger = require('../../utils/logger');
const logger = getLogger();

let iam;

function getIam() {
    if (!iam) {
        iam = new AWS.IAM({
            region: SERVICE_REGION,
            httpOptions: {agent: new https.Agent({keepAlive: true})}
        });
    }
    return iam;
}

async function putUserPolicyAsync(PolicyDocument, PolicyName, UserName) {
    return getIam().putUserPolicy({ PolicyDocument, PolicyName, UserName }).promise();
}

async function createPolicyDocument(commandsQueueArn, statusesQueueArn) {
    const document = await fsPromises.readFile(path.resolve(__dirname, config.get('paths.new-document-policy')), 'utf-8');
    return format(document, {
        commandsQueueArn,
        statusesQueueArn
    });
}

async function createUserAsync(userName) {
    const data = await getIam().createUser({ UserName: userName }).promise();
    return data.User;
}

async function getUserArn(userName) {
    const data = await getIam().getUser({ UserName: userName }).promise();
    return data.User.Arn;
}

async function createAccessKey(userName) {
    const data = await getIam().createAccessKey({ UserName: userName }).promise();
    return data.AccessKey;
}

function listAccessKeys(userName) {
    return getIam().listAccessKeys({ UserName: userName }).promise();
}

function deleteAccessKey(userName, accessKeyId) {
    return getIam().deleteAccessKey({ AccessKeyId: accessKeyId, UserName: userName }).promise();
}

async function deleteKeys(userName) {
    const data = await listAccessKeys(userName);
    return Promise.all(data.AccessKeyMetadata.map(accessKey => deleteAccessKey(userName, accessKey.AccessKeyId)));
}

async function listAttachedUserPolicies(userAndPolicyName) {
    const data = await iam.listUserPolicies({UserName:userAndPolicyName}).promise();
    return data.PolicyNames;
}


async function deleteUserPolicy(userName, policyName) {
    return getIam().deleteUserPolicy({ PolicyName: policyName, UserName: userName }).promise();
}

async function deleteUserPolicies(userName) {
    try {
        const userPolicies = await listAttachedUserPolicies(userName);
        await Promise.all(userPolicies.map(policy => deleteUserPolicy(userName, policy)));
    } catch (err) {
        if (err.code !== 'NoSuchEntity') {
            throw err;
        }
    }
}

async function deleteUser(userName) {
    return getIam().deleteUser({ UserName: userName }).promise();
}

async function waitForUserExists(userName) {
    let interval;
    let timeout;

    const setIntervalTime = '1s';
    const setTimeoutTime = '25s';

    const p1 = new Promise((resolve, reject) => {
        timeout = setTimeout(async() => {
            clearInterval(interval);
            reject(new Error(`Failed to find user ${userName}`));
        }, ms(setTimeoutTime));
    });

    const p2 = new Promise((resolve) => {
        interval = setInterval(async() => {
            try {
                logger.info(`Checking if user ${userName} exists `);
                await getUserArn(userName);
                logger.info(`user ${userName} exists`);
                clearInterval(interval);
                clearTimeout(timeout);
                resolve();
            } catch (err) {
                logger.info(`user ${userName} does not exist`);
            }
        }, ms(setIntervalTime));
    });

    return Promise.race([p1, p2]);
}


module.exports = {
    putUserPolicyAsync,
    createPolicyDocument,
    createUserAsync,
    getUserArn,
    createAccessKey,
    listAccessKeys,
    deleteKeys,
    deleteUserPolicy,
    deleteUser,
    waitForUserExists, deleteUserPolicies
};
