const config = require('config');
const HubSpot = require('hubspot');
const moment = require('moment');
const getUserEmailByJwtToken = require('../operations/auth0-operations');
const getLogger = require('../utils/logger');

const logger = getLogger();

const FIELDS = {
    SINCE: 'cloudsync_since',
    TRIAL_EXPIRY: 'cloudsync_trial_expiry',
    DATA_BROKER_SINCE: 'cloudsync_databroker_since',
    RELATIONSHIP_SINCE: 'cloudsync_relationship_since',
    SUBSCRIPTION_SINCE: 'cloudsync_subscription_since',
    LAST_PAYMENT: 'cloudsync_last_payment_since'
};

function hasHubSpotKey() {
    return config.get('hubspot.api-key') !== null;
}

async function createOrUpdateHubspotContact(account, email, hubSpotClient) {
    if (!hasHubSpotKey()) {
        logger.debug('HubSpot is not enabled');
        return;
    }
    try {
        const client = (hubSpotClient) || new HubSpot({
            apiKey: config.get('hubspot.api-key')
        });

        const properties = {
            properties: [
                {
                    property: 'email',
                    value: email
                },
                {
                    property: FIELDS.SINCE,
                    value: moment(account.createdAt).utc().startOf('day').valueOf()
                },
                {
                    property: FIELDS.TRIAL_EXPIRY,
                    value: moment(account.trialExpiry).utc().startOf('day').valueOf()
                }
            ]
        };

        const data = await client.contacts.createOrUpdate(email, properties);
        const action = data.isNew === true ? 'Created' : 'Updated';
        logger.info(`${action} HubSpot Contact ${data.vid} for user ${email}`);
        return data;
    } catch (err) {
        logger.warn(`Failed creating or updating HubSpot Contact for user ${email}:`, err);
    }
}

async function updateContactProperty(client, contact, property, account) {
    await client.contacts.update(contact.vid, {
        properties: [
            {
                property,
                value: moment().utc().startOf('day').valueOf()
            }
        ]
    });

    logger.info(`Added HubSpot Contact property ${property} mark for accountId: ${account.accountId}`);
}

async function setHubSpotContactPropertyIfNeeded(account, property, usersEmail) {
    if (!hasHubSpotKey()) {
        logger.debug('HubSpot is not enabled');
        return;
    }

    try {
        const client = new HubSpot({apiKey: config.get('hubspot.api-key')});

        Promise.map(usersEmail, async userEmail => {
            const userData = await client.contacts.getByEmail(userEmail);
            let contact;

            if (userData && userData.properties) {
                logger.info('Found existing HubSpot Contact by email: ', userEmail);
                contact = userData;
            } else {
                logger.info(`Existing HubSpot Contact by email ${userEmail} was not found: `, userData);
                logger.info(`Creating new HubSpot Contact for user with email ${userEmail}`);
                await createOrUpdateHubspotContact(account, userEmail, client);
                contact = await client.contacts.getByEmail(userEmail);
            }

            if (!contact.properties[property] || !contact.properties[property].value) {
                return updateContactProperty(client, contact, property, account);
            }

            if (property === FIELDS.LAST_PAYMENT && contact.properties[property] === moment().utc().startOf('day').valueOf()) {
                return updateContactProperty(client, contact, property, userEmail);
            }
        });
    } catch (err) {
        logger.warn(`Failed setting HubSpot Contact property: ${property} for accountId: ${account}`, err);
    }
}

async function setHubSpotContactFirstDataBrokerSinceIfNeeded(account) {
    const userEmail = await getUserEmailByJwtToken();
    return setHubSpotContactPropertyIfNeeded(account, FIELDS.DATA_BROKER_SINCE, [userEmail]);
}

async function setHubSpotContactFirstRelationshipSinceIfNeeded(account) {
    const userEmail = await getUserEmailByJwtToken();
    return setHubSpotContactPropertyIfNeeded(account, FIELDS.RELATIONSHIP_SINCE, [userEmail]);
}

async function setHubSpotContactFirstSubscriptionSinceIfNeeded(account) {
    const userEmail = await getUserEmailByJwtToken();
    return setHubSpotContactPropertyIfNeeded(account, FIELDS.SUBSCRIPTION_SINCE, [userEmail]);
}

module.exports = {
    setHubSpotContactFirstSubscriptionSinceIfNeeded,
    setHubSpotContactFirstRelationshipSinceIfNeeded,
    setHubSpotContactFirstDataBrokerSinceIfNeeded
};
