const axios = require('axios');
const {IPA_AUTHENTICATION_URL, IPA_LICENSE_URL, IPA_SERVER_ADDRESS, IPA_REGISTRATION_PATH, IPA_DE_REGISTRATION_PATH, IPA_REGISTRATION_STATUS_PATH, IPA_SERVICE_USER_ID, IPA_SERVICE_PWD} = require('../utils/consts');
const {validateResponse, validateRequest} = require('../utils/schema-validation');
const {
    verifyNssCredentialsRequestSchema,
    statusResponseSchema,
    registrationUserRequestSchema,
    registrationUserResponseSchema,
    registrationUserStatusRequestSchema,
    deRegistrationUserRequestSchema,
    deRegistrationUserResponseSchema
} = require('../schemas/ipa-schemas');

const moment = require('moment');
const getLogger = require('../utils/logger');
const logger = getLogger();


async function getLicense(nssUserName, nssPassword, serialNumber) {
    logger.info(`Get License async for user: ${nssUserName}`);

    const {data} = await axios.post(IPA_LICENSE_URL, {
        licenseRequest: {
            version: '1',
            serialNumber,
            op: 'get',
            package: 'CLOUD-SYNC-AWS',
            product: 'CLOUDSYNC'
        }
    }, {
        auth: {
            username: nssUserName,
            password: nssPassword
        },
    });

    return data;
}

async function authenticateNssCredentials(nssUserName, nssPassword) {
    logger.info(`Authenticate NSS credentials for user: ${nssUserName}`);

    const body = {
        licenseRequest: {
            version: '1', serialNumber: '', op: 'get', package: ''
        }
    };

    validateRequest(body, verifyNssCredentialsRequestSchema);

    const {data} = await axios.post(IPA_AUTHENTICATION_URL, body, {
        auth: {
            username: nssUserName,
            password: nssPassword
        }
    });

    validateResponse(data, statusResponseSchema);

    return data;
}

async function register(nssUserName, serialNumber, tenancyAccountId) {
    logger.info('Execute registration request: ', {
        nssUserName, serialNumber, tenancyAccountId
    });

    const body = {
        serviceUserID: IPA_SERVICE_USER_ID,
        servicePwd: IPA_SERVICE_PWD,
        nssUserID: nssUserName,
        awsAccountID: tenancyAccountId,
        awsVSAEC2InstanceID: 'na',
        vsaSerialNumber: serialNumber,
        vsaStartDate: moment().format('YYYYMMDD')
    };

    validateRequest(body, registrationUserRequestSchema);

    const {data} = await axios.post(`${IPA_SERVER_ADDRESS}${IPA_REGISTRATION_PATH}`, body);

    validateResponse(data, registrationUserResponseSchema);

    return data;
}

async function deregister(serialNumber, tenancyAccountId) {
    logger.info('Execute deRegistration request: ', {
        serialNumber, tenancyAccountId
    });

    const body = {
        serviceUserID: IPA_SERVICE_USER_ID,
        servicePwd: IPA_SERVICE_PWD,
        vsaSerialNumber: serialNumber,
        cpAccountID: tenancyAccountId
    };

    validateRequest(body, deRegistrationUserRequestSchema);

    const {data} = await axios.post(`${IPA_SERVER_ADDRESS}${IPA_DE_REGISTRATION_PATH}`, body);

    validateResponse(data, deRegistrationUserResponseSchema);

    return data;
}

async function checkRegistrationStatus(serialNumber, nssUserName) {
    const body = {serviceUserID: IPA_SERVICE_USER_ID, servicePwd: IPA_SERVICE_PWD, vsaSerialNumber: serialNumber};

    validateRequest(body, registrationUserStatusRequestSchema);
    const headers = (process.env.IS_SIMULATOR) ? {'X-user-id': nssUserName} : undefined; // in order for the simulator to distinguish between failed requests

    const {data} = await axios.post(`${IPA_SERVER_ADDRESS}${IPA_REGISTRATION_STATUS_PATH}`, body, {headers});

    validateResponse(data, statusResponseSchema);

    return data;
}

module.exports = {
    getLicense,
    authenticateNssCredentials,
    register,
    deregister,
    checkRegistrationStatus
};