const mongoose = require('mongoose');
const { defaultTransform } = require('../utils/utils');


const statisticSchema = new mongoose.Schema({
    bytes: { type: Number, required: true, default: 0 },
    items: { type: Number, required: true, default: 0 }
}, {
    strict: false,
    _id: false,
    toJSON: {
        transform: defaultTransform((doc, ret) => ret)
    }
});

const counterSchema = new mongoose.Schema({
    file: { type: statisticSchema, required: true },
    directory: { type: statisticSchema, required: true },
    symbolic: { type: statisticSchema, required: true },
    other: { type: statisticSchema, required: true }
});

const TransferSchema = new mongoose.Schema({
}, {
    _id: false,
    delete: { type: counterSchema, required: true },
    failedDelete: { type: counterSchema, required: true },
    copy: { type: counterSchema, required: true },
    failedCopy: { type: counterSchema, required: true },
    abort: { type: counterSchema, required: true }
}, {
    strict: false,
    toJSON: {
        transform: defaultTransform((doc, ret) => ret)
    }
});

const ScanSchema = new mongoose.Schema({
}, {
    _id: false,
    failed: { type: counterSchema, required: true },
    delete: { type: counterSchema, required: true },
    copy: { type: counterSchema, required: true },
    grace: { type: counterSchema, required: true },
    ignore: { type: counterSchema, required: true },
    abort: { type: counterSchema, required: true }
}, {
    strict:false,
    toJSON: {
        transform: defaultTransform((doc, ret) => ret)
    }
});


const ListSchema = new mongoose.Schema({
}, {
    _id: false,
    failed: { type: counterSchema, required: true },
    list: { type: counterSchema, required: true }
}, {
    toJSON: {
        transform: defaultTransform((doc, ret) => ret)
    }
});

const relationshipStatisticsSchema = mongoose.Schema({
    relationship: {
        type: mongoose.Schema.Types.ObjectId, ref: 'Relationship', required: true, index: true
    },
    dataBroker: {
        type: mongoose.Schema.Types.ObjectId, ref: 'DataBroker', required: true, index: true
    },
    account: {
        type: mongoose.Schema.Types.ObjectId, ref: 'Account', index: true
    },
    scan: { type: ScanSchema, required: false },
    transfer: { type: TransferSchema, required: false },
    list: { type: ListSchema, required: false },
    status: { type: String, required: false },
    reason: { type: String },
    relationshiperrors: { type: mongoose.Schema.Types.Mixed, required: false }
}, {
    strict:false,
    timestamps: true,
    toJSON: {
        transform: defaultTransform((doc, ret) => ret)
    }
});

relationshipStatisticsSchema.index({ relationship: 1, dataBroker: 1, account: 1 }, { unique: true });

relationshipStatisticsSchema.statics.findRelationshipStatus = async function(relationshipId) {
    return this.aggregate([
        { $match: { relationship: relationshipId } },
        {
            $group: {
                _id: '$relationship',
                statuses: { $addToSet: { status: '$status', reason: '$reason' } }
            }
        }]);
};

const RelationshipStats =  mongoose.model('RelationshipStats', relationshipStatisticsSchema);

async function sumStatsPerRelationship(relationshipId) {
    return RelationshipStats.aggregate([
        { $match: { relationship: mongoose.Types.ObjectId(relationshipId) } },
        {
            $group: {
                _id: '$relationship',
                filesMarkedForCopy: { $sum: '$scan.copy.file.items' },
                bytesMarkedForCopy: { $sum: '$scan.copy.file.bytes' },
                dirsMarkedForCopy: { $sum: '$scan.copy.directory.items' },
                symbolicMarkedForCopy: { $sum: '$scan.copy.symbolic.items' },
                symbolicBytesMarkedForCopy: { $sum: '$scan.copy.symbolic.bytes' },
                filesCopied: { $sum: '$transfer.copy.file.items' },
                dirsCopied: { $sum: '$transfer.copy.directory.items' },
                bytesCopied: { $sum: '$transfer.copy.file.bytes' },
                symbolicBytesCopied : {$sum: '$transfer.copy.symbolic.bytes'},
                symbolicCopied: { $sum: '$transfer.copy.symbolic.items' },
                filesFailed: { $sum: '$transfer.copy_failed.file.items' },
                dirsFailed: { $sum: '$transfer.copy_failed.directory.items' },
                bytesFailed: { $sum: '$transfer.copy_failed.file.bytes' },
                symbolicFailed: { $sum: '$transfer.copy_failed.symbolic.items' },
                symbolicBytesFailed: { $sum: '$transfer.copy_failed.symbolic.bytes' },
                filesRemoved: { $sum: '$transfer.delete_target.file.items' },
                bytesRemoved: { $sum: '$transfer.delete_target.file.bytes' },
                dirsRemoved: { $sum: '$transfer.delete_target.directory.items' },
                symbolicRemoved: { $sum: '$transfer.delete_target.symbolic.items' },
                symbolicBytesRemoved: { $sum: '$transfer.delete_target.symbolic.bytes' },
                bytesMarkedForRemove: { $sum: '$scan.delete_target.file.bytes' },
                filesMarkedForRemove: { $sum: '$scan.delete_target.file.items' },
                dirsMarkedForRemove: { $sum: '$scan.delete_target.directory.items' },
                symbolicMarkedForRemove: { $sum: '$scan.delete_target.symbolic.items' },
                symbolicBytesMarkedForRemove: { $sum: '$scan.delete_target.symbolic.bytes' },
                bytesRemovedFailed: { $sum: '$transfer.delete_failed.file.bytes' },
                filesRemovedFailed: { $sum: '$transfer.delete_failed.file.items' },
                dirsRemovedFailed: { $sum: '$transfer.delete_failed.directory.items' },
                symbolicRemovedFailed: { $sum: '$transfer.delete_failed.symbolic.items' },
                symbolicBytesRemovedFailed: { $sum: '$transfer.delete_failed.symbolic.bytes' },
                filesMarkedForIgnore: { $sum: '$scan.ignore.file.items' },
                dirsMarkedForIgnore: { $sum: '$scan.ignore.directory.items' },
                symbolicMarkedForIgnore: { $sum: '$scan.ignore.symbolic.items' },
                symbolicBytesMarkedForIgnore: { $sum: '$scan.ignore.symbolic.bytes' },
                bytesMarkedForIgnore: { $sum: '$scan.ignore.file.bytes' },
                dirsScanned: { $sum: '$list.list.directory.items' },
                filesScanned: { $sum: '$list.list.file.items' },
                symbolicScanned: { $sum: '$list.list.symbolic.items' },
                symbolicBytesScanned: { $sum: '$list.list.symbolic.bytes' },
                bytesScanned: { $sum: '$list.list.file.bytes' },
                dirsFailedToScan: { $sum: '$list.failed.directory.items' },
                lastMessageTime: { $max: '$updatedAt' },
                statuses: { $addToSet: '$statusMessages.status' },
                relationshipListErrors: { $push: '$list.relationshiperrors' },
                relationshipTransferErrors: { $push: '$transfer.relationshiperrors' }
            }
        }]);
}


module.exports = {RelationshipStats, sumStatsPerRelationship};