const mongoose = require('mongoose');
const {defaultTransform} = require('../utils/utils');

const typesSchema = new mongoose.Schema({
    files: {type: Boolean, required: true, default: true},
    directories: {type: Boolean, required: true, default: true},
    symlinks: {type: Boolean, required: true, default: true}
}, {
    _id: false,
    toJSON: {
        transform: defaultTransform((doc, ret) => ret)
    }
});

const filesSchema = new mongoose.Schema({
    excludeExtensions: {type: Array, required: true, default: []},
    maxSize: {type: Number, required: true, default: Number.MAX_SAFE_INTEGER},
    minSize: {type: Number, required: true, default: 0},
    minDate: {type: String, required: true, default: '1970-01-01'},
    maxDate: {type: String, required: false, default: null}
}, {
    _id: false,
    toJSON: {
        transform: defaultTransform((doc, ret) => ret)
    }
});

const syncSchema = new mongoose.Schema({
    syncInDays: {type: Number, required: true, default: 1},
    syncInHours: {type: Number, required: true, default: 0},
    syncInMinutes: {type: Number, required: true, default: 0},
    nextTime: {type: Date, required: true},
    isEnabled: {
        type: Boolean, required: true, default: true
    },
    syncWhenCreated: {
        type: Boolean, required: true, default: true
    }
}, {
    _id: false,
    toJSON: {
        transform: defaultTransform((doc, ret) => ret)
    }
});

const settingsSchema = new mongoose.Schema({
    gracePeriod: {type: Number, required: true, default: 30},
    deleteOnSource: {type: Boolean, required: true, default: false},
    deleteOnTarget: {type: Boolean, required: true, default: false},
    objectTagging: {type: Boolean, required: true, default: true},
    retries: {type: Number, required: true, default: 3},
    copyAcl: {type: Boolean, required: true, default: false},
    files: {type: filesSchema, required: true, default: {}},
    fileTypes: {type: typesSchema, required: true, default: {}},
    schedule: {type: syncSchema, required: true, default: {}}
}, {
    _id: false,
    toJSON: {
        transform: defaultTransform((doc, ret) => ret)
    }
});


const endPointSchema = new mongoose.Schema({
    protocol: {type: String, enum: ['nfs', 'cifs', 's3', 'local', 'azure', 'gcp'], required: true},
    nfs: {type: mongoose.Schema.Types.Mixed, required: false},
    cifs: {type: mongoose.Schema.Types.Mixed, required: false},
    s3: {type: mongoose.Schema.Types.Mixed, required: false},
    local: {type: mongoose.Schema.Types.Mixed, required: false},
    azure: {type: mongoose.Schema.Types.Mixed, required: false},
    gcp: {type: mongoose.Schema.Types.Mixed, required: false}
}, {
    _id: false,
    toJSON: {
        transform: defaultTransform((doc, ret) => ret)
    }
});

const databrokerTypeSchema = mongoose.Schema({
    dataBrokerId: {type: mongoose.Schema.Types.ObjectId, ref: 'DataBroker', required: true},
    endpointType: {type: String, enum: ['src', 'trg'], required: true}
}, {
    _id: false,
    toJSON: {
        transform: defaultTransform((doc, ret) => ret)
    }
});

const encryptionSchema = mongoose.Schema({
    initiator: {type: databrokerTypeSchema, required: true},
    listener: {type: databrokerTypeSchema, required: true}
}, {
    _id: false,
    toJSON: {
        transform: defaultTransform((doc, ret) => ret)
    }
});

const relationshipSchema = mongoose.Schema({
    account: {
        type: mongoose.Schema.Types.ObjectId, ref: 'Account', index: true
    },
    dataBroker: {type: mongoose.Schema.Types.ObjectId, ref: 'DataBroker'},
    source: {type: endPointSchema, required: true},
    target: {type: endPointSchema, required: true},
    messages: {type: mongoose.Schema.Types.Mixed, required: false},
    encryption: {type: encryptionSchema, required: false},
    settings: {type: settingsSchema, required: true},
    isQstack: {type: Boolean, required: true, default: false},
    isCvo: {type: Boolean, required: true, default: false},
    isCm: {type: Boolean, required: false, default: false},
    scannerQueue: {type: String, required: false},
    transferrerQueue: {type: String, required: false},
    phase: {
        type: String, enum: ['Initial Copy', 'Sync'], required: true, default: 'Initial Copy'
    },
    startTime: {type: Date, required: false},
    endTime: {type: Date, required: false},
    group: {type: mongoose.Schema.Types.ObjectId, ref: 'Group', index: true}
}, {
    timestamps: true,
    toJSON: {
        virtuals: true,
        transform: defaultTransform((doc, ret) => {
            ret.relationshipId = doc._id;
            return ret;
        })
    }
});

relationshipSchema.index({account: 1, _id: 1});
relationshipSchema.index({
    account: 1, dataBroker: 1, source: 1, target: 1, encryption:1
}, {unique: true});


module.exports = mongoose.model('Relationship', relationshipSchema);
