const randomize = require('randomatic');
const ms = require('ms');
const {authenticateNssCredentials, register, deregister, checkRegistrationStatus} = require('../lib/ipa-server');
const {Account} = require('../models/account');
const {
    IPA_PENDING_STATUS, IPA_REGISTERED_STATUS, IPA_UNREGISTERED_STATUS, MESSAGE_UPDATE_SUPPORT_INFO_EVENT, IPA_SERIAL_NUMBER_PREFIX
} = require('../utils/consts');
const {Forbidden, BadRequest} = require('../utils/errors');
const {createMessage} = require('../models/message-v2');
const crypto = require('crypto');
const algorithm = 'aes-128-cbc';
const notEncryptedEncoding = 'utf8';
const encryptedEncoding = 'hex';
const key = Buffer.from('5ebe2294ecd0e0f08eab7690d2a6ee69', encryptedEncoding);
const iv = Buffer.from('26ae5cc854e36b6bdfca366848dea6bb', encryptedEncoding);

const getLogger = require('../utils/logger');

const logger = getLogger();


function encryptNssPassword(password) {
    const cipher = crypto.createCipheriv(algorithm, key, iv);
    let crypted = cipher.update(password, notEncryptedEncoding, encryptedEncoding);
    crypted += cipher.final(encryptedEncoding);
    return crypted;
}

function decryptNssPassword(password) {
    const decipher = crypto.createDecipheriv(algorithm, key, iv);
    let dec = decipher.update(password, encryptedEncoding, notEncryptedEncoding);
    dec += decipher.final(notEncryptedEncoding);
    return dec;
}

async function validateNssCredentials(nssUserName, nssPassword) {
    logger.info('Validate NSS credentials for nssUserName: ', {nssUserName});

    const ipaResponse = await authenticateNssCredentials(nssUserName, nssPassword);
    const {statusResp: {statusCode, message}} = ipaResponse;

    if (statusCode !== 'S001') {
        logger.error(`Failed to validate NSS credentials received statusCode: ${statusCode}, message: ${message}`);
        throw new BadRequest(message);
    }
}

function generateSerialNumber() {
    return `${IPA_SERIAL_NUMBER_PREFIX}${randomize('0', 17)}`;
}

async function registerSerialNumber(serialNumber, nssUserName, tenancyAccountId) {
    logger.info('Register for support: ', {nssUserName, serialNumber, tenancyAccountId});

    const registrationResponse = await register(nssUserName, serialNumber, tenancyAccountId);
    const {regResp: {statusCode, message}} = registrationResponse;

    if (statusCode !== 'S002') {
        logger.error(`Received statusCode: ${statusCode} and message: ${message} after registration`);

        throw new Forbidden(message);
    }
}

async function deregisterSupport(serialNumber, accountId, tenancyAccountId) {
    logger.info('Deregister from support: ', {
        serialNumber, accountId, tenancyAccountId
    });

    await deregister(serialNumber, tenancyAccountId);

    const newSerialNumber = generateSerialNumber();

    await Account.updateOne({tenancyAccountId}, {
        $set: {
            support: {
                serialNumber: newSerialNumber
            }
        },
        $unset: {nssCredentials: undefined}
    }).exec();
    await createMessage(accountId, undefined, undefined, {
        [MESSAGE_UPDATE_SUPPORT_INFO_EVENT]: {
            supportInfo: {
                id: accountId,
                status: IPA_UNREGISTERED_STATUS,
                serialNumber: newSerialNumber
            }
        }
    }
    );
}

async function pollForRegistrationStatus(serialNumber, nssUserName, accountId, tenancyAccountId) {
    logger.info('Polling for registration status from IPA server', {
        serialNumber, nssUserName,  accountId, tenancyAccountId
    });

    let interval;
    let timeout;

    const p1 = new Promise(resolve => {
        timeout = setTimeout(async() => {
            clearInterval(interval);
            await deregisterSupport(serialNumber, accountId, tenancyAccountId);
            logger.warn(`Failed to handle support registration for 1 minute by serialNumber: ${serialNumber}`);
            resolve();
        }, ms('1m'));
    });

    const p2 = new Promise(resolve => {
        interval = setInterval(async() => {
            const data = await checkRegistrationStatus(serialNumber, nssUserName);
            const {statusResp: {statusCode, message}} = data;
            logger.info('Received registration status:', statusCode, 'with message:', message);

            if (statusCode === 'S002') {
                logger.info('Waiting for registration to be completed');
            } else {
                clearInterval(interval);
                clearTimeout(timeout);

                if (statusCode === 'S001') {
                    logger.info('Successfully registered for support');

                    await Account.findByIdAndUpdate(accountId, {$set: {'support.status': IPA_REGISTERED_STATUS}}).exec();
                    await createMessage(accountId, undefined, undefined, {[MESSAGE_UPDATE_SUPPORT_INFO_EVENT]: {
                        supportInfo: {
                            id: accountId,
                            status: IPA_REGISTERED_STATUS,
                            serialNumber
                        }
                    }});
                } else {
                    logger.warn('Got unexpected registration status', statusCode);
                    await deregisterSupport(serialNumber, accountId, tenancyAccountId);
                }

                resolve(statusCode);
            }
        }, ms('5s'));
    });

    await Promise.race([p1, p2]);
}

async function registerForSupport(account, nssUserName, nssPassword) {

    const {id:accountId, tenancyAccountId, support:{serialNumber}} = account;

    logger.info('Starting registration for support: ', {
        accountId, tenancyAccountId, nssUserName
    });

    if (account.support.status === IPA_UNREGISTERED_STATUS) {
        await validateNssCredentials(nssUserName, nssPassword);

        await registerSerialNumber(serialNumber, nssUserName, tenancyAccountId);

        logger.info('Updating now: ', {
            accountId, tenancyAccountId, nssUserName, nssPassword
        });

        await Account.findByIdAndUpdate(accountId, {
            $set: {
                'support.status': IPA_PENDING_STATUS,
                'nssCredentials.nssUsername': nssUserName,
                'nssCredentials.nssPassword': encryptNssPassword(nssPassword),
            }
        }).exec();

        logger.info('Support details were successfully saved in the db for account Id: ', tenancyAccountId);

        // asynchronously poll for registration status and send updates through messages
        pollForRegistrationStatus(serialNumber, nssUserName, accountId, tenancyAccountId);

        return {serialNumber, status: IPA_PENDING_STATUS};
    } else {
        throw new BadRequest(`Account: ${tenancyAccountId} is already registered to support`);
    }
}


module.exports = {
    registerForSupport,
    deregisterSupport,
    encryptNssPassword,
    decryptNssPassword,
    generateSerialNumber
};