const moment = require('moment');
const { get, set } = require('lodash');
const TimelineV2 = require('../models/timeline-v2');
const {
    MESSAGE_STATUS_ABORTED,
    MESSAGE_STATUS_DONE,
    MESSAGE_STATUS_FAILED,
    RUNNING_STATUS,
    TIMELINE_DELETED_RELATIONSHIP_MESSAGE,
    MESSAGE_STATUS_PENDING
} = require('../utils/consts');
const {sendAuditGroup} = require('../operations/audit-operations');

const getLogger = require('../utils/logger');

const logger = getLogger();

async function findTimelineRecordV2(relationship, isAbort) {
    const {account, id: relationshipId} = relationship;
    const filter = {'relationship.relationshipId': relationshipId,
        status: {$nin: ['DONE', 'ABORTED', 'FAILED']}};
    if (isAbort) {
        filter.summary = 'Abort relationship data sync';
    } else {
        filter.summary = {$ne: 'Abort relationship data sync'};
    }
    const timeLineResult = await TimelineV2.findOne(filter).hint({
        'relationship.relationshipId': 1,
        status: 1,
        summary: 1
    }).populate('account').sort({updatedAt: -1})
        .limit(1)
        .exec();
    if (!timeLineResult) {
        logger.info(`TimeLine action with account ${JSON.stringify(account)} and relationshipId ${relationshipId} was not found`);
    } else {
        return timeLineResult;
    }
}

async function updateTimeLineNewDataBroker(relationshipResponse, action, findTimeLineRecordFunc, activityKey) {
    const progressStatuses = [RUNNING_STATUS, 'STARTED'];
    try {
        const timeLineResultRelationship = await findTimeLineRecordFunc(relationshipResponse);
        if (timeLineResultRelationship) {
            switch (action) {
                case MESSAGE_STATUS_ABORTED: {
                    timeLineResultRelationship.status = get(relationshipResponse, 'activity.status', MESSAGE_STATUS_ABORTED);
                    timeLineResultRelationship.failureMessage = get(relationshipResponse, 'activity.failureMessage');
                    set(timeLineResultRelationship, activityKey, relationshipResponse.activity);

                    // there will always be an abort record
                    const timeLineResultAbort = await findTimeLineRecordFunc(relationshipResponse, true);
                    timeLineResultAbort.status = MESSAGE_STATUS_DONE;
                    timeLineResultAbort.save();
                    await sendAuditGroup(timeLineResultAbort, timeLineResultAbort.account.tenancyAccountId,
                        undefined, timeLineResultAbort.relationship);
                }
                    break;
                case 'DELETE': {
                    timeLineResultRelationship.status = MESSAGE_STATUS_FAILED;
                    timeLineResultRelationship.failureMessage = TIMELINE_DELETED_RELATIONSHIP_MESSAGE;
                    set(timeLineResultRelationship, activityKey, relationshipResponse.activity);

                    const timeLineResultAbort = await findTimeLineRecordFunc(relationshipResponse, true);
                    if (timeLineResultAbort) {
                        timeLineResultAbort.status = MESSAGE_STATUS_FAILED;
                        timeLineResultAbort.failureMessage = TIMELINE_DELETED_RELATIONSHIP_MESSAGE;
                        timeLineResultAbort.save();
                    }
                }
                    break;
                case MESSAGE_STATUS_PENDING : {
                    timeLineResultRelationship.status = MESSAGE_STATUS_PENDING;
                    timeLineResultRelationship.relationship.activateRelationshipAt = relationshipResponse.settings.schedule.nextTime;
                }
                    break;
                default: {
                    const statusFromRelationship = get(relationshipResponse, 'activity.status', RUNNING_STATUS);
                    timeLineResultRelationship.status = progressStatuses.includes(statusFromRelationship) ? 'PROGRESS' : statusFromRelationship;
                    timeLineResultRelationship.failureMessage = get(relationshipResponse, 'activity.failureMessage');
                    set(timeLineResultRelationship, activityKey, relationshipResponse.activity);
                }
            }
            await timeLineResultRelationship.save();
            await sendAuditGroup(timeLineResultRelationship, timeLineResultRelationship.account.tenancyAccountId,
                undefined, timeLineResultRelationship.relationship);

        }
    } catch (err) {
        logger.error(`An error occurred in updateTimeLineNewDataBroker function trying to update 
        relationship ${relationshipResponse.id} with message ${JSON.stringify(relationshipResponse.activity.status)}`, err);
        throw err;
    }
}

async function findTotalBytesAndFilesCopiedForAnAccount(accountId, target) {
    const matchQuery = {
        account: accountId,
        status: {$nin: ['FAILED', 'PROGRESS']},
        'relationship.relationshipId': {$exists: true}
    };
    if (target) {
        matchQuery['relationship.source.protocol'] = {$in:['nfs', 'cifs']};
        matchQuery.$or = [{'relationship.target.nfs.provider':target},
            {'relationship.target.cifs.provider':target}];
    }
    //find all bytes and files copied forever - all relationships that has DONE do we want ABORTED or deleted as well?
    const [aggregateResult] = await TimelineV2.aggregate([{
        $match: matchQuery
    },
    {
        $group: {
            _id: '$account',
            allBytesCopied: {$sum: '$relationship.activity.bytesCopied'},
            allFilesCopied: {$sum: '$relationship.activity.filesCopied'}
        }
    }]);
    return {totalBytes: aggregateResult ? aggregateResult.allBytesCopied : 0, totalFiles: aggregateResult ? aggregateResult.allFilesCopied : 0};
}

async function findBytesCopiedForAnAccountInTheLast30Days(accountId, target) {
    //find an array of relationship stats

    const matchQuery = {
        account: accountId,
        status: {$nin: ['FAILED', 'PROGRESS']},
        'relationship.relationshipId': {$exists: true},
        'relationship.activity.endTime': {$gte: new Date(moment().subtract(30, 'day'))}
    };
    if (target) {
        matchQuery['relationship.source.protocol'] = {$in:['nfs', 'cifs']};
        matchQuery.$or = [{'relationship.target.nfs.provider':target},
            {'relationship.target.cifs.provider':target}];
    }

    return TimelineV2.aggregate([{
        $match: matchQuery
    },
    {
        $project: {
            date: {
                $dateToString: {
                    format: '%Y-%m-%d',
                    date: '$relationship.activity.endTime'
                }
            },
            activity: '$relationship.activity'
        }
    },
    {$group: {_id: '$date', bytesCopied: {$sum: '$activity.bytesCopied'}}},
    {$project: {date: '$_id', _id: 0, bytesCopied: 1}}]);

}

module.exports = { findTotalBytesAndFilesCopiedForAnAccount, findBytesCopiedForAnAccountInTheLast30Days, findTimelineRecordV2, updateTimeLineNewDataBroker };
