const express = require('express');
const config = require('config');
const Promise = require('bluebird');
const moment = require('moment');
const {assignIn} = require('lodash');
const {agenda} = require('../utils/agenda');
const DataBroker = require('../models/data-broker');
const Relationship = require('../models/relationship');
const {createMessage} = require('../models/message-v2');
const Group = require('../models/group');
const {inContext} = require('../middlewares/continuation-local-storage');
const {
    MONGO_DB_ERROR_DUPLICATE_KEY,
    DATA_BROKER_CF_FOLDER,
    DFIO_METADATA_BUCKET_REGION,
    DFIO_METADATA_BUCKET_NAME,
    CLOUD_FORMATION_ALL_REGIONS_LINK, ONTAP_S3_PROVIDER,
    MESSAGE_STATUS_DONE, S3_PROVIDER, SGWS_PROVIDER, ALIBABA_PROVIDER, IBM_PROVIDER,
    AGENDA_TASK_NAME_DELETE_NOT_ACTIVE_DATA_BROKERS,
    API_PATH_DATA_BROKERS
} = require('../utils/consts');
const {
    assertIpOrHostname
} = require('../utils/common');
const {pollQueue} = require('../utils/sqs-listener');
const {BadRequest, NotFound} = require('../utils/errors');
const {timeout} = require('../utils/utils');
const {putObject, getS3ObjectUrl} = require('../lib/aws/s3');
const {setHubSpotContactFirstDataBrokerSinceIfNeeded} = require('../lib/hubspot');
const {
    sendListKmsKeys,
    sendS3ListBucketsFolders,
    sendListExportsFoldersCommand, sendListExportsCommand,
    sendListAzureBlobStorageContainersFoldersCommand,
    sendListAzureBlobStorageContainers, sendShutdownCommand,
    sendSharesCommand, sendShareFoldersCommand,
    sendListGcpBucketsCommand, sendListGcpBucketFoldersCommand,
    sendListS3BucketsCommand, createCFN,
    issueDataBrokerCommandAndRespond,
    deleteUserAndPolicyAndQueues, deleteFileLinkIfNeeded,
    removeDataBrokerFromGroupIfNeeded,
    validateDataBrokerDeletion, validateBrokerCreationAndUpdateTimeline, createAndUploadInstaller
} = require('../operations/data-broker-operations');

const getLogger = require('../utils/logger');

const logger = getLogger();

const router = express.Router();


router.get(API_PATH_DATA_BROKERS, async(req, res, next) => {
    try {
        const {account} = req;
        const dataBrokers = await DataBroker.find({account}).sort({createdAt: 1}).exec();
        res.json(dataBrokers);
    } catch (err) {
        next(err);
    }
});

router.post(API_PATH_DATA_BROKERS, async(req, res, next) => {
    let fileLink;
    const {account, body: {
        name
    }} = req;
    let dataBroker;
    let group;
    try {
        const dataBrokerObject = await validateBrokerCreationAndUpdateTimeline(req);
        ({group, dataBroker}  = dataBrokerObject);
        const {type:dataBrokerType, name} = dataBroker;
        const installerUrl = await createAndUploadInstaller(dataBroker);
        // creating fileLink
        if (dataBrokerType === 'ONPREM') {
            fileLink = installerUrl;
        } else {
            const cftKeyPrefix = `${DATA_BROKER_CF_FOLDER}/${dataBroker.externalId}`;
            const cftKey = (dataBrokerType === 'GCP') ? `${cftKeyPrefix}.jinja` : cftKeyPrefix;
            await putObject(DFIO_METADATA_BUCKET_REGION, DFIO_METADATA_BUCKET_NAME, cftKey, createCFN(installerUrl, dataBrokerType), false, true);
            const templateURL = getS3ObjectUrl(DFIO_METADATA_BUCKET_REGION, DFIO_METADATA_BUCKET_NAME, cftKey,
                dataBroker.type !== 'AWS', false);
            fileLink = (dataBrokerType === 'AZURE' || dataBrokerType === 'GCP') ? templateURL :
                `${CLOUD_FORMATION_ALL_REGIONS_LINK}#/stacks/new?stackName=${dataBroker.name}&templateURL=${encodeURIComponent(templateURL)}`;
        }
        dataBroker = await dataBroker.save();

        if (!config.get('one-queue')) {
            pollQueue(dataBroker);
        }

        res.json({type:dataBrokerType, fileLink, id: dataBroker.id, name});
        setHubSpotContactFirstDataBrokerSinceIfNeeded(account);

    } catch (error) {
        if (dataBroker) {
            await deleteUserAndPolicyAndQueues(dataBroker);
            await DataBroker.deleteOne({'_id':dataBroker._id}).exec();
        }

        if (group) {
            // Assumption- if the group has 1 data broker it is the data broker that was not created and it should be deleted
            await removeDataBrokerFromGroupIfNeeded(dataBroker, group);
        }

        if (error.code === MONGO_DB_ERROR_DUPLICATE_KEY || error.errorCode === MONGO_DB_ERROR_DUPLICATE_KEY) {
            logger.error(`Databroker with name  ${name} already exists`, error);
            next(new BadRequest(`Databroker with name  ${name} already exists`));
        } else {
            next(error);
        }
    }
});

router.post(`${API_PATH_DATA_BROKERS}/template`, async(req, res, next) => {
    const {
        account, body: {
            name, type:dataBrokerType
        }
    } = req;
    let dataBroker;
    let group;

    try {
        ({dataBroker, group } = await validateBrokerCreationAndUpdateTimeline(req));

        const installerUrl = await createAndUploadInstaller(dataBroker);
        if (!config.get('one-queue')) {
            pollQueue(dataBroker);
        }

        //generate the template - for now only gcp is supported
        const templateContent = createCFN(installerUrl, dataBrokerType);
        const template = {
            config: {
                // eslint-disable-next-line
                content: 'imports:\n- path: new-gcp-template.jinja\nresources:\n- name: new-gcp-template-jinja\n  properties:\n    network: {{network}}\n    project: {{project}}\n    region: {{region}}\n    service-account: {{serviceAccount}}\n    subnetwork: {{subnetwork}}\n    zone: {{zone}}\n  type: new-gcp-template.jinja\n'
            },
            imports: [
                {
                    content: templateContent,
                    name: 'new-gcp-template.jinja'
                }
            ]
        };


        res.json({type:dataBrokerType, template, id: dataBroker.id, name});
        setHubSpotContactFirstDataBrokerSinceIfNeeded(account);

    } catch (error) {
        if (dataBroker) {
            await deleteUserAndPolicyAndQueues(dataBroker);
            await DataBroker.deleteOne({'_id':dataBroker._id}).exec();
        }
        if (group) {
            // Assumption- if the group has 1 data broker it is the data broker that was not created and it should be deleted
            await removeDataBrokerFromGroupIfNeeded(dataBroker, group);
        }
        if (error.code === MONGO_DB_ERROR_DUPLICATE_KEY || error.errorCode === MONGO_DB_ERROR_DUPLICATE_KEY) {
            logger.error(`Databroker with name  ${name} already exists`, error);
            next(new BadRequest(`Databroker with name  ${name} already exists`));
        } else {
            next(error);
        }
    }
});


router.delete(`${API_PATH_DATA_BROKERS}/:id`, async(req, res, next) => {
    try {
        const {account, params: {id: dataBrokerId}, id: requestId} = req;
        logger.debug(`Delete Data-Broker: ${dataBrokerId} for tenancyAccountId: `, account.tenancyAccountId);

        const dataBroker = await DataBroker.findById(dataBrokerId).populate('groupId').populate({
            path: 'groupId',
            populate: {path: 'dataBrokers'}
        }).exec();

        const {name, groupId: group} = dataBroker || {};

        // update timeline
        req.timeline = assignIn(req.timeline, {dataBroker: name});
        req.timelineV2 = assignIn(req.timelineV2, {
            dataBroker: {name, dataBrokerId, groupId: group ? group.id : undefined},
            group: group ? {
                groupId: group.id,
                name: group.name,
                dataBrokers: group.dataBrokers.map(broker => ({dataBrokerId: broker.id, name: broker.name}))
            } : undefined
        });

        if (!dataBroker) {
            logger.warn(`Data Broker with id ${dataBrokerId} was not found for deletion`);
            next(new NotFound());
        } else {
            const {dataBrokers} = group || [dataBroker];
            await validateDataBrokerDeletion(dataBrokers, group, dataBroker);

            if (group) {
                const anotherDataBrokerFromTheGroup = dataBrokers.find(dataBrokerSample => dataBrokerSample.id !== dataBrokerId);
                await Relationship.updateMany({
                    dataBroker: dataBrokerId,
                    group: group.id
                }, {$set: {dataBroker: anotherDataBrokerFromTheGroup}}).exec();

                await removeDataBrokerFromGroupIfNeeded(dataBroker, group);
            }

            try {
                logger.info(`Sending shutdown command to databroker ${dataBroker._id} of account ${account._id}`);
                await sendShutdownCommand(requestId, dataBroker);
            } catch (err) {
                logger.error('Failed to send shutdown command to dataBroker', dataBroker.id, 'of account', account._id);
            }

            await deleteFileLinkIfNeeded(dataBroker);
            logger.info(`Removing data broker with id ${dataBrokerId} from DB `);
            await dataBroker.deleteOne();
            res.json();
            await timeout(60000);

            deleteUserAndPolicyAndQueues(dataBroker);
        }
    } catch (err) {
        next(err);
    }
});

router.get(`${API_PATH_DATA_BROKERS}/:id/list-cifs-shares`, (req, res, next) => {
    const {query: {host, version}, id: requestId, headers: {username, password, domain}} = req;
    assertIpOrHostname(host, 'cifs');
    issueDataBrokerCommandAndRespond(req, res, next, dataBroker => sendSharesCommand(requestId, dataBroker, host, version, username, password, domain));
});

router.get(`${API_PATH_DATA_BROKERS}/:id/list-cifs-share-folders`, (req, res, next) => {
    const { query: {host, share, path: cifsPath, version}, headers: {username, password, domain}, id: requestId
    } = req;
    assertIpOrHostname(host, 'cifs');
    issueDataBrokerCommandAndRespond(req, res, next, dataBroker => sendShareFoldersCommand(requestId, dataBroker, host, share, cifsPath, version, username, password, domain));
});

router.get(`${API_PATH_DATA_BROKERS}/:id/list-nfs-exports`, async(req, res, next) => {
    const {
        params: {id: dataBrokerId}, account: {id: accountId}, query: {host, isEFS}, id: requestId
    } = req;

    assertIpOrHostname(host, 'nfs');
    if (isEFS) {
        await createMessage(accountId,  requestId, dataBrokerId, {'list-nfs-exports':{
            host,
            status: MESSAGE_STATUS_DONE,
            paths: ['/']
        }});
        res.status(202).json();
    }

    issueDataBrokerCommandAndRespond(req, res, next, dataBroker => sendListExportsCommand(requestId, host, dataBroker));
});

router.get(`${API_PATH_DATA_BROKERS}/:id/list-nfs-export-folders`, (req, res, next) => {
    try {
        const {
            query:
                {exportName, host, path: nfsPath, version}, id: requestId
        } = req;

        assertIpOrHostname(host, 'nfs');
        issueDataBrokerCommandAndRespond(req, res, next, dataBroker => sendListExportsFoldersCommand(requestId, host, exportName, nfsPath, version, dataBroker));
    } catch (err) {
        next(err);
    }
});

router.get(`${API_PATH_DATA_BROKERS}/:id/list-s3-buckets`, (req, res, next) => {
    const {id: requestId} = req;

    issueDataBrokerCommandAndRespond(req, res, next, dataBroker => sendListS3BucketsCommand(requestId, dataBroker, S3_PROVIDER));
});

router.get(`${API_PATH_DATA_BROKERS}/:id/list-s3-bucket-folders`, (req, res, next) => {
    const {query: {bucket, prefix}, id: requestId} = req;

    issueDataBrokerCommandAndRespond(req, res, next, dataBroker => sendS3ListBucketsFolders(requestId, bucket, prefix, dataBroker, S3_PROVIDER));
});

router.get(`${API_PATH_DATA_BROKERS}/:id/list-sgws-buckets`, (req, res, next) => {
    const {query: {host, port}, id: requestId, headers: {'access-key': accessKey, 'secret-key': secretKey}} = req;

    assertIpOrHostname(host, SGWS_PROVIDER);
    issueDataBrokerCommandAndRespond(req, res, next, dataBroker => sendListS3BucketsCommand(requestId, dataBroker, SGWS_PROVIDER, {
        host,
        port,
        accessKey,
        secretKey
    }));
});

router.get(`${API_PATH_DATA_BROKERS}/:id/list-sgws-bucket-folders`, (req, res, next) => {
    const {
        query: {host, port, bucket, prefix}, id: requestId, headers: {'access-key': accessKey, 'secret-key': secretKey}
    } = req;

    assertIpOrHostname(host, SGWS_PROVIDER);
    issueDataBrokerCommandAndRespond(req, res, next, dataBroker =>
        sendS3ListBucketsFolders(requestId, bucket, prefix, dataBroker, SGWS_PROVIDER, {
            host,
            port,
            accessKey,
            secretKey
        }));
});

router.get(`${API_PATH_DATA_BROKERS}/:id/list-aws-kms-keys`, async(req, res, next) => {
    const {
        query: { bucket}, id: requestId } = req;
    issueDataBrokerCommandAndRespond(req, res, next, dataBroker =>
        sendListKmsKeys(requestId, dataBroker, bucket));
});


router.get(`${API_PATH_DATA_BROKERS}/:id/list-ontap-s3-buckets`, (req, res, next) => {
    const {query: {host }, id: requestId, headers: {'access-key': accessKey, 'secret-key': secretKey}} = req;
    assertIpOrHostname(host, ONTAP_S3_PROVIDER);
    issueDataBrokerCommandAndRespond(req, res, next, dataBroker => sendListS3BucketsCommand(requestId, dataBroker, ONTAP_S3_PROVIDER, {
        host,
        accessKey,
        secretKey
    }));
});

router.get(`${API_PATH_DATA_BROKERS}/:id/list-ontap-s3-bucket-folders`, (req, res, next) => {
    const {
        query: {host, bucket, prefix}, id: requestId, headers: {'access-key': accessKey, 'secret-key': secretKey}
    } = req;

    assertIpOrHostname(host, ONTAP_S3_PROVIDER);
    issueDataBrokerCommandAndRespond(req, res, next, dataBroker =>
        sendS3ListBucketsFolders(requestId, bucket, prefix, dataBroker, ONTAP_S3_PROVIDER, {
            host,
            accessKey,
            secretKey
        }));
});


router.get(`${API_PATH_DATA_BROKERS}/:id/list-ibm-buckets`, (req, res, next) => {
    const {id: requestId, query: {host, port}, headers: {'access-key': accessKey, 'secret-key': secretKey}} = req;

    assertIpOrHostname(host, 'ibm');
    issueDataBrokerCommandAndRespond(req, res, next, dataBroker => sendListS3BucketsCommand(requestId, dataBroker, IBM_PROVIDER,
        {host, port, accessKey, secretKey}));
});

router.get(`${API_PATH_DATA_BROKERS}/:id/list-ibm-bucket-folders`, (req, res, next) => {
    const {
        id: requestId, query: {
            host, port, bucket, prefix
        }, headers: {'access-key': accessKey, 'secret-key': secretKey}
    } = req;

    assertIpOrHostname(host, 'ibm');
    issueDataBrokerCommandAndRespond(req, res, next, dataBroker =>
        sendS3ListBucketsFolders(requestId, bucket, prefix, dataBroker, IBM_PROVIDER, {
            host,
            port,
            accessKey,
            secretKey
        }));
});

router.get(`${API_PATH_DATA_BROKERS}/:id/list-alibaba-buckets`, (req, res, next) => {
    issueDataBrokerCommandAndRespond(req, res, next, dataBroker => sendListS3BucketsCommand(req.id, dataBroker, ALIBABA_PROVIDER,
        {accessKey: req.headers['access-key'], secretKey: req.headers['secret-key']}));
});

router.get(`${API_PATH_DATA_BROKERS}/:id/list-alibaba-bucket-folders`, (req, res, next) => {
    const {
        id: requestId,
        query: {bucket, prefix}, headers: {'access-key': accessKey, 'secret-key': secretKey}
    } = req;

    issueDataBrokerCommandAndRespond(req, res, next, dataBroker =>
        sendS3ListBucketsFolders(requestId, bucket, prefix, dataBroker, ALIBABA_PROVIDER, {
            accessKey,
            secretKey
        }));
});

router.get(`${API_PATH_DATA_BROKERS}/:id/list-azure-blob-storage-containers`, (req, res, next) => {
    const {id: requestId, query: {'storage-account-name': storageAccountName}, headers: {'storage-account-connection-string': storageAccountConnectionString}} = req;

    issueDataBrokerCommandAndRespond(req, res, next, dataBroker => sendListAzureBlobStorageContainers(requestId, dataBroker, storageAccountName, storageAccountConnectionString));
});

router.get(`${API_PATH_DATA_BROKERS}/:id/list-azure-blob-storage-container-folders`, (req, res, next) => {
    const {id: requestId, query: {container, prefix, 'storage-account-name': storageAccountName}, headers: {'storage-account-connection-string': storageAccountConnectionString}} = req;

    issueDataBrokerCommandAndRespond(req, res, next, dataBroker => sendListAzureBlobStorageContainersFoldersCommand(requestId, dataBroker, container, prefix, storageAccountName, storageAccountConnectionString));
});

router.get(`${API_PATH_DATA_BROKERS}/:id/list-gcp-buckets`, (req, res, next) => {
    const {id: requestId} = req;

    issueDataBrokerCommandAndRespond(req, res, next, dataBroker => sendListGcpBucketsCommand(requestId, dataBroker));
});

router.get(`${API_PATH_DATA_BROKERS}/:id/list-gcp-bucket-folders`, (req, res, next) => {
    const {id: requestId, query: {bucket, prefix}} = req;

    issueDataBrokerCommandAndRespond(req, res, next, dataBroker => sendListGcpBucketFoldersCommand(requestId, dataBroker, bucket, prefix));
});

agenda.define(AGENDA_TASK_NAME_DELETE_NOT_ACTIVE_DATA_BROKERS, {lockLifetime: 10000}, async(job, done) => {
    inContext('Delete old data brokers without placement task', async() => {
        try {
            const date45DaysAgo = moment().subtract(config.get('number-of-days-before-data-broker-deletion'), 'days').toDate();
            const dataBrokers = await DataBroker.find({
                lastPing: {$exists: false},
                createdAt: {$lt: date45DaysAgo}
            }).sort({'createdAt': -1}).exec();
            logger.info(`Received dataBrokers ${dataBrokers.length} with no placement that were created before ${config.get('number-of-days-before-data-broker-deletion')}`,
                JSON.stringify(dataBrokers, null, 4));
            await Promise.map(dataBrokers, async dataBroker => {
                const {_id: dataBrokerId, groupId} = dataBroker;
                try {
                    const relationships = await Relationship.find({
                        $or: [{dataBroker: dataBrokerId}, {'source.nfs.dataBrokerId': dataBrokerId},
                            {'target.nfs.dataBrokerId': dataBrokerId}]
                    }).exec();

                    if (relationships.length) {
                        logger.error(`data broker with id ${dataBrokerId} has ${relationships.length} relationships`);
                    } else {
                        logger.info(`deleting databroker with id ${dataBrokerId} `);
                        await deleteUserAndPolicyAndQueues(dataBroker);
                        const group = await Group.findById(groupId).exec();
                        await removeDataBrokerFromGroupIfNeeded(dataBroker, group);
                        await DataBroker.deleteOne({_id: dataBrokerId}).exec();
                    }
                } catch (err) {
                    logger.error('>>>>>>>Error failed to delete dataBroker with id', dataBrokerId, err);
                }
            }, {concurrency: 10});
        } catch (err) {
            logger.error(`Failed to execute job ${AGENDA_TASK_NAME_DELETE_NOT_ACTIVE_DATA_BROKERS}`, err);
        } finally {
            done();
        }
    });
});


module.exports = router;
