const moment = require('moment');
const express = require('express');
const {
    get, isArray, mergeWith, pickBy, extend
} = require('lodash');
const {agenda} = require('../utils/agenda');
const {RelationshipStats} = require('../models/relationship-statistics');
const {createMessage} = require('../models/message-v2');
const {
    AGENDA_TASK_NAME_SYNC_SCHEDULER, API_PATH_RELATIONSHIP, API_PATH_RELATIONSHIP_V2, MONGO_DB_ERROR_DUPLICATE_KEY, NETAPP_HEADERS_PREFIX
} = require('../utils/consts');
const {BadRequest, NotFound} = require('../utils/errors');
const {
    updateTimeLineNewDataBroker,
    findTimelineRecordV2
} = require('../operations/timeline-operations');
const getLogger = require('../utils/logger');
const DataBroker = require('../models/data-broker');
const Group = require('../models/group');
const Relationship = require('../models/relationship');
const {generateRequestId} = require('../utils/utils');
const {inContext} = require('../middlewares/continuation-local-storage');
const {setHubSpotContactFirstRelationshipSinceIfNeeded} = require('../lib/hubspot');
const {isThereAliveDataBroker} = require('../operations/group-operations');
const {isFreeOfCharge} = require('../operations/account-operations');
const {
    createActivity, createPendingRelationship, verifyOnlyOne, createRelationship, createTimeLineObjectForRelationship, startNewDataBrokerSync, getNextSyncTime,
    validateRelationship, validateAbortRelationship,
    validateSettings, handleEndOfRelationship, updateRelationshipFields,
    abortNewDataBroker, getRelationshipWithActivity
} = require('../operations/relationship-operations');

const logger = getLogger();

const router = express.Router();


router.get(API_PATH_RELATIONSHIP_V2, async(req, res, next) => {
    const {account} = req;
    try {
        const relationships = await Relationship.find({account}).sort({createdAt: 1}).populate('group').populate({
            path: 'group',
            populate: {path: 'dataBrokers'}
        }).exec();
        const relationshipsResponse = await Promise.all(relationships.map(relationship => createActivity(relationship)));
        res.json(relationshipsResponse);
    } catch (err) {
        next(err);
    }
});

router.get(`${API_PATH_RELATIONSHIP_V2}/:id`, async(req, res, next) => {
    const {params: {id: relationshipId}} = req;

    try {
        const relationship = await Relationship.findById(relationshipId).populate('group').populate({
            path: 'group',
            populate: {path: 'dataBrokers'}
        }).exec();
        if (!relationship) {
            logger.warn(`Relationship with id ${relationshipId} not found `);
            return next(new NotFound(`Relationship with id ${relationshipId} not found`));
        }
        const relationshipResponse = await createActivity(relationship);
        res.json(relationshipResponse);
    } catch (err) {
        next(err);
    }
});

router.post(API_PATH_RELATIONSHIP_V2, async(req, res, next) => {
    let relationship;
    let {body: {settings}} = req;
    const {
        account, id, body: {
            dataBrokerId, source, sourceCredentials, target, encryption,
            targetCredentials, groupId
        }
    } = req;
    const {id:accountId, tenancyAccountId} = account;
    const isCm = req.get('x-cm-integration');
    settings = settings || {};

    if (!settings.schedule) {
        settings.schedule = {
            syncInMinutes: 0, syncInHours: 0, syncInDays: 1,
            nextTime: moment().add(0, 'minutes').add(0, 'hours').add(1, 'days')
        };
    }
    if (!settings.schedule.nextTime) {
        settings.schedule.nextTime = moment().add(settings.schedule.syncInMinutes || 0,
            'minutes').add(settings.schedule.syncInHours || 0, 'hours')
            .add(settings.schedule.syncInDays || 1, 'days');
    }
    updateRelationshipFields(source);
    updateRelationshipFields(target);
    let dataBroker;
    let group;
    try {
        // update isQstack field in case the source or target includes cvs or anf
        const sourceType = get(source, 'nfs.provider') || get(source, 'cifs.provider');
        const targetType = get(target, 'nfs.provider') || get(target, 'cifs.provider');
        if (!dataBrokerId && !groupId) {
            return next(new BadRequest('Use either dataBrokerId or groupId but not both'));
        }
        if (dataBrokerId && groupId) {
            return next(new BadRequest('Use either dataBrokerId or groupId but not both'));
        }
        if (dataBrokerId) {
            dataBroker = await DataBroker.findById(dataBrokerId).exec();
            if (!dataBroker) {
                logger.warn(`Data broker with id ${dataBrokerId} not found`);
                return next(new NotFound(`Data broker with id ${dataBrokerId} not found`));
            }
            // if the dataBroker is phoenix
            if (dataBroker.groupId) {
                group = await Group.findById(dataBroker.groupId).populate('dataBrokers');
            }
        } else if (groupId) {
            group = await Group.findById(groupId).populate('dataBrokers');
            if (!group) {
                logger.warn(`Group with id ${groupId} not found`);
                return next(new NotFound(`Group with id ${groupId} not found`));
            }
            const {dataBrokers} = group;
            dataBroker = await dataBrokers.find(dataBroker => dataBroker.isAlive());
        }
        // check if alive
        isThereAliveDataBroker(group);
        relationship = new Relationship({
            account :accountId,
            isQstack: ['cvs', 'anf'].includes(sourceType) || ['cvs', 'anf'].includes(targetType),
            isCm,
            source,
            target,
            settings,
            encryption,
            dataBroker:dataBroker.id,
            group
        });
        createTimeLineObjectForRelationship(req, relationship);
        validateRelationship(relationship);
        await verifyOnlyOne(relationship);
        const customHeaders = pickBy(req.headers, (value, key) => key.startsWith(NETAPP_HEADERS_PREFIX));
        // start copy according to the dataBroker type
        if (relationship.settings.schedule.syncWhenCreated) {
            relationship = await createRelationship(relationship, id, false, sourceCredentials, targetCredentials, customHeaders);
        } else {
            relationship = await createPendingRelationship(relationship, id, sourceCredentials, targetCredentials); //send only credentials
        }
        const relationshipResponse = await getRelationshipWithActivity(relationship);
        const currentRelationshipsFreeCharge = await isFreeOfCharge(account);
        await createMessage(accountId, undefined, undefined, {'update-account-info':{
            accountId: tenancyAccountId,
            noCharge: currentRelationshipsFreeCharge && relationship.isQstack
        }});
        res.status(202).json(relationshipResponse);
        setHubSpotContactFirstRelationshipSinceIfNeeded(req.userPortalId);
    } catch (err) {
        if (err.code === MONGO_DB_ERROR_DUPLICATE_KEY || err.errorCode === MONGO_DB_ERROR_DUPLICATE_KEY) {
            logger.error(`Relationship already exists source ${JSON.stringify(source)} target ${JSON.stringify(target)}`, err);

            next(new BadRequest('Sync relationship already exists'));
        } else {
            if (relationship) {
                logger.error('Failed creating relationship with error: ', err);
                await Relationship.findOneAndRemove({_id: relationship._id}).exec();
                await RelationshipStats.deleteMany({relationship: relationship._id}).exec();
            }
            next(err);
        }
    }
});

router.delete(`${API_PATH_RELATIONSHIP}/:id`, async(req, res, next) => {
    const {params: {id: relationshipId}, id: requestId, account } = req;
    const {tenancyAccountId, id:accountId} = account;

    try {
        const relationship = await Relationship.findById(relationshipId).populate('dataBroker').populate({
            path: 'group',
            populate: {path: 'dataBrokers'}
        }).exec();
        createTimeLineObjectForRelationship(req, relationship);
        if (!relationship) {
            logger.warn(`Relationship with id ${relationshipId} was not found for deletion`);
            next(new NotFound());
        } else {
            handleEndOfRelationship(relationship, requestId, 'remove');
            const relationshipResponse = await createActivity(relationship);
            await RelationshipStats.deleteMany({relationship: relationshipId}).exec();
            await updateTimeLineNewDataBroker(relationshipResponse, 'DELETE', findTimelineRecordV2, 'relationship.activity');
            await relationship.deleteOne();
            await createMessage(accountId, undefined, undefined, {'update-account-info':{
                accountId: tenancyAccountId,
                noCharge: await isFreeOfCharge(account),
            }});
        }
        res.json();
    } catch (err) {
        logger.warn(`Failed to send delete relationship command to data broker, relationship id ${relationshipId}`, err);
        next(err);
    }
});

router.put(`${API_PATH_RELATIONSHIP_V2}/:id`, async(req, res, next) => {
    const {params: {id: relationshipId}, body} = req;
    try {
        logger.debug('Updating relationshipId: ', relationshipId);

        const relationship = await Relationship.findById(relationshipId).populate({
            path: 'group',
            populate: {path: 'dataBrokers'}
        }).exec();

        if (relationship) {
            createTimeLineObjectForRelationship(req, relationship);

            // we convert the settings from the data base to a javascript regular object, using toObject().
            // we do that because if, after the mergeWith function, we stay with the settings from the data base,
            // then the joi validation won't work.
            // therefore we convert it, merge, validate, and the assign it back to relationship.settings.
            const settingsObj = relationship.settings.toObject();
            const {schedule: scheduleRequest} = body;

            if (scheduleRequest) {
                const {syncInMinutes, syncInHours, syncInDays, nextTime} = scheduleRequest;
                const nextSync = moment().add(syncInMinutes || settingsObj.schedule.syncInMinutes, 'minutes')
                    .add(syncInDays || settingsObj.schedule.syncInDays, 'days')
                    .add(syncInHours || settingsObj.schedule.syncInHours, 'hours')
                    .toISOString();
                scheduleRequest.nextTime = nextTime || nextSync;
            }

            mergeWith(settingsObj, body, (objValue, srcValue) => {
                if (isArray(objValue)) {
                    objValue = srcValue;
                    return objValue;
                }
            });

            validateSettings(settingsObj);
            relationship.settings = settingsObj;
            await relationship.save();

            const response = Object.assign({id: relationshipId}, settingsObj);
            //for audit service
            req.timelineV2 = extend(req.timelineV2, {response});

            return res.status(200).json(response);
        }

        return next(new BadRequest(`relationship with id ${relationshipId} was not found`));
    } catch (err) {
        logger.error(`Error while trying to change settings  of relationship ${relationshipId}`, err);
        return next(err);
    }
});

router.put(`${API_PATH_RELATIONSHIP}/:id/sync`, async(req, res, next) => {
    const {params: {id: relationshipId}, id: requestId} = req;
    try {
        const relationship = await Relationship.findById(relationshipId).populate('account').populate({
            path: 'group',
            populate: {path: 'dataBrokers'}
        }).exec();
        createTimeLineObjectForRelationship(req, relationship);

        if (!relationship) {
            logger.warn(`Relationship with id ${relationshipId} was not found for syncing`);
            return next(new NotFound());
        }

        const {group} = relationship;
        isThereAliveDataBroker(group);
        await startNewDataBrokerSync(relationship, requestId, false, true);

        res.status(202).json();
    } catch (err) {
        next(err);
    }
});

router.put(`${API_PATH_RELATIONSHIP}/:id/abort`, async(req, res, next) => {
    const {params: {id: relationshipId}, id: requestId} = req;

    try {
        logger.debug(`Got abort request for relationship with id ${relationshipId}`);

        const relationship = await Relationship.findById(relationshipId)
            .populate({path: 'group', populate: {path: 'dataBrokers'}}).exec();

        createTimeLineObjectForRelationship(req, relationship);

        if (!relationship) {
            logger.debug(`Relationship with id ${relationshipId} was not found for aborting`);
            return next(new NotFound());
        }

        const {group} = relationship;
        isThereAliveDataBroker(group);
        await validateAbortRelationship(relationshipId);
        abortNewDataBroker(relationship, requestId);

        res.status(202).json();
    } catch (err) {
        next(err);
    }
});

agenda.define(AGENDA_TASK_NAME_SYNC_SCHEDULER, {lockLifetime: 60000}, (job, done) => {
    inContext('Relationships scheduled sync', async() => {
        try {
            logger.info('Relationships scheduled sync task started');
            const relationships = await Relationship.find({
                'settings.schedule.isEnabled': true, 'settings.schedule.nextTime': {$lte: new Date()}
            }).populate('account')
                .populate({path: 'group', populate: {path: 'dataBrokers'}}).exec();
            const promises = relationships.map(async relationship => {
                const {
                    account: {tenancyAccountId}, id: relationshipId, dataBroker, group
                } = relationship;
                logger.debug(`Running scheduled sync for relationship ${relationshipId} of account ${tenancyAccountId} `);
                const rId = generateRequestId();
                try {
                    isThereAliveDataBroker(group, dataBroker);
                    await startNewDataBrokerSync(relationship, rId, true);
                } catch (err) {
                    logger.warn(`Failed syncing relationship ${relationship.id} of account ${tenancyAccountId}:`, err.message);
                } finally {
                    relationship.settings.schedule.nextTime = getNextSyncTime(relationship.settings.schedule.nextTime,
                        relationship.settings.schedule.syncInDays, relationship.settings.schedule.syncInHours, relationship.settings.schedule.syncInMinutes);
                    await relationship.save();
                }
            });
            await Promise.all(promises);
        } catch (err) {
            logger.error('Relationships scheduled sync task failed:', err);
        } finally {
            done();
        }
    });
});


module.exports = router;
