const Joi = require('@hapi/joi');
const moment = require('moment');

function checkIfDateIsValid(joi) {
    return {
        type:'date',
        base: joi.date(),
        messages: {
            datetime: 'Date is not valid - {{date}}'
        },
        rules: {
            datetime: {
                validate(value, state, prefs) {
                    if (moment(value).isValid()) {
                        return value;
                    } else {
                        return this.createError('date.datetime', {value, date: state.parent[state.key]}, state, prefs);
                    }
                }
            }
        }
    };
}

const customJoiDateValidation = Joi.extend(checkIfDateIsValid);

const typesSchema = Joi.object().keys({
    files: Joi.boolean().required(),
    directories: Joi.boolean().required(),
    symlinks: Joi.boolean().required()
});

const filesSchema = Joi.object().keys({
    excludeExtensions: Joi.array().items(Joi.string().regex(/^\./, {name: 'extensions'})).unique().max(15).required(),
    minSize: Joi.number().integer().min(0).max(Number.MAX_SAFE_INTEGER).required(),
    maxSize: Joi.number().integer().min(0).max(Number.MAX_SAFE_INTEGER).required(),
    minDate: customJoiDateValidation.date().iso().min('1970-01-01').datetime().required(),
    maxDate: customJoiDateValidation.date().iso().min('1970-01-01').datetime().allow(null)
});

const schedulerSchema = Joi.object().keys({
    syncInDays: Joi.number().integer().min(0).max(365).required(),
    syncInHours:Joi.number().integer().min(0).max(24).required(),
    syncInMinutes:Joi.number().integer().min(0).max(60).required(),
    nextTime: customJoiDateValidation.date().min('1970-01-01').iso().datetime().allow(null),
    isEnabled:Joi.boolean().required(),
    syncWhenCreated:Joi.boolean()
});

const settingsSchema = Joi.object().keys({
    gracePeriod: Joi.number().min(0).required(),
    deleteOnSource: Joi.boolean().required(),
    deleteOnTarget: Joi.boolean().required().when('deleteOnSource', { // We don't want both of them to be true, o.w all the data wil be deleted on second sync
        is: true,
        then: Joi.valid(false)
    }),
    objectTagging: Joi.boolean().required(),
    retries: Joi.number().integer().min(0).max(5).required(),
    copyAcl: Joi.boolean().required(),
    files: filesSchema.required(),
    fileTypes: typesSchema.required(),
    schedule: schedulerSchema.required()
});

module.exports = {settingsSchema, schedulerSchema};