const { exec } = require('child_process');
const os = require('os');
const fs = require('fs');
const zlib = require('zlib');
const path = require('path');
const hyperid = require('hyperid');
const moment = require('moment');
const config = require('config');
const format = require('string-template');
const {
    last, split, remove, head, drop
} = require('lodash');
const DataBroker = require('../models/data-broker');
const Relationship = require('../models/relationship');
const { MP_PRODUCT_CODE_METERING, SCAN_MODE_SOURCE_AND_TARGET, ONTAP_S3_PROVIDER, S3_PROVIDER, ALIBABA_PROVIDER, IBM_PROVIDER, SGWS_PROVIDER } = require('../utils/consts');
const {
    GCP_INSTANCE_ID, GCP_ZONE, GCP_NETWORK, GCP_SERVICE_ACCOUNT,
    REGION, SUBNET_CIDR_BLOCK, SUBNET_ID, VPC_CIDR_BLOCK, VPC_ID, AVAILABILITY_ZONE,
    PRIVATE_IP, INSTANCE_ID, AWS_ACCOUNT_ID_2, AWS_ACCOUNT_ID_1, RESOURCE_GROUP_NAME, AZURE_INSTANCE_ID, LOCATION
} = require('./consts');
const {getStatusesQueueName} = require('../operations/data-broker-operations');
const getLogger = require('../utils/logger');

const logger = getLogger();

const DELAY_TIME_MS = 3000;
const RECEIVE_TIMEOUT_MS = 10000;
const messagesQueue = [];
const FAILURE_ADDRESS_CREATE_REGEX = /fail\.com/;
const FAILURE_ADDRESS_SYNC_REGEX = /failsync\.com/;
const SUCCESS_ADDRESS_CREATE_REGEX = /success\.com/;
const relationshipsToAbort = [];
const runningRelationshipsNewDataBroker = {};
const GCP = 'gcp';
const AZURE = 'azure';
const VERSION = '1.2.3-mock';


async function execute(command, timeout, env) {
    return new Promise((resolve, reject) => {
        exec(command, { timeout, env }, (error, stdout, stderr) => {
            logger.info('Executing command', { command, timeout });
            if (error) {
                logger.error('Failed to execute command', error);
                reject(stderr || stdout || `Command execution: ${command} - timed out`);
            } else {
                resolve(stdout);
            }
        });
    });
}

async function unzip(zippedString) {
    return new Promise((resolve, reject) => {
        const buffer = Buffer.from(zippedString, 'base64');
        zlib.unzip(buffer, (err, bufferUnzipped) => {
            if (err) {
                reject(err);
            } else {
                resolve(bufferUnzipped.toString());
            }
        });
    });
}

function createRegionObject(s3Region, gcpRegion, alibabaRegion) {
    return {
        [S3_PROVIDER]:s3Region,
        [GCP]: gcpRegion,
        [ALIBABA_PROVIDER]:alibabaRegion
    };
}

const BUCKETS_REGIONS_BY_PROTOCOL = {
    small: createRegionObject('us-west-2', 'us-west2', 'oss-us-west-1'),
    medium:createRegionObject('us-west-1', 'us-west1', 'oss-us-west-1'),
    large:createRegionObject('eu-central-1', 'EU', 'oss-eu-central-1'),
    tiny:createRegionObject('us-east-1', 'us-east1', 'oss-me-east-1'),
    joey:createRegionObject('ap-northeast-1', 'asia-southeast1', 'oss-ap-northeast-1'),
    phoebe:createRegionObject('eu-central-1', 'EU', 'oss-eu-central-1'),
    chandler:createRegionObject('ap-south-1', 'asia-southeast1', 'oss-me-east-1'),
    monica:createRegionObject('ap-south-1', 'asia-southeast1', 'oss-ap-southeast-1'),
    ross:createRegionObject('ca-central-1', 'northamerica-northeast1', 'oss-me-east-1'),
    rachel:createRegionObject('eu-central-1', 'EU', 'oss-eu-central-1'),
    'fabric-pool-a1':createRegionObject('eu-central-1', 'EU', 'oss-eu-central-1')

};

function createSQSMessage(payload, messageAttributes, attributes) {
    delete messageAttributes['zipped'];
    return {
        ReceiptHandle: hyperid().uuid,
        Body: JSON.stringify(payload),
        MessageAttributes: messageAttributes,
        Attributes:attributes
    };
}

function sendHealthMessage(dataBroker) {
    const { id: dataBrokerId, userId } = dataBroker;
    const payload = {
        transferrers: 4,
        scanners: 4,
        memoryUsageRss: 77336576,
        memoryUsageHeapTotal: 29220864,
        memoryUsageHeapUsed: 25610728,
        transferRate: 1223213.5435
    };
    const attributes = { type: { StringValue: 'health', DataType: 'String' }, dataBrokerId: { StringValue: dataBrokerId, DataType: 'String' } };
    messagesQueue.push(createSQSMessage(payload, attributes, {SenderId:userId}));
}

function createFolders(path) {
    let folders;

    const lastDir = last(split(path, '/'));
    switch (lastDir) {
        case 'folder4':
        case 'folder0':
            folders = [];
            break;
        case 'folder1':
            folders = ['folder0', 'folder1', 'folder3'];
            break;
        case 'folder3':
            folders = ['folder0', 'folder1'];
            break;
        case '':
            folders = ['folder1', 'folder2'];
            break;
        default:
            folders = ['folder0', 'folder1', 'folder2', 'folder3', 'folder4', 'folder5'];
    }
    return folders;
}

function createBucketsFoldersList(bucket, prefix) {
    let folders;
    switch (bucket) {
        case 'small': {
            if (!prefix) {
                folders = ['/small1'];
            } else if (['/small1', 'small1'].includes(prefix)) {
                folders = ['small2', 'small3'];
            } else {
                folders = [];
            }
        }
            break;
        case 'medium': {
            if (!prefix) {
                folders = ['medium1'];
            } else if (['/medium1', 'medium1'].includes(prefix)) {
                folders = ['medium2', 'medium3'];
            } else {
                folders = [];
            }
        }
            break;
        case 'tiny': {
            if (!prefix) {
                folders = ['tiny1'];
            } else if (['/tiny1', 'tiny1'].includes(prefix)) {
                folders = ['tiny2', 'tiny3'];
            } else {
                folders = [];
            }
        }
            break;
        case 'joey':
        case 'phoebe':
        case 'chandler':
        case 'monica':
        case 'ross':
        case 'rachel':
            {
                if (!prefix) {
                    folders = ['centralPark1'];
                } else if (['/centralPark1', 'centralPark1'].includes(prefix)) {
                    folders = ['centralPark2', 'centralPark3', 'centralParkIsFullOfSpecialCharacters!@#$%^&*(){}\\[];:~.",\'?|'];
                } else {
                    folders = [];
                }
            }
            break;
        default:
            folders = [];
    }
    return folders;
}

function buildListNfsExportFoldersResponse(host, exportName, path, messageAttributes, attributes,  isNfsExportsFoldersEmpty) {
    const folders = (isNfsExportsFoldersEmpty) ? [] : createFolders(path);
    return createSQSMessage({
        host,
        export: exportName,
        path,
        folders,
        status: 'SUCCEEDED'
    }, messageAttributes, attributes);
}

function buildListNfsExportsResponse(host, messageAttributes, attributes, isNfsExportsEmpty) {
    const payload = {
        host,
        exports: (isNfsExportsEmpty) ? [] : ['/data1', '/data2', '/logs', '/', '/complex/junction/path'],
        status: 'SUCCEEDED'
    };
    return createSQSMessage(payload, messageAttributes, attributes);
}


function createBucketObject(name, provider, tags) {
    return  {
        name: name,
        region: BUCKETS_REGIONS_BY_PROTOCOL[name][provider],
        tags
    };
}

function createBucketsList(provider, withTags) {
    return [createBucketObject('small', provider,
        withTags ? [{ Key: 'price', Value: '100$' }] : undefined),
    createBucketObject('medium', provider, withTags ?
        [{ Key: 'one', Value: '1' }, { Key: 'two', Value: '2' }, { Key: 'three', Value: '3' }] : undefined),
    createBucketObject('large', provider, withTags ? [] : undefined),
    createBucketObject('tiny', provider, withTags ? [] : undefined),
    createBucketObject('joey', provider, withTags ? [] : undefined),
    createBucketObject('phoebe', provider, withTags ? [] : undefined),
    createBucketObject('chandler', provider, withTags ? [] : undefined),
    createBucketObject('monica', provider, withTags ? [] : undefined),
    createBucketObject('rachel', provider, withTags ? [] : undefined),
    createBucketObject('fabric-pool-a1', provider, withTags ? [] : undefined)
    ];
}

function buildListS3BucketsResponse(provider, host, port, messageAttributes, attributes, isEmpty) {
    const withTags = provider === S3_PROVIDER;
    return createSQSMessage({
        buckets: (isEmpty) ? [] : (([SGWS_PROVIDER, IBM_PROVIDER, ONTAP_S3_PROVIDER].includes(provider)) ?
            ([{
                name: 'small'
            }, {
                name: 'medium'
            }, {
                name: 'large'
            }])
            : createBucketsList(provider, withTags)),
        status: 'SUCCEEDED',
        provider,
        host, port
    }, messageAttributes, attributes);
}

function buildListS3BucketFoldersResponse(messageAttributes, attributes, bucket, prefix, provider, options, isBucketFoldersEmpty) {
    const {host, port} = options;
    const withTags = provider === S3_PROVIDER;
    return createSQSMessage({
        provider,
        bucket: (isBucketFoldersEmpty) ? undefined : [S3_PROVIDER, ALIBABA_PROVIDER].includes(provider) ?
            createBucketsList(provider, withTags).find(bucketObj => bucketObj.name === bucket) : {name: bucket},
        prefix,
        folders: (isBucketFoldersEmpty) ? [] : createBucketsFoldersList(bucket, prefix),
        status: 'SUCCEEDED',
        port,
        host
    }, messageAttributes, attributes);
}

function buildListAzureContainerFoldersResponse(messageAttributes, attributes, container, prefix, storageAccountName, isEmptyDataBroker) {
    return createSQSMessage({
        container,
        prefix,
        folders: (isEmptyDataBroker) ? [] : createBucketsFoldersList(container, prefix),
        status: 'SUCCEEDED',
        storageAccountName
    }, messageAttributes, attributes);
}

function buildListAzureContainersResponse(messageAttributes, attributes, storageAccountName, isEmptyDataBroker) {
    return createSQSMessage({
        containers: (isEmptyDataBroker) ? [] : createBucketsList(AZURE, true),
        status: 'SUCCEEDED',
        storageAccountName
    }, messageAttributes, attributes);
}

function buildListGcpBucketFoldersResponse(messageAttributes, attributes, bucket, prefix, projectId, isEmptyDataBroker) {
    return createSQSMessage({
        bucket: (isEmptyDataBroker) ? undefined : createBucketsList(GCP, true).find(bucketObj => bucketObj.name === bucket),
        prefix,
        folders: (isEmptyDataBroker) ? [] : createBucketsFoldersList(bucket, prefix),
        status: 'SUCCEEDED',
        projectId
    }, messageAttributes, attributes);
}

function buildListKmsKeysResponse(messageAttributes, attributes, bucket, isEmpty) {
    return createSQSMessage({
        bucket,
        keys: (isEmpty) ? [] : [ {
            name: 'alias/aws/rds',
            arn: 'arn:aws:kms:us-west-2:464262061435:alias/aws/rds',
            isEnabled:true
        },
        {
            name: 'alias/aws/redshift',
            arn: 'arn:aws:kms:us-west-2:464262061435:alias/aws/redshift',
            isEnabled:true
        },
        {
            name: 'alias/aws/s3',
            arn: 'arn:aws:kms:us-west-2:464262061435:alias/aws/s3',
            isEnabled:false
        }],
        status: 'SUCCEEDED',
    }, messageAttributes, attributes);
}

function buildListGcpBucketsResponse(messageAttributes, attributes, projectId, isEmptyDataBroker) {
    return createSQSMessage({
        buckets: (isEmptyDataBroker) ? [] : createBucketsList(GCP, true),
        status: 'SUCCEEDED',
        projectId
    }, messageAttributes, attributes);
}

function buildListCifsSharesResponseNewDataBroker(host, isCifsSharesEmpty, messageAttributes, attributes) {
    return createSQSMessage({
        status: 'SUCCEEDED',
        shares: (isCifsSharesEmpty) ? [] : ['/data1', '/data2', '/logs'],
        host
    }, messageAttributes, attributes);
}

function buildListCifsShareFoldersResponse(messageAttributes, attributes, host, share, path = '') {
    let folders;

    switch (path.split('/').pop()) {
        case 'folder4':
        case 'folder0':
            folders = [];
            break;
        case 'folder1':
            folders = ['/folder0', '/folder1', '/folder3'];
            break;
        case 'folder3':
            folders = ['/folder0', '/folder1'];
            break;
        case '':
            folders = ['/folder1', '/folder2'];
            break;
        default:
            folders = ['/folder0', '/folder1', '/folder2', '/folder3', '/folder4', '/folder5'];
    }

    return createSQSMessage({
        host,
        folders,
        path,
        share,
        status: 'SUCCEEDED'
    }, messageAttributes, attributes);
}

function createPlacement(dataBroker) {
    const { name, type: dataBrokerType } = dataBroker;
    const placement = {
        version: VERSION,
        platform: `${os.type()} ${os.release()}`,
        hostname: os.hostname(),
        cpus: 4,
        node : '13.9.0',
        osTotalMem : 16343470080,
        processMaxMem : 55373824,
        release : '4.9.76-3.78.amzn1.x86_64',
        os:'linux',
        privateIp: PRIVATE_IP,
        publicIp : '192.168.0.30',
        certificate:'-----BEGIN CERTIFICATE-----\\nMIIDGTCCAgGgAwIBAgIJAPUl95NVYoIzMA0GCSqGSIb3DQEBCwUAMCMxITAfBgNV\\nBAMMGDVkMjc1YzQwY2VkZDI1MDAwYmZmYmE0YzAeFw0xOTA3MTExNjAyMDNaFw0y\\n-----END CERTIFICATE-----\\n'
    };
    switch (dataBrokerType) {
        case 'AWS': {
            const extraParams = {
                awsAccountId:(name.includes('diffAccount')) ? AWS_ACCOUNT_ID_2 : AWS_ACCOUNT_ID_1,
                instanceId:INSTANCE_ID,
                availabilityZone:AVAILABILITY_ZONE,
                vpcId:VPC_ID,
                vpcCidrBlock:VPC_CIDR_BLOCK,
                subnetId:SUBNET_ID,
                subnetCidrBlock:SUBNET_CIDR_BLOCK,
                region:REGION
            };
            return {...extraParams, ...placement};

        }
        case 'ONPREM': {
            const extraParams = {
                awsAccountId:(name.includes('diffAccount')) ? AWS_ACCOUNT_ID_2 : AWS_ACCOUNT_ID_1,
                gcpProjectId:'cloudsync-dev-214020'
            };
            return {...extraParams, ...placement};
        }
        case 'GCP': {
            const extraParams = {
                gcpProjectId:'cloudsync-dev-214020',
                instanceId:GCP_INSTANCE_ID,
                zone:GCP_ZONE,
                network:GCP_NETWORK,
                serviceAccount: GCP_SERVICE_ACCOUNT,
                subnetCidrBlock :SUBNET_CIDR_BLOCK
            };
            return {...extraParams, ...placement};
        }
        case 'AZURE': {
            const extraParams = {
                instanceId:AZURE_INSTANCE_ID,
                subnetCidrBlock:SUBNET_CIDR_BLOCK,
                region:LOCATION,
                resourceGroupName:RESOURCE_GROUP_NAME
            };
            return {...extraParams, ...placement};

        }
        default: {
            throw new Error(`provider ${dataBrokerType} not supported`);
        }
    }
}

function sendPlacementMessage(dataBroker) {
    const placement = createPlacement(dataBroker);
    messagesQueue.push(createSQSMessage(placement,
        {
            type: { StringValue: 'placement', DataType: 'String' },
            dataBrokerId: { StringValue: dataBroker.id, DataType: 'String' }
        }, {SenderId:dataBroker.userId}));
}

AWS.mock('SQS', 'deleteMessage', () => ({ promise: () => ({}) }));
AWS.mock('SQS', 'deleteQueue', () => ({ promise: () => ({}) }));
AWS.mock('SQS', 'setQueueAttributes', () => ({ promise: () => ({}) }));
AWS.mock('SQS', 'getQueueUrl', params => ({
    promise: () => {
        return {QueueUrl : `https://sqs.us-east-1.amazonaws.com/123/${params.QueueName}` };
    }
}));
AWS.mock('SQS', 'getQueueAttributes', params => ({
    promise: () => {
        const { AttributeNames: attributesNames, QueueUrl } = params;
        if (attributesNames.includes('ApproximateNumberOfMessages') || attributesNames.includes('ApproximateNumberOfMessagesNotVisible')) {
            const key = Object.keys(runningRelationshipsNewDataBroker).find(key => key.includes(QueueUrl));
            if (!key) {
                return { Attributes: { QueueArn: 'arn:aws:sqs:us-east-1:123456789012:ARN', ApproximateNumberOfMessages: '0', ApproximateNumberOfMessagesNotVisible: '0' } };
            }
            if (runningRelationshipsNewDataBroker[key].length > 0) {
                return {
                    Attributes: {
                        QueueArn: 'arn:aws:sqs:us-east-1:123456789012:ARN',
                        ApproximateNumberOfMessages: '3',
                        ApproximateNumberOfMessagesNotVisible: '4'
                    }
                };
            }
            return { Attributes: { QueueArn: 'arn:aws:sqs:us-east-1:123456789013:ARN', ApproximateNumberOfMessages: '0', ApproximateNumberOfMessagesNotVisible: '0' } };
        } if (attributesNames.includes('Policy')) {
            return {
                Attributes: {
                    Policy: `{"Statement": [{"Sid": "Sid", "Effect": "Allow", 
                        "Principal": {"AWS": 111}, "Action": ["SQS:GetQueueAttributes", "SQS:DeleteMessage", "SQS:ReceiveMessage", 
                        "SQS:SendMessage", "SQS:ChangeMessageVisibility"], "Resource": "arn:aws:sqs:us-east-1:123456789013:ARN"}]}`
                }
            };
        }
        return { Attributes: { QueueArn: 'arn:aws:sqs:us-east-1:123456789012:ARN' } };
    }
}));

AWS.mock('SQS', 'createQueue', params => ({
    promise: () => ({ QueueUrl: `https://sqs.us-east-1.amazonaws.com/123/${params.QueueName}` })
}));

AWS.mock('SQS', 'receiveMessage', params => ({
    promise: async() => {
        let interval;
        let timeout;

        const p1 = new Promise(resolve => {
            timeout = setTimeout(() => {
                clearInterval(interval);
                resolve({ Messages: [] });
            }, RECEIVE_TIMEOUT_MS);
        });

        const p2 = new Promise((resolve, reject) => {
            interval = setInterval(async() => {
                try {

                    if (params.QueueUrl.includes('MP_SUBSCRIPTION_STATUS_')) {
                        const mpMessages = remove(messagesQueue, message => {
                            const body = JSON.parse(message.Body);
                            if (body.Message) {
                                message = JSON.parse(body.Message);
                                return message['product-code'] === MP_PRODUCT_CODE_METERING;
                            }
                        });

                        if (mpMessages.length > 0) {
                            clearInterval(interval);
                            clearTimeout(timeout);
                            resolve({ Messages: mpMessages });
                        }
                    } else {
                        if (params.QueueUrl === `https://sqs.us-east-1.amazonaws.com/123/${getStatusesQueueName()}`) {
                            const relevantMessages = messagesQueue.splice(0, 9);
                            if (relevantMessages.length > 0) {
                                clearInterval(interval);
                                clearTimeout(timeout);
                                resolve({ Messages: relevantMessages });
                            }
                        } else {
                            const dataBroker = await DataBroker.findOne({ statusesQueueUrl: params.QueueUrl }).exec();
                            if (dataBroker) {
                                const relevantMessages = remove(messagesQueue, message => {
                                    const attr = message.MessageAttributes;
                                    return attr && attr.dataBrokerId && attr.dataBrokerId.StringValue === dataBroker.id;
                                });
                                if (relevantMessages.length > 0) {
                                    clearInterval(interval);
                                    clearTimeout(timeout);
                                    resolve({ Messages: relevantMessages });
                                }
                            }
                        }
                    }
                } catch (err) {
                    logger.error('Received error on sqs receive messages', err);
                    clearInterval(interval);
                    clearTimeout(timeout);
                    reject(err);
                }
            }, 500);
        });

        return Promise.race([p1, p2]);
    }
}));

function sendSaaSSubscriptionMessage(customerIdentifier, action) {
    messagesQueue.push({
        ReceiptHandle: hyperid().uuid,
        Body: JSON.stringify({
            Message: JSON.stringify({
                'customer-identifier': customerIdentifier,
                action,
                'product-code': MP_PRODUCT_CODE_METERING
            }),
            Timestamp: moment().utc().format('YYYY-MM-DDTHH:mm:ss.SSS[Z]')
        })
    });
}

function runScenarioNewDataBroker(key, messageAttributes, attributes, canAbort = false, isUnknown = true) {
    let messages = runningRelationshipsNewDataBroker[key];
    if (messages && messages.length > 0) {
        const headMessage = head(messages);
        if (canAbort && relationshipsToAbort.includes(headMessage.relationshipId)) {
            remove(relationshipsToAbort, relationshipId => relationshipId === headMessage.relationshipId);
        } else if (!isUnknown) {
            messagesQueue.push(createSQSMessage(headMessage, messageAttributes, attributes));
            messages = drop(messages, 1);
            runningRelationshipsNewDataBroker[key] = messages;
            setTimeout(runScenarioNewDataBroker.bind(this, key, messageAttributes, attributes, canAbort, isUnknown), DELAY_TIME_MS);
        }
    }
}


AWS.mock('SQS', 'sendMessage', params => ({
    promise: async() => {
        const messageBody = params.MessageAttributes && params.MessageAttributes.zipped ? JSON.parse(await unzip(params.MessageBody)) : JSON.parse(params.MessageBody);
        const attributes = params.MessageAttributes;
        const dataBrokerId = messageBody.header ? messageBody.header.dataBrokerId : attributes.dataBrokerId.StringValue;
        const commandToHandle = messageBody.command || (attributes.type ? attributes.type.StringValue : undefined);
        const dataBroker = await DataBroker.findOne({ _id: dataBrokerId }).exec();
        if (!dataBroker) {
            logger.debug(`DataBroker with id ${dataBrokerId} was not found`);
            return { MessageId: hyperid().uuid };
        }
        const { name: dataBrokerName, userId } = dataBroker;
        switch (commandToHandle) {
            case 'metadata': {
                const { targetDataBrokerId } = messageBody;
                messagesQueue.push(createSQSMessage({ targetDataBrokerId, metadata: {} }, attributes));
            }
                break;
            case 'startup':
            case 'update':
                // do nothing
                break;
            case 'exports': {
                const { host } = messageBody;
                messagesQueue.push(buildListNfsExportsResponse(host, attributes, {SenderId:userId},
                    (dataBrokerName === 'EmptyDataBroker')));
            }
                break;
            case 'export-folders': {
                const { host, export: exportName, path } = messageBody;
                messagesQueue.push(buildListNfsExportFoldersResponse(host, exportName, path, attributes, {SenderId:userId},
                    (dataBrokerName === 'EmptyDataBroker')));
            }
                break;
            case 'shares': {
                const { host } = messageBody;
                const messageToSend = buildListCifsSharesResponseNewDataBroker(host, (dataBrokerName === 'EmptyDataBroker'), attributes, {SenderId:userId});
                messagesQueue.push(messageToSend);
            }
                break;
            case 'share-folders': {
                const { host, share, path } = messageBody;
                messagesQueue.push(buildListCifsShareFoldersResponse(attributes, {SenderId:userId}, host, share, path));
            }
                break;
            case 's3-buckets' : {
                const {provider, host, port} = messageBody;
                messagesQueue.push(buildListS3BucketsResponse(provider, host, port, attributes, {SenderId:userId},
                    (dataBrokerName === 'EmptyDataBroker')));
            }
                break;
            case 's3-bucket-folders': {
                const { bucket, prefix, provider, host, port } = messageBody;
                messagesQueue.push(buildListS3BucketFoldersResponse(attributes, {SenderId:userId}, bucket, prefix, provider, {host, port}, (dataBrokerName === 'EmptyDataBroker')));
            }
                break;
            case 'azure-blob-storage-containers': {
                const { storageAccountName } = messageBody;
                messagesQueue.push(buildListAzureContainersResponse(attributes, {SenderId:userId}, storageAccountName,
                    dataBrokerName === 'EmptyDataBroker'));
            }
                break;
            case 'azure-blob-storage-container-folders': {
                const { container, prefix, storageAccountName } = messageBody;
                messagesQueue.push(buildListAzureContainerFoldersResponse(attributes, {SenderId:userId}, container, prefix, storageAccountName,
                    dataBrokerName === 'EmptyDataBroker'));
            }
                break;
            case 'gcp-buckets': {
                const { projectId } = messageBody;
                messagesQueue.push(buildListGcpBucketsResponse(attributes, {SenderId:userId}, projectId,
                    dataBrokerName === 'EmptyDataBroker'));
            }
                break;
            case 'gcp-bucket-folders': {
                const { bucket, prefix, projectId } = messageBody;
                messagesQueue.push(buildListGcpBucketFoldersResponse(attributes, {SenderId:userId}, bucket, prefix, projectId,
                    dataBrokerName === 'EmptyDataBroker'));
            }
                break;
            case 'kms-keys': {
                const { bucket } = messageBody;
                messagesQueue.push(buildListKmsKeysResponse(attributes, {SenderId:userId}, bucket, dataBrokerName === 'EmptyDataBroker'));

            }
                break;
            case 'copy': {
                if (messageBody && messageBody.type !== SCAN_MODE_SOURCE_AND_TARGET) {
                    const [sourceType] = Object.keys(messageBody.source);
                    const source = messageBody.source[sourceType];
                    const [targetType] = Object.keys(messageBody.target);
                    const target = messageBody.target[targetType];
                    const { scannerQueue, transferrerQueue, relationshipId } = messageBody;
                    const relationship = await Relationship.findOne({ _id: relationshipId, phase: 'Sync' }).exec();
                    const isRelationshipInSync = Boolean(relationship);
                    const isCopyFailure = (FAILURE_ADDRESS_CREATE_REGEX.test(source.host || source.bucket || source.container || '') && !isRelationshipInSync
                            || FAILURE_ADDRESS_CREATE_REGEX.test(target.host || target.bucket || target.container || '') && !isRelationshipInSync);
                    const isSyncFailure = (FAILURE_ADDRESS_SYNC_REGEX.test(source.host || source.bucket || source.container || '')
                            && isRelationshipInSync || FAILURE_ADDRESS_SYNC_REGEX.test(target.host || target.bucket || target.container || '') && isRelationshipInSync);
                    const isSuccess = (SUCCESS_ADDRESS_CREATE_REGEX.test(source.host || source.bucket || source.container || '')
                            || SUCCESS_ADDRESS_CREATE_REGEX.test(target.host || target.bucket || target.container || ''));
                    const prefix = isRelationshipInSync ? 'syncPhoenix' : 'copy';
                    const suffix = (isCopyFailure || isSyncFailure) ? 'failure' : (isSuccess ? 'success' : 'completed');
                    const scenarioFileName = `${config.get('paths.simulator-scenarios')}/${prefix}-${suffix}.json`;
                    const scenarioFile = path.resolve(__dirname, scenarioFileName);
                    if (!fs.existsSync(scenarioFile)) {
                        logger.error(`Scenario file '${scenarioFileName}' not found to handle:`, messageBody);
                        return;
                    }
                    const scenario = fs.readFileSync(scenarioFile, 'utf-8');
                    const result = JSON.parse(format(scenario, {
                        RELATIONSHIP_ID: relationshipId
                    }));
                    const runningRelationshipsKey = `${scannerQueue}_${transferrerQueue}`;
                    runningRelationshipsNewDataBroker[runningRelationshipsKey] = result;
                    const isUnknown = dataBroker.name === 'Unknown';
                    runScenarioNewDataBroker(runningRelationshipsKey, attributes, {SenderId:userId}, true, isUnknown);
                }
            }
                break;
            case 'abort': {
                const { relationshipId } = messageBody;
                const relationship = await Relationship.findById(relationshipId).exec();
                relationshipsToAbort.push(relationshipId);
                const { scannerQueue, transferrerQueue } = relationship;
                const runningRelationshipsKey = `${scannerQueue}_${transferrerQueue}`;
                delete runningRelationshipsNewDataBroker[runningRelationshipsKey];
            }
                break;
            case 'download-logs':
                {
                    const { name } = messageBody;
                    await execute(`aws s3 cp ./package.json s3://metadata.datafabric.io/data-broker-logs/${name} --acl public-read`);
                }
                break;
            case 'done':
            case 'save-metadata':
            case 'shutdown':
            case 'remove': {
                // nothing to do
            }
                break;
            default:
                logger.error('Got unexpected SQS message', messageBody);
        }
        return { MessageId: hyperid().uuid };
    }
}));

module.exports = { sendHealthMessage, sendSaaSSubscriptionMessage, sendPlacementMessage };
