const { spawn } = require('child_process');
const os = require('os');
const fs = require('fs');
const path = require('path');
const mongoose = require('mongoose');
const moment = require('moment');
const { agenda } = require('./agenda');
const {
    AGENDA_TASK_NAME_BACKUP_DB,
    SERVER_HOST,
    DFIO_METADATA_BUCKET_REGION,
    DFIO_METADATA_BUCKET_NAME,
    DFIO_METADATA_BUCKET_BACKUPS_FOLDER
} = require('../utils/consts');
const { inContext } = require('../middlewares/continuation-local-storage');
const { putObject } = require('../lib/aws/s3');

const getLogger = require('../utils/logger');

const logger = getLogger();

/*
* For restoring a db from archive file use:
* mongorestore -host db.example.com:27017 --oplogReplay --gzip --archive=/path/to/archive
*/

async function discoverMongoNode() {
    logger.info('Discovering source DB host and port');
    const con = mongoose.connection;
    if (con.replica) {
        logger.info('Connected to mongo replica set');
        const status = await mongoose.connection.db.admin().replSetGetStatus();
        logger.trace('Replica set status:', status);
        const secondaryHealthyMembers = status.members.filter(m => m.state === 2 && m.health === 1);
        if (secondaryHealthyMembers.length) {
            logger.info(`Found ${secondaryHealthyMembers.length} healthy secondary members`);
            const secondary = secondaryHealthyMembers[0].name;
            logger.info(`Selecting secondary member: ${secondary}`);
            return secondary;
        }
        logger.warn('Found no healthy secondary member');
        const primary = status.members.find(m => m.state === 1).name;
        logger.info(`Selecting primary member: ${primary}`);
        return primary;
    }
    logger.info('Connected to standalone mongo server');
    const standalone = `${con.host}:${con.port}`;
    logger.info(`Selecting standalone server: ${standalone}`);
    return standalone;
}

function dumpDb(tmpFile, mongoNode) {
    const args = [];
    args.push('--host', mongoNode);
    if (mongoose.connection.replica) {
        args.push('--oplog');
    }
    args.push('--gzip');
    args.push(`--archive=${tmpFile}`);
    logger.info('mongodump args:', args);

    return new Promise((resolve, reject) => {
        logger.info('Spawning mongodump...');
        const mongodump = spawn('mongodump', args);
        mongodump.stdout.on('data', data => logger.info(data.toString().trim()));
        mongodump.stderr.on('data', data => logger.warn(data.toString().trim()));

        mongodump
            .on('exit', (code, signal) => {
                if (signal) {
                    reject(new Error(`mongodump exited due to signal: ${signal}`));
                } else if (code !== 0) {
                    reject(new Error(`mongodump exited with code: ${code}`));
                } else {
                    logger.info(`mongodump exited with code: ${code}`);
                    resolve();
                }
            })
            .on('error', err => {
                logger.error('Failed spawning mongodump:', err);
                reject(err);
            });
    });
}

async function uploadDumpToS3(tmpFile) {
    const key = `${DFIO_METADATA_BUCKET_BACKUPS_FOLDER}/${path.basename(tmpFile)}`;
    logger.info(`Uploading DB backup ${key} to S3 bucket ${DFIO_METADATA_BUCKET_NAME}`);
    await putObject(DFIO_METADATA_BUCKET_REGION, DFIO_METADATA_BUCKET_NAME, key, fs.createReadStream(tmpFile));
    logger.info(`Uploaded DB backup ${key} to S3 bucket ${DFIO_METADATA_BUCKET_NAME} successfully`);
}

function clean(tmpFile) {
    try {
        if (fs.existsSync(tmpFile)) {
            logger.info(`Cleaning temporary backup file ${tmpFile}`);
            fs.unlinkSync(tmpFile);
            logger.info('Clean completed');
        }
    } catch (err) {
        logger.warn(`Failed cleaning temporary backup file ${tmpFile}`, err.message);
    }
}

async function doBackup() {
    const tmpFile = path.resolve(os.tmpdir(), `${SERVER_HOST.split('.').join('-')}_${moment().format('YYYY-MM-DD_HH-mm-ss')}.zmdb`);
    try {
        logger.info(`DB backup file name: ${tmpFile}`);
        const mongoNode = await discoverMongoNode();
        await dumpDb(tmpFile, mongoNode);
        await uploadDumpToS3(tmpFile);
    } finally {
        await clean(tmpFile);
    }
}

agenda.define(AGENDA_TASK_NAME_BACKUP_DB, { lockLifetime: 10000 }, (job, done) => {
    inContext('Database scheduled backup', () => {
        logger.debug('Database scheduled backup task started');
        doBackup()
            .then(() => {
                logger.info('Database scheduled backup task completed successfully');
                done();
            })
            .catch(err => {
                logger.error('Database scheduled backup task failed', err);
                done(err);
            });
    });
});
