const fs = require('fs');
const path = require('path');
const moment = require('moment');
const config = require('config');
const {Cost} = require('../models/cost');
const {Account} = require('../models/account');
const connectAndMigrate = require('../db-connect-and-migrate');
const {initAgenda} = require('./agenda');
const {startListeningStatusesQueue, startListeningDataBrokersStatuses} = require('./sqs-listener');
const startMPSubscriptionStatusListener = require('./mp-subscription-status-listener');
const {isProduction} = require('./environment');
const {TRIAL_EXPIRATION_MS} = require('../utils/consts');
const {relationshipsStatusPoller} = require('../operations/relationship-operations');
const {getDataBrokerLatestVersionAsync} = require('./update-data-broker');

const getLogger = require('../utils/logger');

const logger = getLogger();
const currentTime = Date.now();
const defaultTrialExpiry = new Date(currentTime + TRIAL_EXPIRATION_MS);

async function createFakeAccount(fakeAccount) {
    const {
        id, name, tenancyAccountId
    } = fakeAccount;
    logger.debug(`Creating '${name}' account if needed`);
    const account = await Account.findOne({tenancyAccountId}).exec();
    if (account) {
        logger.debug(`'${name}' account already exist`);
        return account;
    }
    let createdAt;
    logger.debug(`'${name}' account not found. Creating...`);
    if (['OCCM_Fake_User_2', 'Trial_expiry', 'CS_OCCM_Fake_User'].includes(name)) {
        createdAt = new Date('1/1/2016 00:00').valueOf();
    } else {
        createdAt = new Date().valueOf();
    }
    let trialExpiry;
    let automaticExtension = true;
    if (name.includes('CS_OCCM_Fake_User')) {
        trialExpiry = new Date(currentTime + moment.duration(5, 'days').as('milliseconds'));
        automaticExtension = false;
    } else if (name.includes('OCCM_Fake_User_2')) {
        trialExpiry = new Date(currentTime + moment.duration(10, 'days').as('milliseconds'));
        automaticExtension = false;
    } else if (name.includes('Trial_expiry')) {
        trialExpiry = new Date('2/1/2016 00:00').valueOf();
        automaticExtension = false;
    } else {
        trialExpiry = defaultTrialExpiry;
    }
    return new Account({
        _id: id, name, tenancyAccountId, trialExpiry, automaticExtension, createdAt, support: {
            serialNumber: '12345',
            status: 'unregistered'
        }
    }).save();
}

async function createCostRecord(cost) {
    const {
        id, account, timestamp, tier1, tier2, tier3, byol
    } = cost;
    const foundCost = await Cost.findById(id).exec();
    if (foundCost) {
        logger.debug(`'${id}' cost record is already exist`);
        return foundCost;
    }
    logger.debug(`'${id}' cost record was not found. Creating...`);
    return new Cost({
        _id: id, account, timestamp, tier1, tier2, tier3, byol
    }).save();
}

async function createFakeAccounts() {
    if (!isProduction()) {
        const fakeAccounts = JSON.parse(fs.readFileSync(path.resolve(__dirname, config.get('paths.fake-accounts')), 'utf8'));
        await Promise.all(fakeAccounts.map(account => createFakeAccount(account)));
    }
}


async function createCostRecords() {
    if (!isProduction()) {
        const costRecords = JSON.parse(fs.readFileSync(path.resolve(__dirname, config.get('paths.cost-records')), 'utf8'));
        return Promise.all([costRecords.map(createCostRecord)]);
    }
}

async function bootstrap() {
    try {
        logger.info('Booting...');
        await connectAndMigrate();
        // startListeningDataBrokersStatuses dependant on startListeningStatusesQueue
        await Promise.all([initAgenda(), startListeningStatusesQueue()]);
        await Promise.all([createFakeAccounts(), createCostRecords(),
            startListeningDataBrokersStatuses(), startMPSubscriptionStatusListener(),  getDataBrokerLatestVersionAsync()]);
        relationshipsStatusPoller(); // no need to wait for completion
        logger.info('Booting done');
    } catch (err) {
        logger.error('Bootstrapping application failed with:', err);
        throw err;
    }
}

module.exports = bootstrap;
