const Promise = require('bluebird');
const os = require('os');
const {
    createQueue,
    receiveMessage,
    deleteMessage,
    getQueueArn,
    allowArnSendMessage
} = require('../lib/aws/sqs');
const {subscribe, confirmSubscription} = require('../lib/aws/sns');
const {
    SERVER_HOST, MP_SUBSCRIPTION_STATUS_SNS_TOPIC, MP_SELLER_ACCESS_KEY, MP_SELLER_SECRET_KEY, SAAS_SUBSCRIPTION_PROVIDER_AWS,
    PAYGO_SUBSCRIPTION, MP_SUBSCRIPTION_STATUS, MP_SUBSCRIPTION_CONFIRMATION_MESSAGE_TYPE
} = require('../utils/consts');
const {Account} = require('../models/account');
const {updateAccountsSubscription} = require('../operations/account-operations');
const {validateMarketplaceProductCode} = require('./common');

const getLogger = require('../utils/logger');

const logger = getLogger();


async function handleSubscriptionStatusNotification(message, timestamp) {
    const customerIdentifier = message['customer-identifier'];
    const subscriptionType = validateMarketplaceProductCode(message['product-code']);

    // make update multi
    if (subscriptionType === PAYGO_SUBSCRIPTION) {
        const accounts = await Account.find({
            'subscription.customerIdentifier': customerIdentifier,
            'subscription.type': PAYGO_SUBSCRIPTION
        }).exec();

        if (accounts.length > 0) {
            await Promise.all(accounts.map(account => {
                if (timestamp > account.subscription.updatedAt) {
                    logger.info(`Updating Subscription status for tenancyAccountId: ${account.tenancyAccountId}, status ${message.action}, timestamp ${timestamp}`);

                    return updateAccountsSubscription(accounts, message.action, PAYGO_SUBSCRIPTION, customerIdentifier, SAAS_SUBSCRIPTION_PROVIDER_AWS);
                }

                logger.warn(`Received outdated subscription notification [Action: ${message.action}. CustomerIdentifier: ${customerIdentifier}, timestamp ${timestamp}]`);
            }));
        } else {
            logger.warn('Subscription notification\'s customer identifier not found.', {customerIdentifier, action:message.action, timestamp});
        }
    } else {
        logger.warn('Subscription type is CONTRACT and not PAYGO');
    }
}

function handleMessage(sqsMessage) {
    logger.debug('Handling SQS message: ', sqsMessage);

    let subscriptionMessage;
    let message;

    try {
        message = JSON.parse(sqsMessage);
        if (message.Message && !(message.Type && message.Type === MP_SUBSCRIPTION_CONFIRMATION_MESSAGE_TYPE)) {
            subscriptionMessage = JSON.parse(message.Message);
        }
    } catch (err) {
        logger.warn('Got non JSON SQS message', err);
        return;
    }

    logger.info(`Got topic ${MP_SUBSCRIPTION_STATUS_SNS_TOPIC} subscription message type: `, message.Type);
    if (message.Type && message.Type === MP_SUBSCRIPTION_CONFIRMATION_MESSAGE_TYPE) {
        return confirmSubscription(MP_SUBSCRIPTION_STATUS_SNS_TOPIC, message.Token, MP_SELLER_ACCESS_KEY, MP_SELLER_SECRET_KEY);
    }

    if (subscriptionMessage && subscriptionMessage.action && subscriptionMessage['customer-identifier'] && subscriptionMessage['product-code']) {
        return handleSubscriptionStatusNotification(subscriptionMessage, new Date(message.Timestamp));
    }

    logger.warn('Not a supported message:', message);
}

async function pollQueue(queueUrl) {
    try {
        logger.debug(`Polling for messages from ${queueUrl}`);
        const messages = await receiveMessage(queueUrl);
        await Promise.all(messages.map(async message => {
            logger.debug('Got SQS message:', message);
            await handleMessage(message.body);
            await deleteMessage(queueUrl, message.receiptHandle);
        }));

        pollQueue(queueUrl);
    } catch (err) {
        logger.error(`Failed to poll messages from ${queueUrl}`, err);
        if (err.code === 'AWS.SimpleQueueService.NonExistentQueue') {
            logger.warn(`${queueUrl} does not exist - stopping polling`);
        } else {
            setTimeout(() => pollQueue(queueUrl), 5000);
        }
    }
}

async function startMPSubscriptionStatusListener() {
    // #1 - crate queue
    // #2 - translate queue url to arn
    // #3 - allow send message to the queue only from the topic
    // #4 - subscribe queue to topic (in seller account)
    // #5 - start polling from the queue
    // #6 - once SubscriptionConfirmation message is received - confirm the subscription (in seller account)

    const [serverHost] = SERVER_HOST.split('.');
    const name = serverHost === 'local' ? `${MP_SUBSCRIPTION_STATUS}${os.hostname().replace(/[-.]/g, '_')}` : `${MP_SUBSCRIPTION_STATUS}${serverHost}`;
    const queueUrl = await createQueue(name);
    const queueArn = await getQueueArn(queueUrl);
    await allowArnSendMessage(queueUrl, queueArn, MP_SUBSCRIPTION_STATUS_SNS_TOPIC);
    logger.info(`Subscribing queue ${queueArn} to topic ${MP_SUBSCRIPTION_STATUS_SNS_TOPIC}`);
    const subscriptionArn = await subscribe(MP_SUBSCRIPTION_STATUS_SNS_TOPIC, queueArn, MP_SELLER_ACCESS_KEY, MP_SELLER_SECRET_KEY);
    logger.info(`SQS ${queueArn} was successfully subscribed to SNS ${MP_SUBSCRIPTION_STATUS_SNS_TOPIC}, subscription ARN: ${subscriptionArn}`);
    pollQueue(queueUrl);
}


module.exports = startMPSubscriptionStatusListener;
