const { filter, orderBy, last, zipWith } = require('lodash');
const { agenda } = require('./agenda');
const {findRelationshipsForNewDataBrokerStartup} = require('../operations/relationship-operations');
const DataBroker = require('../models/data-broker');
const {sendMessage} = require('../lib/aws/sqs');
const {
    AGENDA_TASK_NAME_UPDATE_DATABROKER,
    DFIO_METADATA_BUCKET_REGION,
    DFIO_METADATA_BUCKET_NAME,
    DATA_BROKER_REPO_FOLDER,
    DATA_BROKER_BUNDLE_LABEL
} = require('./consts');
const { listObjectsAsync, getObjectTaggingAsync, getS3ObjectUrl } = require('../lib/aws/s3');
const { getRuntimeEnvironment } = require('./environment');
const { NotFound } = require('../utils/errors');
const { generateRequestId } = require('./utils');
const { isNewDataBroker } = require('./common');
const getLogger = require('../utils/logger');

const logger = getLogger();

async function sendUpdateVersionCommand(requestId, dataBroker, latestVersion) {
    const {
        commandsQueueUrl, id: dataBrokerId, groupId
    } = dataBroker;
    const relationships = await findRelationshipsForNewDataBrokerStartup(dataBroker, groupId);
    return sendMessage(commandsQueueUrl, {
        bundleUrl: latestVersion.url,
        bundleVersion: latestVersion.version,
        relationships
    }, {type: 'update', requestId, dataBrokerId});
}


async function getDataBrokerLatestVersionAsync() {
    const prefix = `${DATA_BROKER_REPO_FOLDER}/${getRuntimeEnvironment()}/phoenix`;
    logger.info(`Looking for data broker bundle in bucket ${DFIO_METADATA_BUCKET_NAME}, region ${DFIO_METADATA_BUCKET_REGION}, prefix ${prefix}, label ${DATA_BROKER_BUNDLE_LABEL}`);

    const objects = await listObjectsAsync(DFIO_METADATA_BUCKET_REGION, DFIO_METADATA_BUCKET_NAME, prefix);
    const objectsTags = await Promise.all(objects.map(object => getObjectTaggingAsync(DFIO_METADATA_BUCKET_REGION, DFIO_METADATA_BUCKET_NAME, object.key)));
    const objectsWithTags = zipWith(objects, objectsTags, (object, tags) => Object.assign(object, tags));

    logger.info(`${objectsWithTags.length} agent bundles found in ${DFIO_METADATA_BUCKET_NAME}/${prefix}`);
    const filtered = filter(objectsWithTags, o =>
        // use labeled bundles only if bundle name is defined in server configuration
        // use not labeled bundles only if bundle name is not defined
        (DATA_BROKER_BUNDLE_LABEL ? o.label && o.label === DATA_BROKER_BUNDLE_LABEL : !o.label) && o.key.includes('phoenix'));

    const orderedObjects = orderBy(filtered, ['lastModified']);
    const lastObject = last(orderedObjects);
    if (lastObject) {
        // the key is publicly available since create-data-broker-bundle script makes bundle 'public-read'
        const dataBrokerBundleUrl = getS3ObjectUrl(DFIO_METADATA_BUCKET_REGION, DFIO_METADATA_BUCKET_NAME, lastObject.key,
            true, true);
        logger.info('Data broker bundle URL:', dataBrokerBundleUrl);
        lastObject.url = dataBrokerBundleUrl;
        return lastObject;
    }
    throw new NotFound(`Data Broker bundle not found. Bucket ${DFIO_METADATA_BUCKET_NAME}, prefix ${prefix}, label ${DATA_BROKER_BUNDLE_LABEL}`);
}

async function updateDataBrokerWithVersion(dataBrokerId) {
    // #1 get latest data broker version
    const latestVersion = await getDataBrokerLatestVersionAsync();
    logger.info(`Latest online data broker version for phoenix '${latestVersion.version}'`);
    const dataBroker = await DataBroker.findOne({ version: { $ne: latestVersion.version }, _id: dataBrokerId }).exec();
    if (dataBroker) {
        logger.info(`Sending dataBroker with ${dataBrokerId} from version ${dataBroker.version} to version ${latestVersion.version}`);
        await DataBroker.findOneAndUpdate({ _id: dataBroker.id }, { $set: { versionUpdateSent: new Date() } }).exec();
        await sendUpdateVersionCommand(generateRequestId(), dataBroker, latestVersion);
    } else {
        logger.info(`dataBroker with id ${dataBrokerId} doesnt need an update`);
    }
}

async function updateDataBrokers() {
    // #1 get latest data broker version
    const latestVersion = await getDataBrokerLatestVersionAsync();
    logger.info(`Latest online data broker version for phoenix '${latestVersion.version}'`);
    // #2 get all phoenix data brokers
    const allDataBrokers = await DataBroker.find({ version: { $ne: latestVersion.version } }).exec();
    const alivePhoenixDataBrokers = allDataBrokers.filter(dataBroker => isNewDataBroker(dataBroker) && dataBroker.isAlive());
    // #3 check if the version is equal
    // #4 send update command
    const promises = alivePhoenixDataBrokers.map(async dataBroker => {
        logger.info(`Sending ${dataBroker._id} update version command to version '${latestVersion.version}' with url '${latestVersion.url}'`);
        await DataBroker.findOneAndUpdate({ _id: dataBroker.id }, { $set: { versionUpdateSent: new Date() } }).exec();
        await sendUpdateVersionCommand(generateRequestId(), dataBroker, latestVersion);
        logger.info(`Sent ${dataBroker.id} update version command to version '${latestVersion.version}'`);
    });
    await Promise.all(promises);
}

agenda.define(AGENDA_TASK_NAME_UPDATE_DATABROKER, { lockLifetime: 10000 }, async(job, done) => {
    logger.info('update data broker task started');
    try {
        await updateDataBrokers();
        logger.info('update data broker task finished');
    } catch (e) {
        logger.error('update-data-broker task failed:', e);
    } finally {
        done();
    }
});

module.exports = { updateDataBrokerWithVersion, getDataBrokerLatestVersionAsync };
