const moment = require('moment');
const {Account} = require('../../src/models/account');
const {should} = require('../globals');

const name = 'name';
const trialExpiry = moment().subtract(1, 'years');
const support = {
    serialNumber: '12345',
    status: 'unregistered'
};
const tenancyAccountId = 'a';

describe('Account Model', () => {

    beforeEach(() => Account.deleteMany());

    describe('CRUD', () => {
        it('create', async() => {

            const account = await Account.create({
                name, trialExpiry, support, tenancyAccountId
            });
            should.exist(account);
            account.name.should.equal(name);
            account.tenancyAccountId.should.equal(tenancyAccountId);
            should.exist(account.createdAt);
            should.exist(account.updatedAt);

            it('create with optional fields', async() => {
                const subscription = {
                    status: 'subscribe-success',
                    updatedAt: new Date(),
                    type: 'PAYGO',
                    provider: 'aws',
                    customerIdentifier: 'PASTRAMA-MIA'
                };
                const account = await Account.create({
                    name, trialExpiry, support, tenancyAccountId, subscription
                });
                should.exist(account);
                account.name.should.equal(name);
                account.subscription.should.equal(subscription);
                should.exist(account.createdAt);
                should.exist(account.updatedAt);
            });
        });

        it('get', async() => {
            const accountCreated = await Account.create({
                name, trialExpiry, support, tenancyAccountId
            });
            const account = await Account.findOne({_id: accountCreated.id});
            should.exist(account);
            account.name.should.equal(name);
            account.tenancyAccountId.should.equal(tenancyAccountId);
            should.exist(account.createdAt);
            should.exist(account.updatedAt);
        });

        it('update', async() => {
            const updatedName = 'newUser';
            const account = await Account.create({
                name, trialExpiry, support, tenancyAccountId
            });
            account.name = updatedName;
            await account.save();
            const accountFound = await Account.findOne({_id: account.id});
            should.exist(accountFound);
            accountFound.name.should.equal(updatedName);
            accountFound.tenancyAccountId.should.equal(tenancyAccountId);
            should.exist(accountFound.createdAt);
            should.exist(accountFound.updatedAt);
        });

        it('add optional fields', async() => {
            const account = await Account.create({
                name, trialExpiry, support, tenancyAccountId
            });
            const nssUsername = 'a';
            const nssPassword = '2';
            account.nssCredentials = {nssUsername, nssPassword};
            await account.save();
            const foundAccount = await Account.findOne({_id: account.id});
            should.exist(foundAccount);
            foundAccount.name.should.equal(name);
            foundAccount.nssCredentials.nssUsername.should.equal(nssUsername);
            foundAccount.nssCredentials.nssPassword.should.equal(nssPassword);
            should.exist(foundAccount.createdAt);
            should.exist(foundAccount.updatedAt);
        });

        it('delete', async() => {
            await Account.create({
                name, trialExpiry, support, tenancyAccountId
            });
            await Account.deleteOne({tenancyAccountId});
            await Account.findOne({tenancyAccountId}).should.eventually.not.exist;
        });
    });


});