const {Account} = require('../../src/models/account');
const TimelineV2 = require('../../src/models/timeline-v2');
const DataBroker = require('../../src/models/data-broker');
const {should} = require('../globals');

describe('TimelineV2 Model', () => {
    let account;
    let dataBroker;
    const requestId = 'requestId';
    const status = 'status';
    const summary = 'summary';
    const dataBrokerName = 'dataBroker';
    const failureMessage = 'failureMessage';

    beforeEach(async() => {
        await Promise.all([Account.deleteMany(), TimelineV2.deleteMany(), DataBroker.deleteMany()]);
        account = await Account.create({
            name: 'Account Name',
            trialExpiry: new Date(),
            tenancyAccountId: '1',
            support: {
                serialNumber: '12345',
                status: 'unregistered'
            }
        });
        dataBroker = await DataBroker.create({
            name:dataBrokerName,
            type:'AWS',
            account:account.id
        });
    });

    describe('CRUD', () => {
        it('create', async() => {
            const timeline = await TimelineV2.create({
                account,  status, summary, requestId
            });
            should.exist(timeline);
            timeline.account.id.should.equal(account.id);
            timeline.requestId.should.equal(requestId);
            timeline.status.should.equal(status);
            timeline.summary.should.equal(summary);
            should.exist(timeline.createdAt);
            should.exist(timeline.updatedAt);
        });
    });

    it('create with optional fields', async() => {
        const dataBrokerToInsert = {dataBrokerId: dataBroker._id, name: dataBroker.name };
        const timeline = await TimelineV2.create({
            account, requestId, status, summary, dataBroker:dataBrokerToInsert, failureMessage
        });
        should.exist(timeline);
        timeline.account.id.should.equal(account.id);
        timeline.requestId.should.equal(requestId);
        timeline.status.should.equal(status);
        timeline.summary.should.equal(summary);
        timeline.dataBroker.name.should.equal(dataBroker.name);
        should.exist(timeline.createdAt);
        should.exist(timeline.updatedAt);
    });

    it('get', async() => {

        const timelineCreated = await TimelineV2.create({
            account, requestId, status, summary
        });
        const timeLine = await TimelineV2.findOne({_id: timelineCreated.id}).populate('account');
        should.exist(timeLine);
        timeLine.account.id.should.equal(account.id);
        timeLine.requestId.should.equal(requestId);
        timeLine.status.should.equal(status);
        timeLine.summary.should.equal(summary);
        should.exist(timeLine.createdAt);
        should.exist(timeLine.updatedAt);
    });

    it('update', async() => {
        const updateStatus = 'newStatus';
        const timelineCreated =  await TimelineV2.create({
            account, requestId, status, summary
        });
        timelineCreated.status = updateStatus;
        await  timelineCreated.save();
        const timelineFound = await TimelineV2.findOne({_id: timelineCreated.id}).populate('account');
        should.exist(timelineFound);
        timelineFound.account.id.should.equal(account.id);
        timelineFound.status.should.equal(updateStatus);
        timelineFound.requestId.should.equal(requestId);
        timelineFound.summary.should.equal(summary);
        should.exist(timelineFound.createdAt);
        should.exist(timelineFound.updatedAt);
    });
    describe('JSON representation', () => {
        it('successfully transform', async() => {

            const timeline = await TimelineV2.create({
                account, requestId, status, summary
            });
            const json = timeline.toJSON();
            should.exist(json);
            should.not.exist(json.account);
            should.not.exist(json._id);
            should.not.exist(json.__v);
            should.exist(json.createdAt);
            should.not.exist(json.updatedAt);
            should.exist(json.id);
        });
    });
});


