import React, {useEffect} from 'react';
import {buildUrlParams} from './utils/encoders';
import Intercom from './components/intercom/Intercom';
import {updateUrlParams} from './store/wizard/action_creators';
import {logout} from './store/global/action_creators';
import ContentWrapper from './components/ContentWrapper';
import TabHeader from './components/tabs/TabHeader';
import Dashboard from './components/tabs/Dashboard';
import Timeline from './components/tabs/Timeline';
import LicenseSettings from './components/tabs/licenseSettings/LicenseSettings';
import SyncSettingsEdit from './components/tabs/syncSettings/SyncSettingsEdit';
import FailedDetails from './components/tabs/FailedDetails';
import SyncHeader from './components/wizard/SyncHeader';
import DragSourceAndTargetStep from './components/syncWizard/DragSourceAndTargetStep';
import SelectSourceAndTargetHeader from './components/syncWizard/SelectSourceAndTargetHeader';
import WizardContent from './components/wizard/WizardContent';
import WizardHeader from './components/wizard/WizardHeader';
import {ConnectedDialogContainer, openDialog} from './components/dialog/Dialog';
import ThanksForSubscribingDialog from './components/tabs/licenseSettings/ThanksForSubscribingDialog';
import * as actionCreatorsConsts from './store/global/action_creators_consts';
import {Redirect, Route, Switch, withRouter} from 'react-router-dom';
import {connect} from 'react-redux';
import {useIdleTimer} from 'react-idle-timer';
import _ from 'lodash';
import Oops from './components/Oops';
import {getAccountLastUsedPerUser} from './utils/account-utils';
import AccountSelectionDialog from "./components/accounts/AccountSelectionDialog";
import Header from './components/header/Header';
import Footer from './components/footer/Footer';
import CmIntegrationEntryPoint from './components/cmIntegration/CmIntegrationEntryPoint';
import CmHeader from './components/cmIntegration/CmHeader';
import MainLoader from './components/_common_/MainLoader/MainLoader';
import ReloadMessage from './components/cmIntegration/ReloadMessage';
import LoadingBar from './components/_common_/loadingBar/LoadingBar';
import {getPropFromState} from './utils/mapStateToPropsUtils';
import {setActiveAccount} from "./utils/messages-utils";
import useRunOnce from './hooks/useRunOnce';
import DeploymentsTracker from './components/tabs/DeploymentsTracker';

const redirect = ({auth, _ui, _relationships, _licenseSettings}) => {
    if (!_.get(_licenseSettings, 'succeeded')) return; // wait for license data

    // if user is subscribing
    if (_.get(auth, "data.subscribing")) {
        // show thanks dialog on top on the appropriate screen
        openDialog(<ThanksForSubscribingDialog className="confirm thanks-for-subscribing"/>);
        // do not return in order to show the appropriate screen below
    }
    // else if user needs to pay (has relationships that are not payed via subscription or license)
    else if (_ui.showTrialPage) {
        // show the trial screen with payment options
        actionCreatorsConsts.routesContainer.routes.replace({pathname: "/license", query: {trial: true}});
        return;
    }

    if (!_relationships || !_relationships.data) return; // wait for relationships data

    if (_relationships.data.length > 0) actionCreatorsConsts.routesContainer.routes.replace("/dashboard");
    else actionCreatorsConsts.routesContainer.routes.replace("/drag-source-and-target");
};

const RedirectorDumb = props => {
    const {auth, _ui, _relationships, _licenseSettings} = props;

    useEffect(() => {
        redirect({auth, _ui, _relationships, _licenseSettings});
    }, [auth, _ui, _relationships, _licenseSettings]);
    return (<div/>);

};

const redirectorMapStateToProps = (state) => ({
    isCmIntegration: state.syncReducer.isCmIntegration,
    auth: state.global.auth,
    _licenseSettings: state.global._licenseSettings,
    _accounts: state.global._accounts,
    _accountInfo: state.global._accountInfo,
    _ui:  {...state.syncReducer._ui, ...state.global._ui},
    _relationships: getPropFromState(state, "_relationships", {})
});
const Redirector = withRouter(connect(redirectorMapStateToProps)(RedirectorDumb));

const AuthenticatedRoute = props => props.auth.succeeded ?
    <>
        <ConnectedDialogContainer/>
        <Switch>
            <Route path="/cm" component={CmIntegrationEntryPoint}/>
            <Route path="/" render={() => <RouteWithSelectedAccount {...props}/>}/>
        </Switch>
        <DeploymentsTracker/>
    </> : <MainLoader/>;

const RouteWithSelectedAccount = (props) => {
    const {auth, _accounts, setActiveAccount, _accountInfo, _ui: {isCmIntegration}, logout} = props;
    const {data: {uid}} = auth;

    if (auth.accountError) {
        document.body.className = "no-sso"; //to remove the widget
        return <Oops error={auth.accountError} isCmIntegration={isCmIntegration} logout={logout}/>;
    }

    const AppFrame = () => isCmIntegration
        ? <>
            <CmHeader/>
            <MainLoader/>
          </>
        : <>
            <Header auth={auth} showWithoutRightMenu={true}/>
            <LoadingBar/>
            <div className="fixed-footer"><Footer auth={auth} _ui={{}} _generalErrors={{}}/></div>
        </>;

    if (!_.get(_accounts, 'succeeded') || _.get(_accountInfo, 'inProgress') || _.get(_accountInfo, 'failed')) return <AppFrame/>;

    //An account was selected:
    if (_.get(_accountInfo, 'succeeded')) {
        return <AppRoutes {...props}/>;
    }

    //look for previously used account in local storage and use it:
    const accountIdLastUsed = getAccountLastUsedPerUser(uid, _accounts.data);
    const singleAccountId = _accounts.data.length === 1 ? _accounts.data[0].accountId : undefined;
    const accountIdToSelect = accountIdLastUsed || singleAccountId;
    return <AccountSelector accountId={accountIdToSelect} setActiveAccount={setActiveAccount} _accounts={_accounts}
                          uid={uid}/>;
};

const AccountSelector = ({accountId, _accounts, setActiveAccount, uid}) => {
    useRunOnce(() => {
        if (accountId) {
            setActiveAccount({accountId, uid});
        } else {
            openDialog(<AccountSelectionDialog
                className="default-dialog"
                accounts={_accounts.data}
                setActiveAccount={setActiveAccount}
                uid={uid}/>, undefined, true);
        }
    });

    return <div/>;

};

const AppRoutes = props => {
    const {_ui: {isCmIntegration}} = props;

    return (<Switch>
        <Route exact path="/" render={(props) =>
            <ContentWrapper Content={Redirector}
                            contentUiState=""
                            {...props}/>
        }/>
        <Route path="/dashboard" render={(props) =>
            <ContentWrapper PrimaryHeader={TabHeader}
                            ContentHeader={isCmIntegration ? CmHeader : undefined}
                            Content={Dashboard}
                            contentUiState="tab"
                            {...props}/>
        }/>
        <Route path="/timeline" render={(props) =>
            <ContentWrapper PrimaryHeader={TabHeader}
                            Content={Timeline}
                            contentUiState="tab darkula"
                            {...props}/>
        }/>
        <Route path="/license" render={(props) =>
            <ContentWrapper PrimaryHeader={TabHeader}
                            ContentHeader={isCmIntegration ? CmHeader : undefined}
                            Content={LicenseSettings}
                            contentUiState="tab"
                            {...props}/>
        }/>
        <Route path="/settings/:relationshipId" render={(props) =>
            <ContentWrapper PrimaryHeader={TabHeader}
                            ContentHeader={isCmIntegration ? CmHeader : undefined}
                            Content={SyncSettingsEdit}
                            contentUiState="tab"
                            {...props}/>
        }/>
        <Route path="/failed/:relationshipId" render={(props) =>
            <ContentWrapper PrimaryHeader={TabHeader}
                            ContentHeader={isCmIntegration ? CmHeader : undefined}
                            Content={FailedDetails}
                            contentUiState="tab"
                            {...props}/>
        }/>
        {/*<Route path="/cost" render={(props) =>*/}
        {/*    <ContentWrapper PrimaryHeader={TabHeader}*/}
        {/*                    Content={Cost}*/}
        {/*                    contentUiState="tab"*/}
        {/*                    {...props}/>*/}
        {/*}/>*/}
        <Route path="/drag-source-and-target" render={(props) =>
            <ContentWrapper PrimaryHeader={SyncHeader} hideMainHeader={true}
                            Content={DragSourceAndTargetStep}
                            ContentHeader={isCmIntegration ? CmHeader : SelectSourceAndTargetHeader}
                            contentUiState="wizard"
                            {...props}/>

        }/>
        <Route path={`/sync/:protocols/:providers/:isSecure/:firstStep`} render={(props) =>
            <ContentWrapper PrimaryHeader={SyncHeader} hideMainHeader={true}
                            Content={WizardContent}
                            ContentHeader={isCmIntegration ? CmHeader : WizardHeader}
                            contentUiState="wizard"
                            contentKey="sync-wizard"
                            {...props}/>
        }/>
        <Redirect to="/"/>
    </Switch>)
};

const App = props => {
    const {_routing, match, location, auth, _ui: {isCmIntegration}} = props;
    const {updateUrlParams} = props;

    const idleTimeMinutes = Number.parseInt(process.env.REACT_APP_IDLE_TIME_MINUTES);
    const cmIntegrationByPath = location && location.pathname === '/cm';

    useRunOnce(() => {
        const urlParams = buildUrlParams(_routing, match, location, window.location.href);
        updateUrlParams(urlParams);
    });

    useIdleTimer({
        timeout: idleTimeMinutes * 60 * 1000,
        onIdle: window.auth.logout,
        debounce: 500
    });

    if (!auth) return <MainLoader/>;

    if (cmIntegrationByPath && auth.failed) {
        return <ReloadMessage/>;
    }

    if (auth.failed && !auth.needLogin) {
        return <Oops error={auth.error}/>
    }

    if (!auth.data) {
        return <MainLoader/>;
    }
    return (isCmIntegration
            ? <AuthenticatedRoute {...props}/>
            : <Switch>
                <Route path="/cm" render={() => <AuthenticatedRoute {...props}/>}/>
                <Route path="/" render={() => <>
                    <Intercom/>
                    <AuthenticatedRoute {...props}/>
                </>
                }/>
              </Switch>
    );
};

const mapStateToProps = (state, ownProps) => {
    return {
        auth: state.global.auth || {succeeded: false, inProgress: true, failed: false},
        _accounts: state.global._accounts,
        _accountInfo: state.global._accountInfo,
        _ui: {...state.syncReducer._ui, ...state.global._ui},
        _routing: getPropFromState(state, "_routing", {}),
        ...ownProps

    }
};

export default withRouter(connect(mapStateToProps, {updateUrlParams, setActiveAccount, logout})(App));
