import React, {useCallback, useState} from "react";
import InstanceDetailsAzureForm from './InstanceDetailsAzureForm';
import {v4 as uuidv4} from 'uuid';
import {FM_AGENT_TYPE_AZURE} from '../../consts';
import * as azureService from '../../services/azure.api.service';
import {extractErrorAzure} from "../../utils/providers-utils";

export const RESOURCE_GROUP_OPTIONS = ["Generate a new group", "Use an existing group"];

const InstanceDetailsAzure = ({addDeployment, dataBroker, setStepError, onFinish}) => {
    const {id, name, fileLink} = dataBroker;

    const [resourceGroupName, setResourceGroupName] = useState(generateResourceGroupName());

    const handleError = useCallback((error) => setStepError(extractErrorAzure(error)), [setStepError]);

    const handleDeployClick = useCallback(async (values) => {
        try {
            const deployDetails = getDeployDetails(values, resourceGroupName, fileLink);

            await prepareForDeploy(deployDetails);

            await azureService.azureTemplateDeployment(deployDetails);

            addDeployment({
                dataBrokerId: id,
                dataBrokerName: name,
                providerType: FM_AGENT_TYPE_AZURE.name,
                deployDetails
            });

            onFinish();
        }

        catch (err) {
            handleError(err);
            setResourceGroupName(generateResourceGroupName());
        }

    }, [addDeployment, id, name, resourceGroupName, fileLink, handleError, onFinish]);

    return <InstanceDetailsAzureForm handleDeployClick={handleDeployClick} dataBrokerName={name} handleError={handleError}/>;
};

export default InstanceDetailsAzure;

const generateResourceGroupName = () => {
    return `CloudSync-rg-${uuidv4()}`;
};

const prepareForDeploy = async (deployDetails) => {
    if (!deployDetails) return;
    if (!deployDetails.isExistingResourceGroup) {
        await azureService.createResourceGroup(deployDetails.subscriptionId, deployDetails.resourceGroupName, deployDetails.location);
    }
    return await azureService.azureTemplateDeploymentValidation(deployDetails);
};

const getDeployDetails = (values, resourceGroupName, fileLink) => {
    const vnetArr = values.selectedVnet.value.split('/');
    const vnetResourceGroupName = vnetArr[vnetArr.indexOf('resourceGroups') + 1]; // Grab the vnet resource group name

    return {
        subscriptionId: values.selectedSubscription.value,
        resourceGroupName: values.resourceGroupOption === RESOURCE_GROUP_OPTIONS[1] ? values.selectedResourceGroupExisting.value : resourceGroupName,
        isExistingResourceGroup: values.resourceGroupOption === RESOURCE_GROUP_OPTIONS[1],
        deploymentName: `${values.virtualMachineName}-deployment`,
        location: values.selectedRegion.value,
        properties: {
            templateLink: {
                uri: fileLink
            },
            parameters: {
                virtualMachineName: {
                    value: values.virtualMachineName
                },
                virtualNetworkName: {
                    value: values.selectedVnet.label
                },
                subnetName: {
                    value: values.selectedSubnet.value
                },
                adminPublicKey: {
                    value: values.sshKey || ''
                },
                vnetResourceGroupName: {
                    value: vnetResourceGroupName
                },
                adminUsername: {
                    value: values.adminUserName
                },
                authType: {
                    value: values.authMethod === "Password" ? "password" : "ssh"
                },
                adminPassword: {
                    value: values.adminPassword || ''
                },
                enablePublicIP: {
                    value: values.publicIp.value
                }
            },
            mode: 'Incremental'
        }
    }
};
