import React from 'react';
import {
    azureStorageAccounts,
    serverIPStep,
    serverCredentialsStep,
    clearUrlParamsIfChanged,
    clearShowAction,
    updateUrlParams,
    wizardStepStarted,
    wizardStepExited
} from '../../store/wizard/action_creators';
import {connect} from 'react-redux';
import stepinize from "./../wizard/stepinize";
import {Field, reduxForm} from 'redux-form';
import {InputField} from '../_common_/forms/forms.jsx';
import Validator from 'validatorjs';
import {Route, Switch} from 'react-router-dom';
import ServerStepTemplate from "./ServerStepTemplate";
import {
    END_POINT_NAMES_MAP,
    KEYS_REG_EXP,
    SELECTED_PARAMS_AZURE_STORAGE_ACCOUNT,
    SELECTED_PARAMS_TARGET_AZURE_STORAGE_ACCOUNT
} from '../../consts';
import {getCredentialsByType} from '../../utils/sync-utils';
import {getPropFromState, getWizardParamsFromUrlParams} from '../../utils/mapStateToPropsUtils';

const rules = (step) => {
    if (step === "credentials") {
        return {
            serverName: ['required', 'regex:/^[a-z0-9]+$/', 'max:24', 'min:3'],
            connectionString: ['required', 'regex:'+KEYS_REG_EXP]
        }
    }
    if (step === 'ip') {
        return {
            serverName: ['required', 'regex:/^[a-z0-9]+$/', 'max:24', 'min:3'],
        }
    }
};

const shouldValidate = ({values, nextProps, props, initialRender, structure,}) => {
    const {_ui: {azure}} = props;
    const step = azure ? azure.step : "ip";
    return (
        initialRender ||
        !structure.deepEqual(values, nextProps.values) ||
        step === "credentials"
    );
};

const validate = (values, props) => {
    const {_ui: {azure}} = props;
    const step = azure ? azure.step : "ip";
    const validator = new Validator(values, rules(step), {
        "regex.serverName": "Storage Account Name can contain only lowercase letters and numbers",
        "max.serverName": "Azure Storage Account Name must be no more than 24 characters",
        "min.serverName": "Azure Storage Account Name must be no less than 3 characters",
    });
    validator.setAttributeNames({serverName: 'Azure Storage Account Name', connectionString: "Connection String"});
    validator.passes();
    return validator.errors.all();
};

const CredentialsUi = ({isSourceStep}) => (
    <div className="row-medium credentials-container azure no-left-padding">
        <div className="medium-12 columns">
            <Field name="connectionString"
                   component={InputField}
                   autoFocus
                   label={isSourceStep ? "Define a Storage Account SAS (Shared Access Signature) Connection String" : "Define a Storage Account Connection String:"}
                   type="text">

            </Field>
        </div>
    </div>
);

const AzureBlobStep = (props) => {
    const {_azureStorageAccounts, isTargetStepFromSameSourceType, _wizard, _ui: {azure}} = props;
    const {isSourceStep, selectedWizardParams: {selectedStorageAccountName, selectedStorageAccountNameTarget}} = _wizard;
    const selectedServer = isTargetStepFromSameSourceType ? selectedStorageAccountNameTarget : selectedStorageAccountName;
    const stepOptions = {
        storage: _azureStorageAccounts,
        server: selectedServer,
        serverSVG: "azure-default.svg",
        step: azure ? azure.step : "ip",
        serverType: "Azure",
        serverTitle: "Azure",
        selectType: "Storage Account",
        serverPlaceholder: "",
        title: `Select an ${END_POINT_NAMES_MAP.AZURE} ${!isSourceStep || isTargetStepFromSameSourceType ? "Target" : "Source"}`,
        clearURLOptions: [{
            changedParam: isTargetStepFromSameSourceType ? SELECTED_PARAMS_TARGET_AZURE_STORAGE_ACCOUNT : SELECTED_PARAMS_AZURE_STORAGE_ACCOUNT,
            newValueFromState : "selectedIp",
            urlParams: isTargetStepFromSameSourceType ? ["azureBlobContainerPathTarget"] : ["azureBlobContainerPath"]
        }],
        clearShowOptions: {keyToClear: ['_showAzureBlobContainers', selectedServer]},
        validate,
        getCredentialsUi: () => <CredentialsUi isSourceStep={isSourceStep}/>
    };

    return <ServerStepTemplate stepOptions={stepOptions} {...props}/>;
};

const stepId = ({isTargetStepFromSameSourceType}) => {
    return isTargetStepFromSameSourceType ? 'targetAzureBlob' : 'azureBlob';
};

const willMount = (props) => {
    if (!props._azureStorageAccounts || !props._azureStorageAccounts.succeeded) return props.azureStorageAccounts();
};
const isFetching = ({_azureStorageAccounts}) => _azureStorageAccounts.inProgress;
const canShowNextStep = ({_wizard, isTargetStepFromSameSourceType}) => {
    if (isTargetStepFromSameSourceType)
        return _wizard.selectedWizardParams.selectedStorageAccountNameTarget && getCredentialsByType("azure", _wizard.selectedWizardParams.selectedStorageAccountNameTarget, isTargetStepFromSameSourceType);
    return _wizard.selectedWizardParams.selectedStorageAccountName && getCredentialsByType("azure", _wizard.selectedWizardParams.selectedStorageAccountName, isTargetStepFromSameSourceType);
};

const nextRoutes = ({match, isTargetStepFromSameSourceType, nextStep, NextStepComp}) => {
    return (
        <Switch>
            {isTargetStepFromSameSourceType &&
                <Route path={`${match.url}/:storageAccountNameTarget/${nextStep.path}`} render={(props) =>
                    <NextStepComp {...props} isTargetStepFromSameSourceType={nextStep.isTargetStepFromSameSourceType}/>
                }/> }
            {!isTargetStepFromSameSourceType &&
                <Route path={`${match.url}/:storageAccountName/${nextStep.path}`} render={(props) =>
                    <NextStepComp {...props} isTargetStepFromSameSourceType={nextStep.isTargetStepFromSameSourceType}/>
                }/>}
        </Switch>
    )
};

const AzureBloblStepForm = reduxForm({
    form: 'azureBlob',
    validate,
    shouldValidate
})(AzureBlobStep);

const mapStateToProps = (state, ownProps) => {
    return {
        _wizard: getWizardParamsFromUrlParams(state, ownProps),
        _azureStorageAccounts: getPropFromState(state, "_azureStorageAccounts", {}),
        _relationships: getPropFromState(state, "_relationships", {}),
        _ui: {...state.syncReducer._ui, ...state.global._ui},
        routing: state.router,
        ...ownProps
    }
};

const mapDispatchToProps = {
    azureStorageAccounts,
    serverIPStep,
    serverCredentialsStep,
    clearUrlParamsIfChanged,
    clearShowAction,
    updateUrlParams,
    wizardStepStarted,
    wizardStepExited
};

export default connect(mapStateToProps, mapDispatchToProps)(stepinize(AzureBloblStepForm, {
    stepId,
    willMount,
    isFetching,
    canShowNextStep,
    nextRoutes,
    loading: () => {return {text: END_POINT_NAMES_MAP.AZURE, icon: require("../../assets/svgs/azure-default.svg")}}
}));