import DataBrokerCard from "./DataBrokerCard";
import React, {useState, useEffect, useCallback} from 'react';
import CardsContainer from "./../cards/CardsContainer";
import DeleteDataBrokerDialog from "./DeleteDataBrokerDialog";
import ReactPopover from "../_common_/reactPopover/ReactPopover"
import Isvg from 'react-inlinesvg';
import {openDialog} from "./../dialog/Dialog"
import HowDoesItWorkButton from "./../systemFlow/HowDoesItWorkButton";
import {
    FM_AGENT,
    SELECTED_PARAMS_DATA_BROKER_ID,
    SELECTED_PARAMS_DATA_BROKER_ID_TARGET,
    BROKER_STATUS_COMPLETE,
    FM_AGENT_TYPE_AWS,
    CLOUD_FORMATION_LINK
} from '../../consts';
import classNames from "classnames";
import {calculateNextStepObj} from '../../utils/sync-utils';
import "./dataBrokerStep.scss";
import useRunOnce from '../../hooks/useRunOnce';
import ButtonWithNotification from '../widgets/ButtonWithNotification';

const DataBrokers = props => {

    const [selectedId, setSelectedId] = useState(null);
    const [showError, setShowError] = useState(false);

    const {_wizard, data, isTargetStepFromSameSourceType, history, startDataBrokerWizard, shouldDisplayPortBool, isAwsAccessRequired, isGcpAccessRequired, up} = props;
    const {selectedWizardParams: {selectedDataBroker, selectedDataBrokerTarget}, wizardTypeForSystemFlow, isSecure} = _wizard;
    const {clearUrlParamsIfChanged} = props;
    const theRelevantDataBroker = isTargetStepFromSameSourceType ? selectedDataBrokerTarget : selectedDataBroker;

    useRunOnce(() => {
        if (theRelevantDataBroker) {
            setSelectedDataBroker(theRelevantDataBroker.id);
            return;
        }
        if (data && data.length === 1 && data[0].status === BROKER_STATUS_COMPLETE && !data[0].isDisabled) {
            setSelectedDataBroker(data[0].id)
        }
    });

    const setSelectedDataBroker = useCallback((id) => {
        if (selectedId !== id) {
            setSelectedId(id);
            setShowError(false)
        }
    }, [selectedId]);

    useEffect(() => {
        if (selectedId) return;
        if (data && data.length === 1 && data[0].status === 'COMPLETE' && !data[0].isDisabled) {
            setSelectedDataBroker(data[0].id)
        }
    }, [data, selectedId, setSelectedDataBroker]);

    const handleContinue = useCallback(() => {
        if (!selectedId)
            setShowError(true);
        else {
            //clearing everything if data broker changes
            const selectedParamName = isTargetStepFromSameSourceType ? SELECTED_PARAMS_DATA_BROKER_ID_TARGET : SELECTED_PARAMS_DATA_BROKER_ID;
            clearUrlParamsIfChanged(selectedParamName, selectedId,
                [
                    "exportPath", "rootExport", "exportPathTarget", "rootExportTarget",
                    "sharePath", "rootShare", "sharePathTarget", "rootShareTarget",
                    "bucketPath", "s3BasedBucketPath", "s3BasedBucketPathTarget", "gcpBucketPath", "gcpBucketPathTarget",
                    "azureBlobContainerPath", "azureBlobContainerPathTarget"
                ]);
            setShowError(false);
            const nextStep = calculateNextStepObj(_wizard, _wizard.currentStep);
            _wizard.nextStep(`/${selectedId}/${nextStep.path}`, history);
        }
    }, [selectedId, _wizard, clearUrlParamsIfChanged, history, isTargetStepFromSameSourceType]);

    const error = showError ? 'In order to continue to the next step you should select an active Data Broker Instance' : null;
    const disableButton = data.length === 1 && data[0].status === 'PROGRESS' ? 'disabled' : '';
    const selectionOptions = {selector: setSelectedDataBroker, selectedId};
    const containerOptions = {singularTitle: FM_AGENT, pluralTitle: FM_AGENT + 's'};
    const additionalData = {shouldDisplayPort: shouldDisplayPortBool, isAwsAccessRequired, isGcpAccessRequired};

    return (<div className="wizard-step data-mover-step">
        <div className={classNames("scrollable-area", {up, down: !up})}>
            <div className="title-and-actions">
                <div className="actions-left">
                    <HowDoesItWorkButton wizardType={wizardTypeForSystemFlow} isSecure={isSecure}/>
                </div>
                <div className="title-middle">
                    <h3>Select a {FM_AGENT}</h3>
                </div>
                <div className="actions-right">
                    <ReactPopover containerClassName="top-right" trigger="mouseover" popoverClass="hover-tooltip"
                                  container={<button className="button small" style={{fontSize: '28px'}}
                                                     onClick={startDataBrokerWizard}>+</button>}>
                        Add a {FM_AGENT}
                    </ReactPopover>
                </div>
            </div>


            {data && <CardsContainer containerOptions={containerOptions} selectionOptions={selectionOptions}
                                     className="data-brokers"
                                     Card={DataBrokerCard} data={data}
                                     additionalData={additionalData} wizardType={wizardTypeForSystemFlow}
                                     menuItems={dataBrokersMenu}
            />}
        </div>

        <ButtonWithNotification
                    onClick={handleContinue}
                    disable={disableButton}
                    error={error}
                    clearError={() => setShowError(false)}
                    text="Continue"/>
    </div>
    )
};

export default DataBrokers;

const dataBrokersMenu = (element, triggerMenu) => {
    const {type, isWaiting} = element;
    const menuOnlyDelete = [
        <button onClick={(el) => {
            openDialog(<DeleteDataBrokerDialog className="default-dialog" data={element}/>, el.target);
            triggerMenu()
        }} key="delete">
            <span><Isvg className="garbage-bin" src={require('./../../assets/svgs/garbage-bin-icon.svg')}/></span>Remove
        </button>
    ];

    return (type === FM_AGENT_TYPE_AWS.name && isWaiting)
        ? menuOnlyDelete.concat(<button key="link" className="external-link">
            <a href={CLOUD_FORMATION_LINK} target="_blank" rel="noopener noreferrer">
                <div className="external-link-icon"><Isvg src={require('./../../assets/svgs/external-link-icon.svg')}/></div>
                <div>View Cloud Formation</div>
            </a>
        </button>)
        : menuOnlyDelete;
};
