import React, {useState, useEffect, useMemo} from 'react';
import Isvg from 'react-inlinesvg';
import classNames from "classnames";
import {DropTarget} from 'react-dnd';
import { DraggableTypes} from "../../consts";
import _ from "lodash";
import ReactPopover from '../_common_/reactPopover/ReactPopover';
import SmallLoader from '../_common_/SmallLoader/SmallLoader';
import "./dropEndPoint.scss";

const dragEndPointTarget = {
    drop(props) {
        const {title} = props;
        return {
            dropEndPoint: title
        }
    },
    canDrop(props) {
        //allow dropping only when there is no end point yet
        return !props.endPoint;
    }
};

function collect(connect, monitor) {
    return {
        connectDropTarget: connect.dropTarget(),
        isOver: monitor.isOver()
    };
}


const DropEndPoint = ({title, className, connectDropTarget, isOver, endPoint, handleEndPointSelection, oppositeEndPoint, setEndPoint, isSource, showLoaderCondition}) => {
    const [showSelector, setShowSelector] = useState(false);

    const selectedProtocol = endPoint?.protocolOptions && endPoint.protocol
        ? (endPoint.protocol === "cifs"
            ? "SMB"
            : endPoint.protocol.toUpperCase())
        : null;

    //decide if need to show the ProtocolSelector
    useEffect(() => {
        setShowSelector(endPoint?.protocolOptions && !endPoint.protocol);
    }, [endPoint]);

    //decide if can edit protocol
    const allowEdit = useMemo(()=>{
        let isEditable = endPoint?.protocolOptions && endPoint.protocol;
        //in case the opposite endpoint has a protocol that defines the endpoint protocol (nfs or cifs)
        if (isEditable && _.includes(endPoint.protocolOptions, oppositeEndPoint?.protocol)) {
            isEditable = false;
        }
        return isEditable;
    }, [endPoint, oppositeEndPoint]);

    const unselectEndPoint = () => {
        handleEndPointSelection(null);
    };

    const editBlockedText = `You can’t change the protocol because a different protocol isn’t supported with the ${isSource ? "target" : "source"} that you selected. 
    To change the protocol, remove the ${isSource ? "target" : "source"}.`;

    return connectDropTarget(
        <div className={classNames("add-end-point", className)}>

            <div className="add-box">
                <div className="top-area">
                    {endPoint && <Isvg className="unselect-icon" onClick={unselectEndPoint} src={require("./../../assets/svgs/x-delete-icon.svg")}/>}
                </div>

                {!endPoint &&
                <>
                    {!isOver && <Isvg className="add-box-svg" src={require('./../../assets/svgs/add-end-point-icon.svg')}/>}
                    {isOver && <Isvg className="add-box-svg" src={require('./../../assets/svgs/add-end-point-icon-hover.svg')}/>}
                    <div className={classNames("text", {"is-hover" : isOver})}>{title}</div>
                </>}

                {endPoint &&
                <>
                    <Isvg className="add-box-svg" src={endPoint.dropIcon}/>
                    <div className="selected-name">
                        <div className="text">{endPoint.title}{endPoint.protocol && endPoint.protocolOptions ? ` (${selectedProtocol})` : ''}</div>
                        {endPoint.protocolOptions &&
                        (allowEdit
                            ? <span className="pencil-icon"
                                  onClick={(e) => {
                                      setShowSelector(true);
                                      e.stopPropagation();
                                  }}>
                                <Isvg src={require('./../../assets/svgs/pencil-icon.svg')}/>
                            </span>
                            : <ReactPopover trigger="mouseover" container={<span className="pencil-icon disabled">
                                <Isvg src={require('./../../assets/svgs/pencil-icon.svg')}/>
                            </span>}>
                                {editBlockedText}
                            </ReactPopover>)}
                    </div>
                    <div>{endPoint.workingEnvironmentName}</div>
                </>}

            </div>

            {endPoint && showSelector &&
            <ProtocolSelector
                providerIcon={endPoint.protocolSelectorIcon}
                options={endPoint.protocolOptions}
                setProtocol={protocol => setEndPoint({...endPoint, protocol})}/>}

            {endPoint && showLoaderCondition(endPoint) &&
            <div className="loader">
                <SmallLoader/>
            </div>}
        </div>
    )
};

export default DropTarget(DraggableTypes.END_POINT, dragEndPointTarget, collect)(DropEndPoint);

const ProtocolSelector = ({setProtocol, providerIcon, options}) => {
    return <div className="protocol-selector">
        <div className="mask"/>
        <div className="protocol-selector-content">
            <Isvg src={providerIcon}/>
            <div className="select-text">Select Volume Protocol</div>
            <div className="protocol-options">
                {options.map((option, idx) =>
                    <button key={`option-${idx}`} className="outline" onClick={()=>{setProtocol(option)}}>
                        {option === "cifs" ? "SMB" : option.toUpperCase()}
                    </button>)}
            </div>
        </div>
    </div>
};