import React, {useEffect, useMemo} from 'react';
import stepinize from '../wizard/stepinize';
import {
    clearSync,
    clearWizardCache,
    createSync,
    updateUrlParams,
    wizardStepStarted,
    wizardStepExited
} from '../../store/wizard/action_creators';
import {connect} from 'react-redux';
import ReviewEndpointCard from './ReviewEndpointCard';
import ReviewDataBrokerCard from './ReviewDataBrokerCard';
import {buildSyncObject, getSourceAndTargetInfo} from '../../utils/sync-utils';
import * as actionCreatorsConsts from '../../store/global/action_creators_consts';
import _ from 'lodash';
import {isProtocolIncludedWithoutProvider} from '../../utils/reducerUtils';
import Isvg from 'react-inlinesvg';
import classNames from 'classnames';
import './relationshipStep.scss';
import LoadingBar from '../_common_/loadingBar/LoadingBar';
import {FlowEncrypted} from '../systemFlow/SystemFlow';
import useRunOnceWhenTruthy from '../../hooks/useRunOnceWhenTruthy';
import ButtonWithNotification from '../widgets/ButtonWithNotification';
import {getPropFromState, getWizardParamsFromUrlParams} from '../../utils/mapStateToPropsUtils';

const REVIEW = 'REVIEW';
const CREATING = 'CREATING';
const CREATED = 'CREATED';
const PENDING = 'PENDING';
const FAILED = 'FAILED';
const FAILED_TO_START = 'FAILED_TO_START';

const getInfoPerStatus = (status, sync) => {
        switch(status) {
            case CREATING:
                return {
                    headerText: 'Creating Relationship',
                    buttonText: 'View in Dashboard',
                    additionalText: `${sync?.percentCopied || 0}%`,
                    additionalTextClass: 'percentage'
                };
            case CREATED:
                return {
                    headerText: 'Relationship Successfully Created',
                    buttonText: 'View in Dashboard',
                    additionalText: '100%',
                    additionalTextClass: 'percentage'
                };
            case PENDING:
                return {
                    headerText: 'Relationship Successfully Created',
                    buttonText: 'View in Dashboard',
                    additionalText: sync ? `Sync will start at ${sync.startTime}` : "",
                    additionalTextClass: 'pending'
                };
            case FAILED:
                return {
                    headerText: 'Creating Relationship',
                    buttonText: 'View in Dashboard',
                    additionalText: 'Sync Failed',
                    additionalTextClass: 'failed'
                };
            case FAILED_TO_START:
                return {
                    headerText: 'Review Sync Relationship',
                    buttonText: 'Create Relationship',
                    additionalText: 'Sync Failed',
                    additionalTextClass: 'failed'
                };
            case REVIEW:
            default:
                return {
                    headerText: "Review Sync Relationship",
                    buttonText: 'Create Relationship',
                    additionalText: '',
                    additionalTextClass: ''
                };
        }
};

const RelationshipStep = props => {
    const {clearSync, clearWizardCache, _routing: {location: {query, pathname}}, up, _wizard} = props;
    const {currentStep, wizardInfo, wizardTypeForSystemFlow, sync, selectedWizardParams: {
                selectedBucketPath, selectedBucketPathTarget, selectedCIFS, selectedCIFSTarget, selectedS3Based,
                selectedS3BasedTarget, selectedStorageAccountName, selectedStorageAccountNameTarget,
                selectedDataBroker, selectedDataBrokerTarget, selectedIsTargetDataBrokerInitiator}
    } = _wizard;
    const {succeeded, isPending} = sync;
    const isTargetOfSameSource = wizardInfo.source.protocol === wizardInfo.target.protocol;
    const {sourceInfo, targetInfo} = getSourceAndTargetInfo(props);
    const submitting = sync && sync.inProgress;

    const status = useMemo(() => {
        if (sync?.completed) return CREATED;
        if (sync?.isPending) return PENDING;
        if (sync?.failed && sync?.succeeded) return FAILED;
        if (sync?.failed) return FAILED_TO_START;
        if (sync?.succeeded || query.inProgress) return CREATING;
        return REVIEW; //this is the initial status
    }, [sync, query]);

    const syncStarted = sync?.succeeded;

    const {headerText, additionalText, additionalTextClass, buttonText} = getInfoPerStatus(status, sync);

    const gotoDashboard = () => {
        actionCreatorsConsts.routesContainer.routes.push('/dashboard');
        clearWizardCache(!_.isEmpty(selectedCIFS || selectedCIFSTarget), !_.isEmpty(selectedS3Based || selectedS3BasedTarget), !_.isEmpty(selectedBucketPath || selectedBucketPathTarget), !_.isEmpty(selectedStorageAccountName || selectedStorageAccountNameTarget));
    };

    const handleClick = () => {
        if (status === REVIEW || status === FAILED_TO_START) {
            const {_wizard, _ui, createSync} = props;
            createSync(buildSyncObject({_wizard, _ui}), !!_ui.isCmIntegration);
        } else {
            gotoDashboard();
        }
    };

    useRunOnceWhenTruthy(() => {
        clearSync();
        if (query.inProgress) {
            gotoDashboard();
        }
    }, true);

    useEffect(() => {
        if (succeeded && !isPending && !query.inProgress) {
            actionCreatorsConsts.routesContainer.routes.replace(`${pathname}?inProgress=true`);
        }
    }, [succeeded, isPending, query, pathname]);

    useEffect(() => {
        if (isProtocolIncludedWithoutProvider('s3', wizardInfo) && currentStep === 'review' && _.isNull(isTargetOfSameSource ? selectedBucketPath && selectedBucketPathTarget : selectedBucketPath)) {
            //if s3 bucket was added manually and wizard was refreshed
            const separator = _.isNull(selectedBucketPath) ? 'buckets' : 'targetbuckets';
            const redirectToBuckets = pathname.split(separator)[0];
            const reducerUrl = redirectToBuckets ? redirectToBuckets + separator : '/dashboard';
            actionCreatorsConsts.routesContainer.routes.push(reducerUrl);
        }
    }, [isTargetOfSameSource, selectedBucketPath, selectedBucketPathTarget, pathname, currentStep, wizardInfo]);

    return (<div className={classNames('wizard-step relationship-step', {'sync-started': syncStarted})}>
        <div className={classNames("scrollable-area", {up, down: !up})}>
            <h3 className="review-title">{headerText}</h3>
            {selectedDataBrokerTarget &&
            <div className="secure-nfs-relationship">
                <div className="diagram">
                    <FlowEncrypted wizardType={wizardTypeForSystemFlow}/>
                </div>
                <div className="databrokers-function-pair">
                    <span
                        className="databroker-function source"><span>{(selectedIsTargetDataBrokerInitiator === 'true') ? '(Listener)' : '(Initiator)'}</span></span>
                    <span
                        className="databroker-function target"><span>{(selectedIsTargetDataBrokerInitiator === 'true') ? '(Initiator)' : '(Listener)'}</span></span>
                </div>
                <div className="secure-relationship-description-pair">
                        <span className="secure-relationship-description source">
                            <div>
                                <div><span
                                    className="field-name">{`${sourceInfo.title} :`}</span>{sourceInfo.description}</div>
                                <div><span
                                    className="field-name">NFS Directory: </span>{`${sourceInfo.description} ${sourceInfo.subtitleDescription}`}</div>
                                <div><span
                                    className="field-name">Data Broker: </span>{`${selectedDataBroker.ui.type.displayName} | ${selectedDataBroker.name}`}</div>
                                {selectedDataBroker.port && <div><span
                                    className="field-name">Data Broker Port: </span>{`${selectedDataBroker.port}`}
                                </div>}
                            </div>
                        </span>
                    <span className="secure-relationship-description target">
                        <div>
                            <div><span
                                className="field-name">{`${targetInfo.title} :`}</span>{targetInfo.description}</div>
                            <div><span
                                className="field-name">NFS Directory: </span>{`${targetInfo.description} ${targetInfo.subtitleDescription}`}</div>
                            <div><span
                                className="field-name">Data Broker: </span>{`${selectedDataBrokerTarget.ui.type.displayName} | ${selectedDataBrokerTarget.name}`}</div>
                            {selectedDataBrokerTarget.port && <div><span className="field-name">Data Broker Port: </span>{`${selectedDataBrokerTarget.port}`}
                            </div>}
                        </div>
                    </span>
                </div>
            </div>}

            {!selectedDataBrokerTarget &&
            <div className="cards">
                <div className="end-points-area">
                    <ReviewEndpointCard protocol={wizardInfo.source.protocol} provider={wizardInfo.source.provider}
                                title={sourceInfo.title}
                                description={sourceInfo.description} subtitle={sourceInfo.subtitle}
                                subtitleDescription={sourceInfo.subtitleDescription}/>
                    <ReviewEndpointCard protocol={wizardInfo.target.protocol} provider={wizardInfo.target.provider}
                                title={targetInfo.title}
                                description={targetInfo.description} subtitle={targetInfo.subtitle}
                                subtitleDescription={targetInfo.subtitleDescription}/>
                </div>
                <div className="data-broker-area">
                    <div className="data-broker-container">
                        <Isvg className="left-arrow" src={require('./../../assets/svgs/corner-square-arrow-left.svg')}/>
                        <div className="data-broker-card-position">
                            <ReviewDataBrokerCard selectedDataBroker={selectedDataBroker}/>
                        </div>
                        <Isvg className="right-arrow" src={require('./../../assets/svgs/corner-square-arrow-right.svg')}/>
                    </div>
                </div>
            </div>}

            <div className={`additional-text ${additionalTextClass}`}>{additionalText}</div>

        </div>
        <ButtonWithNotification onClick={handleClick} disabled={submitting} submitting={submitting} text={buttonText}>
            {status === CREATING && <LoadingBar/>}
        </ButtonWithNotification>

    </div>);
};

const mapStateToProps = (state, ownProps) => {
    return {
        _wizard: getWizardParamsFromUrlParams(state, ownProps),
        routing: state.router,
        _routing:  getPropFromState(state, "_routing", {}),
        _ui: {...state.syncReducer._ui, ...state.global._ui},
        ...ownProps
    }
};

const mapDispatchToProps = {
    clearSync,
    clearWizardCache,
    createSync,
    updateUrlParams,
    wizardStepStarted,
    wizardStepExited
};

export default connect(mapStateToProps, mapDispatchToProps)(stepinize(RelationshipStep, {stepId: () => 'review'}));
