import React, {Component} from 'react';
import {reduxForm, Field, FieldArray, initialize} from 'redux-form';
import {InputField} from '../_common_/forms/forms.jsx';
import ButtonWithNotification from "../widgets/ButtonWithNotification";
import {MAX_S3_TAGS} from '../../consts';
import Isvg from 'react-inlinesvg';
import classNames from "classnames";
import _ from "lodash";
import HowDoesItWorkButton from '../systemFlow/HowDoesItWorkButton';
import {getObjectStorageTargetTypeFromWizardInfo} from '../../utils/sync-utils';

const validationMap = {
    s3_metadata: {
        maxKeyLength: 128,
        maxValueLength: 256,
        keyValidationRegex: /[^\d\w+-=._:/@]/,
        keyValidationRegexErr: `Amazon metadata names may contain letters, numbers, or the following special characters: + - . _ : / @`,
        valueValidationRegex: /[^\x00-\x7F]+/, // eslint-disable-line
        valueValidationRegexErr: `Amazon metadata values may contain only ASCII characters`,
    },
    s3: {
        maxKeyLength: 128,
        maxValueLength: 256,
        keyValidationRegex: /^(aws:)|[^\d\s\w+-=._:/@]/,
        keyValidationRegexErr: `Amazon tag keys may not start with "aws:" and may contain letters, numbers, spaces, or the following special characters: + - . _ : / @`,
        valueValidationRegex: /[^\d\s\w+-=._:/@]/,
        valueValidationRegexErr: `Amazon tag values may contain letters, numbers, spaces, or the following special characters + - . _ : / @`,
    },
    azure: {
        maxKeyLength: 512,
        maxValueLength: 256,
        keyValidationRegex: /^[^a-zA-Z_]|[^\w]/,
        keyValidationRegexErr: `Azure metadata keys may not start with a number, and may contain letters, numbers or the special character _`,
        valueValidationRegex: /[^\x00-\x7F]+/, // eslint-disable-line
        valueValidationRegexErr: `Azure metadata values may contain only ASCII characters`
    }
}

const validationFunc = (tag, targetProtocol, objectTaggingDisabled) => {
    const validationKey = targetProtocol + (objectTaggingDisabled ? '_metadata' : '');
    const {
        maxKeyLength,
        maxValueLength,
        keyValidationRegex,
        keyValidationRegexErr,
        valueValidationRegex,
        valueValidationRegexErr
    } = validationMap[validationKey];
    let errors = {};

    if (!_.isEmpty(tag)) {
        if (!_.isEmpty(tag.key) && _.isEmpty(tag.value))
            errors.value = "Tag Value is required for a tag with Key";
        if (_.isEmpty(tag.key) && !_.isEmpty(tag.value))
            errors.key = "Tag Key is required for a tag with Value";
        if (tag.key && tag.key.length > maxKeyLength - 1)
            errors.key = `Tag Key must be no longer than ${maxKeyLength} characters`;
        if (tag.value && tag.value.length > maxValueLength - 1)
            errors.value = `Tag Value must be no longer than ${maxValueLength} characters`;
        if (tag.value && valueValidationRegex.test(tag.value)) {
            errors.value = valueValidationRegexErr;
        }
        if (tag.key && keyValidationRegex.test(tag.key)) {
            errors.key = keyValidationRegexErr;
        }
    }
    return errors;
};

const validate = (values, {targetProtocol, objectTaggingDisabled}) => {
    let errors = {};
    if (values.tags)
        errors.tags = values.tags.map((tag) => validationFunc(tag, targetProtocol, objectTaggingDisabled));
    //check if all the tags are valid, hence no error for any tag
    const errorsEmpty = _.find(errors.tags, (obj) => !_.isEmpty(obj)) === undefined;
    return errorsEmpty ? {} : errors;
};

const renderTags = ({fields, targetProtocol}) => {
    const {maxKeyLength, maxValueLength} = validationMap[targetProtocol];
    return (
        <div>
            {fields.map((tag, index) =>
                <div key={index} className="row tag">
                    <div className="6-small columns key">
                        <Field component={InputField} label="Tag Key" name={`${tag}.key`}
                               placeholder={`Up to ${maxKeyLength} characters`}/>
                    </div>
                    <div className="6-small columns value">
                        <Field component={InputField} label="Tag Value" name={`${tag}.value`}
                               placeholder={`Up to ${maxValueLength} characters`}/>
                    </div>
                    {index !== 0 && <div className="close" onClick={() => fields.remove(index)}><span><Isvg
                        src={require("./../../assets/svgs/x.svg")}/></span></div>}
                </div>
            )}
            <div className="row add-tag">
                {fields.length < MAX_S3_TAGS && <div>
                    <div className="circle" onClick={() => fields.push()}>+</div>
                    <div>Add Relationship Tag</div>
                </div>}
                <div className="info">Optional Field | [Up to {MAX_S3_TAGS} tags]</div>
            </div>
        </div>
    )
};


class RelationshipTags extends Component {
    componentDidMount() {
        const {initialTags, dispatch} = this.props;
        if (initialTags) {
            dispatch(initialize("relationshipTags", {tags: initialTags}, ['tags[].key', 'tags[].value']));
        }
    }

    submit(complete, values) {
        return Promise.resolve(true)
            .then(() => {
                complete(values);
            });
    }

    render() {
        const {complete, handleSubmit, targetProtocol = 's3'} = this.props;
        const { wizardTypeForSystemFlow, isSecure, wizardInfo, up, objectTaggingDisabled} = this.props;

        const error = "";
        const target = getObjectStorageTargetTypeFromWizardInfo(wizardInfo);

        return (<form className="wizard-step-form" onSubmit={handleSubmit(this.submit.bind(this, complete))}>
            <div className={classNames("scrollable-area", {up, down: !up})}>
                <div className="title-and-actions">
                    <div className="actions-left">
                        <HowDoesItWorkButton wizardType={wizardTypeForSystemFlow} isSecure={isSecure}/>
                    </div>
                    <div className="title-middle">
                        <h3>Relationship Tags</h3>
                    </div>
                    <div className="actions-right"/>
                </div>
                <div className="tags-content">
                    <div className="row-medium">
                        <p>Cloud Sync assigns the relationship tags to all of the files transferred to the {target}.<br/>
                            This enables you to search for the transferred files by using the tag values.</p>
                        {objectTaggingDisabled &&
                        <p className="note">Note: Because you disabled tagging of S3 objects in the relationship’s settings, Cloud Sync stores the key-value pairs in the metadata of the object and not in the tags.</p>}
                    </div>
                </div>
                <FieldArray name="tags" component={renderTags} targetProtocol={targetProtocol}/>
            </div>
                <ButtonWithNotification type="submit" error={error} text="Continue"/>
            </form>
        )
    }
}

export const RelationshipTagsForm = reduxForm({
    form: 'relationshipTags',
    validate
})(RelationshipTags);
