import React from 'react';
import {
    serverCredentialsStep,
    getServersByType,
    serverIPStep,
    clearUrlParamsIfChanged,
    clearShowAction,
    updateUrlParams,
    wizardStepStarted,
    wizardStepExited
} from '../../store/wizard/action_creators';
import {connect} from 'react-redux';
import stepinize from "./../wizard/stepinize";
import {Field, reduxForm} from 'redux-form';
import {InputField} from '../_common_/forms/forms.jsx';
import {END_POINT_NAMES_MAP, HOST_IPV4_IPV6_REG_EXP, HOST_NAME_REG_EXP, IP_REG_EXP, KEYS_REG_EXP} from '../../consts';
import Validator from 'validatorjs';
import {Route, Switch} from 'react-router-dom';
import ServerStepTemplate from "./ServerStepTemplate";
import {calculateStepId, getCredentialsByType} from '../../utils/sync-utils';
import classNames from "classnames";
import _ from "lodash";
import url from "is-url";
import {buildServersList, getPropFromState, getWizardParamsFromUrlParams} from '../../utils/mapStateToPropsUtils';

const rules = (step, providerUpper) => {
    switch (step) {
        case "credentials": {
            const portNotRequired = providerUpper === "ONTAP" || providerUpper === "SGWS";
            const portRules = ['required', 'numeric', 'between:1,65535'];
            if (portNotRequired) {
                portRules.shift();
            }
            return{
                serverName: ['required', 'max:256'],
                port: portRules,
                accessKey: ['required', 'regex:'+KEYS_REG_EXP],
                secretKey: ['required', 'regex:'+KEYS_REG_EXP]
            };
        }
        case "ip":
        default: {
            return {serverName: ['required', 'max:256']}
        }
    }
};

const shouldValidate = ({values, nextProps, props, initialRender, structure,}) => {
    const {_ui: {objectStorage}} = props;
    const step = objectStorage ? objectStorage.step : "ip";
    return (
        initialRender ||
        !structure.deepEqual(values, nextProps.values) ||
        step === "credentials"
    );
};

const validate = (values, props) => {
    let errors = {};
    const {serverName} = values;
    const {_ui: {objectStorage}, endpointObj} = props;
    const step = objectStorage ? objectStorage.step : "ip";
    const providerUpper = endpointObj.provider ? _.toUpper(endpointObj.provider) : _.toUpper(endpointObj.protocol);
    const serverType = END_POINT_NAMES_MAP[providerUpper];
    const validator = new Validator(values, rules(step, providerUpper), {
        "max.serverName": `${serverType} hostname or IP address must be less than 256 characters`,
        "between.port": "Port must be between 1 to 65535"});
    validator.setAttributeNames({serverName : serverType, accessKey: "Access Key", secretKey: 'Secret Key' });
    validator.passes();
    errors = validator.errors.all();
    if (serverName && !IP_REG_EXP.test(serverName) && !HOST_NAME_REG_EXP.test(serverName) && !HOST_IPV4_IPV6_REG_EXP.test(serverName) && !url(serverName)) {
        errors.serverName = errors.serverName ||[];
        errors.serverName.push("Invalid hostname or IP address");
    }
    return errors;
};

const CredentialsUi = ({wizardInfo, isTargetStepFromSameSourceType}) => {
    const stepId = calculateStepId(wizardInfo, isTargetStepFromSameSourceType, "s3");
    const portNotNeeded = stepId.includes("Ontap");
    return (
        <div className={classNames("row-medium credentials-container", {"narrow": portNotNeeded})}>
            <div className="credentials-title">Object Storage Credentials:</div>
            <div className={classNames("medium-4 columns no-padding-left", {"hide": portNotNeeded})}>
                <Field name="port" component={InputField} label="Port" type="text"/>
            </div>
            <div className={classNames("medium-4 columns", {"no-padding-left": portNotNeeded})}>
                <Field name="accessKey" component={InputField} label="Access Key" type="text"/>
            </div>
            <div className="medium-4 columns no-padding-right">
                <Field name="secretKey" component={InputField} label="Secret Key" type="password" autoComplete="off"/>
            </div>
        </div>
    )
};

const S3BasedServerStep = (props) => {
    const {_servers, protocolAndProviderKey, endpointObj, isTargetStepFromSameSourceType, _wizard, _ui: {objectStorage}} = props;
    const {isSourceStep, wizardInfo, selectedWizardParams: {selectedS3Based, selectedS3BasedTarget}} = _wizard;
    const stepId = calculateStepId(wizardInfo, isTargetStepFromSameSourceType, "s3");
    const portNotNeeded = stepId.includes("Ontap");
    const step = objectStorage ? objectStorage.step : "ip";
    const selectedServer = isTargetStepFromSameSourceType ? selectedS3BasedTarget : selectedS3Based;
    const stepOptions = {
        storage: _servers[protocolAndProviderKey],
        server: selectedServer,
        serverSVG: `${endpointObj.provider}-default.svg`,
        step,
        serverType: "Object Storage",
        serverTitle: "Object Storage",
        selectType: "Server",
        serverPlaceholder: "For example: 217.70.21.18 or netapp.com",
        title: `Select an Object Storage ${!isSourceStep || isTargetStepFromSameSourceType ? "Target" : "Source"}`,
        titlePrefix: "Select a",
        clearURLOptions: [{
            changedParam: isTargetStepFromSameSourceType ? "s3BasedTarget" : "s3Based",
            newValueFromState: "selectedIp",
            urlParams: [isTargetStepFromSameSourceType ? "s3BasedBucketPathTarget" : "s3BasedBucketPath"]
        }],
        clearShowOptions: {keyToClear: ['_showS3BasedBuckets', selectedServer]},
        validate,
        getCredentialsUi: () => <CredentialsUi isTargetStepFromSameSourceType={isTargetStepFromSameSourceType} wizardInfo={wizardInfo}/>,
        selectedFieldIsNarrow: portNotNeeded
    };

    return <ServerStepTemplate stepOptions={stepOptions} {...props}/>;
};

const stepId = ({_wizard: {wizardInfo}, isTargetStepFromSameSourceType}) => {
    return calculateStepId(wizardInfo, isTargetStepFromSameSourceType, "s3");
};

const willMount = (props) => {
    const { endpointObj, protocolAndProviderKey } = props;
    if (!props._servers || !props._servers[protocolAndProviderKey] || !props._servers[protocolAndProviderKey].succeeded) {
        return props.getServersByType(endpointObj.protocol, endpointObj.provider, protocolAndProviderKey);
    }
};

const isFetching = (props) => {
    const {_servers, protocolAndProviderKey} = props;
    return _servers[protocolAndProviderKey].inProgress;
};

const canShowNextStep = ({_wizard, isTargetStepFromSameSourceType}) => {
    if (isTargetStepFromSameSourceType) {
        return _wizard.selectedWizardParams.selectedS3BasedTarget && getCredentialsByType("objectStorage", _wizard.selectedWizardParams.selectedS3BasedTarget, isTargetStepFromSameSourceType);
    }
    else {
        return _wizard.selectedWizardParams.selectedS3Based && getCredentialsByType("objectStorage", _wizard.selectedWizardParams.selectedS3Based, isTargetStepFromSameSourceType);
    }
};

const nextRoutes = ({match, isTargetStepFromSameSourceType, nextStep, NextStepComp}) => {

    return (
        <Switch>
            {isTargetStepFromSameSourceType &&
                <Route path={`${match.url}/:s3BasedTarget/${nextStep.path}`} render={(props) =>
                    <NextStepComp {...props} isTargetStepFromSameSourceType={nextStep.isTargetStepFromSameSourceType}/>
                }/> }
            {!isTargetStepFromSameSourceType &&
                <Route path={`${match.url}/:s3Based/${nextStep.path}`} render={(props) =>
                    <NextStepComp {...props} isTargetStepFromSameSourceType={nextStep.isTargetStepFromSameSourceType}/>
                }/> }
        </Switch>
    )
};

const S3BasedServerStepForm = reduxForm({
    form: 's3BasedServer',
    validate,
    shouldValidate
})(S3BasedServerStep);

const mapStateToProps = (state, ownProps) => {
    return {
        _servers: buildServersList(state),
        _wizard: getWizardParamsFromUrlParams(state, ownProps),
        _relationships: getPropFromState(state, "_relationships", {}),
        _ui: {...state.syncReducer._ui, ...state.global._ui},
        routing: state.router,
        ...ownProps
    }
};

const mapDispatchToProps = {
    getServersByType,
    serverIPStep,
    serverCredentialsStep,
    clearUrlParamsIfChanged,
    clearShowAction,
    updateUrlParams,
    wizardStepStarted,
    wizardStepExited
};

export default connect(mapStateToProps, mapDispatchToProps)(stepinize(S3BasedServerStepForm, {stepId, willMount, isFetching, canShowNextStep, nextRoutes,
    loading: ({endpointObj}) => {
        const serverType = endpointObj.provider || endpointObj.protocol;
        return {text: "Object Storage Servers", icon: require(`./../../assets/svgs/${serverType}-default.svg`)}}
}));