import React, {useEffect} from "react";
import * as actionCreatorsConsts from '../../store/global/action_creators_consts';
import * as consts from "../../consts";
import {connect} from 'react-redux';
import RelationshipTable from "./RelationshipTable";
import RelationshipCard from "./relationshipCard/RelationshipCard";
import CardsContainer from "./../cards/CardsContainer";
import {SWITCH} from "./../../utils/directives"
import LoadingBar from "../_common_/loadingBar/LoadingBar";
import SyncNowConfirmDialog from "./SyncNowConfirmDialog";
import AbortSyncConfirmDialog from "./AbortSyncConfirmDialog"
import DeleteRelationshipDialog from "./DeleteRelationshipDialog";
import SearchableTableContainer from "./../table/SearchableSelectableTable";
import Isvg from 'react-inlinesvg';
import {openDialog} from "./../dialog/Dialog"
import SubscriptionRequiredDialog from "./SubscriptionRequiredDialog";
import _ from "lodash";
import {getRelationships, clearWizardCache} from "../../store/wizard/action_creators";
import {clearCmIntegrationDetails, updateDashboardView} from "../../store/global/action_creators";
import {checkRelationshipRecommendations} from "../../utils/relationshipUtils";
import useRunOnce from '../../hooks/useRunOnce';
import useRightPanel from '../../hooks/useRightPanel';
import DataBrokerWizard from '../dataBrokerWizard/DataBrokerWizard';
import MainLoader from '../_common_/MainLoader/MainLoader';
import "./dashboard.scss";

//Exclude non-relevant relationship properties from the search filter, to avoid crash and improve performance (CS-5275)
const keysToExcludeFromSearch = ["activity", "settings", "ui", "lastCopyMessage", "lastRequests", "lastScanMessage", "lastSyncMessage"];

const Dashboard = (props) => {
    const {_relationships, _ui, dashboardView} = props;
    const {data} = _relationships;
    const {isCmIntegration, hasNewRelationship, shouldBlockCreatingRelationship} = _ui;
    //action creators:
    const {getRelationships, clearWizardCache, updateDashboardView, clearCmIntegrationDetails} = props;

    const {setPanelContent, closePanel} = useRightPanel();

    const startDataBrokerWizard = (props) => {
        setPanelContent(<DataBrokerWizard
            closePanel={closePanel}
            {...props}
        />)
    };

    useRunOnce(() => {
        getRelationships();
        clearWizardCache(true, true, true, true);//make sure all Credentials are cleared from before
        clearCmIntegrationDetails();//make sure all entry point input details are cleared
    });

    useEffect(() => {
        if (hasNewRelationship && data) {
            getRelationships();
            return;
        }
        if (isCmIntegration && data?.length === 0) {
            actionCreatorsConsts.routesContainer.routes.push("/drag-source-and-target");
        }
    }, [hasNewRelationship, data, isCmIntegration, getRelationships]);

    const toggleSwitch = () => {
        updateDashboardView(dashboardView === consts.CARDS_VIEW ? consts.TABLE_VIEW : consts.CARDS_VIEW);

    };

    const containerOptions = {
        singularTitle: "Sync",
        pluralTitle: "Syncs",
        viewToggle: toggleSwitch,
        keysToExcludeFromSearch
    };

    return (
        <div className="scrollable-area">
            <div className={`sync-dashboard ${dashboardView === consts.TABLE_VIEW ? 'table' : ''}`}>
                <div className="clearfix">
                    {!isCmIntegration && <h1 className="float-left">Sync Relationships</h1>}
                    <button className={"float-right button create-button"}
                            onClick={(el) => createNewRelationship(el, shouldBlockCreatingRelationship, isCmIntegration)}>
                        {isCmIntegration ? "Create New Sync" : "Create New Sync Relationship"}
                    </button>
                </div>
                <SWITCH on={_relationships.inProgress || !data}>
                    <div data-when={true}>
                        {!isCmIntegration && <LoadingBar className="top"/>}
                        <MainLoader/>
                    </div>
                    <div data-when={false}>
                        <div>
                            {dashboardView === consts.TABLE_VIEW &&
                            <SearchableTableContainer containerOptions={containerOptions} Table={RelationshipTable}
                                                      data={data} isMenuDisabled={false}
                                                      menuItems={relationshipMenu(startDataBrokerWizard)}/>}
                            {dashboardView === consts.CARDS_VIEW &&
                            <CardsContainer containerOptions={containerOptions} Card={RelationshipCard}
                                            data={data} isMenuDisabled={false}
                                            menuItems={relationshipMenu(startDataBrokerWizard)}/>}
                        </div>
                    </div>
                </SWITCH>
            </div>
        </div>
    );
};

const mapStateToProps = (state) => ({
    _relationships: checkRelationshipRecommendations(state.syncReducer._relationships, state.global._ui.isCmIntegration) || {},
    dashboardView: state.global._ui.dashboardView
});

export default connect(mapStateToProps, {
    getRelationships,
    clearWizardCache,
    updateDashboardView,
    clearCmIntegrationDetails
})(Dashboard);

const createNewRelationship = (element, shouldBlockCreatingRelationship) => {
    // check if user should blocked from creating a new relationship
    if (shouldBlockCreatingRelationship) {
        openDialog(<SubscriptionRequiredDialog
                className="default-dialog"
                errorMessage={shouldBlockCreatingRelationship}/>,
            element.target);
        return;
    }

    actionCreatorsConsts.routesContainer.routes.push("/drag-source-and-target");
};

const gotoSettings = (el) => {
    actionCreatorsConsts.routesContainer.routes.push(`/settings/${el.relationshipId}`);
};

const relationshipMenu = (startDataBrokerWizard) => {
    return (element, triggerMenu) => {
        const syncProgress = element.ui.status === "RUNNING" || element.ui.status === "ABORTING" || element.ui.status === "STARTING";
        const syncAborting = element.ui.status === "ABORTING";
        const failedDataBroker = _.includes(element.ui.failureMessage, consts.FAILED_AFTER_STARTUP_MESSAGE);
        const disableTitleSyncNow = failedDataBroker ? element.ui.failureMessage : "";
        const isReady = !syncProgress && !failedDataBroker;
        const existingGroupId = element.group.id;
        const existingAgentType = _.get(element.ui, 'dataBrokers[0].type');
        const isSecureRelationship = !!element.encryption;
        const isGcpAccessRequired = element.source.protocol === "gcp" || element.target.protocol === "gcp";
        const isAwsAccessRequired = (element.source.protocol === "s3" && element.source[element.source.protocol].provider === "s3") ||
            (element.target.protocol === "s3" && element.target[element.target.protocol].provider === "s3");

        const deleteButton = [<button key="delete"
                                      onClick={(el) => {
                                          openDialog(<DeleteRelationshipDialog className="default-dialog" data={element}/>, el.target);
                                          triggerMenu();
                                      }}>
            <span className="garbage-bin"><Isvg src={require("./../../assets/svgs/garbage-bin-icon.svg")}/></span>
            Delete
        </button>];

        const accelerateAndSettingsButtons = [
            <button className="add-data-broker"
                    title={isSecureRelationship ? "Accelerating sync performance is currently not supported for relationships that use data-in-flight encryption." : "Enhance sync performance by adding an additional data broker."}
                    disabled={isSecureRelationship}
                    key="addDataBroker"
                    onClick={() => {
                        startDataBrokerWizard({
                            existingGroupId,
                            existingAgentType,
                            isGcpAccessRequired,
                            isAwsAccessRequired
                        });
                        triggerMenu();
                    }}
            >
                <span className="plus-circle"><Isvg src={require("./../../assets/svgs/plus-circle-icon.svg")}/></span>
                Accelerate
            </button>,

            <button title="View and edit sync relationship settings"
                    key="editSettings"
                    onClick={() => {
                        gotoSettings(element);
                        triggerMenu();
                    }}>
                <span className="settings"><Isvg src={require("./../../assets/svgs/settings.svg")}/></span>
                Settings
            </button>
        ];

        const syncOrAbortButton = (syncProgress || syncAborting) ?
            [<button title={"Abort current sync"}
                     disabled={syncAborting}
                     key="abortSync"
                     onClick={(el) => {
                         openDialog(<AbortSyncConfirmDialog className="default-dialog" data={element}/>, el.target);
                         triggerMenu();
                     }}>
                <span className="clock"><Isvg src={require("./../../assets/svgs/x-circle-icon.svg")}/></span>
                Abort Sync
            </button>]
            :
            [<button title={disableTitleSyncNow}
                     disabled={!isReady}
                     key="syncNow"
                     onClick={(el) => {
                         openDialog(<SyncNowConfirmDialog className="default-dialog" data={element}/>, el.target);
                         triggerMenu();
                     }}>
                <span className="rotating-arrows"><Isvg src={require("./../../assets/svgs/rotating-arrows-icon.svg")}/></span>
                Sync Now
            </button>];

        return syncOrAbortButton.concat(accelerateAndSettingsButtons).concat(deleteButton);
    }
};