import React, {Component} from 'react';
import {openDialog} from '../../dialog/Dialog'
import {
    DOCUMENTATION_LICENSING_URL, FREE_TRIAL_PERIOD, AWS_MP_PRICING_MODEL_PAYGO_URL,
    AWS_MP_PRICING_MODEL_ANNUAL_URL, AZURE_MP_PRICING_MODEL_URL,
    AWS_MP_SUBSCRIBE_URL, AWS_MP_UNSUBSCRIBE_URL
} from '../../../consts';
import SubscriptionFlowDialog from './SubscriptionFlowDialog';
import Isvg from 'react-inlinesvg';
require("./subscribe.scss");

export default class extends Component {

    renderSubscribeTitle() {
        const {subscribing, subscribed, failedToSubscribe, isPayGoSubscription, provider} = this.props;

        if (subscribed) {
            return (
            <div className="icon-with-title">
                <span><Isvg src={require("../../../assets/svgs/checkmark-circle-icon.svg")}/></span>
                <h5>Subscribed ({provider} {isPayGoSubscription ? 'Pay-As-You-Go' : 'annual'} subscription)</h5>
            </div>
            );
        }

        if (failedToSubscribe) {
            return (
            <div className="icon-with-title failed">
                <span><Isvg src={require("../../../assets/svgs/x-circle-icon.svg")}/></span>
                <h5>Failed to subscribe from the {provider} Marketplace</h5>
            </div>
            );
        }

        if (subscribing) return <h5>Subscribing from {provider}</h5>;

        return <h5>Billing from your {provider} monthly bill</h5>;
    }

    renderSubscribeText() {
        const {subscribing, subscribed, failedToSubscribe, freeTrial, hasLicense, isPayGoSubscription, provider} = this.props;
        const unsubscribed = !subscribed && !subscribing;
        const priceModelLink = provider === 'AWS' ? (isPayGoSubscription ? AWS_MP_PRICING_MODEL_PAYGO_URL : AWS_MP_PRICING_MODEL_ANNUAL_URL) : AZURE_MP_PRICING_MODEL_URL;

        if (subscribing) {
            return <span>Your subscription will be completed soon.</span>
        }

        if (failedToSubscribe) {
            return <span>Unsubscribe from the {provider} Marketplace and start the subscription process again.</span>
        }

        // build text dynamically
        let subscribeText;

        if (subscribed && freeTrial) {
            subscribeText = `After the ${FREE_TRIAL_PERIOD}-day free trial is over, you will be charged hourly according to the number of sync relationships that you created`
        }

        if (subscribed && !freeTrial && isPayGoSubscription) {
            subscribeText = `You will be charged hourly according to the number of sync relationships that you created`
        }

        if (subscribed && freeTrial && !isPayGoSubscription) {
            subscribeText = `You have an annual subscription to Cloud Sync. You can create up to 20 sync relationships using your annual subscription${provider=== 'Azure' ? '. Additional relationships will be charged by the hour' : ''}`;
        }

        if (unsubscribed && freeTrial) {
            subscribeText = `After the ${FREE_TRIAL_PERIOD}-day free trial is over, pay-as-you-go with hourly rates based on the number of sync relationships that you create, 
                                or purchase a credit of relationships by paying up front for a year`
        }

        if (unsubscribed && !freeTrial) {
            subscribeText = `Pay-as-you-go with hourly rates based on the number of sync relationships that you create, 
                                or purchase a credit of relationships by paying up front for a year`
        }

        // is has license - note that the subscription will only affect the relationships that are not in the license capacity
        if (!subscribing && hasLicense) {
            subscribeText += ` (additional to the reserved relationships that your licenses allow)`
        }

        subscribeText += '.';

        // only show price model link when subscribed (otherwise pricing model is shown on subscribe page itself)
        if (subscribed) {
            return (
                <span>
                    {subscribeText}
                    <br/>
                    <a href={priceModelLink} target="_blank" rel="noopener noreferrer">{`View the pricing model`}</a>
                </span>
            );
        }


        return <span>{subscribeText}</span>;
    }

    renderSubscribeAction() {
        const {subscribing, subscribed, failedToSubscribe, provider} = this.props;

        if (subscribing) {
            return <img className="loading" src={require('../../../assets/images/loading.gif')} alt=""/>
        }
        if (failedToSubscribe) {
            return <button className="button" onClick={(clickElement) => this.showUnsubscribeConfirmation(clickElement, provider)}>Unsubscribe</button>
        }
        if (!subscribed) {
            return <button className="button" onClick={(clickElement) => this.showSubscribeConfirmation(clickElement, provider)}>Subscribe</button>
        }

        return <div/>
    }

    showSubscribeConfirmation(clickElement, provider) {
        openDialog(<SubscriptionFlowDialog className="default-dialog subscription-flow-dialog"
                                           providerLink={provider === 'AWS' ? AWS_MP_SUBSCRIBE_URL : process.env.REACT_APP_AZURE_SUBSCRIPTION_URL}
                                           providerInstructions={provider === 'AWS' ? `In the AWS Marketplace, you should click Continue, then Subscribe, and then Set Up Your Account.` :
                                               `In the Azure Portal, click Create, select your options, and click Subscribe (select Monthly to pay by the hour). When deployment is
                                                complete, click the name of the SaaS resource in the notification popup, and then click Configure Account.`}
                                           backToCloudSyncInstructions="You are then redirected back to Cloud Sync."
                                           videoLink= {`${DOCUMENTATION_LICENSING_URL}#${provider.toLowerCase()}`}
                                           provider={provider}
                                           iconFileName={`${provider.toLowerCase()}-marketplace`}/>, clickElement.target);
    }

    showUnsubscribeConfirmation(clickElement, provider) {
        openDialog(<SubscriptionFlowDialog className="default-dialog subscription-flow-dialog"
                                           providerLink={provider === 'AWS' ? AWS_MP_UNSUBSCRIBE_URL : process.env.REACT_APP_AZURE_SUBSCRIPTION_URL}
                                           providerInstructions={`At the ${provider} Marketplace, look for Data Fabric Cloud Sync Service and click CANCEL SUBSCRIPTION.`}
                                           backToCloudSyncInstructions="You should then return to Cloud Sync to start the subscription process again."
                                           provider={provider}
                                           iconFileName={`${provider.toLowerCase()}-marketplace`}/>, clickElement.target);
    }

    render() {
        const {provider} = this.props;
        const iconFileName = provider === 'AWS' ? 's3-default' : 'azure-default';
        return (
            <div className="subscribe payment-option row medium">
                <div className="payment-icon">
                    <span><Isvg src={require(`../../../assets/svgs/${iconFileName}.svg`)}/></span>
                </div>

                <div className="medium-8 columns payment-text">
                    {this.renderSubscribeTitle()}

                    {this.renderSubscribeText()}
                </div>

                <div className="medium-1 columns payment-action">
                    {this.renderSubscribeAction()}
                </div>

            </div>
        );
    }
}
