import axios from 'axios';
import _ from "lodash";
import * as consts from "./consts";

axios.defaults.headers.post['Content-Type'] = 'application/json';
axios.defaults.headers.put['Content-Type'] = 'application/json';

const axiosInstance = axios.create({
    baseURL: process.env.REACT_APP_API_BASE_URL + "/api",
});

export const setAuthHeader = (t) => {
    axiosInstance.defaults.headers.common['Authorization'] = t.authorization;
    return t;
};

export const setAccountHeader = (accountId) => {
    axiosInstance.defaults.headers.common['x-account-id'] = accountId;
};

const extractData = (response) => {
    const data = response.data;
    const requestId = response.headers["x-dfio-req-id"];
    if (data && !_.isString(data)) {
        data.requestId = requestId;
        return data;
    } else return {requestId};
};

export default {
    getAccounts: () => {
        return axiosInstance.get('accounts').then(extractData);
    },
    getAccountInfo: (accountId) => {
        setAccountHeader(accountId);
        return axiosInstance.get('accounts/info').then(extractData);
    },
    registerToMarketplace: (defaultAccountId, mpCustomerToken, mpProvider) => {
        const config = { headers: { "x-account-id": defaultAccountId } };
        if (mpCustomerToken && mpProvider) {
            return axiosInstance.put('accounts/register',{mpCustomerToken, mpProvider}, config).then(extractData);
        }
        return Promise.resolve();
    },
    createDataBroker: function (opts) {
        return axiosInstance.post('/data-brokers', opts).then(extractData);
    },
    createDataBrokerWithTemplate: function (opts) {
        return axiosInstance.post('/data-brokers/template', opts).then(extractData);
    },
    deleteDataBroker: function (id) {
        return axiosInstance.delete(`/data-brokers/${id}`).then(extractData);
    },
    dataBrokers: function () {
        return axiosInstance.get('/data-brokers').then(extractData);
    },

    getServers: function (serverTypeBase, provider) {
        const providerParam = provider ? `?provider=${provider}` : '';
        return axiosInstance.get(`/storages/${serverTypeBase}${providerParam}`).then(extractData);
    },
    azureStorageAccounts: function () {
        return axiosInstance.get('/storages/azure').then(extractData);
    },
    messages: function (lastMessage) {
        const filter = (lastMessage) ? '?last=' + lastMessage : '';
        return axiosInstance.get(`/messages/client${filter}`).then(extractData);
    },
    showExports: function (dataBrokerId, nfsServer) {
        return axiosInstance.get(`/data-brokers/${dataBrokerId}/${consts.PAYLOAD_SHOW_EXPORTS}`, {
            params: {
                host: nfsServer,
            }
        }).then(extractData);
    },
    showEFSExports: function (dataBrokerId, nfsServer) {
        return axiosInstance.get(`/data-brokers/${dataBrokerId}/${consts.PAYLOAD_SHOW_EXPORTS}?host=${nfsServer}&isEFS=${true}`).then(extractData);
    },
    listFolder: function (dataBrokerId, nfsServer, path, exportName, nfsVersion) {
        return axiosInstance.get(`/data-brokers/${dataBrokerId}/${consts.PAYLOAD_LIST_FOLDERS}`, {
            params: {
                host: nfsServer,
                path,
                exportName,
                version: nfsVersion || undefined
            }
        }).then(() => ({dataBrokerId, nfsServer, path}));
    },
    showShares: function (dataBrokerId, cifsServer, username, password, domain, cifsVersion) {
        let headers = {
            username,
            password
        };
        if (domain) {
            headers.domain = domain;
        }

        return axiosInstance.get(`/data-brokers/${dataBrokerId}/${consts.PAYLOAD_LIST_CIFS_SHARES}`, {
            params: {
                host: cifsServer,
                version: cifsVersion || undefined
            },
            headers
        }).then(extractData);
    },
    listShareFolders: function (dataBrokerId, host, share, path, username, password, domain, cifsVersion) {
        let headers = {
            username,
            password
        };
        if (domain) {
            headers.domain = domain;
        }
        return axiosInstance.get(`/data-brokers/${dataBrokerId}/${consts.PAYLOAD_LIST_CIFS_SHARE_FOLDERS}`, {
            params: {
                host,
                share,
                path,
                version: cifsVersion || undefined
            },
            headers
        }).then(extractData);
    },
    fetchS3BasedBuckets: function (dataBrokerId, s3Based, osCredentials, provider) {
        const payloadName = `PAYLOAD_LIST_${provider.toUpperCase()}_BUCKETS`;
        return axiosInstance.get(`/data-brokers/${dataBrokerId}/${consts[payloadName]}`, {
            params: {
                host: s3Based,
                port: osCredentials.port
            },
            headers: {
                "access-key": osCredentials.accessKey,
                "secret-key": osCredentials.secretKey
            }
        }).then(extractData);
    },

    listS3BasedBucketFolders: function (dataBrokerId, host, bucket, path, accessKey, secretKey, port, provider) {
        const payloadName = `PAYLOAD_LIST_${provider.toUpperCase()}_BUCKETS_FOLDERS`;
        return axiosInstance.get(`/data-brokers/${dataBrokerId}/${consts[payloadName]}`, {
            params: {
                host,
                bucket,
                prefix: path,
                port

            },
            headers: {
                "access-key": accessKey,
                "secret-key": secretKey
            }
        }).then(extractData);
    },

    fetchBuckets: function (dataBrokerId) {
        return axiosInstance.get(`/data-brokers/${dataBrokerId}/${consts.PAYLOAD_LIST_BUCKETS}`).then(extractData);
    },

    listBucketFolders: function (dataBrokerId, bucket, prefix) {
        return axiosInstance.get(`/data-brokers/${dataBrokerId}/${consts.PAYLOAD_LIST_SUB_DIRECTORIES}?bucket=${bucket}&prefix=${prefix}`).then(extractData);
    },

    fetchAzureBlobContainers: function (dataBrokerId, storageAccountName, connectionString) {
        return axiosInstance.get(`/data-brokers/${dataBrokerId}/${consts.PAYLOAD_LIST_CONTAINERS}`, {
            params: {
                'storage-account-name': storageAccountName
            },
            headers: {
                'storage-account-connection-string': connectionString
            }
        }).then(extractData);
    },

    listContainerFolders: function (dataBrokerId, storageAccountName, connectionString, container, path) {
        return axiosInstance.get(`/data-brokers/${dataBrokerId}/${consts.PAYLOAD_LIST_CONTAINER_SUB_DIRECTORIES}`, {
            params: {
                'storage-account-name': storageAccountName,
                container,
                prefix:path !== "" ? path : undefined
            },
            headers: {
                'storage-account-connection-string': connectionString
            }
        }).then(extractData);
    },

    listGCPBuckets: function (dataBrokerId) {
        return axiosInstance.get(`/data-brokers/${dataBrokerId}/${consts.PAYLOAD_LIST_GCP_BUCKETS}`).then(extractData);
    },

    listGCPBucketFolders: function (dataBrokerId, bucket, path) {
        return axiosInstance.get(`/data-brokers/${dataBrokerId}/${consts.PAYLOAD_LIST_GCP_BUCKET_FOLDERS}`, {
            params: {
                bucket,
                prefix:path !== "" ? path : undefined
            }
        }).then(extractData);
    },

    createSync: function (opts, isCmIntegration) {
        const config = { headers: {  "x-cm-integration": isCmIntegration } };
        return axiosInstance.post(`/relationships-v2`, opts, config).then(extractData);
    },
    syncNow: function (relationshipId) {
        return axiosInstance.put(`/relationships/${relationshipId}/sync`, {}).then(extractData);
    },
    abortSync: function (relationshipId) {
        return axiosInstance.put(`/relationships/${relationshipId}/abort`, {}).then(extractData);
    },
    getRelationships: function () {
        return axiosInstance.get("/relationships-v2").then(extractData);
    },
    deleteRelationship: function (relationshipId) {
        return axiosInstance.delete(`/relationships/${relationshipId}`).then(extractData);
    },
    updateRelationship: function (relationshipId, opts) {
        return axiosInstance.put(`/relationships-v2/${relationshipId}`, opts).then(extractData);
    },
    getTimeline: function () {
        return axiosInstance.get("/timelines").then(extractData);
    },
    getCost: function (params) {
        return axiosInstance.get("/accounts/cost", {params}).then(extractData);
    },
    getCostTimes: function () {
        return axiosInstance.get("/accounts/cost-times").then(extractData);
    },
    getLicenseSettings: function () {
        return axiosInstance.get("/accounts/license-settings").then(extractData);
    },
    getBYOLLicenses: function () {
        return axiosInstance.get("/accounts/licenses").then(extractData);
    },
    updateNSSCredentials: function (values) {
        return axiosInstance.put("/accounts/nss-credentials", values).then(extractData);
    },
    addBYOLLicense: function (values) {
        return axiosInstance.put("/accounts/licenses", values).then(extractData);
    },
    registerToSupport: (credentials) => {
        return axiosInstance.post(`/accounts/support/register`, credentials).then(extractData);
    },
    sendAsup: () => {
        return axiosInstance.post(`/accounts/support/send-asup`, null, {maxContentLength: 0}).then(extractData);
    },
    downloadDataBrokerLogs: (groupId) => {
        return axiosInstance.put(`/groups/${groupId}/download-data-brokers-logs`, null, {maxContentLength: 0}).then(extractData);
    },
    listAwsKmsKeys: (dataBrokerId, bucket) => {
        return axiosInstance.get(`/data-brokers/${dataBrokerId}/list-aws-kms-keys`, {params: {bucket}}).then(extractData);
    },
    interceptors: function () {
        return axiosInstance.interceptors;
    }
};
