import React from 'react';
import {RELATIONSHIP_RECOMMENDATIONS} from "../consts";
import get from 'lodash/get';

const {manyRelationshipsOnFewDb, targetAnfRelationship, manySmallFiles} = RELATIONSHIP_RECOMMENDATIONS;

const recommendationsMap = {
    [manyRelationshipsOnFewDb]: {
        id: manyRelationshipsOnFewDb,
        title: 'Acceleration recommendation',
        description: 'The data broker associated with this sync relationship is either managing more than 6 relationships by itself or is part of a group that is managing more than 6 relationships per data broker. Both of these scenarios can affect sync performance.',
        suggestion: <span>We recommend that you accelerate the relationship from the action menu on the right.</span>
    },
    [targetAnfRelationship]: {
        id: targetAnfRelationship,
        title: 'Optimization recommendation',
        description: `Verify that you're using the Premium or Ultra service level with Azure NetApp Files. You might experience failures and performance issues if the disk service level is Standard.`,
        suggestion: <span>Consult a solutions architect if you need help determining the right service level. The volume size and volume tier determines the <a target="_blank" rel="noopener noreferrer"
            href="https://docs.microsoft.com/en-us/azure/azure-netapp-files/azure-netapp-files-service-levels#throughput-limits">throughput</a> that you can get.</span>
    },
    [manySmallFiles]: {
        id: manySmallFiles,
        title: 'Acceleration recommendation',
        description: `We noticed that a large amount of directories are being transferred, which can affect sync performance.`,
        suggestion: <span>We recommend that you accelerate the relationship from the action menu on the right.</span>
    }
};
const getRecommendation = ({id, title, description, suggestion}) => ({
    id: id,
    title: title,
    text: <span>{description} {suggestion}</span>,
});

export const checkRelationshipRecommendations = (relationships) => {
    if (!relationships || !relationships.data) {
        return relationships
    }
    const dbGroupsMap = relationships.data.reduce(dbGroupsReducer, {});
    return relationships.updateIn(["data"], relationships => {
        return relationships.map((relationship) => {
            relationship = isManyRelationshipOnFewDb(relationship, dbGroupsMap);
            relationship = isAnfRelationship(relationship);
            relationship = isManySmallFiles(relationship);
            return relationship;
        })
    })
};

const isManyRelationshipOnFewDb = (relationship, dbGroupsMap) => {
    const {group: {id:groupId}} = relationship;
    const {relationships, dbsInGroup} = dbGroupsMap[groupId];
    if (relationships > dbsInGroup * 6) {
        return relationship.updateIn(['ui', 'recommendations'], recommendations => [...recommendations, getRecommendation(recommendationsMap.manyRelationshipsOnFewDb)])
    }
    return relationship;
};
const isManySmallFiles = (relationship) => {
    const {activity: {dirsScanned}, group: {dataBrokers}} = relationship;
    if (dirsScanned > dataBrokers.length * 5000) {
        return relationship.updateIn(['ui', 'recommendations'], recommendations => [...recommendations, getRecommendation(recommendationsMap.manySmallFiles)])
    }
    return relationship;
};

const isAnfRelationship = (relationship) => {
    if (get(relationship, 'target.cifs.provider') === "anf" ||
        get(relationship, 'target.nfs.provider') === "anf" ||
        get(relationship, 'source.nfs.provider') === "anf" ||
        get(relationship, 'source.cifs.provider') === "anf") {
        return relationship.updateIn(['ui', 'recommendations'], recommendations => [...recommendations, getRecommendation(recommendationsMap.targetAnfRelationship)])
    }
    return relationship;

};


const dbGroupsReducer = (acc, {group: {id, dataBrokers}}) => {
    if (!acc[id]) {
        acc[id] = {
            relationships: 1,
            dbsInGroup: dataBrokers.length
        }
    } else {
        acc[id].relationships++;
    }
    return acc;
};