import _ from 'lodash';
import React from 'react';
import Isvg from 'react-inlinesvg';
import moment from "moment";
import {getSelectOptions, convertBytesToSize, convertSizeToBytes} from './helpers';

const SETTINGS_KEY = "settings";

//General functions/consts for all types of settings:

export const SETTINGS_DEFAULTS = {
    gracePeriod: 30,
    deleteOnTarget: false,
    deleteOnSource: false,
    objectTagging: true,
    fileTypes: {
        files: true,
        directories: true,
        symlinks: true
    },
    retries: 3,
    files: {
        excludeExtensions: [],
        minSize: 0,
        maxSize: Number.MAX_SAFE_INTEGER,
        minDate: "1970-01-01",
        maxDate: null
    },
    schedule: {
        syncInDays: 1,
        syncInHours: 0,
        syncInMinutes: 0,
        isEnabled: true,
        syncWhenCreated: true,
        nextTime: moment().set({"minute": 0}).toISOString() //The UI only uses HH:mm , the date doesn't matter.
    },
    copyAcl: false
};

export const getSummaryLines = (formValues) => {
    const includeAllTypes = _.get(formValues.fileTypes, 'length') === fileTypesOrder.length;
    return {
        gracePeriod: getGraceSummary(formValues.gracePeriodOn, formValues.graceValue, formValues.graceResolution),
        deleteOnTarget: getDeleteOnTargetSummary(formValues.deleteOnTarget),
        deleteOnSource: getDeleteOnSourceSummary(formValues.deleteOnSource),
        objectTagging: formValues.objectTagging,
        fileTypes: `Include ${includeAllTypes ? 'All' : 'Only'}: ${getFileTypesSummary(formValues.fileTypes)}`,
        retries: `Retry ${_.get(formValues.retries, 'value')} ${_.get(formValues.retries, 'value') === 1 ? 'time' : 'times'} before skipping file`,
        filesExtensions: _.get(formValues.filesExtensions, 'length', 0) === 0 ? 'None' : formValues.filesExtensions.map(item => item.value).join(' '),
        fileSize: formValues.fileSizeOption === FILE_SIZE_OPTION_DISABLED ? 'All' :
            `Min size: ${formValues.fileMinSizeValue} ${_.get(formValues.fileMinSizeResolution, 'value')}, Max size:  ${formValues.fileMaxSizeValue} ${_.get(formValues.fileMaxSizeResolution, 'value')}`,
        dateModified: getDateModifiedSummary(_.get(formValues.fileDateModifiedOption, 'value'), formValues.fileDateModifiedMin, formValues.fileDateModifiedMax),
        schedule: getScheduleSummary(formValues.scheduleStatus, formValues.scheduleInterval, _.get(formValues.scheduleResolution, 'value')),
        acl: formValues.copyAcl ? "Active": "Inactive"
    };
};

export const createSettingsObjectFromFormValues = (values) => {
    return {
        gracePeriod: getGracePeriod(values.graceValue, values.graceResolution, !_.includes(values.gracePeriodOn, NOT_EXCLUDE_FILES_OPTION)),
        deleteOnTarget: getDeleteOnTargetBool(values.deleteOnTarget),
        deleteOnSource: getDeleteOnSourceBool(values.deleteOnSource),
        objectTagging: getObjectTaggingBool(values.objectTagging),
        fileTypes: getFileTypesObj(values.fileTypes),
        retries: values.retries.value,
        files: {
            excludeExtensions: values.filesExtensions.map(item => item.value),
            minSize: values.fileSizeOption === FILE_SIZE_OPTION_DISABLED ? SETTINGS_DEFAULTS.files.minSize : convertSizeToBytes(values.fileMinSizeValue, values.fileMinSizeResolution.value),
            maxSize: values.fileSizeOption === FILE_SIZE_OPTION_DISABLED ? SETTINGS_DEFAULTS.files.maxSize : convertSizeToBytes(values.fileMaxSizeValue, values.fileMaxSizeResolution.value),
            minDate: getMinDate(values),
            maxDate: getMaxDate(values)
        },
        schedule: getScheduleData(values),
        copyAcl: values.copyAcl
    };
};

export const getFormValuesFromSettings = (settings, isEditMode) => {
    const formGracePeriod = getFormGrace(settings.gracePeriod);
    const formFileMinSize = getFormFileSize(settings.files.minSize);
    const formFileMaxSize = getFormFileSize(settings.files.maxSize);
    const isFileSizeDefaults = settings.files.minSize === SETTINGS_DEFAULTS.files.minSize && settings.files.maxSize === SETTINGS_DEFAULTS.files.maxSize;
    const dateModifiedOption = getDateModifiedOption(settings.files.minDate, settings.files.maxDate);
    const scheduleNextTime = getNextTimeHour(settings.schedule.nextTime);

    return {
        graceValue: formGracePeriod.count,
        graceResolution: formGracePeriod.resolution,
        gracePeriodOn: settings.gracePeriod > 0 ? EXCLUDE_FILES_OPTION : NOT_EXCLUDE_FILES_OPTION,
        deleteOnTarget: getFormDeleteOnTarget(settings.deleteOnTarget),
        deleteOnSource: getFormDeleteOnSource(settings.deleteOnSource),
        objectTagging: getFormObjectTagging(settings.objectTagging),
        fileTypes: getFormFileTypes(settings.fileTypes),
        retries: {label: `${settings.retries}`, value: settings.retries},
        filesExtensions: getSelectOptions(settings.files.excludeExtensions),
        fileSizeOption: isFileSizeDefaults ? FILE_SIZE_OPTION_DISABLED : FILE_SIZE_OPTION,
        fileMinSizeValue: formFileMinSize.value,
        fileMinSizeResolution: {label: `${formFileMinSize.resolution}`, value: formFileMinSize.resolution},
        fileMaxSizeValue: formFileMaxSize.value,
        fileMaxSizeResolution: {label: `${formFileMaxSize.resolution}`, value: formFileMaxSize.resolution},
        fileDateModifiedMin: convertStringToDate(settings.files.minDate),
        fileDateModifiedMax: convertStringToDate(settings.files.maxDate),
        fileDateModifiedOption: {label: dateModifiedOption, value: dateModifiedOption},
        scheduleStatus: getFormScheduleStatus(settings.schedule.isEnabled, settings.schedule.syncWhenCreated, isEditMode),
        scheduleNextTime: {label: scheduleNextTime, value: scheduleNextTime},
        scheduleInterval: settings.schedule.syncInDays > 0 ? settings.schedule.syncInDays : (settings.schedule.syncInHours > 0 ? settings.schedule.syncInHours : settings.schedule.syncInMinutes),
        scheduleResolution: settings.schedule.syncInDays > 0 ? {label: 'Days', value: 'Days'} : (settings.schedule.syncInHours > 0 ? {label: 'Hours', value: 'Hours'} : {label: 'Minutes', value: 'Minutes'}),
        copyAcl: settings.copyAcl
    };
};

//Specific functions/consts:

export const UI_MAX_FILE_SIZE_LIMIT = convertSizeToBytes('4', 'PB');
export const UI_MAX_FILE_SIZE_DEFAULT = convertSizeToBytes('500', 'TB');

export const NOT_EXCLUDE_FILES_OPTION = "Don’t exclude files that are modified before a scheduled sync";
export const EXCLUDE_FILES_OPTION = "Exclude files that are modified up to ";

export const NOT_DELETE_FILES_FROM_TARGET_OPTION_TEXT = "Never delete files from the target location";
export const DELETE_FILES_FROM_TARGET_OPTION_TEXT = "Delete files from the target location if they were deleted from the source";

export const NOT_DELETE_FILES_FROM_SOURCE_OPTION_TEXT = "Never delete files from the source location";
export const DELETE_FILES_FROM_SOURCE_OPTION_TEXT = "Delete files from the source location after they are copied to the target";

export const OBJECT_TAGGING_USE_OPTION_TEXT = "Allow Cloud Sync to tag S3 objects";
export const OBJECT_TAGGING_DISABLE_OPTION_TEXT = "Disable tagging of S3 objects";

export const FILE_SIZE_OPTION_DISABLED = 'Sync all file sizes';
export const FILE_SIZE_OPTION = 'Select a file size range';

export const MAX_RETRIES = 5;

export const DATE_MODIFIED_OPTIONS = {
    ALL: 'All',
    ONLY_MIN: 'Sync files modified after a specific date',
    ONLY_MAX: 'Sync files modified before a specific date',
    RANGE: 'Sync files modified between a time range'
};

export const SCHEDULE_OPTIONS = {
    NOW: 'Start sync now',
    ON: 'Future sync',
    ONCE: 'One time copy',
    OFF: 'Turn sync schedule off'
};

export const MOCK_DATE_FOR_TESTS = "2018–01–30T11:00:0+00:00";

export const MIN_DATE_POSSIBLE = '01/01/1970';

export const saveSettingsToSessionStorage = (settings) => {
    window.sessionStorage.setItem(SETTINGS_KEY, JSON.stringify(settings));
};

export const getSettingsFromSessionStorage = () => {
    let settings = JSON.parse(window.sessionStorage.getItem(SETTINGS_KEY));
    if (settings === null) {
        settings = SETTINGS_DEFAULTS;
    }
    return settings;
};

export const removeSettingsFromSessionStorage = () => {
    window.sessionStorage.removeItem(SETTINGS_KEY);
};

export const fileTypesMap = {
    files: 'Files',
    directories: 'Directories',
    symlinks: 'Symbolic Links'
};

export const fileTypesOrder = [fileTypesMap.files, fileTypesMap.directories, fileTypesMap.symlinks];

export const getFileTypesOptions = () => {
    return getSelectOptions(fileTypesOrder);
};

export const getRetriesOptions = () => {
    const valuesArray = Array.from(Array(MAX_RETRIES + 1), (x, index) => index);
    return getSelectOptions(valuesArray);
};

export const getScheduleResolutionOptions = () => {
    return getSelectOptions(["Minutes", "Hours", "Days"]);
};

export const getScheduleNextTimeOptions = () => {
    return getSelectOptions(["00:00", "01:00", "02:00", "03:00", "04:00", "05:00", "06:00", "07:00", "08:00", "09:00", "10:00", "11:00", "12:00", "13:00", "14:00", "15:00", "16:00", "17:00", "18:00", "19:00", "20:00", "21:00", "22:00", "23:00"]);
};

export const getFileSizeResolutionOptions = () => {
    return getSelectOptions(["B", "KB", "MB", "GB", "TB"]);
};

export const getGraceResolutionOptions = () => {
    return getSelectOptions(["Seconds", "Minutes", "Hours"]);
};

export const getDateModifiedOptions = () => {
    return getSelectOptions(Object.values(DATE_MODIFIED_OPTIONS));
};

export const extensionsInputTransformation = (inputValue) => (inputValue.startsWith('.') ? inputValue : `.${inputValue}`);

export const getGraceSummary = (gracePeriodOn, graceValue, graceResolution) => {
    return _.includes(gracePeriodOn, NOT_EXCLUDE_FILES_OPTION) ? NOT_EXCLUDE_FILES_OPTION :
        <div>{EXCLUDE_FILES_OPTION}<b> {graceValue} {_.get(graceResolution, 'label')}</b> before a scheduled sync</div>
};

const getDeleteOnTargetSummary = (deleteOnTarget) => {
    const warningIcon = (<div className="warning"><span><Isvg src={require("../assets/svgs/exclamation-mark-triangle-icon.svg")}/></span></div>);
    return deleteOnTarget === DELETE_FILES_FROM_TARGET_OPTION_TEXT ? (<div>{warningIcon}{deleteOnTarget}</div>) : deleteOnTarget;
};

const getDeleteOnSourceSummary = (deleteOnSource) => {
    const warningIcon = (<div className="warning"><span><Isvg src={require("../assets/svgs/exclamation-mark-triangle-icon.svg")}/></span></div>);
    return deleteOnSource === DELETE_FILES_FROM_SOURCE_OPTION_TEXT ? (<div>{warningIcon}{deleteOnSource}</div>) : deleteOnSource;
};

export const getFileTypesSummary = (fileTypes) => {
    if (!fileTypes) return;
    fileTypes.sort((typeA, typeB) => {
        return (fileTypesOrder.indexOf(typeA) - fileTypesOrder.indexOf(typeB));
    });
    return fileTypes.join(', ');
};

const getDateModifiedSummary = (option, dateMin, dateMax) => {
    const format = "MM/DD/YYYY";
    switch (option) {
        case DATE_MODIFIED_OPTIONS.ALL: {
            return DATE_MODIFIED_OPTIONS.ALL;
        }
        case DATE_MODIFIED_OPTIONS.ONLY_MIN: {
            return `Sync files modified after ${moment(dateMin).format(format)}`
        }
        case DATE_MODIFIED_OPTIONS.ONLY_MAX: {
            return `Sync files modified before ${moment(dateMax).format(format)}`
        }
        case DATE_MODIFIED_OPTIONS.RANGE: {
            return `Sync files modified between ${moment(dateMin).format(format)} and ${moment(dateMax).format(format)}`
        }
        default:
            return '';
    }
};

const getScheduleSummary = (status, interval, resolution) => {
    if ( _.includes([SCHEDULE_OPTIONS.OFF, SCHEDULE_OPTIONS.ONCE], status)) return 'OFF';
    const summary =  `ON | Every ${interval} ${resolution}`;
    return (interval === '1' || interval === 1) ? summary.substring(0, summary.length - 1) : summary;
};

export const isScheduleActive = (status) => {
    return _.includes([SCHEDULE_OPTIONS.ON, SCHEDULE_OPTIONS.NOW], status);
};

export const getGracePeriod = (graceValue, graceResolution, isGraceEnabled) => {
    const res = graceResolution.value;
    const power = res === "Hours" ? 3600 : (res === "Minutes" ? 60 : 1);
    const grace = isGraceEnabled ? graceValue : 0;
    return grace * power;
};

export const getFormGrace = (seconds) => {
    if (seconds < 60) return {count: seconds, resolution: {label: "Seconds", value: "Seconds"}};
    else if (seconds < 3600) return {count: (seconds / 60), resolution: {label: "Minutes", value: "Minutes"}};
    else return {count: (seconds / 3600), resolution: {label: "Hours", value: "Hours"}};
};

export const getDeleteOnTargetBool = (deleteOnTarget) => {
    return _.includes(deleteOnTarget, DELETE_FILES_FROM_TARGET_OPTION_TEXT);
};

export const getDeleteOnSourceBool = (deleteOnSource) => {
    return _.includes(deleteOnSource, DELETE_FILES_FROM_SOURCE_OPTION_TEXT);
};

export const getObjectTaggingBool = (objectTagging) => {
    return _.includes(objectTagging, OBJECT_TAGGING_USE_OPTION_TEXT);
};

export const getFormDeleteOnTarget = (isDeleteOnTarget) => {
    return isDeleteOnTarget ? DELETE_FILES_FROM_TARGET_OPTION_TEXT : NOT_DELETE_FILES_FROM_TARGET_OPTION_TEXT;
};

export const getFormDeleteOnSource = (isDeleteOnSource) => {
    return isDeleteOnSource ? DELETE_FILES_FROM_SOURCE_OPTION_TEXT : NOT_DELETE_FILES_FROM_SOURCE_OPTION_TEXT;
};

export const getFormObjectTagging = (isObjectTagging) => {
    return isObjectTagging ? OBJECT_TAGGING_USE_OPTION_TEXT : OBJECT_TAGGING_DISABLE_OPTION_TEXT;
};

export const getFileTypesObj = (typesArray) => {
    return {
        files: _.includes(typesArray, fileTypesMap.files),
        directories: _.includes(typesArray, fileTypesMap.directories),
        symlinks: _.includes(typesArray, fileTypesMap.symlinks)
    }
};

export const getFormFileTypes = (fileTypesObj) => {
    const typesArray = [];
    _.forEach(fileTypesObj, (value, type) => {
        if (value) typesArray.push(fileTypesMap[type]);
    });
    return typesArray;
};

export const getFormFileSize = (size) => {
    return (size < UI_MAX_FILE_SIZE_DEFAULT) ? convertBytesToSize(size) : convertBytesToSize(UI_MAX_FILE_SIZE_DEFAULT);
};

export const convertDateToString = (date) => {
    if (!date) return null;
    return moment(date).format("YYYY-MM-DD");
};

export const convertStringToDate = (string) => {
    if (!string || string === SETTINGS_DEFAULTS.files.minDate) return null;
    return new Date(string);
};

const getDateModifiedOption = (minDate, maxDate) => {
    if (minDate !== SETTINGS_DEFAULTS.files.minDate && maxDate) return DATE_MODIFIED_OPTIONS.RANGE;
    if (maxDate) return DATE_MODIFIED_OPTIONS.ONLY_MAX;
    if (minDate !== SETTINGS_DEFAULTS.files.minDate) return DATE_MODIFIED_OPTIONS.ONLY_MIN;
    return DATE_MODIFIED_OPTIONS.ALL;
};

const getMinDate = (values) => {
    if (_.includes([DATE_MODIFIED_OPTIONS.ONLY_MIN, DATE_MODIFIED_OPTIONS.RANGE], values.fileDateModifiedOption.value))
        return !values.fileDateModifiedMin ? SETTINGS_DEFAULTS.files.minDate : convertDateToString(values.fileDateModifiedMin);
    return SETTINGS_DEFAULTS.files.minDate;
};

const getMaxDate = (values) => {
    if (_.includes([DATE_MODIFIED_OPTIONS.ONLY_MAX, DATE_MODIFIED_OPTIONS.RANGE], values.fileDateModifiedOption.value))
        return convertDateToString(values.fileDateModifiedMax);
    return SETTINGS_DEFAULTS.files.maxDate;
};

export const getScheduleData = (values) => ({
    syncInMinutes: values.scheduleResolution.value === 'Minutes' ? Number.parseInt(values.scheduleInterval, 10) : 0,
    syncInDays: values.scheduleResolution.value === 'Days' ? Number.parseInt(values.scheduleInterval, 10) : 0,
    syncInHours: values.scheduleResolution.value === 'Hours' ? Number.parseInt(values.scheduleInterval, 10) : 0,
    isEnabled: isScheduleActive(values.scheduleStatus),
    syncWhenCreated: _.includes([SCHEDULE_OPTIONS.NOW, SCHEDULE_OPTIONS.ONCE], values.scheduleStatus),
    nextTime: getNextTimeIsoString(_.get(values.scheduleNextTime, 'value'))
});

export const getNextTimeHour = (isoString) => {

    const testingEnv = process.env.NODE_ENV === 'test';
    if (testingEnv) return "10:00";

    if (!isoString) return undefined;
    const nextTime = moment(isoString); //this is local time
    return nextTime.format("HH:mm");
};

export const getNextTimeIsoString = (nextTimeHour) => {

    const testingEnv = process.env.NODE_ENV === 'test';
    if (testingEnv) return MOCK_DATE_FOR_TESTS;

    const nowHour = moment().format("HH:mm");
    const isTomorrow = nextTimeHour <= nowHour;
    const date = isTomorrow ? moment().add(1, 'd').format("YYYY-MM-DD") : moment().format("YYYY-MM-DD");
    return moment(date + " " + nextTimeHour).toISOString();
};

const getFormScheduleStatus = (isEnabled, syncWhenCreated, isEditMode) => {
    if (isEditMode) return isEnabled ? SCHEDULE_OPTIONS.ON : SCHEDULE_OPTIONS.OFF;
    if (isEnabled && syncWhenCreated) return SCHEDULE_OPTIONS.NOW;
    if (!isEnabled ) return SCHEDULE_OPTIONS.ONCE;
    return SCHEDULE_OPTIONS.ON;
};