import {getInitialState, deployReducer} from "../../src/store/deploy/reducer";
import * as deployActions from "../../src/store/deploy/action_creators_consts";
import {initTests} from "../globals";
import {DEPLOY_STATUS_ERROR, DEPLOY_STATUS_PROGRESS, DEPLOY_STATUS_SUCCESS} from '../../src/consts';

beforeEach(() => {
    initTests();
    window.Intercom = jest.fn();
});

describe("deploy Reducer", () => {
    describe("Set credentials", () => {
        test("should set AWS credentials", () => {
            const state = deployReducer(getInitialState(), {
                type: deployActions.SET_AWS_CREDENTIALS,
                payload: {
                    accessKeyId: "accessKey",
                    secretAccessKey: "secretKey"
                },
            });
            expect(state.credentials.aws).toEqual({
                accessKeyId: "accessKey",
                secretAccessKey: "secretKey"
            })
        });
        test("should set Azure credentials", () => {
            const state = deployReducer(getInitialState(), {
                type: deployActions.SET_AZURE_CREDENTIALS,
                payload: {
                    accessToken: "accessToken"
                },
            });
            expect(state.credentials.azure).toEqual({
                accessToken: "accessToken"
            })
        });
    });

    describe("Add deployment", () => {
        const firstDeploy = {
            dataBrokerId: "1",
            dataBrokerName: "name",
            providerType: "AWS",
            deployDetails: {detail: "detail"},
            deployStatus: DEPLOY_STATUS_PROGRESS
        };
        const secondDeploy = {
            dataBrokerId: "2",
            dataBrokerName: "anotherName",
            providerType: "GCP",
            deployDetails: {detail: "detail"}
        };
        test("should add first deployment", () => {
            const state = deployReducer(getInitialState(), {
                type: deployActions.ADD_DEPLOYMENT,
                payload: firstDeploy,
            });
            expect(state.deployments).toEqual([{...firstDeploy, deployStatus: DEPLOY_STATUS_PROGRESS}]);
        });
        test("should add second deployment", () => {
            const initialState = getInitialState().set("deployments", [{...firstDeploy, deployStatus: DEPLOY_STATUS_PROGRESS}]);
            const finalState = deployReducer(initialState, {
                type: deployActions.ADD_DEPLOYMENT,
                payload: secondDeploy
            });
            expect(finalState.deployments).toEqual([
                {...firstDeploy, deployStatus: DEPLOY_STATUS_PROGRESS},
                {...secondDeploy, deployStatus: DEPLOY_STATUS_PROGRESS}
            ])
        });
    });

    describe("Set deployment status", () => {
        const firstDeploy = {
            dataBrokerId: "1",
            dataBrokerName: "name",
            providerType: "AWS",
            deployDetails: {detail: "detail"},
            deployStatus: DEPLOY_STATUS_PROGRESS
        };
        const secondDeploy = {
            dataBrokerId: "2",
            dataBrokerName: "anotherName",
            providerType: "GCP",
            deployDetails: {detail: "detail"}
        };
        test("should set status to error", () => {
            const initialState = getInitialState().set("deployments", [
                    {...firstDeploy, deployStatus: DEPLOY_STATUS_PROGRESS},
                    {...secondDeploy, deployStatus: DEPLOY_STATUS_PROGRESS}
                ]);
            const finalState = deployReducer(initialState, {
                type: deployActions.SET_DEPLOYMENT_STATUS,
                payload: {
                    dataBrokerId: "1",
                    deployStatus: DEPLOY_STATUS_ERROR,
                    errorMessage: "This is an error"
                }
            });
            expect(finalState.deployments).toEqual([
                    {...firstDeploy, deployStatus: DEPLOY_STATUS_ERROR, errorMessage: "This is an error"},
                    {...secondDeploy, deployStatus: DEPLOY_STATUS_PROGRESS}
                ]);
        });
        test("should set status to success", () => {
            const initialState = getInitialState().set("deployments", [
                {...firstDeploy, deployStatus: DEPLOY_STATUS_PROGRESS},
                {...secondDeploy, deployStatus: DEPLOY_STATUS_PROGRESS}
            ]);
            const finalState = deployReducer(initialState, {
                type: deployActions.SET_DEPLOYMENT_STATUS,
                payload: {
                    dataBrokerId: "2",
                    deployStatus: DEPLOY_STATUS_SUCCESS
                }
            });
            expect(finalState.deployments).toEqual([
                {...firstDeploy, deployStatus: DEPLOY_STATUS_PROGRESS},
                {...secondDeploy, deployStatus: DEPLOY_STATUS_SUCCESS}
            ]);
        });
    });
});
