import {syncDefaultState} from "../../src/store/wizard/reducer";
import {globalDefaultState, globalReducer} from "../../src/store/global/reducer";
import * as consts from "../../src/consts";
import * as globalActions from "../../src/store/global/action_creators_consts";
import * as wizardActions from "../../src/store/wizard/action_creators_consts";
import {bootGoogleAnalytics} from "../../src/utils/analytics";
import {initTests} from "../globals";

const withAuth = () => syncDefaultState({}).set("auth", {succeeded: true, data: {email: "blabla"}});
const withGlobalAuth = () => globalDefaultState().set("auth", {succeeded: true, data: {email: "blabla"}});

jest.mock("./../../src/utils/analytics");

beforeEach(() => {
    initTests();
    window.Intercom = jest.fn();
});

describe("global Reducer", () => {
    describe("User Login", () => {
        test("should update user info on auth", () => {
            const state = globalReducer(syncDefaultState(), {
                type: globalActions.AUTH_REFRESH_SSO_SUCCESS,
                payload: {
                    idTokenPayload: {
                        "http://cloud.netapp.com/user_metadata": {name: "name1", email: "email1", company: "company1"},
                        sub: "123"
                    }
                },
            });
            expect(state.auth).toEqual({
                data: {userName: "name1", email: "email1", company: "company1", uid: "123"},
                succeeded: true,
                inProgress: false,
                failed: false
            })
        });
    });

    describe("Get Accounts - Update _accounts with built state", () => {
        test("should update inProgress", () => {

            const state = globalReducer(withAuth(), {type: globalActions.GET_ACCOUNTS, status: globalActions.STATUS_PROGRESS, id: 1, extra: {}});

            expect(state._accounts).toEqual({
                id: 1,
                failed: false,
                inProgress: true,
                succeeded: false,
                data: undefined,
                error: undefined,
            })
        });

        test("should update failed", () => {
            const state = globalReducer(withGlobalAuth(), {
                type: globalActions.GET_ACCOUNTS,
                payload: {data: {message: "error"}},
                status: globalActions.STATUS_ERROR,
                id: 1,
                extra: {}
            });

            expect(state._accounts).toEqual({
                id: 1,
                failed: true,
                inProgress: false,
                succeeded: false,
                data: undefined,
                error: {data: {message: "error"}},
            });
            expect(state.auth.accountError).toEqual("error")
        });

        test("should update success", () => {
            const state = globalReducer(withGlobalAuth(), {
                type: globalActions.GET_ACCOUNTS,
                payload: [{
                    accountId: "123",
                    name:"aaa"
                }],
                status: globalActions.STATUS_SUCCESS,
                id: 1,
                extra: {}
            });

            expect(state._accounts).toEqual({
                id: 1,
                failed: false,
                inProgress: false,
                succeeded: true,
                error: undefined,
                data: [{accountId: "123", name: "aaa"}]
            });
        });
    });

    describe("Get Account Info - Update _accountInfo with built state", () => {
        test("should update inProgress", () => {

            const state = globalReducer(withAuth(), {
                type: globalActions.GET_ACCOUNT_INFO,
                status: globalActions.STATUS_PROGRESS,
                id: 1,
                extra: {}
            });

            expect(state._accountInfo).toEqual({
                id: 1,
                failed: false,
                inProgress: true,
                succeeded: false,
                data: undefined,
                error: undefined,
            })
        });

        test("should update failed", () => {
            const state = globalReducer(withGlobalAuth(), {
                type: globalActions.GET_ACCOUNT_INFO,
                payload: {data: {message: "error"}},
                status: wizardActions.STATUS_ERROR,
                id: 1,
                extra: {}
            });

            expect(state._accountInfo).toEqual({
                id: 1,
                failed: true,
                inProgress: false,
                succeeded: false,
                data: undefined,
                error: {data: {message: "error"}}
            });
            expect(state.auth.accountError).toEqual("error")
        });

        test("should update success", () => {
            const accountData = {
                accountId: "123",
                name: "aaa",
                noCharge: false,
                daysLeftForTrial: 14,
                subscriptionStatus: consts.SAAS_REGISTRATION_STATUS_INIT,
                supportRegistrationStatus: consts.SUPPORT_NOT_REGISTERED
            };
            const state = globalReducer(withGlobalAuth(), {
                type: globalActions.GET_ACCOUNT_INFO,
                payload: accountData,
                status: globalActions.STATUS_SUCCESS,
                id: 1,
                extra: {}
            });

            expect(state._accountInfo).toEqual({
                id: 1,
                failed: false,
                inProgress: false,
                succeeded: true,
                error: undefined,
                data: accountData
            });
            expect(state.auth.data).toEqual({
                email: "blabla",
                ...accountData,
                subscribed: false,
                unsubscribed: true,
                failedToSubscribe: false,
                isPayGoSubscription: false,
                subscribing: false
            })
        });

    });

    describe("Clear account data", () => {
        test("should clear the account's resources data", () => {
            const startState = withAuth()
                .set("auth", {data: "1"})
                .set("_accounts", {data: "1"})
                .set("_accountInfo", {data: "1"})
                .set("_relationships", {data: "1"})
                .set("_dataBrokers", {data: "1"})
                .set("_licenseSettings", {data: "1"});

            const updatedState = globalReducer(startState, {type: globalActions.CLEAR_ACCOUNT_DATA});
            expect(updatedState).toEqual(expect.not.objectContaining({
                _relationships: {data: "1"},
                _dataBrokers: {data: "1"},
                _licenseSettings: {data: "1"}
            }));
        });
    });

    describe("Boot analytics", () => {
        test("should boot analytics", () => {
            bootGoogleAnalytics.mockReturnValue();

            globalReducer(withGlobalAuth().setIn(["_routing","location","query","analytics"], "true"), {type: globalActions.BOOT_ANALYTICS, payload: {isCmIntegration: false}});

            expect(bootGoogleAnalytics).toHaveBeenCalledWith("Stand alone app", "blabla");
        })
    });

    describe("Logout", () => {
        test("should shutdown intercom, remove auth from state", () => {
            const state = globalReducer(withGlobalAuth(), {type: globalActions.LOGOUT});

            expect(state.auth).toEqual(undefined);
            expect(window.Intercom.mock.calls[0][0]).toEqual("shutdown");
        })
    });

    describe("Add BYOL License", () => {

        test("should add a license to license array", () => {
            const state = globalReducer(withGlobalAuth(), {
                type: globalActions.ADD_BYOL_LICENSE,
                status: globalActions.STATUS_SUCCESS,
                id: 1,
                payload: {
                    expirationDate: 0,
                    reservedRelationships: 20,
                    serialNumber: "991"
                }
            });

            expect(state._licenses.data.licenses).toEqual([
                    {
                        expirationDate: 0,
                        reservedRelationships: 20,
                        serialNumber: "991"
                    }
                ]
            );
        });

    });

    describe("Get License Settings", () => {

        const initialState = globalDefaultState().set("auth", {succeeded: true})
            .set("_licenses", {succeeded: true, data: {licenses: []}});

        const onTrialState = initialState.setIn(["auth", "data", "daysLeftForTrial"], 7);
        const trialEndedState = initialState.setIn(["auth", "data", "daysLeftForTrial"], 0);

        const onTrialNotSubscribedState = onTrialState.setIn(["auth", "data", "subscriptionStatus"], 'unsubscribed-init');
        const trialEndedNotSubscribedState = trialEndedState.setIn(["auth", "data", "subscriptionStatus"], 'unsubscribed-init');
        const initialStateSubscribed = onTrialState.setIn(["auth", "data", "subscriptionStatus"], consts.SAAS_REGISTRATION_STATUS_SUBSCRIBE_SUCCESS);

        const reduce = (state, payload) => {
            return globalReducer(state, {
                type: globalActions.GET_LICENSE_SETTINGS,
                status: globalActions.STATUS_SUCCESS,
                id: 1,
                payload
            });
        };

        test("should construct the available license for Free Trial", () => {
            const payload = {
                freeTrial: true,
                existingRelationships: 2
            };
            const licenseType = consts.LICENSE_TYPE_FREE_TRIAL;

            const state = reduce(onTrialNotSubscribedState, payload);

            // user doesn't need to pay - still in free trial
            expect(state._licenseSettings.data).toEqual(Object.assign({}, payload, {licenseType, needToPay: false}));
        });

        describe("should construct the available license for PayGo", () => {
            const payload = {
                freeTrial: false,
                existingRelationships: 3,
                reservedRelationships: 0,
                totalReservedRelationships: 0,
                chargedRelationships: 3
            };
            const licenseType = consts.LICENSE_TYPE_PAYGO;

            test("when not subscribed and on trial", () => {
                const state = reduce(onTrialNotSubscribedState, payload);

                // user needs to pay - the marketplace charged relationships are not payed with subscription
                expect(state._licenseSettings.data).toEqual(Object.assign({}, payload, {licenseType, needToPay: true}));

                // user should see trial page and not be blocked
                expect(state._ui.showTrialPage).toEqual(true);
                expect(state._ui.blocked).toEqual(false);
            });

            test("when not subscribed and not on trial", () => {
                const state = reduce(trialEndedNotSubscribedState, payload);

                // user needs to pay - the marketplace charged relationships are not payed with subscription
                expect(state._licenseSettings.data).toEqual(Object.assign({}, payload, {licenseType, needToPay: true}));

                // user should not see trial page and be blocked
                expect(state._ui.showTrialPage).toEqual(false);
                expect(state._ui.blocked).toEqual(true);
            });

            test("when subscribed", () => {
                const state = reduce(initialStateSubscribed, payload);

                // user doesn't need to pay - the marketplace charged relationships are payed with subscription
                expect(state._licenseSettings.data).toEqual(Object.assign({}, payload, {
                    licenseType,
                    needToPay: false
                }));

                // user should not see trial page and not be blocked
                expect(state._ui.showTrialPage).toEqual(false);
                expect(state._ui.blocked).toEqual(false);
            });

        });

        describe("should construct the available license for BYOL", () => {
            const payload = {
                freeTrial: false,
                existingRelationships: 19,
                reservedRelationships: 19,
                totalReservedRelationships: 20,
                chargedRelationships: 0
            };
            const licenseType = consts.LICENSE_TYPE_BYOL;

            const addLicenseToState = (state) => state.setIn(["_licenses", "data", "licenses"], ['one license']);

            test("when not subscribed and on trial", () => {
                const stateWithLicense = addLicenseToState(onTrialNotSubscribedState);
                const nextState = reduce(stateWithLicense, payload);

                // user doesn't need to pay - all relationships are within the license capacity
                expect(nextState._licenseSettings.data).toEqual(Object.assign({}, payload, {
                    licenseType,
                    needToPay: false
                }));

                // user should not see trial page and not be blocked
                expect(nextState._ui.showTrialPage).toEqual(false);
                expect(nextState._ui.blocked).toEqual(false);
            });

            test("when not subscribed and not on trial", () => {
                const stateWithLicense = addLicenseToState(trialEndedNotSubscribedState);
                const nextState = reduce(stateWithLicense, payload);

                // user doesn't need to pay - all relationships are within the license capacity
                expect(nextState._licenseSettings.data).toEqual(Object.assign({}, payload, {
                    licenseType,
                    needToPay: false
                }));

                // user should not see trial page and not be blocked
                expect(nextState._ui.showTrialPage).toEqual(false);
                expect(nextState._ui.blocked).toEqual(false);
            });

            test("when subscribed", () => {
                const stateWithLicense = addLicenseToState(initialStateSubscribed);
                const nextState = reduce(stateWithLicense, payload);

                // user doesn't need to pay - all relationships are within the license capacity
                expect(nextState._licenseSettings.data).toEqual(Object.assign({}, payload, {
                    licenseType,
                    needToPay: false
                }));

                // user should not see trial page and not be blocked
                expect(nextState._ui.showTrialPage).toEqual(false);
                expect(nextState._ui.blocked).toEqual(false);
            });
        });

        describe("should construct the available license for PayGo with BYOL", () => {
            const payload = {
                freeTrial: false,
                existingRelationships: 24,
                reservedRelationships: 20,
                totalReservedRelationships: 20,
                chargedRelationships: 4
            };
            const licenseType = consts.LICENSE_TYPE_PAYGO_WITH_BYOL;

            test("when not subscribed and on trial", () => {
                const state = reduce(onTrialNotSubscribedState, payload);

                // user needs to pay - there are marketplace charged relationships that are not payed with subscription
                expect(state._licenseSettings.data).toEqual(Object.assign({}, payload, {licenseType, needToPay: true}));

                // user should see trial page and not be blocked
                expect(state._ui.showTrialPage).toEqual(true);
                expect(state._ui.blocked).toEqual(false);
            });

            test("when not subscribed and not on trial", () => {
                const state = reduce(trialEndedNotSubscribedState, payload);

                // user needs to pay - there are marketplace charged relationships that are not payed with subscription
                expect(state._licenseSettings.data).toEqual(Object.assign({}, payload, {licenseType, needToPay: true}));

                // user should not see trial page and not be blocked, just can't create a relationship
                expect(state._ui.showTrialPage).toEqual(false);
                expect(state._ui.blocked).toEqual(false);
                expect(state._ui.shouldBlockCreatingRelationship).toBeDefined(); // not null or undefined
            });

            test("when subscribed", () => {
                const state = reduce(initialStateSubscribed, payload);

                // user doesn't need to pay - the marketplace charged relationships are payed with subscription
                expect(state._licenseSettings.data).toEqual(Object.assign({}, payload, {
                    licenseType,
                    needToPay: false
                }));

                // user should not see trial page and not be blocked
                expect(state._ui.showTrialPage).toEqual(false);
                expect(state._ui.blocked).toEqual(false);
            });
        });

    });

    describe("Update support info",() => {
        test("should update support info in auth", () => {
            const state = globalReducer(withGlobalAuth(), {
                type: globalActions.UPDATE_SUPPORT_INFO,
                id: 1,
                payload: {
                    [consts.PAYLOAD_UPDATE_SUPPORT_INFO]: {
                        supportInfo: {
                            status: consts.SUPPORT_REGISTERED,
                            serialNumber: "1234"
                        }
                    }
                }
            });
            expect(state.auth.data).toEqual({
                    email: "blabla",
                    supportRegistrationStatus: consts.SUPPORT_REGISTERED,
                    supportSerialNumber: "1234",
                }
            );
        })
    });

    describe("Register to support",() => {
        test("should update support status to in-progress in auth", () => {
            const state = globalReducer(withGlobalAuth(), {
                type: globalActions.START_REGISTER_TO_SUPPORT,
                status: globalActions.STATUS_SUCCESS,
                id: 1
            });
            expect(state.auth.data).toEqual({
                    email: "blabla",
                    supportRegistrationStatus: consts.SUPPORT_IN_PROGRESS
                }
            );
        })
    });

    describe("Update account info",() => {
        test("should update account info", () => {
            const initialState = globalDefaultState().setIn(["auth", "data", "noCharge"], true).setIn(["_accountInfo", "data"], {accountId: "123", noCharge: true});
            const action = {
                type: globalActions.UPDATE_ACCOUNT_INFO,
                payload: {[consts.PAYLOAD_UPDATE_ACCOUNT_INFO]: {accountId: "123", noCharge: false}}
            };
            const finalState = globalReducer(initialState, action);
            expect(finalState._accountInfo.data.noCharge).toEqual(false);
            expect(finalState.auth.data.noCharge).toEqual(false);
        });
    });

    describe("Update cm integration mode",() => {
        test("should update cm integration to true", () => {
            const initialState = withGlobalAuth().setIn(["_ui", "isCmIntegration"], false);
            const action = {
                type: globalActions.UPDATE_CM_INTEGRATION
            };
            const finalState = globalReducer(initialState, action);
            expect(finalState._ui.isCmIntegration).toEqual(true);
        });
    });

    describe("Update dashboard view",() => {
        test("should update dashboard view to table view", () => {
            const initialState = withGlobalAuth();
            const action = {
                type: globalActions.UPDATE_DASHBOARD_VIEW,
                payload: consts.TABLE_VIEW
            };
            const finalState = globalReducer(initialState, action);
            expect(finalState._ui.dashboardView).toEqual(consts.TABLE_VIEW);
        });
        test("should update dashboard view to cards view", () => {
            const initialState = withGlobalAuth().setIn(["_ui", "dashboardView"], consts.TABLE_VIEW);
            const action = {
                type: globalActions.UPDATE_DASHBOARD_VIEW,
                payload: consts.CARDS_VIEW
            };
            const finalState = globalReducer(initialState, action);
            expect(finalState._ui.dashboardView).toEqual(consts.CARDS_VIEW);
        });
    });
    describe("Update cm integration input details",() => {
        test("should update cm integration input details", () => {
            const initialState = withGlobalAuth();
            const payload = {workingEnvironmentName: "we1", workingEnvironmentType: "ANF", isSource: true, viewDashboard: false};
            const action = {
                type: globalActions.SET_CM_INTEGRATION_DETAILS,
                payload
            };
            const finalState = globalReducer(initialState, action);
            expect(finalState._cmIntegration.inputDetails).toEqual(payload);
        });
    });
    describe("Clear cm integration input details",() => {
        test("should clear cm integration input details", () => {
            const initialState = withGlobalAuth().setIn(["_cmIntegration", "inputDetails"], {workingEnvironmentName: "we1", workingEnvironmentType: "ANF", isSource: true, viewDashboard: false});
            const action = {
                type: globalActions.CLEAR_CM_INTEGRATION_DETAILS,
            };
            const finalState = globalReducer(initialState, action);
            expect(finalState._cmIntegration.inputDetails).toEqual(null);
        });
    });
});
