import {syncDefaultState, syncReducer} from "../../src/store/wizard/reducer"
import * as wizardActions from "../../src/store/wizard/action_creators_consts"
import {routesContainer} from "../../src/store/global/action_creators_consts"
import * as consts from "./../../src/consts";
import {END_POINT_NAMES_MAP, FM_AGENT} from "./../../src/consts";
import {initTests} from "./../globals";
import {encodeAll} from '../../src/utils/encoders';

jest.mock("./../../src/utils/analytics");

beforeEach(() => {
    initTests();
    window.Intercom = jest.fn();
});


const defaultState = () => syncDefaultState({});

const stepsOrder = ["dataBroker", "s3Buckets", "nfs", "directories", "settings", "review"];

const stepInfoById = {
    dataBroker: {name: FM_AGENT, path: 'agents'},
    settings: {name: 'Settings', path: 'settings'},
    review: {name: 'Review', path: 'review'},
    nfs: {name: END_POINT_NAMES_MAP.NFS, path: 'nfs'},
    directories: {name: 'Directories', path: 'exports'},
    s3Buckets: {name: 'AWS S3 Bucket', path: 'buckets'}
};

const wizardInfo = {source: {protocol: 's3'}, target: {protocol: 'nfs'}};

const callSyncReducer = (state, action, urlParams = {location: {pathname: ""}, params: {}}) => {
    const updatedState = state.set("_routing", urlParams);
    return syncReducer(updatedState, action);
};


describe("reducer", () => {
    test("should return the initial state on call with undefined and no matched action", () => {
        expect(syncReducer(undefined, {}))
            .toEqual(syncDefaultState());
    });

    describe("UI STATE", () => {
        test("should push the payload on push", () => {
            const state = syncReducer(defaultState().set("_ui", {states: []}), {
                type: wizardActions.PUSH_UI_STATE,
                payload: {test: "test"}
            });

            expect(state._ui.states).toEqual([{test: "test"}]);
        });

        test("should start with an empty list if there is nothing when being pushed", () => {
            const state = syncReducer(defaultState().set("_ui", {}), {
                type: wizardActions.PUSH_UI_STATE,
                payload: {test: "test"}
            });

            expect(state._ui.states).toEqual([{test: "test"}]);
        });

        test("should pop on pop", () => {
            const state = syncReducer(defaultState().set("_ui", {states: [{test: "test"}]}), {type: wizardActions.POP_UI_STATE});
            expect(state._ui.states).toEqual([])
        });
    });

    describe("Get Data Brokers - Update _dataBrokers with built state", () => {
        test("should update inProgress", () => {

            const state = syncReducer(defaultState(), {type: wizardActions.GET_DATA_BROKERS, status: wizardActions.STATUS_PROGRESS, id: 1, extra: {}});

            expect(state._dataBrokers).toEqual({
                id: 1,
                failed: false,
                inProgress: true,
                succeeded: false,
                data: undefined,
                error: undefined,
            })
        });

        test("should update failed", () => {
            const state = syncReducer(defaultState().setIn(["_dataBrokers", "id"], 1), {
                type: wizardActions.GET_DATA_BROKERS,
                payload: new Error("error"),
                status: wizardActions.STATUS_ERROR,
                id: 1,
                extra: {}
            });

            expect(state._dataBrokers).toEqual({
                id: 1,
                failed: true,
                inProgress: false,
                succeeded: false,
                data: undefined,
                error: new Error("error"),
            })
        });

        test("should update success", () => {
                const state = syncReducer(defaultState().setIn(["_dataBrokers", "id"], 1), {
                    type: wizardActions.GET_DATA_BROKERS,
                    payload: [{
                        creationFrom: "A few seconds ago",
                        isWaiting: true,
                        isFailed: false,
                        isReady: false
                    }],
                    status: wizardActions.STATUS_SUCCESS,
                    id: 1,
                    extra: {}
                });

                expect(state._dataBrokers).toHaveProperty("id", 1);
                expect(state._dataBrokers).toHaveProperty("failed", false);
                expect(state._dataBrokers).toHaveProperty("inProgress", false);
                expect(state._dataBrokers).toHaveProperty("succeeded", true);
                expect(state._dataBrokers).toHaveProperty("error", undefined);

                const dataBroker = state._dataBrokers.data[0];

                if (dataBroker) {
                    expect(dataBroker).toHaveProperty("isWaiting", true);
                    expect(dataBroker).toHaveProperty("isReady", false);
                    expect(dataBroker).toHaveProperty("isFailed", false);
                    expect(dataBroker).toHaveProperty("ui.type.name", "AWS");
                    expect(dataBroker).toHaveProperty("ui.type.displayName", "AWS Data Broker");
                }
            });
    });

    describe("Create Data Broker - Update _dataBrokers in case of success", () => {

        test("should update success", () => {
            const state = syncReducer(defaultState().setIn(["_dataBrokers", "data"], []), {
                type: wizardActions.DATA_BROKER,
                payload: {id: "123", fileLink: "link", type: "AWS", name: "name"},
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra: {}
            });

            expect(state._dataBrokers.data).toEqual([{
                id: "123",
                fileLink: "link",
                type: "AWS",
                name: "name",
                creationFrom: "A few seconds ago",
                isDisabled: false,
                isFailed: false,
                isReady: false,
                isWaiting: true,
                ui: {transferRate: "", type: {name: 'AWS', displayName: 'AWS Data Broker', disabledIcon: '', colorIcon: '', circledIcon: '', disabledCircledIcon: '', "company": "Amazon", "providerName": "Amazon Web Services"}}
            }]);
        });
    });

    describe("Delete Data Broker - update _deleteDataBroker with built state", () => {
        test("should update inProgress", () => {
            const state = syncReducer(defaultState(), {type: wizardActions.DELETE_DATA_BROKER, status: wizardActions.STATUS_PROGRESS, id: 1, extra: {}});

            expect(state._deleteDataBroker).toEqual({
                id: 1,
                failed: false,
                inProgress: true,
                succeeded: false,
                data: undefined,
                error: undefined,
            })
        });
        test("should update failed", () => {
            const state = syncReducer(defaultState(), {
                type: wizardActions.DELETE_DATA_BROKER,
                payload: new Error("error"),
                status: wizardActions.STATUS_ERROR,
                id: 1,
                extra: {}
            });

            expect(state._deleteDataBroker).toEqual({
                id: 1,
                failed: true,
                inProgress: false,
                succeeded: false,
                data: undefined,
                error: new Error("error"),
            })
        });
        test("should update success", () => {
            const state = syncReducer(defaultState(), {
                type: wizardActions.DELETE_DATA_BROKER,
                payload: {something: "something"},
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra: {}
            });

            expect(state._deleteDataBroker).toHaveProperty("id", 1);
            expect(state._deleteDataBroker).toHaveProperty("failed", false);
            expect(state._deleteDataBroker).toHaveProperty("inProgress", false);
            expect(state._deleteDataBroker).toHaveProperty("succeeded", true);
            expect(state._deleteDataBroker).toHaveProperty("error", undefined);
        });

        });

    describe("Update Data Broker status", () => {

        test("should update _dataBrokers", () => {
            const initialState = defaultState()
                .setIn(["_dataBrokers", "data"], [{id: "1"}]);
            const finalState = syncReducer(initialState, {
                type: wizardActions.UPDATE_DATA_BROKER_STATUS,
                payload: {[consts.PAYLOAD_DATA_BROKER_STATUS]: {id: "1", status: consts.BROKER_STATUS_COMPLETE, placement: "1.1.1.1"}}
            });

            expect(finalState._dataBrokers.data).toEqual([{
                id: "1",
                isDisabled: false,
                isFailed: false,
                isReady: true,
                isWaiting: false,
                creationFrom: "A few seconds ago",
                placement: "1.1.1.1",
                status: consts.BROKER_STATUS_COMPLETE
            }]);
        });
    });

    describe("Sync Now - update relationship.ui.status" , () => {
        const baseRelationship = {
            id: 1,
            activity: {status: "COMPLETED"},
            target: {protocol: 'nfs', nfs: {}},
            source: {protocol: 'nfs', nfs: {}},
            group: {dataBrokers: []}
        };
        const stateOrigin = defaultState().set("_relationships", {data: [baseRelationship]});

        test("should update STARTING if in progress", () => {
            const state = syncReducer(stateOrigin, {type: wizardActions.SYNC_NOW, status: wizardActions.STATUS_PROGRESS, extra: {relationshipId: 1}});
            expect(state._relationships.data[0].ui.status).toEqual("STARTING")
        });

        test("should leave COMPLETED if error", () => {
            const state = syncReducer(stateOrigin, {type: wizardActions.SYNC_NOW, status: wizardActions.STATUS_ERROR, extra: {relationshipId: 1}});
            expect(state._relationships.data[0].ui.status).toEqual("COMPLETED")
        });
    });

    describe("Get relationships", () => {

        const nfsToS3RelationshipResponse = {
            activity: {
                type: "Initial Copy",
                status: "RUNNING",
                capacityCopiedBytes: 900,
                executionTime: 72000000,
                lastMessageTime: undefined,
                progress: 94,
                timeEstimation: 60
            },
            source: {protocol: "nfs", "nfs" : {host: "1.1.1.1", export: "/data1"}},
            target: {protocol: "s3" , "s3": {bucket: "small"}},
            host: "nfsServer",
            actualTarget: "s3://small/1.1.1.1:@_data1",
            dataBroker: {transferRate: 350000000},
            group: {dataBrokers: [
                    {id: '123', placement: {awsAccountId: '123'}, transferRate: 350000000},
                    {id: '456'}
                ]},
            settings: {
                schedule: {
                    isEnabled: true,
                    syncInMinutes: 5,
                    syncInHours: 0,
                    syncInDays: 0
                },
            }


        };

        const nfsToS3RelationshipResponseProcessed = Object.assign({}, nfsToS3RelationshipResponse, {
            group: {dataBrokers: [
                    {id: '123', placement: {awsAccountId: '123'}, creationFrom: "A few seconds ago", isDisabled: false, isFailed: false, isReady: true, isWaiting: false, transferRate: 350000000},
                    {id: '456', creationFrom: "A few seconds ago", isDisabled: false, isFailed: false, isReady: false, isWaiting: true},
                ]}
        });

        const nfsToS3RelationshipResult = {
            ...nfsToS3RelationshipResponseProcessed,
            ui: {
                actualTarget: 's3://small/1.1.1.1:@_data1',
                copyDoneByCloudProvider: false,
                dataBrokers: [
                    {id: '123', placement: {awsAccountId: '123'}, creationFrom: "A few seconds ago", isDisabled: false, isFailed: false, isReady: true, isWaiting: false, transferRate: 350000000,
                        ui: {transferRate: "(333.79MB/s)", type: {name: 'AWS', displayName: 'AWS Data Broker', disabledIcon: '', colorIcon: '', circledIcon: ''}}},
                    {id: '456', creationFrom: "A few seconds ago", isDisabled: false, isFailed: false, isReady: false, isWaiting: true,
                        ui: {transferRate: "", type: {name: 'AWS', displayName: 'AWS Data Broker', disabledIcon: '', colorIcon: '', circledIcon: ''}}},
                ],
                estimatedTimeText: "About a minute remaining",
                host: 'nfsServer',
                isScheduleOn: "ON",
                pendingSyncStartTime: "",
                lastTransferredHeaderText: 'Initial Copy | Duration: 20 hours',
                progress: '(94%)',
                showUnknown: false,
                source: {
                    nfs: {
                        'export': '/data1',
                        host: '1.1.1.1'
                    },
                    protocol: 'nfs'
                },
                status: 'RUNNING',
                statusText: 'Sync In Progress',
                target: {
                    protocol: 's3',
                    s3: {
                        bucket: 'small'
                    }
                },
                type: 'Initial Copy',
                failureMessage: undefined,
                tags: undefined
            },
            lastCopyMessage: {},
            lastRequests: [],
            lastScanMessage: {},
            lastSyncMessage: undefined
        };

        test("should update inProgress", () => {
            const state = syncReducer(defaultState(), {
                type: wizardActions.GET_RELATIONSHIPS,
                status: wizardActions.STATUS_PROGRESS,
                id: 1
            });

            expect(state._relationships).toEqual({
                failed: false,
                error: undefined,
                inProgress: true,
                succeeded: false,
                id: 1,
                data: undefined,
                requestId: undefined
            })
        });

        test("should update failed", () => {
            const state = syncReducer(defaultState().setIn(["_relationships", "id"], 1), {
                type: wizardActions.GET_RELATIONSHIPS,
                status: wizardActions.STATUS_ERROR,
                id: 1
            });

            expect(state._relationships).toEqual({
                failed: true,
                inProgress: false,
                succeeded: false,
                data: undefined,
                id: 1,
                error: undefined,
                requestId: undefined
            })
        });

        test("should update success", () => {
            const state = syncReducer(defaultState().setIn(["_relationships", "id"], 1), {
                type: wizardActions.GET_RELATIONSHIPS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                payload: [nfsToS3RelationshipResponse]
            });

            expect(state._relationships).toMatchObject({
                failed: false,
                inProgress: false,
                succeeded: true,
                error: undefined,
                id: 1,
                data: [nfsToS3RelationshipResult]
            })
        });
    });

    describe("Show exports", () => {
        const host = '1.1.1.1';
        const hostEncoded = encodeAll(host);
        const dataBrokerId = 1;
        const extra = {host, dataBrokerId};

        const buildStateWithExportsFolders = () => defaultState()
            .set("_showExports", {
                [`${hostEncoded}-${dataBrokerId}`] : {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        '-': {
                            data: [{
                                actualPath: '/data1',
                                host,
                                path: '/data1'
                            }]/*,
                             retrievedAll: true*/
                        },
                    },
                    id: 1
                }
            });

        const buildStateWithExportsDeepFolders = () => defaultState()
            .set("_showExports", {
                [`${hostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        '-': {
                            data: [{
                                actualPath: '/data1',
                                host,
                                path: '/data1'
                            }]/*,
                             retrievedAll: true*/
                        },
                        '/logs': {
                            data: [{
                                actualPath: '/logs/folder1',
                                path: '/folder1'
                            }]
                        }
                    },
                    id: 1
                }
            });

        test("should update inProgress", () => {
            const state = syncReducer(defaultState(), {
                type: wizardActions.SHOW_EXPORTS,
                status: wizardActions.STATUS_PROGRESS,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: true,
                    succeeded: false,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should update failed", () => {
            const state = syncReducer(defaultState(), {
                type: wizardActions.SHOW_EXPORTS,
                status: wizardActions.STATUS_ERROR,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    host,
                    failed: true,
                    inProgress: false,
                    succeeded: false,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should update success", () => {
            const state = syncReducer(defaultState(), {
                type: wizardActions.SHOW_EXPORTS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should keep all the folders from the state ( host & dataBrokerId are identical)", () => {
            const state = syncReducer(buildStateWithExportsFolders(), {
                type: wizardActions.SHOW_EXPORTS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {data: [{actualPath: '/data1', host, path: '/data1'}]}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should keep all the folders from the state -deep folders check ( host & dataBrokerId are identical)", () => {
            const state = syncReducer(buildStateWithExportsDeepFolders(), {
                type: wizardActions.SHOW_EXPORTS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {data: [{actualPath: '/data1', host, path: '/data1'}]},
                        '/logs': {
                            data: [{
                                actualPath: '/logs/folder1',
                                path: '/folder1'
                            }]
                        }
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should not keep all the folders from the state ( host is different)", () => {
            const newHost = "2.2.2.2";
            const newHostEncoded = encodeAll(newHost);
            const extra = {dataBrokerId, host: newHost};

            const initialState = buildStateWithExportsFolders();
            const state = syncReducer(initialState, {
                type: wizardActions.SHOW_EXPORTS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    "failed": false,
                    "folders": {
                        "-": {
                            "data": [
                                {
                                    "actualPath": "/data1",
                                    "host": "1.1.1.1",
                                    "path": "/data1"
                                }
                            ]
                        }
                    },
                    "host": "1.1.1.1",
                    "id": 1,
                    "inProgress": false,
                    "succeeded": true
                },
                [`${newHostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    host: newHost,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should not keep all the folders from the state -deep folders check ( host is different)", () => {
            const newHost = "2.2.2.2";
            const newHostEncoded = encodeAll(newHost);
            const extra = {dataBrokerId, host: newHost};

            const state = syncReducer(buildStateWithExportsDeepFolders(), {
                type: wizardActions.SHOW_EXPORTS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    "failed": false,
                    "folders": {
                        "-": {
                            "data": [
                                {
                                    "actualPath": "/data1",
                                    "host": "1.1.1.1",
                                    "path": "/data1"
                                }
                            ]
                        },
                        "/logs": {
                            "data": [
                                {
                                    "actualPath": "/logs/folder1",
                                    "path": "/folder1"
                                }
                            ]
                        }
                    },
                    host,
                    "id": 1,
                    "inProgress": false,
                    "succeeded": true
                },
                [`${newHostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    host: newHost,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should not keep all the folders from the state ( dataBrokerId is different)", () => {
            const newDataBrokerId = 2;
            const extra = {dataBrokerId: newDataBrokerId, host};
            const state = syncReducer(buildStateWithExportsFolders(), {
                type: wizardActions.SHOW_EXPORTS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual({
                [`${hostEncoded}-${dataBrokerId}`] : {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        '-': {
                            data: [{
                                actualPath: '/data1',
                                host,
                                path: '/data1'
                            }]/*,
                             retrievedAll: true*/
                        },
                    },
                    id: 1
                },
                [`${hostEncoded}-${newDataBrokerId}`]: {
                    dataBrokerId: newDataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should not keep all the folders from the state -deep folders check ( dataBrokerId is different)", () => {
            const newDataBrokerId = 2;
            const extra = {dataBrokerId: newDataBrokerId, host};
            const state = syncReducer(buildStateWithExportsDeepFolders(), {
                type: wizardActions.SHOW_EXPORTS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        '-': {
                            data: [{
                                actualPath: '/data1',
                                host,
                                path: '/data1'
                            }]/*,
                             retrievedAll: true*/
                        },
                        '/logs': {
                            data: [{
                                actualPath: '/logs/folder1',
                                path: '/folder1'
                            }]
                        }
                    },
                    id: 1
                },
                [`${hostEncoded}-${newDataBrokerId}`]: {
                    dataBrokerId: newDataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });


    });

    describe("Show shares", () => {
        const username = 'testUser';
        const host = '1.1.1.1';
        const hostEncoded = encodeAll(host);
        const dataBrokerId = 1;
        const domain = 'netapp';
        const password = 'Netapp234';
        const extra = {host, dataBrokerId, username, password, domain};

        const buildStateWithSharesFolders = () => defaultState()
            .set("_showShares", {
                [`${hostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    domain,
                    host,
                    username,
                    password,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        '-': {
                            data: [{
                                actualPath: '/data1',
                                host,
                                path: '/data1'
                            }]/*,
                             retrievedAll: true*/
                        }
                    },
                    id: 1
                }
            });

        test("should update inProgress", () => {
            const state = syncReducer(defaultState(), {
                type: wizardActions.SHOW_SHARES,
                status: wizardActions.STATUS_PROGRESS,
                id: 1,
                extra
            });

            expect(state._showShares).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    host,
                    username,
                    password,
                    domain,
                    failed: false,
                    inProgress: true,
                    succeeded: false,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should update failed", () => {
            const state = syncReducer(defaultState(), {
                type: wizardActions.SHOW_SHARES,
                status: wizardActions.STATUS_ERROR,
                id: 1,
                extra
            });

            expect(state._showShares).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    host,
                    username,
                    password,
                    domain,
                    failed: true,
                    inProgress: false,
                    succeeded: false,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should update success", () => {
            const state = syncReducer(defaultState(), {
                type: wizardActions.SHOW_SHARES,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showShares).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    host,
                    username,
                    password,
                    domain,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should keep all the folders from the state ( username, password and domain are identical)", () => {
            const state = syncReducer(buildStateWithSharesFolders(), {
                type: wizardActions.SHOW_SHARES,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showShares).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    host,
                    username,
                    password,
                    domain,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {data: [{actualPath: '/data1', host, path: '/data1'}]}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });


        test("should delete all the folders from the state  (different password)", () => {
            const newPassword = 'Netapp456';
            const state = syncReducer(buildStateWithSharesFolders(), {
                type: wizardActions.SHOW_SHARES,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra: {host, dataBrokerId, username, password: newPassword, domain}
            });

            expect(state._showShares).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    host,
                    username,
                    password: newPassword,
                    domain,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should delete all the folders from the state  (different username)", () => {
            const username = 'newUser2';
            const state = syncReducer(buildStateWithSharesFolders(), {
                type: wizardActions.SHOW_SHARES,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra: {host, dataBrokerId, username, password, domain}
            });

            expect(state._showShares).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    host,
                    username,
                    password,
                    domain,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should delete all the folders from the state  (different domain)", () => {
            const domain = 'domain2';
            const state = syncReducer(buildStateWithSharesFolders(), {
                type: wizardActions.SHOW_SHARES,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra: {host, dataBrokerId, username, password, domain}
            });

            expect(state._showShares).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    host,
                    username,
                    password,
                    domain,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should delete all the folders from the state  (different host)", () => {
            const newHost = '2.2.2.2';
            const newHostEncoded = encodeAll(newHost);
            const state = syncReducer(buildStateWithSharesFolders(), {
                type: wizardActions.SHOW_SHARES,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra: {host: newHost, dataBrokerId, username, password, domain}
            });

            expect(state._showShares).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    "domain": "netapp",
                    "failed": false,
                    "folders": {
                        "-": {
                            "data": [
                                {
                                    "actualPath": "/data1",
                                    "host": "1.1.1.1",
                                    "path": "/data1"
                                }
                            ]
                        }
                    },
                    host,
                    "id": 1,
                    "inProgress": false,
                    "password": "Netapp234",
                    "succeeded": true,
                    "username": "testUser"
                },
                [`${newHostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    host: newHost,
                    username,
                    password,
                    domain,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });

        test("should delete all the folders from the state  (different dataBrokerId)", () => {
            const newDataBrokerId = 2;
            const state = syncReducer(buildStateWithSharesFolders(), {
                type: wizardActions.SHOW_SHARES,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra: {host, dataBrokerId: newDataBrokerId, username, password, domain}
            });

            expect(state._showShares).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    domain,
                    host,
                    username,
                    password,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        '-': {
                            data: [{
                                actualPath: '/data1',
                                host,
                                path: '/data1'
                            }]/*,
                             retrievedAll: true*/
                        }
                    },
                    id: 1
                },
                [`${hostEncoded}-${newDataBrokerId}`]: {
                    dataBrokerId: newDataBrokerId,
                    host,
                    username,
                    password,
                    domain,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }
            })
        });
    });

    describe("Update shares", () => {

        const username = 'testUser';
        const host = '1.1.1.1';
        const hostEncoded = encodeAll(host);
        const dataBrokerId = 1;
        const domain = 'netapp';
        const password = 'Netapp234';
        const extra = {host, dataBrokerId, username, password, domain};
        const path = 'data1';

        const buildStateWithSharesFolders = () => defaultState()
            .set("_showShares", {
                [`${hostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    domain,
                    host,
                    username,
                    password,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: '/data1',
                        host,
                        path: 'data1'
                    }],
                    folders: {
                        '-': {
                            data: [{
                                actualPath: '/data1',
                                host,
                                path: '/data1'
                            }]/*,
                             retrievedAll: true*/
                        }
                    },
                    id: 1
                }
            });

        const buildStateWithoutSharesFolders = () => defaultState()
            .set("_showShares", {
                [`${hostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    domain,
                    host,
                    username,
                    password,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        '-': {}
                    },
                    id: 1
                }
            });

        test("should update retrievedAll and failedRetrieve upon failure in list-cifs-share-folders", () => {
            const state = syncReducer(buildStateWithSharesFolders(), {
                type: wizardActions.UPDATE_SHARES, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_CIFS_SHARES]: {
                        status: "FAILED",
                        host
                    }
                }, status: wizardActions.STATUS_ERROR, id: 1, extra
            });
            expect(state._showShares).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                        dataBrokerId,
                        host,
                        username,
                        password,
                        domain,
                        failed: false,
                        failedRetrieve: true,
                        inProgress: false,
                        succeeded: true,
                        data: [{
                            actualPath: '/data1',
                            host,
                            path: 'data1'
                        }],
                        folders: {
                            "-": {
                                data: [{
                                    actualPath: "/data1",
                                    host: "1.1.1.1",
                                    path: "/data1"
                                }],
                                retrievedAll: true
                            }
                        },
                        retrievedAll: true,
                        id: 1
                    }
                }
            );
        });


        test("should update exports upon status done in list-cifs-shares and the state doesnt have any exports", () => {
            const state = syncReducer(buildStateWithoutSharesFolders(), {
                type: wizardActions.UPDATE_SHARES, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_CIFS_SHARES]: {
                        status: "DONE",
                        host,
                        paths: [path]
                    }
                }, status: wizardActions.STATUS_SUCCESS, id: 1, extra
            });
            expect(state._showShares).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                        dataBrokerId,
                        username,
                        password,
                        domain,
                        host,
                        failedRetrieve: false,
                        retrievedAll: true,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        folders: {
                            "-": {
                                data: [
                                    {
                                        actualPath: '/' + path,
                                        host,
                                        path
                                    }
                                ],
                                retrievedAll: true
                            }
                        },
                        data: [
                            {
                                actualPath: '/' + path,
                                host,
                                path
                            }
                        ],
                        id: 1
                    }
                }
            );
        });

        test("should update exports upon status done in list-cifs-shares and the state has previous exports", () => {
            const state = syncReducer(buildStateWithSharesFolders(), {
                type: wizardActions.UPDATE_SHARES, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_CIFS_SHARES]: {
                        status: "DONE",
                        host,
                        paths: ['data2']
                    }
                }, status: wizardActions.STATUS_SUCCESS, id: 1, extra
            });
            expect(state._showShares).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                        dataBrokerId,
                        username,
                        password,
                        domain,
                        host,
                        retrievedAll: true,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        failedRetrieve: false,
                        folders: {
                            "-": {
                                data: [
                                    {
                                        actualPath: '/' + path,
                                        host,
                                        path: '/' + path
                                    },
                                    {
                                        actualPath: "/data2",
                                        host,
                                        path: "data2"
                                    }
                                ],
                                retrievedAll: true
                            }
                        },
                        data: [
                            {
                                actualPath: '/' + path,
                                host,
                                path
                            },
                            {
                                actualPath: "/data2",
                                host,
                                path: "data2"
                            }
                        ],
                        id: 1
                    }
                }
            );
        });

        test("should update retrievedAll and export upon status DONE in list-cifs-shares and the state doesnt have any exports", () => {
            const state = syncReducer(buildStateWithoutSharesFolders(), {
                type: wizardActions.UPDATE_SHARES, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_CIFS_SHARES]: {
                        status: "DONE",
                        host,
                        paths: []
                    }
                }, status: wizardActions.STATUS_SUCCESS, id: 1, extra
            });
            expect(state._showShares).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                        dataBrokerId,
                        domain,
                        host,
                        username,
                        password,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        failedRetrieve: false,
                        data: [],
                        folders: {
                            "-": {
                                retrievedAll: true,
                                data: []
                            }
                        },
                        retrievedAll: true,
                        id: 1
                    }
                }
            );
        });

        test("should update retrievedAll and export upon status DONE in list-cifs-shares and the state has previous exports", () => {
            const state = syncReducer(buildStateWithSharesFolders(), {
                type: wizardActions.UPDATE_SHARES, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_CIFS_SHARES]: {
                        status: "DONE",
                        host,
                        paths: []
                    }
                }, status: wizardActions.STATUS_SUCCESS, id: 1, extra
            });
            expect(state._showShares).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                        dataBrokerId,
                        host,
                        username,
                        password,
                        domain,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        failedRetrieve: false,
                        folders: {
                            "-": {
                                data: [
                                    {
                                        actualPath: '/' + path,
                                        host,
                                        path: '/' + path
                                    }
                                ],
                                retrievedAll: true
                            }
                        },
                        data: [
                            {
                                actualPath: '/' + path,
                                host,
                                path
                            }
                        ],
                        retrievedAll: true,
                        id: 1
                    }
                }
            );
        });

        test("should return state with unhandledMessages - _showShares is undefined", () => {
            const action = {
                type: wizardActions.UPDATE_SHARES, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_CIFS_SHARES]: {
                        status: "DONE",
                        host: '2.2.2.2'
                    }, status: wizardActions.STATUS_SUCCESS, id: 1, extra
                }
            };

            const state = syncReducer(defaultState(), action);
            action["unhandledCounter"] = 2;
            expect(state).toEqual(defaultState().set("unhandledMessages", [action]));
        });

        test("should return state with unhandledMessages - host is not like _showShares.host", () => {

            const action = {
                type: wizardActions.UPDATE_SHARES, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_CIFS_SHARES]: {
                        status: "DONE",
                        host: '2.2.2.2'
                    }, status: wizardActions.STATUS_SUCCESS, id: 1, extra
                }
            };

            const state = syncReducer(buildStateWithoutSharesFolders(), action);
            action["unhandledCounter"] = 2;
            expect(state).toEqual(buildStateWithoutSharesFolders().set("unhandledMessages", [action]));

        });

    });

    describe("list folders", () => {
        const dataBrokerId = 1;
        const fileServer = '1.1.1.1';
        const fileServerEncoded = encodeAll(fileServer);
        const path = '/data1';
        const extra = {fileServer, path, dataBrokerId};

        const buildStateWithExportsFolders = () => defaultState()
            .set("_showExports", {
                [`${fileServerEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    host: fileServer,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: '/data1',
                        host: fileServer,
                        path: '/data1'
                    }],
                    folders: {
                        '-': {
                            data: [{
                                actualPath: '/data1',
                                host: fileServer,
                                path: '/data1'
                            }],
                            retrievedAll: true
                        }
                    },
                    retrievedAll: true,
                    id: 1
                }
            });

        test("should update inProgress", () => {
            const state = syncReducer(buildStateWithExportsFolders(), {
                type: wizardActions.LIST_FOLDERS,
                status: wizardActions.STATUS_PROGRESS,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual({
                [`${fileServerEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    host: fileServer,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: '/data1',
                        host: fileServer,
                        path: '/data1'
                    }],
                    folders: {
                        "-": {
                            data: [{actualPath: '/data1', host: fileServer, path}],
                            retrievedAll: true
                        },
                        "/data1": {
                            data: [],
                            error: undefined,
                            failed: false,
                            id: 1,
                            inProgress: true,
                            requestId: undefined,
                            succeeded: false
                        }
                    },
                    retrievedAll: true,
                    id: 1
                }
            })
        });

        test("should update failed", () => {
            const state = syncReducer(buildStateWithExportsFolders(), {
                type: wizardActions.LIST_FOLDERS,
                status: wizardActions.STATUS_ERROR,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual({
                [`${fileServerEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    host: fileServer,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: '/data1',
                        host: fileServer,
                        path: '/data1'
                    }],
                    folders: {
                        "-": {
                            data: [{actualPath: '/data1', host: fileServer, path}],
                            retrievedAll: true
                        },
                        "/data1": {
                            data: [],
                            error: undefined,
                            failed: true,
                            id: 1,
                            inProgress: false,
                            requestId: undefined,
                            succeeded: false
                        }
                    },
                    retrievedAll: true,
                    id: 1
                }
            });

        });

        test("should update success", () => {

            const state = syncReducer(buildStateWithExportsFolders(), {
                type: wizardActions.LIST_FOLDERS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual({
                [`${fileServerEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    host: fileServer,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: '/data1',
                        host: fileServer,
                        path: '/data1'
                    }],
                    folders: {
                        "-": {
                            data: [{actualPath: '/data1', host: fileServer, path}],
                            retrievedAll: true
                        },
                        "/data1": {
                            data: [],
                            error: undefined,
                            failed: false,
                            id: 1,
                            inProgress: false,
                            requestId: undefined,
                            succeeded: true
                        }
                    },
                    retrievedAll: true,
                    id: 1
                }
            });
        });

        test("should return empty object", () => {

            const state = syncReducer(defaultState(), {
                type: wizardActions.LIST_FOLDERS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual({});
            expect(state).toEqual(defaultState());
        });

        test("should return empty object and the same state that was sent- state._showExports.data is not exist", () => {

            const state = syncReducer(defaultState(), {
                type: wizardActions.LIST_FOLDERS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showExports).toEqual({});
            expect(state).toEqual(defaultState());
        });

        test("Should return the same state that was sent - state._showExports.data = []", () => {
            const prevState = defaultState()
                .set("_showExports", {
                    [`${fileServerEncoded}-${dataBrokerId}`]: {
                        dataBrokerId,
                        host: fileServer,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [],
                        folders: {
                            "-": {
                                data: [],
                                retrievedAll: true
                            }
                        },
                        id: 1
                    }
                });
            const currState = syncReducer(prevState, {
                type: wizardActions.LIST_FOLDERS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(currState).toEqual(prevState);
        });

        test("Should return the same state that was sent not state._showExports.data", () => {
            const prevState = defaultState()
                .set("_showExports", {
                    [`${fileServerEncoded}-${dataBrokerId}`]: {
                        dataBrokerId,
                        host: fileServer,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        folders: {
                            '-': {}
                        },
                        id: 1
                    }
                });
            const currState = syncReducer(prevState, {
                type: wizardActions.LIST_FOLDERS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(currState).toEqual(prevState);
        });

    });

    describe("list buckets folders", () => {
        const dataBrokerId = 1;
        const bucket = 'thisBucket';
        const extra = {path: bucket, bucket, dataBrokerId};

        const buildStateWithBucketsFolders = () => defaultState()
            .set("_showBuckets",
                {
                    "1": {
                        dataBrokerId,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket
                        }],
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: bucket,
                                    name: bucket,
                                    path: bucket
                                }],
                                retrievedAll: true
                            }
                        },
                        retrievedAll: true,
                        id: 1
                    }
                });

        test("should update in progress for data1", () => {
            const state = syncReducer(buildStateWithBucketsFolders(), {
                type: wizardActions.LIST_BUCKETS_FOLDERS,
                status: wizardActions.STATUS_PROGRESS,
                id: 1,
                extra
            });

            expect(state._showBuckets["1"]).toEqual({
                dataBrokerId,
                failed: false,
                inProgress: false,
                succeeded: true,
                data: [{
                    actualPath: bucket,
                    name: bucket,
                    path: bucket
                }],
                folders: {
                    '-': {
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket
                        }],
                        retrievedAll: true
                    },
                    'thisBucket': {
                        data: [],
                        failed: false,
                        inProgress: true,
                        succeeded: false,
                        error: undefined,
                        requestId: undefined,
                        id: 1
                    }
                },
                retrievedAll: true,
                id: 1
            })
        });

        test("should update failed", () => {
            const state = syncReducer(buildStateWithBucketsFolders(), {
                type: wizardActions.LIST_BUCKETS_FOLDERS,
                status: wizardActions.STATUS_ERROR,
                id: 1,
                extra
            });

            expect(state._showBuckets).toEqual({
                "1" : {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: bucket,
                        name: bucket,
                        path: bucket
                    }],
                    folders: {
                        '-': {
                            data: [{
                                actualPath: bucket,
                                name: bucket,
                                path: bucket
                            }],
                            retrievedAll: true
                        },
                        "thisBucket": {
                            data: [],
                            failed: true,
                            inProgress: false,
                            succeeded: false,
                            error: undefined,
                            requestId: undefined,
                            id: 1
                        }
                    },
                    retrievedAll: true,
                    id: 1
                }
                }
                );

        });

        test("should update success", () => {

            const state = syncReducer(buildStateWithBucketsFolders(), {
                type: wizardActions.LIST_BUCKETS_FOLDERS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showBuckets).toEqual({
                "1" : {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: bucket,
                        name: bucket,
                        path: bucket
                    }],
                    folders: {
                        '-': {
                            data: [{
                                actualPath: bucket,
                                name: bucket,
                                path: bucket
                            }],
                            retrievedAll: true
                        },
                        "thisBucket": {
                            data: [],
                            failed: false,
                            inProgress: false,
                            succeeded: true,
                            error: undefined,
                            requestId: undefined,
                            id: 1
                        }
                    },
                    retrievedAll: true,
                    id: 1
                }
                }
                );
        });

        test("should return empty object", () => {

            const state = syncReducer(defaultState(), {
                type: wizardActions.LIST_BUCKETS_FOLDERS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showBuckets).toEqual({});
            expect(state).toEqual(defaultState());
        });

        test("should return undefined and the same state that was sent- state._showBuckets.data is not exist", () => {
            const show =
                {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {
                            data: [],
                            retrievedAll: true
                        }
                    },
                    id: 1
                };

            const state = syncReducer(defaultState()
                    .set("_showBuckets", show)
                , {
                    type: wizardActions.LIST_BUCKETS_FOLDERS,
                    status: wizardActions.STATUS_SUCCESS,
                    id: 1,
                    extra
                });

            expect(state._showBuckets).toEqual(show);
            expect(state).toEqual(defaultState().set("_showBuckets", show));
        });

        test("Should return the same state that was sent - state._showBuckets.data = []", () => {
            const prevState = defaultState()
                .set("_showBuckets", {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [],
                    folders: {
                        "-": {
                            data: [],
                            retrievedAll: true
                        }
                    },
                    id: 1
                });
            const currState = syncReducer(prevState, {
                type: wizardActions.LIST_BUCKETS_FOLDERS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(currState).toEqual(prevState);
        });

        test("Should return the same state that was sent not state._showBuckets.data", () => {
            const prevState = defaultState()
                .set("_showBuckets", {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        '-': {}
                    },
                    id: 1
                });
            const currState = syncReducer(prevState, {
                type: wizardActions.LIST_BUCKETS_FOLDERS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(currState).toEqual(prevState);
        });

        test("Should return the same state that was sent - bucket name does not exist in data", () => {
            const extra = {bucket: 'test', path: bucket};

            const state = syncReducer(buildStateWithBucketsFolders(), {
                type: wizardActions.LIST_BUCKETS_FOLDERS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showBuckets).toEqual({
                "1" : {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: bucket,
                        name: bucket,
                        path: bucket
                    }],
                    folders: {
                        '-': {
                            data: [{
                                actualPath: bucket,
                                name: bucket,
                                path: bucket
                            }],
                            retrievedAll: true
                        }
                    },
                    retrievedAll: true,
                    id: 1
                }
                }
                );

        });
    });

    describe("list SG buckets folders", () => {
        const dataBrokerId = 1;
        const bucket = 'thisBucket';
        const host = '1.1.1.1';
        const path = 'data1';
        const extra = {path, bucket, host};
        const port = 8080;
        const accessKey = 'a';
        const secretKey = 'a';

        const buildStateWithSGBucketsFolders = () => defaultState()
            .set("_showS3BasedBuckets", {'1.1.1.1' : {
                dataBrokerId,
                failed: false,
                inProgress: false,
                succeeded: true,
                data: [{
                    actualPath: bucket,
                    name: bucket,
                    path: bucket
                }],
                folders: {
                    '-': {
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket
                        }],
                        retrievedAll: true
                    }
                },
                accessKey,
                secretKey,
                port,
                retrievedAll: true,
                id: 1
            }});

        test("should update in progress for data1", () => {
            const state = syncReducer(buildStateWithSGBucketsFolders(), {
                type: wizardActions.LIST_S3BASED_BUCKETS_FOLDERS,
                status: wizardActions.STATUS_PROGRESS,
                id: 1,
                extra
            });

            expect(state._showS3BasedBuckets).toEqual({ '1.1.1.1': {
                dataBrokerId,
                failed: false,
                inProgress: false,
                succeeded: true,
                data: [{
                    actualPath: bucket,
                    name: bucket,
                    path: bucket
                }],
                folders: {
                    '-': {
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket
                        }],
                        retrievedAll: true
                    },
                    "data1": {
                        data: [],
                        failed: false,
                        inProgress: true,
                        succeeded: false,
                        error: undefined,
                        requestId: undefined,
                        id: 1
                    }
                },
                retrievedAll: true,
                accessKey,
                secretKey,
                port,
                id: 1
            }})
        });

        test("should update failed", () => {
            const state = syncReducer(buildStateWithSGBucketsFolders(), {
                type: wizardActions.LIST_S3BASED_BUCKETS_FOLDERS,
                status: wizardActions.STATUS_ERROR,
                id: 1,
                extra
            });

            expect(state._showS3BasedBuckets).toEqual({ '1.1.1.1' : {
                dataBrokerId,
                failed: false,
                inProgress: false,
                succeeded: true,
                data: [{
                    actualPath: bucket,
                    name: bucket,
                    path: bucket
                }],
                folders: {
                    '-': {
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket
                        }],
                        retrievedAll: true
                    },
                    "data1": {
                        data: [],
                        failed: true,
                        inProgress: false,
                        succeeded: false,
                        error: undefined,
                        requestId: undefined,
                        id: 1
                    }
                },
                retrievedAll: true,
                accessKey,
                secretKey,
                port,
                id: 1
            }});

        });

        test("should update success", () => {

            const state = syncReducer(buildStateWithSGBucketsFolders(), {
                type: wizardActions.LIST_S3BASED_BUCKETS_FOLDERS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showS3BasedBuckets).toEqual({ '1.1.1.1' : {
                dataBrokerId,
                failed: false,
                inProgress: false,
                succeeded: true,
                data: [{
                    actualPath: bucket,
                    name: bucket,
                    path: bucket
                }],
                folders: {
                    '-': {
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket
                        }],
                        retrievedAll: true
                    },
                    "data1": {
                        data: [],
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        error: undefined,
                        requestId: undefined,
                        id: 1
                    }
                },
                retrievedAll: true,
                accessKey,
                secretKey,
                port,
                id: 1
            }});
        });

        test("should return empty object", () => {

            const state = syncReducer(defaultState(), {
                type: wizardActions.LIST_S3BASED_BUCKETS_FOLDERS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showBuckets).toEqual({});
            expect(state).toEqual(defaultState());
        });

        test("should return empty object and the same state that was sent- state._showS3BasedBuckets.data is not exist", () => {

            const state = syncReducer(defaultState(), {
                type: wizardActions.LIST_S3BASED_BUCKETS_FOLDERS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showS3BasedBuckets).toEqual({});
            expect(state).toEqual(defaultState());
        });

        test("Should return the same state that was sent - state._showS3BasedBuckets.data = []", () => {
            const prevState = defaultState()
                .set("_showS3BasedBuckets", {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [],
                    folders: {
                        "-": {
                            data: [],
                            retrievedAll: true
                        }
                    },
                    accessKey,
                    secretKey,
                    port,
                    id: 1
                });
            const currState = syncReducer(prevState, {
                type: wizardActions.LIST_S3BASED_BUCKETS_FOLDERS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(currState).toEqual(prevState);
        });

        test("Should return the same state that was sent not state._showS3BasedBuckets.data", () => {
            const prevState = defaultState()
                .set("_showS3BasedBuckets", {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        '-': {}
                    },
                    accessKey,
                    secretKey,
                    port,
                    id: 1
                });
            const currState = syncReducer(prevState, {
                type: wizardActions.LIST_S3BASED_BUCKETS_FOLDERS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(currState).toEqual(prevState);
        });

        test("Should return the same state that was sent bucket name does not exist in data", () => {
            const extra = {bucket: 'test', path};

            const state = syncReducer(buildStateWithSGBucketsFolders(), {
                type: wizardActions.LIST_S3BASED_BUCKETS_FOLDERS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showS3BasedBuckets).toEqual({ '1.1.1.1' : {
                dataBrokerId,
                failed: false,
                inProgress: false,
                succeeded: true,
                data: [{
                    actualPath: bucket,
                    name: bucket,
                    path: bucket
                }],
                folders: {
                    '-': {
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket
                        }],
                        retrievedAll: true
                    }
                },
                accessKey,
                secretKey,
                port,
                retrievedAll: true,
                id: 1
            }});

        });
    });

    describe('update folders', () => {
        const dataBrokerId = 1;
        const host = '1.1.1.1';
        const hostEncoded = encodeAll(host);

        const buildStateWithExportsFoldersAndAfterListFolders = () => defaultState()
            .set("_showExports", {
                [`${hostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: '/data1',
                        host,
                        path: '/data1'
                    }],
                    folders: {
                        '-': {
                            data: [{
                                actualPath: '/data1',
                                host,
                                path: '/data1'
                            }],
                            retrievedAll: true
                        },
                        '/data1': {data: [], failed: false, inProgress: false, succeeded: true}
                    },
                    id: 1
                }
            });


        test("should update retrievedAll  upon failure in list-folders", () => {
            const action = {
                type: wizardActions.UPDATE_FOLDERS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_FOLDERS]: {
                        status: "FAILED",
                        path: '/data1',
                         host
                    }
                }, status: wizardActions.STATUS_ERROR, id: 1
            };

            const state = syncReducer(buildStateWithExportsFoldersAndAfterListFolders(), action);

            expect(state._showExports).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                        dataBrokerId,
                        host,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [{
                            actualPath: '/data1',
                            host,
                            path: '/data1'
                        }],
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            },
                            '/data1': {data: [], failed: false, inProgress: false, succeeded: true, retrievedAll: true}
                        },
                        id: 1
                    }
                }
            );
        });

        test("should update retrievedAll after success in list-folders", () => {

            const action = {
                type: wizardActions.UPDATE_FOLDERS,
                payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_FOLDERS]: {
                        status: "DONE",
                        path: '/data1',
                        host,
                        folders: ['a', 'ab', 'ac']
                    }
                }, status: wizardActions.STATUS_SUCCESS, id: 1
            };

            const state = syncReducer(buildStateWithExportsFoldersAndAfterListFolders(), action);

            expect(state._showExports).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                        dataBrokerId,
                        host,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [{
                            actualPath: '/data1',
                            host,
                            path: '/data1'
                        }],
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            },
                            '/data1': {
                                data: [
                                    {
                                        actualPath: "/data1/a",
                                        path: "a"
                                    },
                                    {
                                        actualPath: "/data1/ab",
                                        path: "ab"
                                    },
                                    {
                                        actualPath: "/data1/ac",
                                        path: "ac"
                                    }
                                ], failed: false, inProgress: false, succeeded: true, retrievedAll: true
                            }
                        },
                        id: 1
                    }
                }
            );
        });

        test("should update retrievedAll after success in list-folders and to unite between the folders ( the ones from the action and ones from the state ) ", () => {
            const action = {
                type: wizardActions.UPDATE_FOLDERS,
                payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_FOLDERS]: {
                        status: "DONE",
                        path: '/data1',
                        host,
                        folders: ['a', 'ab', 'ac']
                    }
                }, status: wizardActions.STATUS_SUCCESS, id: 1
            };

            const state = syncReducer(defaultState()
                    .set("_showExports", {
                        [`${hostEncoded}-${dataBrokerId}`]: {
                            dataBrokerId,
                            host,
                            failed: false,
                            inProgress: false,
                            succeeded: true,
                            data: [{
                                actualPath: '/data1',
                                host,
                                path: '/data1'
                            }],
                            folders: {
                                '-': {
                                    data: [{
                                        actualPath: '/data1',
                                        host,
                                        path: '/data1'
                                    }],
                                    retrievedAll: true
                                },
                                '/data1': {
                                    data: [{
                                        actualPath: "/data1/b",
                                        path: "b"
                                    },
                                        {
                                            actualPath: "/data1/c",
                                            path: "c"
                                        },
                                        {
                                            actualPath: "/data1/d",
                                            path: "d"
                                        }], failed: false, inProgress: false, succeeded: true
                                }
                            },
                            id: 1
                        }
                    })
                , action);

            expect(state._showExports).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                        dataBrokerId,
                        host,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [{
                            actualPath: '/data1',
                            host,
                            path: '/data1'
                        }],
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            },
                            '/data1': {
                                data: [
                                    {
                                        actualPath: "/data1/b",
                                        path: "b"
                                    },
                                    {
                                        actualPath: "/data1/c",
                                        path: "c"
                                    },
                                    {
                                        actualPath: "/data1/d",
                                        path: "d"
                                    },
                                    {
                                        actualPath: "/data1/a",
                                        path: "a"
                                    },
                                    {
                                        actualPath: "/data1/ab",
                                        path: "ab"
                                    },
                                    {
                                        actualPath: "/data1/ac",
                                        path: "ac"
                                    }
                                ], failed: false, inProgress: false, succeeded: true, retrievedAll: true
                            }
                        },
                        id: 1
                    }
                }
            );
        });

        test("should return empty object and the same state that was sent- state._showExports is not exist", () => {

            const action = {
                type: wizardActions.UPDATE_FOLDERS,
                payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_FOLDERS]: {
                        status: "DONE",
                        path: '/data1',
                        host,
                        folders: ['a', 'ab', 'ac']
                    }
                }, status: wizardActions.STATUS_SUCCESS, id: 1
            };

            const state = syncReducer(defaultState(), action);

            expect(state._showExports).toEqual({});
            expect(state).toEqual(defaultState());

        });

        test("should return undefined and the same state that was sent- state._showExports.data is not exist", () => {
            const prevState = defaultState()
                .set("_showExports", {
                    [`${hostEncoded}-${dataBrokerId}`]: {
                        dataBrokerId,
                        host,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            },
                            '/data1': {
                                data: [{
                                    actualPath: "/data1/b",
                                    path: "b"
                                },
                                    {
                                        actualPath: "/data1/c",
                                        path: "c"
                                    },
                                    {
                                        actualPath: "/data1/d",
                                        path: "d"
                                    }], failed: false, inProgress: false, succeeded: true
                            }
                        },
                        id: 1
                    }
                });

            const currState = syncReducer(prevState, {
                type: wizardActions.UPDATE_FOLDERS,
                payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_FOLDERS]: {
                        status: "DONE",
                        path: '/data1',
                        nfsServer: host,
                        folders: ['a', 'ab', 'ac']
                    }
                }, status: wizardActions.STATUS_SUCCESS, id: 1
            });

            expect(prevState).toEqual(currState);

        });

        test("should return undefined and the same state that was sent- state._showExports.data=[] ", () => {
            const prevState = defaultState()
                .set("_showExports", {
                    [`${hostEncoded}-${dataBrokerId}`]: {
                        dataBrokerId,
                        host,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [],
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            },
                            '/data1': {
                                data: [{
                                    actualPath: "/data1/b",
                                    path: "b"
                                },
                                    {
                                        actualPath: "/data1/c",
                                        path: "c"
                                    },
                                    {
                                        actualPath: "/data1/d",
                                        path: "d"
                                    }], failed: false, inProgress: false, succeeded: true
                            }
                        },
                        id: 1
                    }
                });

            const currState = syncReducer(prevState, {
                type: wizardActions.UPDATE_FOLDERS,
                payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_FOLDERS]: {
                        status: "DONE",
                        path: '/data1',
                        nfsServer: host,
                        folders: ['a', 'ab', 'ac']
                    }
                }, status: wizardActions.STATUS_SUCCESS, id: 1
            });

            expect(currState).toEqual(prevState);

        });

        test("Should return the same state that was sent _showExports.folders[path] in undefined", () => {
            const prevState = defaultState()
                .set("_showExports", {
                    [`${hostEncoded}-${dataBrokerId}`]: {
                        dataBrokerId,
                        host,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [{
                            actualPath: '/data1',
                            host: '2.2.2.2',
                            path: '/data1'
                        }],
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            }
                        },
                        id: 1
                    }
                });

            const currState = syncReducer(prevState, {
                type: wizardActions.UPDATE_FOLDERS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_FOLDERS]: {
                        status: "DONE",
                        path: '/data1',
                        nfsServer: host,
                        folders: ['a', 'ab', 'ac']
                    }
                }, status: wizardActions.STATUS_SUCCESS, id: 1
            });

            expect(currState).toEqual(prevState);

        });

    });

    describe("list shares folders", () => {
        const dataBrokerId = 1;
        const host = '1.1.1.1';
        const hostEncoded = encodeAll(host);
        const path = '/data1';
        const extra = {host, path, dataBrokerId};


        const buildStateWithExportsFolders = () => defaultState()
            .set("_showShares", {
                [`${hostEncoded}-${dataBrokerId}`]:  {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: '/data1',
                        host,
                        path: '/data1'
                    }],
                    folders: {
                        '-': {
                            data: [{
                                actualPath: '/data1',
                                host,
                                path: '/data1'
                            }],
                            retrievedAll: true
                        }
                    },
                    retrievedAll: true,
                    id: 1
                }
            });

        test("should update inProgress", () => {
            const state = syncReducer(buildStateWithExportsFolders(), {
                type: wizardActions.LIST_SHARE_FOLDERS,
                status: wizardActions.STATUS_PROGRESS,
                id: 1,
                extra
            });

            expect(state._showShares).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]:{
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: '/data1',
                        host,
                        path: '/data1'
                    }],
                    folders: {
                        "-": {
                            data: [{actualPath: '/data1', host, path}],
                            retrievedAll: true
                        },
                        "/data1": {
                            data: [],
                            error: undefined,
                            failed: false,
                            id: 1,
                            inProgress: true,
                            requestId: undefined,
                            succeeded: false
                        }
                    },
                    retrievedAll: true,
                    id: 1
                }
            })
        });

        test("should update failed", () => {
            const state = syncReducer(buildStateWithExportsFolders(), {
                type: wizardActions.LIST_SHARE_FOLDERS,
                status: wizardActions.STATUS_ERROR,
                id: 1,
                extra
            });

            expect(state._showShares).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]:{
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: '/data1',
                        host,
                        path: '/data1'
                    }],
                    folders: {
                        "-": {
                            data: [{actualPath: '/data1', host, path}],
                            retrievedAll: true
                        },
                        "/data1": {
                            data: [],
                            error: undefined,
                            failed: true,
                            id: 1,
                            inProgress: false,
                            requestId: undefined,
                            succeeded: false
                        }
                    },
                    retrievedAll: true,
                    id: 1
                }
            });

        });

        test("should update success", () => {

            const state = syncReducer(buildStateWithExportsFolders(), {
                type: wizardActions.LIST_SHARE_FOLDERS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showShares).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: '/data1',
                        host,
                        path: '/data1'
                    }],
                    folders: {
                        "-": {
                            data: [{actualPath: '/data1', host, path}],
                            retrievedAll: true
                        },
                        "/data1": {
                            data: [],
                            error: undefined,
                            failed: false,
                            id: 1,
                            inProgress: false,
                            requestId: undefined,
                            succeeded: true
                        }
                    },
                    retrievedAll: true,
                    id: 1
                }
            });
        });

        test("should return empty object", () => {

            const state = syncReducer(defaultState(), {
                type: wizardActions.LIST_SHARE_FOLDERS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showShares).toEqual({});
            expect(state).toEqual(defaultState());
        });

        test("should return empty object and the same state that was sent- state._showShares.data is not exist", () => {

            const state = syncReducer(defaultState(), {
                type: wizardActions.LIST_SHARE_FOLDERS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showShares).toEqual({});
            expect(state).toEqual(defaultState());
        });

        test("Should return the same state that was sent - state._showShares.data = []", () => {
            const prevState = defaultState()
                .set("_showShares", {
                    [`${hostEncoded}-${dataBrokerId}`]:  {
                        dataBrokerId,
                        host,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [],
                        folders: {
                            "-": {
                                data: [],
                                retrievedAll: true
                            }
                        },
                        id: 1
                    }
                });
            const currState = syncReducer(prevState, {
                type: wizardActions.LIST_SHARE_FOLDERS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(currState).toEqual(prevState);
        });

        test("Should return the same state that was sent not state._showShares.data", () => {
            const prevState = defaultState()
                .set("_showShares", {
                    [`${hostEncoded}-${dataBrokerId}`]: {
                        dataBrokerId,
                        host,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        folders: {
                            '-': {}
                        },
                        id: 1
                    }
                });
            const currState = syncReducer(prevState, {
                type: wizardActions.LIST_SHARE_FOLDERS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(currState).toEqual(prevState);
        });

    });

    describe("update shares folders", () => {
        const username = 'testUser';
        const host = '1.1.1.1';
        const hostEncoded = encodeAll(host);
        const dataBrokerId = 1;
        const domain = 'netapp';
        const password = 'Netapp234';

        const buildStateWithSharesFoldersAndAfterListFolders = () => defaultState()
            .set("_showShares", {
                [`${hostEncoded}-${dataBrokerId}`]:{
                    dataBrokerId,
                    domain,
                    host,
                    username,
                    password,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [{
                        actualPath: '/data1',
                        host,
                        path: '/data1'
                    }],
                    folders: {
                        '-': {
                            data: [{
                                actualPath: '/data1',
                                host,
                                path: '/data1'
                            }],
                            retrievedAll: true
                        },
                        '/data1': {data: [], failed: false, inProgress: false, succeeded: true}
                    },
                    id: 1
                }
            });


        test("should update retrievedAll  upon failure in list-cifs-share-folders", () => {
            const state = syncReducer(buildStateWithSharesFoldersAndAfterListFolders(), {
                type: wizardActions.UPDATE_SHARE_FOLDERS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_CIFS_SHARE_FOLDERS]: {
                        status: "FAILED",
                        path: '',
                        share: '/data1',
                        host
                    }
                }, status: wizardActions.STATUS_ERROR, id: 1
            });

            expect(state._showShares).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]:  {
                        dataBrokerId,
                        domain,
                        host,
                        username,
                        password,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [{
                            actualPath: '/data1',
                            host,
                            path: '/data1'
                        }],
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            },
                            '/data1': {data: [], failed: false, inProgress: false, succeeded: true, retrievedAll: true}
                        },
                        id: 1
                    }
                }
            );
        });

        test("should update retrievedAll after success in list-cifs-share-folders", () => {
            const state = syncReducer(buildStateWithSharesFoldersAndAfterListFolders(), {
                type: wizardActions.UPDATE_SHARE_FOLDERS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_CIFS_SHARE_FOLDERS]: {
                        status: "DONE",
                        path: '',
                        share: '/data1',
                        host,
                        folders: ['a', 'ab', 'ac']
                    }
                }, status: wizardActions.STATUS_SUCCESS, id: 1
            });

            expect(state._showShares).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                        dataBrokerId,
                        domain,
                        host,
                        username,
                        password,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [{
                            actualPath: '/data1',
                            host,
                            path: '/data1'
                        }],
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            },
                            '/data1': {
                                data: [
                                    {
                                        actualPath: "/data1/a",
                                        path: "a"
                                    },
                                    {
                                        actualPath: "/data1/ab",
                                        path: "ab"
                                    },
                                    {
                                        actualPath: "/data1/ac",
                                        path: "ac"
                                    }
                                ], failed: false, inProgress: false, succeeded: true, retrievedAll: true
                            }
                        },
                        id: 1
                    }
                }
            );
        });

        test("should update retrievedAll after success in list-cifs-share-folders and to unite between the folders ( the ones from the action and ones from the state ) ", () => {
            const state = syncReducer(defaultState()
                    .set("_showShares", {
                        [`${hostEncoded}-${dataBrokerId}`]:   {
                            dataBrokerId,
                            domain,
                            host,
                            username,
                            password,
                            failed: false,
                            inProgress: false,
                            succeeded: true,
                            data: [{
                                actualPath: '/data1',
                                host,
                                path: '/data1'
                            }],
                            folders: {
                                '-': {
                                    data: [{
                                        actualPath: '/data1',
                                        host,
                                        path: '/data1'
                                    }],
                                    retrievedAll: true
                                },
                                '/data1': {
                                    data: [{
                                        actualPath: "/data1/b",
                                        path: "b"
                                    },
                                        {
                                            actualPath: "/data1/c",
                                            path: "c"
                                        },
                                        {
                                            actualPath: "/data1/d",
                                            path: "d"
                                        }], failed: false, inProgress: false, succeeded: true
                                }
                            },
                            id: 1
                        }
                    })
                , {
                    type: wizardActions.UPDATE_SHARE_FOLDERS, payload: {
                        dataBrokerId,
                        [consts.PAYLOAD_LIST_CIFS_SHARE_FOLDERS]: {
                            status: "DONE",
                            path: '',
                            share: '/data1',
                            host,
                            folders: ['a', 'ab', 'ac']
                        }
                    }, status: wizardActions.STATUS_SUCCESS, id: 1
                });

            expect(state._showShares).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]:  {
                        dataBrokerId,
                        domain,
                        host,
                        username,
                        password,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [{
                            actualPath: '/data1',
                            host,
                            path: '/data1'
                        }],
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            },
                            '/data1': {
                                data: [
                                    {
                                        actualPath: "/data1/b",
                                        path: "b"
                                    },
                                    {
                                        actualPath: "/data1/c",
                                        path: "c"
                                    },
                                    {
                                        actualPath: "/data1/d",
                                        path: "d"
                                    },
                                    {
                                        actualPath: "/data1/a",
                                        path: "a"
                                    },
                                    {
                                        actualPath: "/data1/ab",
                                        path: "ab"
                                    },
                                    {
                                        actualPath: "/data1/ac",
                                        path: "ac"
                                    }
                                ], failed: false, inProgress: false, succeeded: true, retrievedAll: true
                            }
                        },
                        id: 1
                    }
                }
            );
        });

        test("should return the full path as required  ", () => {
            const state = syncReducer(defaultState()
                    .set("_showShares", {
                        [`${hostEncoded}-${dataBrokerId}`]:{
                            dataBrokerId,
                            domain,
                            host,
                            username,
                            password,
                            failed: false,
                            inProgress: false,
                            succeeded: true,
                            data: [{
                                actualPath: '/data1',
                                host,
                                path: '/data1'
                            }],
                            folders: {
                                '-': {
                                    data: [{
                                        actualPath: '/data1',
                                        host,
                                        path: '/data1'
                                    }],
                                    retrievedAll: true
                                },
                                '/data1': {
                                    data: [{
                                        actualPath: "/data1/b",
                                        path: "b"
                                    }], failed: false, inProgress: false, succeeded: true
                                },
                                '/data1/b': {data: [], failed: false, inProgress: false, succeeded: true}
                            },
                            id: 1
                        }
                    })
                , {
                    type: wizardActions.UPDATE_SHARE_FOLDERS, payload: {
                        dataBrokerId,
                        [consts.PAYLOAD_LIST_CIFS_SHARE_FOLDERS]: {
                            status: "DONE",
                            path: 'b',
                            share: '/data1',
                            host,
                            folders: ['a', 'ab', 'ac']
                        }
                    }, status: wizardActions.STATUS_SUCCESS, id: 1
                });

            expect(state._showShares).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]:{
                        dataBrokerId,
                        domain,
                        host,
                        username,
                        password,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [{
                            actualPath: '/data1',
                            host,
                            path: '/data1'
                        }],
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            },
                            '/data1': {
                                data: [
                                    {
                                        actualPath: "/data1/b",
                                        path: "b"
                                    }], failed: false, inProgress: false, succeeded: true
                            },
                            '/data1/b': {
                                data: [
                                    {
                                        actualPath: "/data1/b/a",
                                        path: "a"
                                    },
                                    {
                                        actualPath: "/data1/b/ab",
                                        path: "ab"
                                    },
                                    {
                                        actualPath: "/data1/b/ac",
                                        path: "ac"
                                    }], failed: false, inProgress: false, retrievedAll: true, succeeded: true
                            }
                        },
                        id: 1
                    }
                }
            );
        });


        test("should return empty object and the same state that was sent- state._showShares is not exist", () => {
            const state = syncReducer(defaultState()
                , {
                    type: wizardActions.UPDATE_SHARE_FOLDERS, payload: {
                        dataBrokerId,
                        [consts.PAYLOAD_LIST_CIFS_SHARE_FOLDERS]: {
                            status: "DONE",
                            path: '',
                            share: '/data1',
                            host,
                            folders: ['a', 'ab', 'ac']
                        }
                    }, status: wizardActions.STATUS_SUCCESS, id: 1
                });

            expect(state._showShares).toEqual({});
            expect(state).toEqual(defaultState());

        });

        test("should return undefined and the same state that was sent- state._showShares.data is not exist", () => {
            const prevState = defaultState()
                .set("_showShares", {
                    [`${hostEncoded}-${dataBrokerId}`]:{
                        dataBrokerId,
                        domain,
                        host,
                        username,
                        password,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            },
                            '/data1': {
                                data: [{
                                    actualPath: "/data1/b",
                                    path: "b"
                                },
                                    {
                                        actualPath: "/data1/c",
                                        path: "c"
                                    },
                                    {
                                        actualPath: "/data1/d",
                                        path: "d"
                                    }], failed: false, inProgress: false, succeeded: true
                            }
                        },
                        id: 1
                    }
                });
            const currState = syncReducer(prevState
                , {
                    type: wizardActions.UPDATE_SHARE_FOLDERS, payload: {
                        dataBrokerId,
                        [consts.PAYLOAD_LIST_CIFS_SHARE_FOLDERS]: {
                            status: "DONE",
                            path: '',
                            share: '/data1',
                            host,
                            folders: ['a', 'ab', 'ac']
                        }
                    }, status: wizardActions.STATUS_SUCCESS, id: 1
                });

            expect(prevState).toEqual(currState);

        });

        test("should return undefined and the same state that was sent- state._showShares.data=[] ", () => {
            const prevState = defaultState()
                .set("_showShares", {
                    [`${hostEncoded}-${dataBrokerId}`]: {
                        dataBrokerId,
                        domain,
                        host,
                        username,
                        password,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [],
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            },
                            '/data1': {
                                data: [{
                                    actualPath: "/data1/b",
                                    path: "b"
                                },
                                    {
                                        actualPath: "/data1/c",
                                        path: "c"
                                    },
                                    {
                                        actualPath: "/data1/d",
                                        path: "d"
                                    }], failed: false, inProgress: false, succeeded: true
                            }
                        },
                        id: 1
                    }
                });
            const currState = syncReducer(prevState, {
                type: wizardActions.UPDATE_SHARE_FOLDERS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_CIFS_SHARE_FOLDERS]: {
                        status: "DONE",
                        path: '',
                        share: '/data1',
                        host,
                        folders: ['a', 'ab', 'ac']
                    }
                }, status: wizardActions.STATUS_SUCCESS, id: 1
            });

            expect(currState).toEqual(prevState);

        });

        test("Should return the same state that was sent _showShares.folders[path] in undefined", () => {
            const prevState = defaultState()
                .set("_showShares", {
                    [`${hostEncoded}-${dataBrokerId}`]: {
                        dataBrokerId,
                        domain,
                        host,
                        username,
                        password,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        data: [{
                            actualPath: '/data1',
                            host: '2.2.2.2',
                            path: '/data1'
                        }],
                        folders: {
                            '-': {
                                data: [{
                                    actualPath: '/data1',
                                    host,
                                    path: '/data1'
                                }],
                                retrievedAll: true
                            }
                        },
                        id: 1
                    }
                });
            const currState = syncReducer(prevState
                , {
                    type: wizardActions.UPDATE_SHARE_FOLDERS, payload: {
                        dataBrokerId,
                        [consts.PAYLOAD_LIST_CIFS_SHARE_FOLDERS]: {
                            status: "DONE",
                            path: '',
                            share: '/data1',
                            host,
                            folders: ['a', 'ab', 'ac']
                        }
                    }, status: wizardActions.STATUS_SUCCESS, id: 1
                });

            expect(currState).toEqual(prevState);

        });

    });

    describe("Show s3based buckets", () => {
        const accessKey = '1';
        const secretKey = '1';
        const host = '1.1.1.1';
        const dataBrokerId = 1;
        const port = 1;
        const extra = {host, dataBrokerId, accessKey, secretKey, port};

        const buildStateWithSGBuckets = () => defaultState().set("_showS3BasedBuckets", { '1.1.1.1' : {
            dataBrokerId,
            port,
            host,
            accessKey,
            secretKey,
            failed: false,
            inProgress: false,
            succeeded: true,
            folders: {'-': {data: [{actualPath: 'small', name: 'small', path: 'small'}], retrievedAll: true}},
            id: 1
        }});

        test("should update inProgress", () => {
            const state = syncReducer(defaultState(), {
                type: wizardActions.SHOW_S3BASED_BUCKETS,
                status: wizardActions.STATUS_PROGRESS,
                id: 1,
                extra
            });

            expect(state._showS3BasedBuckets).toEqual({ '1.1.1.1' : {
                dataBrokerId,
                host,
                accessKey,
                secretKey,
                port,
                failed: false,
                inProgress: true,
                succeeded: false,
                folders: {
                    "-": {}
                },
                error: undefined,
                requestId: undefined,
                id: 1
            }})
        });

        test("should update failed", () => {
            const state = syncReducer(defaultState(), {
                type: wizardActions.SHOW_S3BASED_BUCKETS,
                status: wizardActions.STATUS_ERROR,
                id: 1,
                extra
            });

            expect(state._showS3BasedBuckets).toEqual({ '1.1.1.1' : {
                dataBrokerId,
                host,
                accessKey,
                secretKey,
                port,
                failed: true,
                inProgress: false,
                succeeded: false,
                folders: {
                    "-": {}
                },
                error: undefined,
                requestId: undefined,
                id: 1
            }})
        });

        test("should update success", () => {
            const state = syncReducer(defaultState(), {
                type: wizardActions.SHOW_S3BASED_BUCKETS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showS3BasedBuckets).toEqual({ '1.1.1.1' : {
                dataBrokerId,
                host,
                accessKey,
                secretKey,
                port,
                failed: false,
                inProgress: false,
                succeeded: true,
                folders: {
                    "-": {}
                },
                error: undefined,
                requestId: undefined,
                id: 1
            }})
        });

        test("should keep all the folders from the state ( port, awsAccessKey and awsSecretKey are identical)", () => {
            const state = syncReducer(buildStateWithSGBuckets(), {
                type: wizardActions.SHOW_S3BASED_BUCKETS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra
            });

            expect(state._showS3BasedBuckets).toEqual({ '1.1.1.1' : {
                dataBrokerId,
                host,
                port,
                secretKey,
                accessKey,
                failed: false,
                inProgress: false,
                succeeded: true,
                folders: {
                    "-": {data: [{actualPath: 'small', name: 'small', path: 'small'}], retrievedAll: true}
                },
                error: undefined,
                requestId: undefined,
                id: 1
            }})
        });


        test("should delete all the folders from the state  (different accessKey and secretKey)", () => {
            const accessKey = '2';
            const secretKey = '2';
            const state = syncReducer(buildStateWithSGBuckets(), {
                type: wizardActions.SHOW_S3BASED_BUCKETS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra: {host, dataBrokerId, accessKey, secretKey, port}
            });

            expect(state._showS3BasedBuckets).toEqual({ '1.1.1.1' : {
                dataBrokerId,
                host,
                accessKey,
                secretKey,
                port,
                failed: false,
                inProgress: false,
                succeeded: true,
                folders: {
                    "-": {}
                },
                error: undefined,
                requestId: undefined,
                id: 1
            }})
        });

        test("should delete all the folders from the state  (different port)", () => {
            const port = 2;
            const state = syncReducer(buildStateWithSGBuckets(), {
                type: wizardActions.SHOW_S3BASED_BUCKETS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra: {host, dataBrokerId, accessKey, secretKey, port}
            });

            expect(state._showS3BasedBuckets).toEqual({ '1.1.1.1' : {
                    dataBrokerId,
                    host,
                    accessKey,
                    secretKey,
                    port,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }})
        });

        test("should delete all the folders from the state  (different host)", () => {
            const host = "2.2.2.2";
            const state = syncReducer(buildStateWithSGBuckets(), {
                type: wizardActions.SHOW_S3BASED_BUCKETS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra: {host, dataBrokerId, accessKey, secretKey, port}
            });

            expect(state._showS3BasedBuckets[host]).toEqual({
                dataBrokerId,
                host,
                accessKey,
                secretKey,
                port,
                failed: false,
                inProgress: false,
                succeeded: true,
                folders: {
                    "-": {}
                },
                error: undefined,
                requestId: undefined,
                id: 1
            })
        });

        test("should delete all the folders from the state  (different dataBrokerId)", () => {
            const dataBrokerId = 2;
            const state = syncReducer(buildStateWithSGBuckets(), {
                type: wizardActions.SHOW_S3BASED_BUCKETS,
                status: wizardActions.STATUS_SUCCESS,
                id: 1,
                extra: {host, dataBrokerId, accessKey, secretKey, port}
            });

            expect(state._showS3BasedBuckets).toEqual({ '1.1.1.1' : {
                    dataBrokerId,
                    host,
                    accessKey,
                    secretKey,
                    port,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {}
                    },
                    error: undefined,
                    requestId: undefined,
                    id: 1
                }})
        });

    });

    describe("Show buckets", () => {
        const extra = {dataBrokerId : 1};
        test("should update inProgress", () => {
            const state = syncReducer(defaultState(), {type: wizardActions.SHOW_BUCKETS, status: wizardActions.STATUS_PROGRESS, id: 1, extra});

            expect(state._showBuckets).toEqual({
                "1": {
                    "failed": false,
                    "folders": {
                    "-": {}
                    },
                    "id": 1,
                    "inProgress": true,
                    "succeeded": false
                }
            })
        });

        test("should update failed", () => {
            const state = syncReducer(defaultState(), {type: wizardActions.SHOW_BUCKETS, status: wizardActions.STATUS_ERROR, id: 1, extra});

            expect(state._showBuckets).toEqual({
                "1": {
                failed: true,
                inProgress: false,
                error: undefined,
                requestId: undefined,
                succeeded: false,
                id: 1,
                folders: {
                    "-": {}
                }
                }
            })
        });

        test("should update success no showBuckets field", () => {
            const state = syncReducer(defaultState(), {type: wizardActions.SHOW_BUCKETS, status: wizardActions.STATUS_SUCCESS, id: 1, extra});

            expect(state._showBuckets).toEqual({
                "1": {
                failed: false,
                inProgress: false,
                error: undefined,
                requestId: undefined,
                succeeded: true,
                id: 1,
                folders: {
                    "-": {}
                }
                }
            })
        });

        test("should update success with showBuckets field", () => {
            const state = syncReducer(defaultState()
                .setIn(["_showBuckets","1"], {
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        '-': {
                            data: [{
                                actualPath: 'large',
                                name: 'large',
                                path: 'large',
                                region: {displayName: "EU (Frankfurt)", name: "eu-central-1"}, tags: []
                            }],
                            retrievedAll: true
                        }
                    },
                    id: 1
                }), {type: wizardActions.SHOW_BUCKETS, status: wizardActions.STATUS_SUCCESS, id: 1, extra});

            expect(state._showBuckets).toEqual({
                "1": {
                failed: false,
                inProgress: false,
                error: undefined,
                requestId: undefined,
                succeeded: true,
                id: 1,
                folders: {
                    '-': {
                        data: [{
                            actualPath: 'large',
                            name: 'large',
                            path: 'large',
                            region: {displayName: "EU (Frankfurt)", name: "eu-central-1"}, tags: []
                        }],
                        retrievedAll: true
                    }
                }
                }
            })
        });
    });

    describe("Start wizard", () => {
        test("should set the _wizard state to the payload", () => {
            const state = callSyncReducer(defaultState(), {
                type: wizardActions.START_WIZARD,
                payload: {protocols: "s3-nfs", wizardInfo, isSecure: false, stepsOrder, stepInfoById}
            });

            expect(state._wizard).toEqual({
                protocols: "s3-nfs",
                wizardInfo,
                isSecure: false,
                stepsOrder,
                stepInfoById,
                sync: {},
                sourceType: "storage",
                targetType: "fileSystem",
                wizardTypeForSystemFlow: "s3-nfs"
            })
        })
    });

    describe("Started wizard step", () => {
        test("should add the step name to the steps portion of wizard", () => {
            const wizardStartedState = callSyncReducer(defaultState(), {
                type: wizardActions.START_WIZARD,
                payload: {protocols: "s3-nfs", wizardInfo, isSecure: false, stepsOrder, stepInfoById}
            });
            const state = callSyncReducer(wizardStartedState, {
                type: wizardActions.STARTED_WIZARD_STEP,
                payload: "dataBroker"
            });

            expect(state._wizard).toEqual({
                isSourceStep: false,
                urlParams: {},
                protocols: "s3-nfs",
                wizardInfo,
                isSecure: false,
                stepsOrder,
                stepInfoById,
                sync: {},
                sourceType: "storage",
                targetType: "fileSystem",
                wizardTypeForSystemFlow: "s3-nfs",
                currentStep: "dataBroker",
                steps: {
                    "dataBroker": {started: true}
                }
            })
        });
    });

    describe("clear url params", () => {
        const paramsWithExports = {dataBrokerId: "1",
            exportPath:"---252Fdata1---252Ffolder1",
            nfsServer:"1.2.2.2",
            rootExport:"---252Fdata1"};

        const paramsWithoutExports = {dataBrokerId: "1",
            nfsServer:"1.2.2.2"};

        const buildStateWithWizardParams = () => defaultState()
            .setIn(["_wizard","urlParams"], paramsWithExports)
            .setIn(["_routing", "decodedParams"], paramsWithExports)
            .setIn(["_showExports", "1---252E2---252E2---252E2"], {id: "2"});

        const state1 = syncReducer(buildStateWithWizardParams(), {
            type: wizardActions.CLEAR_URL_PARAMS,
            payload: {changedParam: consts.SELECTED_PARAMS_NFS_SERVER, newVal:"1.3.2.2", without: ["exportPath", "rootExport"], extraKeyToClear: ["_showExports", "1---252E2---252E2---252E2"]}
        });

        test("should clear wizard and routing params, and also extraKeyToClear", () => {
            expect(state1._wizard).toEqual({"urlParams": paramsWithoutExports});
            expect(state1._routing.decodedParams).toEqual(undefined);
            expect(state1._showExports["1---252E2---252E2---252E2"]).toEqual(undefined);
        });
        const state2 = syncReducer(buildStateWithWizardParams(), {
            type: wizardActions.CLEAR_URL_PARAMS,
            payload: {changedParam: consts.SELECTED_PARAMS_NFS_SERVER, newVal:"1.3.2.2", without: ["exportPath", "rootExport"]}
        });

        test("should clear only urlParams if there is no extraKeyToClear", () => {
            expect(state2._wizard).toEqual({"urlParams": paramsWithoutExports});
            expect(state2._routing.decodedParams).toEqual(undefined);
            expect(state2._showExports["1---252E2---252E2---252E2"]).toEqual({id: "2"});

        });

        const state3 = syncReducer(buildStateWithWizardParams(), {
            type: wizardActions.CLEAR_URL_PARAMS,
            payload: {changedParam: consts.SELECTED_PARAMS_NFS_SERVER, newVal:"1.2.2.2", without: ["exportPath", "rootExport"], extraKeyToClear: ["_showExports", "1---252E2---252E2---252E2"]}
        });

        test("should not clear any params, nfs is the same", () => {
            expect(state3._wizard).toEqual({"urlParams": paramsWithExports});
            expect(state3._routing.decodedParams).toEqual(paramsWithExports);
            expect(state3._showExports["1---252E2---252E2---252E2"]).toEqual({id: "2"});
        });
    });

    describe("Clear a specific show action", () => {
        const buildStateWithShowFolders = () => defaultState()
            .set("_showShares", {
                "1---252E1---252E1---252E1": {id: 1},
                "2---252E2---252E2---252E2": {id: 2}
            });

        const state1 = syncReducer(buildStateWithShowFolders(), {
            type: wizardActions.CLEAR_SHOW_ACTION,
            payload: {keyToClear: ["_showShares", "1---252E1---252E1---252E1"]}
        });

        test("Should clear the relevant _showShares", () => {
            expect(state1._showShares).toEqual({
                "1---252E1---252E1---252E1": undefined,
                "2---252E2---252E2---252E2": {id: 2}
            });
        });
    });

    describe("Exited wizard step", () => {
        test("should remove the step name from the wizard steps", () => {
            const wizardStartedState = callSyncReducer(defaultState(), {
                type: wizardActions.START_WIZARD,
                payload: {protocols: "s3-nfs", wizardInfo, isSecure: false, stepsOrder, stepInfoById}
            });
            const wizardStep1StartedState = callSyncReducer(wizardStartedState, {
                type: wizardActions.STARTED_WIZARD_STEP,
                payload: "dataBroker"
            });
            const wizardStep2StartedState = callSyncReducer(wizardStep1StartedState, {
                type: wizardActions.STARTED_WIZARD_STEP,
                payload: "s3Buckets"
            });
            const state = callSyncReducer(wizardStep2StartedState, {
                type: wizardActions.EXITED_WIZARD_STEP,
                payload: "s3Buckets"
            });

            expect(state._wizard).toEqual({
                isSourceStep: false,
                urlParams: {},
                protocols: "s3-nfs",
                wizardInfo,
                isSecure: false,
                stepsOrder,
                stepInfoById,
                sync: {},
                sourceType: "storage",
                targetType: "fileSystem",
                wizardTypeForSystemFlow: "s3-nfs",
                currentStep: "dataBroker",
                steps: {
                    "dataBroker": {
                        started: true
                    }
                }
            })
        })
    });

    describe("Update Exports", () => {

        const host = '1.1.1.1';
        const hostEncoded = encodeAll(host);
        const dataBrokerId = 1;
        const extra = {host, dataBrokerId};
        const path = 'data1';

        const buildStateWithoutExportsFolders = () => defaultState()
            .set("_showExports", {
                [`${hostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        '-': {}
                    },
                    id: 1
                }
            });

        const buildStateWithExportsFolders = () => defaultState()
            .set("_showExports", {
                [`${hostEncoded}-${dataBrokerId}`]: {
                    dataBrokerId,
                    host,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {
                            data: [
                                {
                                    actualPath: '/' + path,
                                    host,
                                    path
                                }
                            ]
                        }
                    },
                    data: [
                        {
                            actualPath: '/' + path,
                            host,
                            path
                        }
                    ],
                    id: 1
                }
            });

        test("should update retrievedAll and failedRetrieve upon failure in show-exports", () => {

            const state = syncReducer(buildStateWithoutExportsFolders(), {
                type: wizardActions.UPDATE_EXPORTS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_SHOW_EXPORTS]: {
                        status: "FAILED",
                        host
                    }
                },
                status: wizardActions.STATUS_ERROR, id: 1, extra
            });


            expect(state._showExports).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                        dataBrokerId,
                        host,
                        failed: false,
                        failedRetrieve: true,
                        inProgress: false,
                        succeeded: true,
                        folders: {
                            "-": {
                                retrievedAll: true
                            }
                        },
                        retrievedAll: true,
                        id: 1
                    }
                }
            );
        });

        test("should update exports upon status running in show-exports and the state doesnt have any exports", () => {

            const state = syncReducer(buildStateWithoutExportsFolders(), {
                type: wizardActions.UPDATE_EXPORTS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_SHOW_EXPORTS]: {
                        status: "DONE",
                        host,
                        paths: [path]
                    }
                },
                status: wizardActions.STATUS_SUCCESS, id: 1, extra
            });
            expect(state._showExports).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                        dataBrokerId,
                        host,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        failedRetrieve: false,
                        retrievedAll: true,
                        folders: {
                            "-": {
                                data: [
                                    {
                                        actualPath: '/' + path,
                                        host,
                                        path
                                    }
                                ],
                                retrievedAll: true
                            }
                        },
                        data: [
                            {
                                actualPath: '/' + path,
                                host,
                                path
                            }
                        ],
                        id: 1
                    }
                }
            );
        });

        test("should update exports upon status running in show-exports and the state has previous exports", () => {

            const state = syncReducer(buildStateWithExportsFolders(), {
                type: wizardActions.UPDATE_EXPORTS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_SHOW_EXPORTS]: {
                        status: "DONE",
                        host,
                        paths: ['data2']
                    }
                },
                status: wizardActions.STATUS_SUCCESS, id: 1, extra
            });

            expect(state._showExports).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                        dataBrokerId,
                        host,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        failedRetrieve: false,
                        retrievedAll: true,
                        folders: {
                            "-": {
                                data: [
                                    {
                                        actualPath: '/' + path,
                                        host,
                                        path
                                    },
                                    {
                                        actualPath: "/data2",
                                        host,
                                        path: "data2"
                                    }
                                ],
                                retrievedAll: true,
                            }
                        },
                        data: [
                            {
                                actualPath: '/' + path,
                                host,
                                path
                            },
                            {
                                actualPath: "/data2",
                                host,
                                path: "data2"
                            }
                        ],
                        id: 1
                    }
                }
            );
        });

        test("should update retrievedAll and export upon status DONE in show-exports and the state doesnt have any exports", () => {

            const state = syncReducer(buildStateWithoutExportsFolders(), {
                type: wizardActions.UPDATE_EXPORTS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_SHOW_EXPORTS]: {
                        status: "DONE",
                        host,
                        paths: []
                    }
                },
                status: wizardActions.STATUS_SUCCESS, id: 1, extra
            });
            expect(state._showExports).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                        dataBrokerId,
                        host,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        failedRetrieve: false,
                        data: [],
                        folders: {
                            "-": {
                                data: [],
                                retrievedAll: true
                            }
                        },
                        retrievedAll: true,
                        id: 1
                    }
                }
            );
        });

        test("should update retrievedAll and export upon status DONE in show-exports and the state has previous exports", () => {

            const state = syncReducer(buildStateWithExportsFolders(), {
                type: wizardActions.UPDATE_EXPORTS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_SHOW_EXPORTS]: {
                        status: "DONE",
                        host,
                        paths: []
                    }
                },
                status: wizardActions.STATUS_SUCCESS, id: 1, extra
            });
            expect(state._showExports).toEqual({
                [`${hostEncoded}-${dataBrokerId}`]: {
                        dataBrokerId,
                        host,
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        failedRetrieve: false,
                        folders: {
                            "-": {
                                data: [
                                    {
                                        actualPath: '/' + path,
                                        host,
                                        path
                                    }
                                ],
                                retrievedAll: true
                            }
                        },
                        data: [
                            {
                                actualPath: '/' + path,
                                host,
                                path
                            }
                        ],
                        retrievedAll: true,
                        id: 1
                    }
                }
            );
        });

        test("should return state with unhandledMessages - _showExports is undefined", () => {
            const action = {
                type: wizardActions.UPDATE_EXPORTS, payload: {
                    [consts.PAYLOAD_SHOW_EXPORTS]: {
                        status: "DONE",
                        host: '2.2.2.2'
                    },
                    dataBrokerId:1
                },
                status: wizardActions.STATUS_SUCCESS, id: 1, extra
            };

            const state = syncReducer(defaultState(), action);
            action["unhandledCounter"] = 2;
            expect(state).toEqual(defaultState().set("unhandledMessages", [action]));
        });

        test("should return state with unhandledMessages - host is not like _showExports.host", () => {

            const action = {
                type: wizardActions.UPDATE_EXPORTS, payload: {
                    [consts.PAYLOAD_SHOW_EXPORTS]: {
                        status: "DONE",
                        host: '2.2.2.2'
                    }, status: wizardActions.STATUS_SUCCESS, id: 1, extra
                }
            };

            const state = syncReducer(buildStateWithoutExportsFolders(), action);
            action["unhandledCounter"] = 2;
            expect(state).toEqual(buildStateWithoutExportsFolders().set("unhandledMessages", [action]));

        });

    });

    describe("Update s3based Buckets", () => {
        const host = '1.1.1.1';
        const port = 8080;
        const accessKey = 'a';
        const secretKey = 'b';

        const stateWithShowSGBucketsField = () => defaultState().set("_showS3BasedBuckets", { '1.1.1.1' : {
            failed: false,
            inProgress: false,
            error: undefined,
            requestId: undefined,
            succeeded: true,
            id: 1,
            folders: {
                "-": {}
            },
            host,
            port,
            accessKey,
            secretKey

        }});

        test("should return state with unhandledMessages - _showS3BasedBuckets is undefined", () => {

            const action = {
                type: wizardActions.UPDATE_S3BASED_BUCKETS, payload: {
                    [consts.PAYLOAD_LIST_SGWS_BUCKETS]: {
                        buckets: [{name: 'large'},
                            {name: 'small'}],
                        host,
                        port,
                        status: 'DONE'
                    }
                }, status: wizardActions.STATUS_SUCCESS, id: 1
            };

            const state = syncReducer(defaultState(), action);
            action["unhandledCounter"] = 2;
            expect(state).toEqual(defaultState().set("unhandledMessages", [action]));
        });

        test("should return retrievedAll true and not update the list - action status is failed", () => {

            const state = syncReducer(stateWithShowSGBucketsField(), {
                type: wizardActions.UPDATE_S3BASED_BUCKETS, payload: {
                    [consts.PAYLOAD_LIST_SGWS_BUCKETS]: {
                        status: 'FAILED',
                        host: '1.1.1.1'
                    }, status: wizardActions.STATUS_ERROR, id: 1
                }
            });
            expect(state._showS3BasedBuckets).toEqual({ '1.1.1.1': {

                failed: false,
                inProgress: false,
                error: undefined,
                requestId: undefined,
                succeeded: true,
                id: 1,
                folders: {
                    "-": {retrievedAll: true}
                },
                retrievedAll: true,
                failedRetrieve: true,
                host,
                port,
                accessKey,
                secretKey
            }});
        });

        test("should return retrievedAll true and check list was updated - action status is DONE", () => {

            const state = syncReducer(stateWithShowSGBucketsField(), {
                type: wizardActions.UPDATE_S3BASED_BUCKETS, payload: {
                    [consts.PAYLOAD_LIST_SGWS_BUCKETS]: {
                        buckets: [{name: 'large'},
                            {name: 'small'}],
                        host,
                        port,
                        status: 'DONE'
                    }
                }
                , status: wizardActions.STATUS_SUCCESS, id: 1
            });
            expect(state._showS3BasedBuckets).toEqual({ '1.1.1.1' : {

                failed: false,
                inProgress: false,
                error: undefined,
                requestId: undefined,
                succeeded: true,
                id: 1,
                data: [{actualPath: 'large', path: 'large', name: 'large'},
                    {actualPath: 'small', path: 'small', name: 'small'}],
                folders: {
                    "-": {
                        data: [{actualPath: 'large', path: 'large', name: 'large'},
                            {actualPath: 'small', path: 'small', name: 'small'}],
                        retrievedAll: true
                    }
                },
                retrievedAll: true,
                failedRetrieve: false,
                host,
                port,
                accessKey,
                secretKey
            }});
        });

        test("should return retrievedAll true and check list was updated - no buckets", () => {

            const state = syncReducer(stateWithShowSGBucketsField(), {
                type: wizardActions.UPDATE_S3BASED_BUCKETS, payload: {
                    [consts.PAYLOAD_LIST_SGWS_BUCKETS]: {
                        buckets: [], host, port, status: 'DONE'
                    }, status: wizardActions.STATUS_SUCCESS, id: 1
                }
            });
            expect(state._showS3BasedBuckets).toEqual({ '1.1.1.1': {
                    failed: false,
                    inProgress: false,
                    error: undefined,
                    requestId: undefined,
                    succeeded: true,
                    id: 1,
                    data: [],
                    folders: {
                        "-": {
                            data: [],
                            retrievedAll: true
                        }
                    },
                    retrievedAll: true,
                    failedRetrieve: false,
                    host,
                    port,
                    accessKey,
                    secretKey
                }}
                );
        });
    });

    describe("Update Buckets", () => {
        const dataBrokerId = "1";
        const stateWithShowBucketsField = () => defaultState().set("_showBuckets", {
            "1" : {
            failed: false,
            inProgress: false,
            error: undefined,
            requestId: undefined,
            succeeded: true,
            id: 1,
            folders: {
                "-": {}
            }
        }
        } );

        test("should return state with unhandledMessages - _showBuckets is undefined", () => {

            const action = {
                type: wizardActions.UPDATE_BUCKETS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_BUCKETS]: [{
                        name: 'large',
                        region: {displayName: 'EU (Frankfurt)', name: 'eu-central-1'},
                        tags: []
                    },
                        {name: 'small', region: {displayName: 'US West (Oregon)', name: 'us-west-2'}, tags: []}]
                }, status: wizardActions.STATUS_SUCCESS, id: 1
            };

            const state = syncReducer(defaultState(), action);
            action["unhandledCounter"] = 2;
            expect(state).toEqual(defaultState().set("unhandledMessages", [action]));
        });

        test("should return retrievedAll true and not update the list - action status is failed", () => {

            const state = syncReducer(stateWithShowBucketsField(), {
                type: wizardActions.UPDATE_BUCKETS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_BUCKETS]: {
                        status: 'FAILED'
                    }
                }, status: wizardActions.STATUS_ERROR, id: 1
            });
            expect(state._showBuckets).toEqual({
                "1" : {
                    failed: false,
                    inProgress: false,
                    error: undefined,
                    requestId: undefined,
                    succeeded: true,
                    id: 1,
                    folders: {
                        "-": {retrievedAll: true}
                    },
                    retrievedAll: true,
                    failedRetrieve: true
                }
                }
                );
        });

        test("should return retrievedAll true and check list was updated - action status is DONE", () => {

            const state = syncReducer(stateWithShowBucketsField(), {
                type: wizardActions.UPDATE_BUCKETS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_BUCKETS]: {buckets:[{
                        name: 'large',
                        region: {displayName: 'EU (Frankfurt)', name: 'eu-central-1'},
                        tags: []
                    },
                        {name: 'small', region: {displayName: 'US West (Oregon)', name: 'us-west-2'}, tags: []}]}
                }, status: wizardActions.STATUS_SUCCESS, id: 1
            });
            expect(state._showBuckets).toEqual({
                "1" : {
                    failed: false,
                    inProgress: false,
                    error: undefined,
                    requestId: undefined,
                    succeeded: true,
                    id: 1,
                    data: [{
                        actualPath: 'large',
                        path: 'large',
                        name: 'large',
                        region: {displayName: 'EU (Frankfurt)', name: 'eu-central-1'},
                        tags: []
                    },
                        {
                            actualPath: 'small',
                            path: 'small',
                            name: 'small',
                            region: {displayName: 'US West (Oregon)', name: 'us-west-2'},
                            tags: []
                        }],
                    folders: {
                        "-": {
                            data: [{
                                actualPath: 'large',
                                path: 'large',
                                name: 'large',
                                region: {displayName: 'EU (Frankfurt)', name: 'eu-central-1'},
                                tags: []
                            },
                                {
                                    actualPath: 'small',
                                    path: 'small',
                                    name: 'small',
                                    region: {displayName: 'US West (Oregon)', name: 'us-west-2'},
                                    tags: []
                                }],
                            retrievedAll: true
                        }
                    },
                    retrievedAll: true,
                    failedRetrieve: false

                }
                }
                );
        });

        test("should return retrievedAll true and check list was updated - no buckets", () => {

            const state = syncReducer(stateWithShowBucketsField(), {
                type: wizardActions.UPDATE_BUCKETS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_BUCKETS]: {buckets:[]}
                }, status: wizardActions.STATUS_SUCCESS, id: 1
            });
            expect(state._showBuckets).toEqual({
                "1" : {
                    failed: false,
                    inProgress: false,
                    error: undefined,
                    requestId: undefined,
                    succeeded: true,
                    id: 1,
                    data: [],
                    folders: {
                        "-": {
                            data: [],
                            retrievedAll: true
                        }
                    },
                    retrievedAll: true,
                    failedRetrieve: false
                }
                }
                );
        });
    });

    describe("Update Buckets Folders", () => {
        const dataBrokerId = 1;
        const bucket = 'thisBucket';
        const path = 'medium2';
        const stateWithShowBucketsFieldWithBuckets = () => defaultState().setIn(["_showBuckets", "1"], {
            error: undefined,
            failed: false,
            inProgress: false,
            succeeded: true,
            data: [{
                actualPath: bucket,
                name: bucket,
                path: bucket,
                region: {
                    displayName: 'US WEST (Oregon)',
                    name: 'us-west-2'
                },
                tags: []
            }],
            folders: {
                '-': {
                    data: [{
                        actualPath: bucket,
                        name: bucket,
                        path: bucket,
                        region: {
                            displayName: 'US WEST (Oregon)',
                            name: 'us-west-2'
                        },
                        tags: []
                    }],
                    retrievedAll: true
                },
                "thisBucket": {
                    data: [],
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    requestId: undefined,
                    id: 1
                }
            },
            retrievedAll: true,
            id: 1
        });


        const stateWithBucketsAndExtraListFolders = () => defaultState().setIn(["_showBuckets", "1"], {
            error: undefined,
            failed: false,
            inProgress: false,
            succeeded: true,
            data: [{
                actualPath: bucket,
                name: bucket,
                path: bucket,
                region: {
                    displayName: 'US WEST (Oregon)',
                    name: 'us-west-2'
                },
                tags: []
            }],
            folders: {
                '-': {
                    data: [{
                        actualPath: bucket,
                        name: bucket,
                        path: bucket,
                        region: {
                            displayName: 'US WEST (Oregon)',
                            name: 'us-west-2'
                        },
                        tags: []
                    }],
                    retrievedAll: true
                },
                "thisBucket": {
                    data: [{actualPath: `${bucket}/${path}`, bucket, name: path, path}],
                    failed: false,
                    inProgress: false,
                    retrievedAll: true,
                    succeeded: true,
                    requestId: undefined,
                    id: 1
                },
                "thisBucket/medium2": {
                    data: [],
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    requestId: undefined,
                    id: 1
                }
            },
            retrievedAll: true,
            id: 1
        });

        test("should return state with unhandledMessages - _showBuckets is undefined", () => {
            const data = {
                path: bucket,
                status: 'DONE',
                bucket
            };
            const action = {
                type: wizardActions.UPDATE_BUCKETS_FOLDERS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_SUB_DIRECTORIES]: data,
                    status: undefined, id: 1,
                    data
                }
            };

            const state = syncReducer(defaultState(), action);
            action["unhandledCounter"] = 2;
            expect(state).toEqual(defaultState().set("unhandledMessages", [action]));
        });

        test("should return undefined and the same state that was sent- state._showBuckets.data is not exist", () => {
            const show =
                {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {
                            data: [],
                            retrievedAll: true
                        }
                    },
                    id: 1
                };

            const data = {
                path: bucket,
                status: 'DONE',
                bucket
            };
            const action = {
                type: wizardActions.UPDATE_BUCKETS_FOLDERS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_SUB_DIRECTORIES]: data,
                    status: undefined, id: 1,
                    data
                }
            };

            const state = syncReducer(defaultState().setIn(["_showBuckets", "1"], show),action );

            expect(state._showBuckets["1"]).toEqual(show);
        });

        test("Should return the same state that was sent - state._showBuckets.data = []", () => {
            const prevState = defaultState()
                .setIn(["_showBuckets","1"], {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [],
                    folders: {
                        "-": {
                            data: [],
                            retrievedAll: true
                        }
                    },
                    id: 1
                });

            const data = {
                path: bucket,
                status: 'DONE',
                bucket
            };
            const action = {
                type: wizardActions.UPDATE_BUCKETS_FOLDERS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_SUB_DIRECTORIES]: data,
                    status: undefined, id: 1,
                    data
                }
            };
            const currState = syncReducer(prevState, action);
            expect(currState).toEqual(prevState);
        });

        test("Should return the same state that was sent - bucket name does not exist in data", () => {

            const prevState = defaultState()
                .setIn(["_showBuckets","1"], {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [],
                    folders: {
                        "-": {
                            data: [],
                            retrievedAll: true
                        }
                    },
                    id: 1
                });

            const data = {
                path: 'bucket1',
                status: 'DONE',
                bucket: 'bucket1'
            };

            const state = syncReducer(prevState, {
                type: wizardActions.UPDATE_BUCKETS_FOLDERS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_SUB_DIRECTORIES]: data,
                    status: undefined, id: 1,
                    data
                }
            });
            expect(state._showBuckets).toEqual(prevState._showBuckets);
        });

        test("Should return the same state that was sent - path does not exist in folders", () => {

            const prevState = defaultState()
                .setIn(["_showBuckets", "1" ], {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [],
                    folders: {
                        "-": {
                            data: [],
                            retrievedAll: true
                        }
                    },
                    id: 1
                });
            const data = {
                path: `${bucket}/bucket11`,
                status: 'DONE',
                bucket
            };

            const state = syncReducer(prevState, {
                type: wizardActions.UPDATE_BUCKETS_FOLDERS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_SUB_DIRECTORIES]: data,
                    status: undefined, id: 1,
                    data
                }
            });
            expect(state._showBuckets).toEqual(prevState._showBuckets);

        });


        test("should return retrievedAll true and check list was updated - action status is DONE", () => {
            const data = {
                status: 'DONE',
                list: ['medium2'],
                bucket
            };

            const state = syncReducer(stateWithShowBucketsFieldWithBuckets(), {
                type: wizardActions.UPDATE_BUCKETS_FOLDERS,
                payload: {
                    dataBrokerId,
                    status: undefined,
                    id: 1,
                    data,
                    [consts.PAYLOAD_LIST_SUB_DIRECTORIES]: data
                }
            });
            expect(state._showBuckets["1"]).toEqual({

                failed: false,
                inProgress: false,
                error: undefined,
                succeeded: true,
                id: 1,
                data: [{
                    actualPath: bucket,
                    name: bucket,
                    path: bucket,
                    region: {
                        displayName: 'US WEST (Oregon)',
                        name: 'us-west-2'
                    },
                    tags: []
                }],
                folders: {
                    "-": {
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket,
                            region: {
                                displayName: 'US WEST (Oregon)',
                                name: 'us-west-2'
                            },
                            tags: []
                        }],
                        retrievedAll: true
                    },
                    'thisBucket': {
                        data: [{actualPath: 'thisBucket/medium2', bucket, name: 'medium2', path: 'medium2'}],
                        failed: false,
                        inProgress: false,
                        id: 1,
                        retrievedAll: true,
                        requestId: undefined,
                        succeeded: true
                    }
                },
                retrievedAll: true
            });
        });

        test("should return retrievedAll true and check list was updated - action status is DONE - deep dive", () => {
            const data = {
                prefix: '/medium2',
                status: 'DONE',
                list: ['medium3'],
                bucket
            };

            const state = syncReducer(stateWithBucketsAndExtraListFolders(), {
                    type: wizardActions.UPDATE_BUCKETS_FOLDERS,
                    payload: {
                        dataBrokerId,
                        status: undefined,
                        id: 1,
                        data,
                        [consts.PAYLOAD_LIST_SUB_DIRECTORIES]: data
                    }
                }
            );
            expect(state._showBuckets["1"]).toEqual({
                error: undefined,
                failed: false,
                inProgress: false,
                succeeded: true,
                data: [{
                    actualPath: bucket,
                    name: bucket,
                    path: bucket,
                    region: {
                        displayName: 'US WEST (Oregon)',
                        name: 'us-west-2'
                    },
                    tags: []
                }],
                folders: {
                    '-': {
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket,
                            region: {
                                displayName: 'US WEST (Oregon)',
                                name: 'us-west-2'
                            },
                            tags: []
                        }],
                        retrievedAll: true
                    },
                    "thisBucket": {
                        data: [{actualPath: `${bucket}/${path}`, bucket, name: path, path}],
                        failed: false,
                        retrievedAll: true,
                        inProgress: false,
                        succeeded: true,
                        requestId: undefined,
                        id: 1
                    },
                    "thisBucket/medium2": {
                        data: [
                            {
                                actualPath: "thisBucket/medium2/medium3",
                                bucket,
                                name: "medium3",
                                path: "medium3"
                            }
                        ],
                        failed: false,
                        retrievedAll: true,
                        inProgress: false,
                        succeeded: true,
                        requestId: undefined,
                        id: 1
                    }
                },
                retrievedAll: true,
                id: 1
            });
        });


        test("should return retrievedAll true and check list was updated - action status is FAILED", () => {
            const data = {
                path: bucket,
                status: 'FAILED',
                bucket,
                list: ['medium2']
            };

            const state = syncReducer(stateWithShowBucketsFieldWithBuckets(), {
                type: wizardActions.UPDATE_BUCKETS_FOLDERS, payload: {
                    dataBrokerId,
                    [consts.PAYLOAD_LIST_SUB_DIRECTORIES]: data,
                    status: undefined, id: 1,
                    data
                }
            });
            expect(state._showBuckets["1"]).toEqual({

                failed: false,
                inProgress: false,
                error: undefined,
                succeeded: true,
                id: 1,
                data: [{
                    actualPath: bucket,
                    name: bucket,
                    path: bucket,
                    region: {
                        displayName: 'US WEST (Oregon)',
                        name: 'us-west-2'
                    },
                    tags: []
                }],
                folders: {
                    "-": {
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket,
                            region: {
                                displayName: 'US WEST (Oregon)',
                                name: 'us-west-2'
                            },
                            tags: []
                        }],
                        retrievedAll: true
                    },
                    'thisBucket': {
                        data: [],
                        failed: false,
                        inProgress: false,
                        id: 1,
                        retrievedAll: true,
                        requestId: undefined,
                        succeeded: true
                    }
                },
                retrievedAll: true
            });
        });
    });

    describe("Update s3based Buckets Folders", () => {
        const dataBrokerId = 1;
        const bucket = 'thisBucket';
        const port = 8080;
        const accessKey = 'a';
        const secretKey = 'a';
        const host = '1.1.1.1';
        const path = 'medium2';

        const stateWithShowSGBucketsFieldWithBuckets = () => defaultState().set("_showS3BasedBuckets", { '1.1.1.1' : {
            error: undefined,
            failed: false,
            inProgress: false,
            port,
            accessKey,
            secretKey,
            succeeded: true,
            data: [{
                actualPath: bucket,
                name: bucket,
                path: bucket
            }],
            folders: {
                '-': {
                    data: [{
                        actualPath: bucket,
                        name: bucket,
                        path: bucket
                    }],
                    retrievedAll: true
                },
                "thisBucket": {
                    data: [],
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    requestId: undefined,
                    id: 1
                }
            },
            retrievedAll: true,
            id: 1
        }});


        const stateWithSGBucketsAndExtraListFolders = () => defaultState().set("_showS3BasedBuckets", { '1.1.1.1': {
                error: undefined,
                failed: false,
                inProgress: false,
                port,
                accessKey,
                secretKey,
                succeeded: true,
                data: [{
                    actualPath: bucket,
                    name: bucket,
                    path: bucket
                }],
                folders: {
                    '-': {
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket
                        }],
                        retrievedAll: true
                    },
                    "thisBucket": {
                        data: [{actualPath: 'thisBucket/medium2', bucket, name: 'medium2', path: 'medium2'}],
                        failed: false,
                        inProgress: false,
                        id: 1,
                        retrievedAll: true,
                        requestId: undefined,
                        succeeded: true
                    },
                    "thisBucket/medium2": {
                        data: [],
                        failed: false,
                        inProgress: false,
                        succeeded: true,
                        requestId: undefined,
                        id: 1
                    }
                },
                retrievedAll: true,
                id: 1
            }});

        test("should return state with unhandledMessages - _showS3BasedBuckets is undefined", () => {
            const data = {
                path: bucket,
                status: 'DONE',
                bucket,
                host
            };
            const action = {
                type: wizardActions.UPDATE_S3BASED_BUCKETS_FOLDERS,
                payload: {
                    [consts.PAYLOAD_LIST_SGWS_BUCKETS_FOLDERS]: data,
                    status: undefined,
                    id: 1,
                    data
                }
            };

            const state = syncReducer(defaultState(), action);
            action["unhandledCounter"] = 2;
            expect(state).toEqual(defaultState().set("unhandledMessages", [action]));
        });

        test("should return undefined and the same state that was sent- state._showS3BasedBuckets.data is not exist", () => {
            const show =
                { '1.1.1.1': {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    folders: {
                        "-": {
                            data: [],
                            retrievedAll: true
                        }
                    },
                    id: 1
                }};

            const state = syncReducer(defaultState().set("_showS3BasedBuckets", show), {
                type: wizardActions.UPDATE_S3BASED_BUCKETS_FOLDERS, payload: {
                    [consts.PAYLOAD_LIST_SGWS_BUCKETS_FOLDERS]: {
                        path: bucket,
                        status: 'DONE',
                        bucket,
                    }, status: undefined, id: 1,
                    data: {
                        path: bucket,
                        status: 'DONE',
                        bucket,
                        host
                    }
                }
            });

            expect(state._showS3BasedBuckets).toEqual(show);
            expect(state).toEqual(defaultState().set("_showS3BasedBuckets", show));
        });

        test("Should return the same state that was sent - state._showS3BasedBuckets.data = []", () => {
            const prevState = defaultState()
                .set("_showS3BasedBuckets", { '1.1.1.1': {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [],
                    folders: {
                        "-": {
                            data: [],
                            retrievedAll: true
                        }
                    },
                    id: 1
                }});
            const currState = syncReducer(prevState, {
                type: wizardActions.UPDATE_S3BASED_BUCKETS_FOLDERS, payload: {
                    [consts.PAYLOAD_LIST_SGWS_BUCKETS_FOLDERS]: {
                        path: bucket,
                        status: 'DONE',
                        bucket,
                    }, status: undefined, id: 1
                    ,
                    data: {
                        path: bucket,
                        status: 'DONE',
                        bucket,
                        host
                    }
                }
            });


            expect(currState).toEqual(prevState);
        });

        test("Should return the same state that was sent - bucket name does not exist in data", () => {

            const prevState = defaultState()
                .set("_showS3BasedBuckets", { '1.1.1.1': {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [],
                    folders: {
                        "-": {
                            data: [],
                            retrievedAll: true
                        }
                    },
                    id: 1
                }});

            const state = syncReducer(prevState, {
                type: wizardActions.UPDATE_S3BASED_BUCKETS_FOLDERS, payload: {
                    [consts.PAYLOAD_LIST_SGWS_BUCKETS_FOLDERS]: {
                        path: 'bucket1',
                        status: 'DONE',
                        bucket: 'bucket1',
                    }, status: undefined, id: 1
                    ,
                    data: {
                        path: 'bucket1',
                        status: 'DONE',
                        bucket: 'bucket1',
                        host
                    }
                }
            });
            expect(state._showS3BasedBuckets).toEqual(prevState._showS3BasedBuckets);

        });

        test("Should return the same state that was sent - path does not exist in folders", () => {

            const prevState = defaultState()
                .set("_showS3BasedBuckets", { '1.1.1.1': {
                    dataBrokerId,
                    failed: false,
                    inProgress: false,
                    succeeded: true,
                    data: [],
                    folders: {
                        "-": {
                            data: [],
                            retrievedAll: true
                        }
                    },
                    id: 1
                }});

            const state = syncReducer(prevState, {
                type: wizardActions.UPDATE_S3BASED_BUCKETS_FOLDERS, payload: {
                    [consts.PAYLOAD_LIST_SGWS_BUCKETS_FOLDERS]: {
                        path: `${bucket}/bucket11`,
                        status: 'DONE',
                        bucket,
                    }, status: undefined, id: 1
                    ,
                    data: {
                        path: `${bucket}/bucket11`,
                        status: 'DONE',
                        bucket,
                        host
                    }
                }
            });
            expect(state._showS3BasedBuckets).toEqual(prevState._showS3BasedBuckets);

        });


        test("should return retrievedAll true and check list was updated - action status is DONE", () => {
            const data = {
                status: 'DONE',
                bucket,
                list: ['medium2'],
                host,
                port
            };
            const action = {
                type: wizardActions.UPDATE_S3BASED_BUCKETS_FOLDERS, payload: {
                    [consts.PAYLOAD_LIST_SGWS_BUCKETS_FOLDERS]: data, status: undefined, id: 1,
                    data
                }
            };

            const state = syncReducer(stateWithShowSGBucketsFieldWithBuckets(), action);
            expect(state._showS3BasedBuckets).toEqual({ '1.1.1.1': {
                accessKey,
                port,
                secretKey,
                failed: false,
                inProgress: false,
                error: undefined,
                succeeded: true,
                id: 1,
                data: [{
                    actualPath: bucket,
                    name: bucket,
                    path: bucket

                }],
                folders: {
                    "-": {
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket

                        }],
                        retrievedAll: true
                    },
                    'thisBucket': {
                        data: [{actualPath: 'thisBucket/medium2', bucket, name: 'medium2', path: 'medium2'}],
                        failed: false,
                        inProgress: false,
                        id: 1,
                        retrievedAll: true,
                        requestId: undefined,
                        succeeded: true
                    }
                },
                retrievedAll: true
            }});
        });


        test("should return retrievedAll true and check list was updated - action status is FAILED", () => {

            const data = {
                status: 'FAILED',
                bucket,
                list: ['medium2'],
                host,
                port
            };
            const action = {
                type: wizardActions.UPDATE_S3BASED_BUCKETS_FOLDERS, payload: {
                    [consts.PAYLOAD_LIST_SGWS_BUCKETS_FOLDERS]: data, status: undefined,
                    id: 1,
                    data
                }
            };

            const state = syncReducer(stateWithShowSGBucketsFieldWithBuckets(), action);
            expect(state._showS3BasedBuckets).toEqual({ '1.1.1.1': {
                accessKey,
                port,
                secretKey,
                failed: false,
                inProgress: false,
                error: undefined,
                succeeded: true,
                id: 1,
                data: [{
                    actualPath: bucket,
                    name: bucket,
                    path: bucket

                }],
                folders: {
                    "-": {
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket

                        }],
                        retrievedAll: true
                    },
                    'thisBucket': {
                        data: [],
                        failed: false,
                        inProgress: false,
                        id: 1,
                        retrievedAll: true,
                        requestId: undefined,
                        succeeded: true
                    }
                },
                retrievedAll: true
            }});
        });

        test("should return retrievedAll true and check list was updated - action status is DONE -  deep dive", () => {
            const data = {
                status: 'DONE',
                list: ['medium3'],
                prefix: `/${path}`,
                bucket,
                host,
                port
            };

            const state = syncReducer(stateWithSGBucketsAndExtraListFolders(), {
                type: wizardActions.UPDATE_S3BASED_BUCKETS_FOLDERS,
                payload: {
                    [consts.PAYLOAD_LIST_SGWS_BUCKETS_FOLDERS]: data,
                    status: undefined,
                    id: 1,
                    data
                }
            });
            expect(state._showS3BasedBuckets).toEqual({ '1.1.1.1': {
                accessKey,
                secretKey,
                port,
                error: undefined,
                failed: false,
                inProgress: false,
                succeeded: true,
                data: [{
                    actualPath: bucket,
                    name: bucket,
                    path: bucket
                }],
                folders: {
                    '-': {
                        data: [{
                            actualPath: bucket,
                            name: bucket,
                            path: bucket
                        }],
                        retrievedAll: true
                    },
                    "thisBucket": {
                        data: [{actualPath: `${bucket}/${path}`, bucket, name: path, path}],
                        failed: false,
                        retrievedAll: true,
                        inProgress: false,
                        succeeded: true,
                        requestId: undefined,
                        id: 1
                    },
                    "thisBucket/medium2": {
                        data: [
                            {
                                actualPath: "thisBucket/medium2/medium3",
                                bucket,
                                name: "medium3",
                                path: "medium3"
                            }
                        ],
                        failed: false,
                        retrievedAll: true,
                        inProgress: false,
                        succeeded: true,
                        requestId: undefined,
                        id: 1
                    }
                },
                retrievedAll: true,
                id: 1
            }});
        });

    });

    describe("Clear Endpoint cache",() => {
        test("should clear endpint cache", () => {
            const initialState = defaultState().setIn(["_showExports", "netapp---252Ecom-5e9453c1521e0d2838f9c16e"], {})
            const action = {
                type: wizardActions.CLEAR_FOLDERS_CACHE,
                payload: {cacheName:'_showExports', storageKey:'netapp---252Ecom-5e9453c1521e0d2838f9c16e'}
            };
            const finalState = syncReducer(initialState, action);
            expect(finalState._showExports['netapp---252Ecom-5e9453c1521e0d2838f9c16e']).toEqual(undefined);
        });
    });
});

describe("handleErrors", () => {
    beforeEach(() => {
        routesContainer.routes = {};
    });
    test("should update the error in the errors map and set errorsPanel to open, and clear it when success for the same message arrives", function () {
        const state = syncReducer(defaultState(), {
            "type": "DELETE_STORAGE_SYSTEM",
            "payload": {
                "data": {
                    "message": "Error deleting storage system"
                }
            },
            "status": wizardActions.STATUS_ERROR
        });

        expect(state._generalErrors._errorsMap).toEqual(expect.objectContaining({"DELETE_STORAGE_SYSTEM": expect.anything()}));
        expect(state._generalErrors._errorsMap["DELETE_STORAGE_SYSTEM"]).toEqual(expect.objectContaining({
            id: "DELETE_STORAGE_SYSTEM",
            message: "Error deleting storage system"
        }));
        expect(state._generalErrors._errorsPanelOpen).toEqual(true);

        //now clear the error
        const clearedState = syncReducer(state, {
            "type": "DELETE_STORAGE_SYSTEM",
            "payload": {},
            "status": wizardActions.STATUS_SUCCESS
        });
        expect(clearedState._generalErrors._errorsMap).toEqual({});
    });

    test("should network errors properly", function () {
        const state = syncReducer(defaultState(), {
            "type": "DELETE_STORAGE_SYSTEM",
            "payload": {status: 504},
            "status": wizardActions.STATUS_ERROR
        });

        expect(state._generalErrors._errorsMap).toEqual(expect.objectContaining({"NETWORK": expect.anything()}));
        expect(state._generalErrors._errorsMap["NETWORK"]).toEqual(expect.objectContaining({
            id: "NETWORK",
            message: "Please check your network connectivity and refresh the browser."
        }));
        expect(state._generalErrors._errorsPanelOpen).toEqual(true);

        //now claer the error on any success
        const clearedState = syncReducer(state, {
            "type": "DELETE_STORAGE_SYSTEM",
            "payload": {},
            "status": wizardActions.STATUS_SUCCESS
        });
        expect(clearedState._generalErrors._errorsMap).toEqual({});
    });

    test("should handle client errors properly", function () {
        const state = syncReducer(defaultState(), {
            "type": "DELETE_STORAGE_SYSTEM",
            "payload": new Error("Not a network error"),
            "status": wizardActions.STATUS_ERROR
        });

        expect(state._generalErrors._errorsMap).toEqual(expect.objectContaining({"DELETE_STORAGE_SYSTEM": expect.anything()}));
        expect(state._generalErrors._errorsMap["DELETE_STORAGE_SYSTEM"]).toEqual(expect.objectContaining({
            id: "DELETE_STORAGE_SYSTEM",
            message: "Client Error: Not a network error"
        }));
        expect(state._generalErrors._errorsPanelOpen).toEqual(true);

        //now claer the error on any success
        const clearedState = syncReducer(state, {
            "type": "DELETE_STORAGE_SYSTEM",
            "payload": {},
            "status": wizardActions.STATUS_SUCCESS
        });
        expect(clearedState._generalErrors._errorsMap).toEqual({});
    });

    test("should remove the right error when there are more than 1 and only one of them is success", function () {
        const state = syncReducer(defaultState(), {
            "type": "FIRST_ERROR",
            "payload": {
                "data": {
                    "message": "Error deleting storage system"
                }
            },
            "status": wizardActions.STATUS_ERROR
        });
        expect(state._generalErrors._errorsMap).toEqual(expect.objectContaining({"FIRST_ERROR": expect.anything()}));

        const state1 = syncReducer(state, {
            "type": "ANOTHER_ERROR",
            "payload": {
                "data": {
                    "message": "this is another error"
                }
            },
            "status": wizardActions.STATUS_ERROR
        });
        expect(state1._generalErrors._errorsMap).toEqual(expect.objectContaining({
            "FIRST_ERROR": expect.anything(),
            "ANOTHER_ERROR": expect.anything()
        }));

        //now clear the first
        const clearedState = syncReducer(state1, {
            "type": "FIRST_ERROR",
            "payload": {},
            "status": wizardActions.STATUS_SUCCESS
        });
        expect(clearedState._generalErrors._errorsMap).not.toEqual(expect.objectContaining({"FIRST_ERROR": expect.anything()}));
    });
});