import {
    tagsToString,
    stringToTags,
    removeKeyFromPath,
    createStringFromEndPointObject,
    formatActivityValues,
    getSelectOptions,
    convertSizeToBytes,
    convertBytesToSize,
    createSplitPath
} from "./../../src/utils/helpers";

describe('createStringFromEndPointObject', () => {
    const host = '1.1.1.1';
    const nfsExport = 'export';
    const path = 'folder1/folder2';
    const bucket = 'bucket';
    const port = '80';

    function validateStringCreatedFromEndPoint(endPoint, expectedString) {
        expect(createStringFromEndPointObject(endPoint)).toEqual(expectedString);
    }

    function buildEndPoint(protocol, details) {
        return {protocol, [protocol]: details};
    }

    function buildFileSystemDetailObject(protocol, host, share, path) {
        return (protocol === 'nfs') ? {host, export: share, path} : {host, share, path};
    }

    function buildStorageDetailObject(protocol, provider, bucket, prefix, host, port) {
        return (protocol === 's3' && !provider) ? {bucket, prefix} : {bucket, prefix, host, port, provider};
    }

    test('nfs endPoint object with no path', () => {
        validateStringCreatedFromEndPoint(buildEndPoint('nfs', buildFileSystemDetailObject('nfs', host, nfsExport)), `nfs://${host}/${nfsExport}`);
    });

    test('nfs endPoint object with path', () => {
        validateStringCreatedFromEndPoint(buildEndPoint('nfs', buildFileSystemDetailObject('nfs', host, nfsExport, path)), `nfs://${host}/${nfsExport}/${path}`);
    });

    test('cifs endPoint object with no path', () => {
        validateStringCreatedFromEndPoint(buildEndPoint('cifs', buildFileSystemDetailObject('cifs', host, nfsExport)), `smb://${host}/${nfsExport}`);
    });

    test('cifs endPoint object with path', () => {
        validateStringCreatedFromEndPoint(buildEndPoint('cifs', buildFileSystemDetailObject('cifs', host, nfsExport, path)), `smb://${host}/${nfsExport}/${path}`);
    });

    test('s3 endPoint object with no prefix', () => {
        validateStringCreatedFromEndPoint(buildEndPoint('s3', buildStorageDetailObject('s3', undefined, bucket)), `s3://${bucket}`);
    });

    test('s3 endPoint object with prefix', () => {
        validateStringCreatedFromEndPoint(buildEndPoint('s3', buildStorageDetailObject('s3', undefined, bucket, path)), `s3://${bucket}/${path}`);
    });

    test('sgws endPoint object with no prefix and no port', () => {
        validateStringCreatedFromEndPoint(buildEndPoint('s3', buildStorageDetailObject('s3', 'sgws', bucket, undefined, host)), `sgws://${host}/${bucket}`);
    });

    test('sgws endPoint object with no prefix and port', () => {
        validateStringCreatedFromEndPoint(buildEndPoint('s3', buildStorageDetailObject('s3', 'sgws', bucket, undefined, host, port)), `sgws://${host}:${port}/${bucket}`);
    });
    test('sgws endPoint object with  prefix and port', () => {
        validateStringCreatedFromEndPoint(buildEndPoint('s3', buildStorageDetailObject('s3', 'sgws', bucket, path, host, port)), `sgws://${host}:${port}/${bucket}/${path}`);
    });

});

describe("createSplitPath", () => {
    test("should split to host and path", () => {
        expect(createSplitPath("aaa://bbbb/b/b")).toEqual(["aaa","://bbbb/b/b"]);
    });
    test("should return empty string", () => {
        expect(createSplitPath("")).toEqual([""]);
    });
    test("should return same string when there is no delimiter", () => {
        expect(createSplitPath("aaa")).toEqual(["aaa"]);
    });
});

describe("tagsToString", () => {
    test("should return the right string for a tags array with one tag", () => {
        expect(tagsToString([{key: "key", value: "value"}])).toEqual("key=value");
    });

    test("should return the right string for a tags array with more than one tag", () => {
        expect(tagsToString([{key: "key", value: "value"}, {
            key: "key2",
            value: "value2"
        }])).toEqual("key=value&key2=value2");
    });

    test("should return 'no tags' when there are not tags", () => {
        expect(tagsToString([{key: undefined, value: undefined}])).toEqual("noTags");
    });

    test("should skip empty tags and not include them in the list", () => {
        expect(tagsToString([{key: "key", value: "value"}, {key: undefined, value: undefined}, {
            key: "key2",
            value: "value2"
        }])).toEqual("key=value&key2=value2");
    });
    test("should support empty array", () => {
        expect(tagsToString([])).toEqual("noTags");
    });
    test("should support array with empty objects", () => {
        expect(tagsToString([{}, {}])).toEqual("noTags");
    });
    test("should support array with null objects", () => {
        expect(tagsToString([null, null])).toEqual("noTags");
    });
});

describe("stringToTags", () => {
    test("should return the right array with one object for one tagString", () => {
        expect(stringToTags("key=value")).toEqual([{key: "key", value: "value"}]);
    });

    test("should return the right array with more than one tag for string with more than one tag", () => {
        expect(stringToTags("key=value&key2=value2")).toEqual([{key: "key", value: "value"}, {
            key: "key2",
            value: "value2"
        }]);
    });
});

describe("removeKeyFromPath", () => {
    test("should return the right path - /aa/bb/cc - key in the middle of path", () => {
        expect(removeKeyFromPath("/aa/bb/cc", "/bb")).toEqual("/aa/bb/cc");
    });
    test("should return the right path - /aa/bb/cc - path ends in key", () => {
        expect(removeKeyFromPath("/aa/bb/cc", "/cc")).toEqual("/aa/bb/cc");
    });
    test("should return the right path - /bb/cc - path starts in key", () => {
        expect(removeKeyFromPath("/aa/bb/cc", "/aa")).toEqual("/bb/cc");
    });
    test("should return the right path - /aa/bb/cc- - key is twice in path", () => {
        expect(removeKeyFromPath("/aa/aa/bb/cc", "/aa")).toEqual("/aa/bb/cc");
    });
    test("should return the right path - /bb/cc/aa- key is twice in path", () => {
        expect(removeKeyFromPath("/aa/bb/cc/aa", "/aa")).toEqual("/bb/cc/aa");
    });
    test("should return the right path - key not in path", () => {
        expect(removeKeyFromPath("/bb/cc/aa", "guy")).toEqual("/bb/cc/aa");
    });


});

describe("formatActivityValues", () => {
    const activity = {
        bytesCopied: 5581252300000,
        bytesFailed: 253000000,
        bytesMarkedForCopy: 11163010600000,
        bytesMarkedForGrace: 0,
        bytesMarkedForRemove: 0,
        bytesRemoved: 890,
        bytesRemovedFailed: 0,
        bytesScanned: 5581505300445,
        dirsCopied: 4,
        dirsFailed: 0,
        dirsFailedToScan: 6,
        dirsMarkedForCopy: 4,
        dirsMarkedForGrace: 0,
        dirsMarkedForRemove: 0,
        dirsRemoved: 0,
        dirsScanned: 2,
        filesCopied: 692,
        filesFailed: 4,
        filesMarkedForCopy: 1392,
        filesMarkedForIgnore: 3,
        filesMarkedforRemove: 0,
        filesRemoved: 6,
        filesRemovedFailed: 0,
        filesScanned: 699,
        type: 'Sync',
        executionTime: 92734,
        lastMessageTime: '2019-11-19T09:26:56.586Z',
        status: 'DONE'
    };
    const resultActivityValues = {
        copy: [
            { "name":"Succeeded", "data": {
                    "dirs": {"formatted":"4","label":"Directories","original":4},
                    "files": {"formatted":"692","label":"Files","original":692},
                    "bytes": {"formatted":"5.08","label":"TB","original":5581252300000}},
            },
            { "name": "Failed", "data": {
                    "dirs": {"formatted": "0", "label": "Directories", "original": 0},
                    "files": {"formatted": "4", "label": "Files", "original": 4},
                    "bytes": {"formatted": "241.28", "label": "MB", "original": 253000000}},
            },
            {"name": "Deleted", "data": {
                    "dirs": {"formatted": "0", "label": "Directories", "original": 0},
                    "files": {"formatted": "6", "label": "Files", "original": 6},
                    "bytes": {"formatted": "890", "label": "B", "original": 890}},
            },
        ],
        scan: [
            {"name": "Succeeded", "data": {
                    "dirs": {"formatted": "2", "label": "Directories", "original": 2},
                    "files": {"formatted": "699", "label": "Files", "original": 699},
                    "bytes": {"formatted": "5.08", "label": "TB", "original": 5581505300445}},
            },
            {"name": "Failed", "data": {
                    "dirs": {"formatted": "6", "label": "Directories", "original": 6},
                    "files": {"formatted": null, "label": "Files", "original": null},
                    "bytes": {"formatted": null, "label": "Bytes", "original": null}},
            },
            {"name": "Marked for Copy", "data": {
                    "dirs": {"formatted": "4", "label": "Directories", "original": 4},
                    "files": {"formatted": "1.4k", "label": "Files", "original": 1392},
                    "bytes": {"formatted": "10.15", "label": "TB", "original": 11163010600000}},
            },
            {"name": "Marked for Delete", "data": {
                    "dirs": {"formatted": "0", "label": "Directories", "original": 0},
                    "files": {"formatted": "0", "label": "Files", "original": 0},
                    "bytes": {"formatted": "0", "label": "B", "original": 0}},
            }
        ],
    }
    test("should format the activity values correctly", () => {
        expect(formatActivityValues(activity)).toEqual({
            activityValues: resultActivityValues,
            lastTransferredHeaderText: expect.stringMatching(/^Sync \| Duration: 2 minutes | \|/)
        });
    });

    test("should add the bytesRemovedFailed and FilesRemovedFailed to the failed counters", () => {
        const activityWithFailedRemove = {
            ...activity,
            filesRemovedFailed: 10,
            bytesRemovedFailed: 23454524234
        };
        const resultWithFailedRemoved = resultActivityValues;
        resultWithFailedRemoved.copy[1].data.files = {"formatted": "14", "label": "Files", "original": 14};
        resultWithFailedRemoved.copy[1].data.bytes = {"formatted": "22.08", "label": "GB", "original": 23707524234};

        expect(formatActivityValues(activityWithFailedRemove)).toEqual({
            activityValues: resultWithFailedRemoved,
            lastTransferredHeaderText: expect.stringMatching(/^Sync \| Duration: 2 minutes | \|/)
        });
    });

});

describe("getSelectOptions", () => {
    test("should build select options array for strings", () => {
        const input = ['a', 'b', 'c'];
        expect(getSelectOptions(input)).toEqual([{label: 'a', value: 'a'}, {label: 'b', value: 'b'}, {label: 'c', value: 'c'}]);
    });
    test("should build select options array for numbers", () => {
        const input = [0, 1, 2];
        expect(getSelectOptions(input)).toEqual([{label: '0', value: 0}, {label: '1', value: 1}, {label: '2', value: 2}]);
    });
});

describe("convertSizeToBytes", () => {
    test("should convert the size to bytes", () => {
        expect(convertSizeToBytes('999', 'GB')).toEqual(1072668082176);
        expect(convertSizeToBytes('0', 'KB')).toEqual(0);
        expect(convertSizeToBytes('0.4', 'TB')).toEqual(439804651110);
    });
});

describe("convertBytesToSize", () => {
    test("should convert bytes to size", () => {
        expect(convertBytesToSize(1072668082176)).toEqual({value: '999', resolution: 'GB'});
        expect(convertBytesToSize(0)).toEqual({value: '0', resolution: 'B'});
        expect(convertBytesToSize(439804651110)).toEqual({value: '409.6', resolution: 'GB'});
        expect(convertBytesToSize(4886229673836544)).toEqual({value: '4.34', resolution: 'PB'});
    });
});