import * as licenseUtils from "./../../src/utils/license-utils";

describe("licenseUtils", () => {

    // tested according to docs/ui/license-status-footer-truth-table.xlsx
    describe("calculateLicenseStatusFooter", () => {
        test("should return LOADING when store is not ready", () => {
            const result = licenseUtils.calculateLicenseStatusFooter(undefined);
            expect(result).toEqual({
                licenseStatusText: "LOADING...",
                warning: false,
                error: false,
                title: ""
            });
        });


        // (# row numbers according to truth table on SER-1635)

        test("should return TRIAL when on trial and has license and subscribed (#1)", () => {
            const store = {
                auth: {succeeded: true, data: {daysLeftForTrial: 101, subscribed: true}},
                _licenseSettings: {succeeded: true, data: {totalReservedRelationships: 20, needToPay: false}}
            };
            const result = licenseUtils.calculateLicenseStatusFooter(store);
            expect(result).toEqual({
                licenseStatusText: "FREE TRIAL [101 Days left]",
                warning: false,
                error: false,
                title: ""
            });
        });

        test("should return ACTIVE when trial has ended and user has license and subscribed (#2)", () => {
            const store = {
                auth: {succeeded: true, data: {daysLeftForTrial: 0, subscribed: true}},
                _licenseSettings: {succeeded: true, data: {totalReservedRelationships: 20, needToPay: false}}
            };

            const result = licenseUtils.calculateLicenseStatusFooter(store);

            expect(result).toEqual({
                licenseStatusText: "ACTIVE SUBSCRIPTION",
                warning: false,
                error: false,
                title: ""
            });

        });

        test("should return ACTIVE when trial has ended and user has license and subscribed (#2 even when exceeded relationships)", () => {
            const store = {
                auth: {succeeded: true, data: {daysLeftForTrial: 0, subscribed: true}},
                _licenseSettings: {succeeded: true, data: {totalReservedRelationships: 20, needToPay: false}}
            };

            licenseUtils.rewire$checkLicensesCapacity(() => ({reachedLicensesCapacity: true}));

            const result = licenseUtils.calculateLicenseStatusFooter(store);

            licenseUtils.restore();

            expect(result).toEqual({
                licenseStatusText: "ACTIVE SUBSCRIPTION",
                warning: false,
                error: false,
                title: ""
            });

        });

        test("should return TRIAL when on trial and not licensed but subscribed (#3)", () => {
            const store = {
                auth: {succeeded: true, data: {daysLeftForTrial: 104, subscribed: true}},
                _licenseSettings: {succeeded: true, data: {needToPay: false}}
            };
            const result = licenseUtils.calculateLicenseStatusFooter(store);
            expect(result).toEqual({
                licenseStatusText: "FREE TRIAL [104 Days left]",
                warning: false,
                error: false,
                title: ""
            });
        });

        test("should return ACTIVE when trial has ended and user is subscribed (#4)", () => {
            const store = {
                auth: {succeeded: true, data: {daysLeftForTrial: 0, subscribed: true}},
                _licenseSettings: {succeeded: true, data: {needToPay: false}}
            };
            const result = licenseUtils.calculateLicenseStatusFooter(store);
            expect(result).toEqual({
                licenseStatusText: "ACTIVE SUBSCRIPTION",
                warning: false,
                error: false,
                title: ""
            });
        });

        test("should return warning TRIAL when on trial and user has license but exceeded its relationships capacity and not subscribed (#5)", () => {
            const store = {
                auth: {succeeded: true, data: {daysLeftForTrial: 106, subscribed: false}},
                _licenseSettings: {succeeded: true, data: {totalReservedRelationships: 20, needToPay: false}}
            };

            licenseUtils.rewire$checkLicensesCapacity(() => ({reachedLicensesCapacity: true}));

            const result = licenseUtils.calculateLicenseStatusFooter(store);

            licenseUtils.restore();

            expect(result).toEqual({
                licenseStatusText: "FREE TRIAL [106 Days left]",
                warning: true,
                error: false,
                title: ""
            });

        });

        test("should return error SUBSCRIPTION REQUIRED when trial ended, user has license but exceeded its capacity and not subscribed (#6)", () => {
            const store = {
                auth: {succeeded: true, data: {daysLeftForTrial: 0, subscribed: false}},
                _licenseSettings: {succeeded: true, data: {totalReservedRelationships: 20, needToPay: true}}
            };
            const result = licenseUtils.calculateLicenseStatusFooter(store);
            expect(result).toEqual({
                licenseStatusText: "SUBSCRIPTION REQUIRED",
                warning: false,
                error: true,
                title: ""
            });
        });

        test("should return warning TRIAL when on trial and not licensed and not subscribed (need to pay) (#7)", () => {
            const store = {
                auth: {succeeded: true, data: {daysLeftForTrial: 108, subscribed: false}},
                _licenseSettings: {succeeded: true, data: {needToPay: true}}
            };
            const result = licenseUtils.calculateLicenseStatusFooter(store);
            expect(result).toEqual({
                licenseStatusText: "FREE TRIAL [108 Days left]",
                warning: true,
                error: false,
                title: ""
            });
        });

        test("should return error SUBSCRIPTION REQUIRED when trial has ended and user is not subscribed and has no license (need to pay) (#8)", () => {
            const store = {
                auth: {succeeded: true, data: {daysLeftForTrial: 0, subscribed: false}},
                _licenseSettings: {succeeded: true, data: {needToPay: true}}
            };
            const result = licenseUtils.calculateLicenseStatusFooter(store);
            expect(result).toEqual({
                licenseStatusText: "SUBSCRIPTION REQUIRED",
                warning: false,
                error: true,
                title: ""
            });
        });

        test("should return TRIAL when on trial and user has license but didn't exceed its relationships capacity and not subscribed (#9)", () => {
            const store = {
                auth: {succeeded: true, data: {daysLeftForTrial: 114, subscribed: false}},
                _licenseSettings: {succeeded: true, data: {totalReservedRelationships: 20, needToPay: false}}
            };

            const result = licenseUtils.calculateLicenseStatusFooter(store);

            expect(result).toEqual({
                licenseStatusText: "FREE TRIAL [114 Days left]",
                warning: false,
                error: false,
                title: ""
            });

        });

        test("should return ACTIVE when trial has ended and user has license but didn't exceed its relationships capacity and not subscribed (#10)", () => {
            const store = {
                auth: {succeeded: true, data: {daysLeftForTrial: 0, subscribed: false}},
                _licenseSettings: {succeeded: true, data: {totalReservedRelationships: 20, needToPay: false}}
            };

            const result = licenseUtils.calculateLicenseStatusFooter(store);

            expect(result).toEqual({
                licenseStatusText: "ACTIVE SUBSCRIPTION",
                warning: false,
                error: false,
                title: ""
            });

        });

        test("should return error SUBSCRIPTION REQUIRED when trial has ended and user has no license and not subscribed even without exceeding its relationships capacity (no need to pay) (#11)", () => {
            const store = {
                auth: {succeeded: true, data: {daysLeftForTrial: 0, subscribed: false}},
                _licenseSettings: {succeeded: true, data: {needToPay: false}}
            };

            const result = licenseUtils.calculateLicenseStatusFooter(store);

            expect(result).toEqual({
                licenseStatusText: "SUBSCRIPTION REQUIRED",
                warning: false,
                error: true,
                title: ""
            });

        });


        test("should return warning ACTIVE when trial has ended and user is not subscribed and has license and just reached its relationships capacity (no need to pay yet)", () => {
            const store = {
                auth: {succeeded: true, data: {daysLeftForTrial: 0, subscribed: false}},
                _licenseSettings: {succeeded: true, data: {totalReservedRelationships: 20, needToPay: false}}
            };

            licenseUtils.rewire$checkLicensesCapacity(() => ({reachedLicensesCapacity: true}));

            const result = licenseUtils.calculateLicenseStatusFooter(store);

            licenseUtils.restore();

            expect(result).toEqual({
                licenseStatusText: "ACTIVE SUBSCRIPTION",
                warning: true,
                error: false,
                title: ""
            });


        });

        test("should return error TRIAL when user needs to pay and trial ends in less than a week", () => {
            const store = {
                auth: {succeeded: true, data: {daysLeftForTrial: 7, subscribed: false}},
                _licenseSettings: {succeeded: true, data: {needToPay: true}}
            };
            const result = licenseUtils.calculateLicenseStatusFooter(store);
            expect(result).toEqual({
                licenseStatusText: "FREE TRIAL [7 Days left]",
                warning: true,
                error: true,
                title: ""
            });
        });

        test("should return FREE OF CHARGE", () => {
            const store = {
                auth: {succeeded: true, data: {daysLeftForTrial: 7, subscribed: false, noCharge: true}},
                _licenseSettings: {succeeded: true, data: {needToPay: true}}
            };
            const result = licenseUtils.calculateLicenseStatusFooter(store);
            expect(result).toEqual({
                licenseStatusText: "FREE OF CHARGE",
                warning: false,
                error: false,
                title: "There is no charge for sync relationships that include Azure NetApp Files or Cloud Volumes Service."
            });
        });
    });

    describe("checkLicensesCapacity", () => {

        test("should return false when store is not ready", () => {
            const result = licenseUtils.checkLicensesCapacity(undefined);
            expect(result).toEqual({
                reachedLicensesCapacity: false,
                licensesCapacityWarning: ''
            });
        });

        test("should return false when there's no license", () => {
            const store = {
                _licenseSettings: {succeeded: true, data: {existingRelationships: 2, totalReservedRelationships: 0}}
            };
            const result = licenseUtils.checkLicensesCapacity(store);
            expect(result).toEqual({
                reachedLicensesCapacity: false,
                licensesCapacityWarning: ''
            });
        });

        test("should return false when there are less relationships than the capacity of a license", () => {
            const store = {
                _licenseSettings: {succeeded: true, data: {existingRelationships: 2, totalReservedRelationships: 3}}
            };
            const result = licenseUtils.checkLicensesCapacity(store);
            expect(result).toEqual({
                reachedLicensesCapacity: false,
                licensesCapacityWarning: ''
            });
        });

        test("should return true when there's the same number of relationships as the capacity of the licenses", () => {
            const store = {
                _licenseSettings: {succeeded: true, data: {existingRelationships: 3, totalReservedRelationships: 3}}
            };
            const result = licenseUtils.checkLicensesCapacity(store);
            expect(result).toEqual({
                reachedLicensesCapacity: true, licensesCapacityWarning: `You have reached the maximum number of relationships that your licenses provide.\n`
                + `To create a new relationship, go to the License Settings page and either subscribe to Cloud Sync from the AWS Marketplace or add another BYOL license.`
            });
        });

        test("should return true when there are more relationships than the capacity of the licenses", () => {
            const store = {
                _licenseSettings: {succeeded: true, data: {existingRelationships: 4, totalReservedRelationships: 3}}
            };
            const result = licenseUtils.checkLicensesCapacity(store);
            expect(result).toEqual({
                reachedLicensesCapacity: true, licensesCapacityWarning: `You created 4 sync relationships, but you have licenses for only 3.\n` +
                `To pay for the remaining 1 relationship, go to the License Settings page and either subscribe to Cloud Sync from the AWS Marketplace or add another BYOL license.`
            });
        });

    });

});